#!/usr/bin/perl

############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

###############################################################################=pod

=head1 admin_object.cgi

admin_object.cgi - Content Manger Administrator

=head1 SYNOPSIS

http://YOURDOMAIN/cgi-bin/om/admin/admin_object.cgi

=head1 DESCRIPTION

This script is responsible for displaying and submitting information for
creation and modification of objects.  

=head1 NOTES & BUGS

=cut
###############################################################################

&main();


##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main driver is responsible for determining the flow of the admin_object
script.  It displays all pages dependant on the mode that is passed
in by the FORM.  The templates are loaded dependant of the mode and the mode
is changed to prepare for submission after it is loaded.  Handles search, 
add/modify prices, add/modify admin_objects, and/modify inventory.

The flow of the main driver is to do any functional processing first, and then
change the mode to a display mode to display the next form.

=item INPUT

NONE

=item OUTPUT

The FORM is populated and the display is posted according the the choices made
by the user.

=item NOTES

The main processing of the FORM data is done in os_allcommerce/lib/admin_object.pm

=back

=cut
###############################################################################
sub main {
    my ($q, $numRows, @lstock, $admin_object, $stock, @pulldowns, $v, $l, $spine);
    my (%FORM, $site, $dbh, $select_hash, $object_info, $error);
    
    ($site, $dbh, %FORM) = &initscript();

    &setup_session(\%FORM, $site, $dbh);

    &debug("After initscript", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if (!$FORM{'mode'}) {
	$FORM{'mode'} = "home";
    }

    # Take the input from the form, and permform a search for the requested data, and then
    # display the search results
    if ($FORM{'mode'} eq "search") {
	&debug("Before do_admin_object_search, set previous start row to $FORM{'previous_startrow'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&do_admin_object_search(\%FORM, $site, $dbh);
    }

    # Take the input from the form, and create an object in the tables
    if ($FORM{'mode'} eq "create_object") {
	$error = &create_object(\%FORM, $site, $dbh);
	if (defined $error) {
	    if ($FORM{'page_error'} eq "") {
		$FORM{'page_error'} = $error;
	    }
	    $FORM{'new_object'} = "yes";   # this is still a new object!  Hasn't been created yet!
	}
	&debug("Just created $FORM{'objid'}", 3, $site->{'debug_log'}, $site->{'debug_level'});
	$FORM{'mode'} = "display_object";
    }

    # Take the input from the form, and update the object in the tables
    if ($FORM{'mode'} eq "update_object") {
	$error = &update_object($FORM{'objid'}, \%FORM, $site, $dbh);
	if (defined $error) {
	    if ($FORM{'page_error'} eq "") {
		$FORM{'page_error'} = $error;
	    }
	}
	&debug("Just updated $FORM{'objid'}", 3, $site->{'debug_log'}, $site->{'debug_level'});
	$FORM{'mode'} = "display_object";
    }

    # This screen can have multiple rows, some update and some inserts
    # need to sort them into new and modifying information, and update
    # or create as required
    if ($FORM{'mode'} eq "update_rows") {
	$object_info = &get_object_info(\%FORM);
	$error = &update_rows(\%FORM, $object_info, $site, $dbh);
	if ($FORM{'page_error'} eq "") {
	    $FORM{'page_error'} = $error;
	}
	&debug("Just updated $FORM{'objid'}", 3, $site->{'debug_log'}, $site->{'debug_level'});
	$FORM{'mode'} = "display_object";
    }

    # Attach or detach the requested attributes to/from the object, and then
    # redisplay the attach/detatch option in case there is more to attach or
    # detach
    if ($FORM{'mode'} eq "attach_detach") {
	$object_info = &get_object_info(\%FORM);
	$error = &update_late_binding_attributes(\%FORM, $object_info, $site, $dbh);
	if ($FORM{'page_error'} eq "") {
	    $FORM{'page_error'} = $error;
	}
	&debug("Just updated $FORM{'objid'}", 3, $site->{'debug_log'}, $site->{'debug_level'});
	$FORM{'mode'} = "display_attach";

    }
    # Show the attach/detach screen to attach or detach attributes to an actual object
    if ($FORM{'mode'} eq "display_attach") {
	$object_info = &get_object_info(\%FORM);
	$object_info->{'gui_name'} = "attach_attributes_gui";
	&display_update_attributes(\%FORM, $object_info, $site, $dbh);
    }

    # Show the object, in either edit mode or create mode
    if ($FORM{'mode'} eq "display_object") {
	$object_info = &get_object_info(\%FORM);
	&display_object(\%FORM, $object_info, $site, $dbh);
    }

    # Show the home page
    if ($FORM{'mode'} eq "home") {
	&debug("Before display_home", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&display_home(\%FORM, $site);
	&debug("After display_home", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    $dbh->disconnect if ( $dbh ) ;
}

##############################################################################
=pod

=head2 display_object

display_object - &display_object(\%FORM, $object_info, $site, $dbh);

=over 4

=item DESCRIPTION

Objects that already exist are loaded in, and displayed according the the 
current render language, the user permissions for the object and attributes.

If it is a new object, the FORM is displayed to the user, with all attributes
that the user has permissions to in a INPUT form, and all others in read only 
form.  If the object already exists, the FORM is displayed to the user with
the object loaded, all attributes that the user has permissions to edit
are in a INPUT form, read only attributes are displayed and attributes that
are excluded are not displayed.

If this is a new object, the mode is set to create_object, otherwise it is
set to update_object.

All appropriate links are displayed on the the spine of the display.

=item INPUT

$FORM - Pointer to the FORM

$object_info - Hashref with flags of information.  See &get_object_info in
               admin_object.pm OUTPUT to see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

FORM output to the browser.  Mode is set to update_object or create_object.

=item NOTES

=back

=cut
###############################################################################
sub display_object {
    my ($FORM) = shift;
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;


    my ($object_hashref, $template, $object_body, $template, @object_rows, $edit);
    my ($enter_button, $attach_button, $attach_perms, @rows, $title, $desc, @select_array, $permute_with);
    my ($price_link, $inventory_link, $back_to_search, $search_link,@tmp);
    my (@checkbox_array, $key);


    &debug("display_object:  In with objid $object_inf->{'objid'}, objclass $object_inf->{'objclass'}, objtype $object_inf->{'objtype'}, page error is $$FORM{'page_error'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($object_inf, 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($object_inf->{'new_object'}) {
	$permute_with = &set_new_defaults($FORM, $site);
    }

    if ($object_inf->{'master_objid'} ne "") {
	&debug("display_object: Have master object $object_inf->{'master_objid'}, need to modify variants", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$object_inf->{'variants'} = "yes";
	$object_inf->{'new_object'} = "yes";
	# this is variant, so deal with it
	return (&display_variant_products($FORM, $object_inf, $site, $dbh));
    }

    # There was an error in the input, to just regenerate from the page
    if ($$FORM{'page_error'} ne "") {
	$permute_with = &age_form($FORM, $site);
	foreach $key (keys %{$permute_with}) {
	    if (($key =~ /axis/) && ($key =~ /_pre/)) {
		$permute_with->{$key} = "";
	    }

	}
        $permute_with->{'previous_error'} = "yes";			     
    }

    # It was successful, or the first time so get the object from the database
    else {

	if (defined $object_inf->{'objid'}) {
	    if ($$FORM{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'contact'}) {
		$object_hashref = &load_object($object_inf->{'objid'}, "contacts", "", \@tmp, $site, $dbh, 1);
	    }
	    # default to things
	    else {
		$object_hashref = &load_object($object_inf->{'objid'}, "things", "", \@tmp, $site, $dbh, 1);
	    }

	    $permute_with = &get_hashref_to_permute($object_hashref->{'c'}, $site, $object_inf);
	}
	&debug("display_object: Is this a variant product?  Do I have a vcode?  $permute_with->{'vcode_pre'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug("display_object: Is this a master product?  Do I have a axisnames?  $permute_with->{'axisnames_pre'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	
	# This is a variant Product, it has already been permuted, so use the _pre values
	if ($permute_with->{'vcode_pre'} ne "") {
	    $object_inf->{'master_objid'} = $permute_with->{'ref_from_pre'};
#	    $object_inf->{'objid'} = "";
	    $object_inf->{'variants'} = "yes";
	    $object_inf->{'new_object'} = "yes";
	    # this is variant, so deal with it
	    return (&display_variant_products($FORM, $object_inf, $site, $dbh));
	}
	
	# This is a master product, so flag it
	if ($permute_with->{'axisnames_pre'} ne "") {
	    $object_inf->{'master_objid'} = $permute_with->{'objid_pre'};
	}

    }


    $template = &load_main_admin_template("os_allcommerce-admin_object-object", $site);

    $object_body = &build_object_template($object_inf, $site, $dbh, \@select_array, \@checkbox_array);

    # Have something to enter in, so put the enter changes button on there
#    if ((length($object_body) > 0) || ($object_inf->{'need_enter_button'} eq "YES")) {
    if ($object_inf->{'need_enter_button'} eq "YES") {
	my @rows = ('row_buttons');
	$template = &permute_rows($template, $site, \@rows);
    }

#    $enter_button = &localize_template("row_enter_button", "admin", 0, $site);
#    $template =~ s/__row_enter_button__/$enter_button/gms;
    
    # put in the body
    $template =~ s/__body__/$object_body/gms;
    if (defined $permute_with) {
	$template = &permutebit($template, $permute_with);
    }

    &initialize_selects($FORM, $object_inf, $permute_with, $site, \@select_array);
    &debug("AFTER initialize_selects", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &initialize_checkboxes($FORM, $permute_with, $template, $site, \@checkbox_array);
    &debug("AFTER initialize_checkboxes", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($object_inf->{'new_object'}) {
	$$FORM{'mode'} = "create_object";
    }
    else {
	$$FORM{'mode'} = "update_object";
    }

    $attach_perms = &get_attribute_permissions($site->{'OBJCLASSES'}->{'attribute'}, $site);
    &debug("display_object: $attach_perms is permissions for attributes, mode is $$FORM{'mode'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ((($attach_perms !~ /x/) && ($attach_perms =~ /m/)) 
	&& ($$FORM{'mode'} eq "update_object")) {
	&debug("display_object: $attach_perms is permissions for attributes, setting attach_link to yes", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$object_inf->{'attach_link'} = "yes";
    }

    $template = &set_links($object_inf, $FORM, $template, $site);
    &debug("AFTER set_links", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &permutebit($template, $FORM);
    $template = &set_titles($object_inf, $FORM, $template, $site);
    &debug("AFTER set_titles", 3, $site->{'debug_level'}, $site->{'debug_log'});


    $template = &permutebit($template, $site);

    $template = &permuteclean($template, $site);

    &debug("Going to print template $template", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $template = &set_headers("content", $template, $site);

    print $template;
}

##############################################################################
=pod

=head2 set_new_defaults

set_new_defaults - &set_new_defaults($FORM, $site)

=over 4

=item DESCRIPTION

To set the appropriate fields to default values before displaying the object.

=item INPUT

$FORM - Pointer to the FORM

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$permute_with - A hashref to permute with, with the default values set.

=item NOTES

=back

=cut
###############################################################################
sub set_new_defaults {
    my ($FORM) = shift;
    my ($site) = shift;

    my ($permute_with);

    $permute_with->{'status_pre'} = "15000-A";

    if ($$FORM{'objtype_inp'} ne $site->{'OBJTYPES'}->{'internal_doc'}) {
	$permute_with->{'visibility_pre'} = "110000-A";
    }
    else {
	$permute_with->{'visibility_pre'} = "110001-A";
    }

    if ($$FORM{'objtype_inp'} eq $site->{'OBJTYPES'}->{'product'}) {
	$permute_with->{'behavior_pre'} = "90002-A";
    }
    return ($permute_with);
}

##############################################################################
=pod

=head2 display_update_attributes

display_update_attributes - &display_update_attributes(\%FORM, $object_info, $site, $dbh);

=over 4

=item DESCRIPTION

In order to attach attributes to individual objects, it is necessary to provide
the user with a FORM which allows them to select pre-defined attributes and 
attach them to objects.  This routine displays the FORM, showing all attributes
that the user is allowed to detach from the object, and giving them a selection 
of attributes tha they can attach to the object.  Once displayed, the mode is
changed to attach_detach to wait for input from the user.  

All appropriate links are displayed on the the spine of the display.

=item INPUT

$FORM - Pointer to the FORM

$object_info - Hashref with flags of information.  See &get_object_info in
               admin_object.pm OUTPUT to see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

FORM output to the browser.  Mode is set to attach_detach.

=item NOTES

=back

=cut
###############################################################################
sub display_update_attributes {
    my ($FORM) = shift;
    my ($object_info) = shift;
    my ($site) = shift;
    my ($dbh) = shift;


    my ($class_defined, $can_detach, $can_attach, $lang_order);
    my (@WHERE, $num_attributes, $next_start, @attributes, $where_indx, $all_attributes, $attached_sel);
    my ($detach_sel, $permute_with, @select_array, @checkbox_array, $object_body, $tmp_info);
    my (@attach_raw, @detach_raw, $next_attribute, $num_detach, $num_attach, $field, $template, $enter_button);

    &debug("display_update_attributes:  In with objid $object_info->{'objid'}, objclass $object_info->{'objclass'}, objtype $object_info->{'objtype'}, page error is $$FORM{'page_error'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    ($class_defined, $can_detach, $can_attach) = &load_late_binding_attributes($object_info, $site, $dbh);

    $where_indx = 0;
    $WHERE[$where_indx++] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'attribute'}, "and");
    ($num_attributes, $next_attribute, @attributes) = &load_objects("things", "title,brief,objid", \@WHERE, "", "", $site, $dbh, $site->{'override_permissions'});
    $all_attributes = &format_array_into_hashref("title", \@attributes);
    
    $num_detach = $num_attach = 0;
    foreach $field (keys %{$can_detach}) {
	next if ((!defined $all_attributes->{$field}) || ($all_attributes->{$field}->{'brief'} eq ""));
	push @detach_raw, "$field|$all_attributes->{$field}->{'brief'}";
	$num_detach++;
    }

    foreach $field (keys %{$can_attach}) {
	next if ((!defined $all_attributes->{$field}) || ($all_attributes->{$field}->{'brief'} eq ""));
	push @attach_raw, "$field|$all_attributes->{$field}->{'brief'}";
	$num_attach++;
    }

    $permute_with->{'detach'} = &build_html_select(\@detach_raw, "detach", "", "cell",  "MULTIPLE SIZE=10",  $site);	
    $permute_with->{'detach_tag'} = $site->{'script_msg'}{'detach_row_tag'};
    $permute_with->{'attach'} = &build_html_select(\@attach_raw, "attach", "", "cell", "MULTIPLE SIZE=10",  $site);
    $permute_with->{'attach_tag'} = $site->{'script_msg'}{'attach_row_tag'};

    $template = &load_main_admin_template("os_allcommerce-admin_object-object", $site);
    $tmp_info = {};
    $tmp_info->{'gui_name'} = $object_info->{'gui_name'};
    $tmp_info->{'objclass'} = $object_info->{'objclass'};
    $tmp_info->{'objtype'} = $object_info->{'objtype'};
    $object_body = &build_object_template($tmp_info, $site, $dbh, \@select_array, \@checkbox_array);

    my $attach_perms = &get_attribute_permissions($site->{'OBJCLASSES'}->{'attribute'}, $site);
    if ((($attach_perms !~ /x/) && ($attach_perms =~ /m/))) {
	my @rows = ('row_buttons');
	$template = &permute_rows($template, $site, \@rows);
    }

    &debug("The body that was built for $tmp_info->{'gui_name'} was $object_body", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
#    $enter_button = &localize_template("row_enter_button", "admin", 0, $site);
#    $template =~ s/__row_enter_button__/$enter_button/gms;
    
    # put in the body
    $template =~ s/__body__/$object_body/gms;
    if (defined $permute_with) {
	$template = &permutebit($template, $permute_with);
    }


    &initialize_selects($FORM, $object_info, $permute_with, $site, \@select_array);
    &debug("AFTER initialize_selects", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &initialize_checkboxes($FORM, $permute_with, $template, $site, \@checkbox_array);
    &debug("AFTER initialize_checkboxes", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $$FORM{'mode'} = "attach_detach";

    $template = &set_links($object_info, $FORM, $template, $site);
    &debug("AFTER set_links", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &permutebit($template, $FORM);
    $template = &set_titles($object_info, $FORM, $template, $site);
    &debug("AFTER set_titles", 3, $site->{'debug_level'}, $site->{'debug_log'});


    $template = &permutebit($template, $site);
    $template = &permuteclean($template, $site);

    &debug("Going to print template $template", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &set_headers("content", $template, $site);
    print $template;
}


##############################################################################
=pod

=head2 display_variant_products

=over 4

display_variant_products - &display_variant_products(\%FORM, $object_inf, $site, $dbh);

=item DESCRIPTION

Variant Products are different than most objects in AllCommerce.  On one display,
we are adding new variant products as well as updating old ones.  This is
one instance where we require multiple rows, updating some, inserting some,
and doing nothing to others.  

Each time that you have a multiple row generation, it is necessary to create it's
own display routine.  There are 'rows' utilities available to ease this (like
generating unique tags for each row, and mode 'update_rows'), but it is still 
necessary to write a distinct display routine.

This routine finds all of the existing variant products for the master product,
and displays them based on the users permissions.  Then, a row for creating
new variant products is provided based on the users permissions.  The FORM is
set to 'update_rows' mode where it awaits submition of the rows data.

=item INPUT

$FORM - Pointer to the FORM

$object_info - Hashref with flags of information.  See &get_object_info in
               admin_object.pm OUTPUT to see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

FORM output to the browser.  Mode is set to update_rows.

=item NOTES

=back

=cut
###############################################################################
sub display_variant_products {
    my ($FORM) = shift;
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($num_variant_products, @variant_products, $permute_with, $template, @rows);
    my ($object_body, @select_array,@checkbox_array, $enter_button, $attach_perms);
    my ($variant_product, $master_hashref, @where, @variational_axis, $num_axis);

    my ($axis_select, $obj_perms);

    $obj_perms = &get_attribute_permissions($object_inf->{'objtype'}, $site);
    if ($obj_perms =~ /x/) {
	return "";
    }

    &debug("display_variant_products:  In with master objid $object_inf->{'master_objid'}, page error is $$FORM{'page_error'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # There was an error in the input, to just regenerate from the page
    if ($$FORM{'page_error'} ne "") {
	$permute_with = &age_form($FORM, $site);
    }

    $$FORM{'master_objid'} = $object_inf->{'master_objid'};
    # Get the master object information
    $master_hashref = &load_default_object($object_inf->{'master_objid'}, "things", "title,brief,axisnames,language", \@where, $site, $dbh);
    &debug("load_default_objects:  Master info: $master_hashref->{'title'}, $master_hashref->{'brief'}, $master_hashref->{'axisnames'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    
    # Only display if we are allowed to modify things
    if ($obj_perms =~ /c/) {
         # Display a row of new data to define
	$object_inf->{'row_number'} = "0";
	$object_body .= &build_header("<header>:align=left|$site->{'script_msg'}{'new_vars'}", $site);
	$object_body .= &display_new_variant_products($master_hashref, $object_inf, $site, $dbh, \@select_array, \@checkbox_array);
    }
    
    
    # Get the existing varianat products & display them
    $num_variant_products = &get_existing_variant_products($object_inf->{'master_objid'}, $site, $dbh, \@variant_products);
    &debug("display_variant_products:  There are $num_variant_products variant products", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($num_variant_products > 0) {
	$object_body .= &build_header("<header>:align=left|$site->{'script_msg'}{'existing_vars'}", $site);
	$object_body .= &display_existing_variant_products($num_variant_products, \@variant_products, $master_hashref, $object_inf, $site, $dbh);
    }
    
    $template = &load_main_admin_template("os_allcommerce-admin_object-object", $site);
    # See if we need an enter button
    if ($object_inf->{'need_enter_button'} eq "YES") {
	my @rows = ('row_buttons');
	$template = &permute_rows($template, $site, \@rows);
    }

#    $enter_button = &localize_template("row_enter_button", "admin", 0, $site);
#    $template =~ s/__row_enter_button__/$enter_button/gms;
    
    # put in the body
    $template =~ s/__body__/$object_body/gms;
    if (defined $permute_with) {
	$template = &permutebit($template, $permute_with);
    }

    &initialize_selects($FORM, $object_inf, $permute_with, $site, \@select_array);
    $template = &initialize_checkboxes($FORM, $permute_with, $template, $site, \@checkbox_array);

    $$FORM{'mode'} = "update_rows";

    $template = &set_links($object_inf, $FORM, $template, $site);
    &debug("AFTER set_links", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &permutebit($template, $FORM);
    # editing variant products flag
    $object_inf->{'variants'} = "yes";
    $template = &set_titles($object_inf, $FORM, $template, $site);
    &debug("AFTER set_titles", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # Get rid of non-initialized space holders, as for some reason, they get
    # permuted with junk from the $site permutebit call
    $template =~ s/__axis0__//gms;
    $template =~ s/__axis1__//gms;
    $template =~ s/__axis2__//gms;
    $template =~ s/__axis3__//gms;

    $template = &permutebit($template, $site);
    $template = &permuteclean($template, $site);
    $template = &set_headers("content", $template, $site);
    print $template;
}

##############################################################################
=pod

=head2 display_new_variant_products

=over 4

display_new_variant_products - &display_new_variant_products($master_hashref, $object_inf, $site, $dbh, \@select_array, \@check_array);

=item DESCRIPTION

This creates the row to define the new variant products.  Variant products are also
unique in the fact that they require automatic SELECT pulldown generation based
on their Variational Sets.  This routine determine what Variants are used by the
definition of the master product, and dynamically builds the appropriate variant
SELECT input fields.  It also displays the other information associated with a 
variant product.

=item INPUT

$master_hashref - Hashref with the master objid loaded into memory (via load_object).

$object_inf - Hashref with flags of information.  See &get_object_info in
               admin_object.pm OUTPUT to see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$select_array - Pointer to an array, where all SELECT items in the new variant
                product will be recorded, to flag the calling display routine
                to call initialize_selects.

$check_array - Pointer to an array, where all check box items in the new variant
                product will be recorded, to flag the calling display routine
                to call initialize_check_boxes.

=item OUTPUT

Builds the body for the creation of a new variant product.  It is then added
to the display_variant_product template.

=item NOTES

TARA, do we need select_array or check_box array?

=back

=cut
###############################################################################
sub display_new_variant_products {
    my ($master_hashref) = shift;
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($select_array) = shift;
    my ($check_array) = shift;

    my ($object_body, $variant_product, @where, $axis_select, @variation_axis, $num_axis, @vcodes);
    my ($i, @variational_axis, @select_array, @checkbox_array, $permute_with, %tmp_form, $permute_with);
    my ($tag, $saved_objid);
 
    # Get the axisnames from the master, these are the keys to the variational sets that we need to retreive to build the pulldowns
    $saved_objid = $object_inf->{'objid'};
    $object_inf->{'objid'} = "";
    @variational_axis = split /\:/, $master_hashref->{'axisnames'};
    $num_axis = $#variational_axis + 1;
    &debug("display_new_variant_products: There are $num_axis variational axis.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $object_body = "";
    # load in each of the existing ones
    # build the row template
    $object_inf->{'gui_name'} = $object_inf->{'new'};
    $object_inf->{'row_number'} = "0";
    $object_inf->{'additional_cols'} = $num_axis;  # flag that there are additional columns to consider
    $object_body .= &build_object_template($object_inf, $site, $dbh, $select_array, $check_array);
    &debug("display_new_variants: **********passing into initialize_axis_selects\n\n$object_body\n\n", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # reset it back one to permute everything for this 'row'
    $object_inf->{'row_number'} =  $object_inf->{'row_number'} - 1;
    $object_body = &initialize_axis_selects($object_body, $object_inf,"", $num_axis, \@variational_axis, $site, $dbh);
    &debug("display_new_variant_products: Going into initialize_select.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $tag = &get_row_number("status_pre", $object_inf);
    $permute_with->{$tag} = "15000-A";
    &debug("display_new_variant_products: Just set $tag = $permute_with->{$tag}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $tag = &get_row_number("visibility_pre", $object_inf);
    $permute_with->{$tag} = "110000-A";
    &debug("display_new_variant_products: Just set $tag = $permute_with->{$tag}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
    $tag = &get_row_number("ship_weight_pre", $object_inf);
    $permute_with->{$tag} = "";
    $tag = &get_row_number("weight_man_pre", $object_inf);
    $permute_with->{$tag} = "";

    &initialize_selects(\%tmp_form, $object_inf, $permute_with, $site, $select_array);
    $object_body = &permutebit($object_body, $permute_with);
    $object_body = &permutebit($object_body, \%tmp_form);

    # put it back to where it was
    $object_inf->{'row_number'} =  $object_inf->{'row_number'} + 1;

    $object_inf->{'objid'} = $saved_objid;
    return $object_body;
}

##############################################################################
=pod

=head2 display_existing_variant_products

=over 4

display_existing_variant_products - &display_existing_variant_products(12, \@variants, $master_hashref, $object_inf, $site, $dbh);

=item DESCRIPTION

This creates the rows of existing variant products, based on the render language and
user permissions. The existing variant products are displayed and the appropriate attributes
are set up for input.

=item INPUT

$num_variants - The number of variants that already exist for the master product.

$variant_products - A pointer to an array of variant products that belong to the
                    master product, all which have been loaded into memory via load_objects.

$master_hashref - Hashref with the master objid loaded into memory (via load_object).

$object_inf - Hashref with flags of information.  See &get_object_info in
               admin_object.pm OUTPUT to see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

Builds the body for the display of existing variant products. It is then added
to the display_variant_product template.

=item NOTES

=back

=cut
###############################################################################
sub display_existing_variant_products {
    my ($num_variants) = shift;
    my ($variant_products) = shift;
    my ($master_hashref) = shift;
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($object_body, $variant_product, @where, $axis_select, @variation_axis, $num_axis, @vcodes);
    my ($i, @variational_axis, @select_array, @checkbox_array, $permute_with, %tmp_form);

    my ($init_objid);
    &debug("display_existing_variant_products: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $init_objid = $object_inf->{'objid'};
    $object_body = "";
    # load in each of the existing ones
    foreach $variant_product (@$variant_products) {
	next if (!defined $variant_product->{'objid'});  # This shouldn't have to be here, but somewhere in the load_objects routine, it adds a null row
	# build the row template
	$object_inf->{'gui_name'} = $object_inf->{'exists'};
	$object_inf->{'row_number'} = "0";
	# set the objid so it can figure out row numbers
	$object_inf->{'objid'} = $variant_product->{'objid'};
	$object_body .= &build_object_template($object_inf, $site, $dbh, \@select_array, \@checkbox_array);	
	$permute_with = &get_hashref_to_permute($variant_product, $site, $object_inf);
	&initialize_selects(\%tmp_form, $object_inf, $permute_with, $site, \@select_array);
	&debug_hashref(\%tmp_form, 3, $site->{'debug_log'}, $site->{'debug_level'});
	&debug_hashref($permute_with, 3, $site->{'debug_log'}, $site->{'debug_level'});
	$object_body = &permutebit($object_body, $permute_with);
	$object_body = &permutebit($object_body, \%tmp_form);
	&debug("display_existing_variant_products: Just built $object_body", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    #set back to default
    $object_inf->{'objid'} = $init_objid;
    return $object_body;
}

##############################################################################
=pod

=head2 display_search_results

display_search_results - &display_search_results($link_details, $$FORM{'foundcount'}, $$FORM{'startrow'}, $$FORM{'next_startrow'}, $site, $FORM, \@results);

=over 4

=item DESCRIPTION

To display the search results to the browser

=item INPUT

$link_details - Details of the link to build the link to the next page with.

$found - Total number of rows found

$previousRow - The previous starting code

$nextRow - The next starting code

$site - A hashref containing information required throughout about the current
        site and session

$FORM -	Pointer to FORM hash

$p_results - Pointer to array of search results

=item OUTPUT

The search results are displayed

=item NOTES

=back

=cut
###############################################################################
sub display_search_results {
    my ($link_details) = shift;
    my ($previousRow) = shift;
    my ($nextRow) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($p_results) = shift;

    my ($encrypt_sid);
    my ($template, $result, $row, $objid, $prices, $inventory, $linkage, $title, $variants, $main, $lang);
    my ($back, $next, $pages, $firstrecord, @rows, $start, $link_info, $link_row, $link_info, $search_info, $object_inf, $saved);

    $encrypt_sid = &encrypt_sid($site->{'sid'}, $site);
    &debug("In display_search_results with $link_details", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &localize_template("os_allcommerce-admin_object-search", "admin", 0, $site);
    $template =~ s/__admin_title__/$site->{'script_msg'}{'5'}/gms;
    $template =~ s/__admin_descript__/$site->{'script_msg'}{'6'}/gms;
    @rows = ('row_spine','row_menu','row_headers','row_copyright','row_help','row_site_info','row_search_results','row_search_navigation');
    $template = &permute_rows($template, $site, \@rows);

    # Save the objtype and objclass of the main form
    my $saved_form = &set_form_temporary($FORM, "came_from_search,objid,objclass_inp,objtype_inp,objtype_pre,objclass_pre", {}, $site);

    foreach $result (@$p_results) {
	next if (!defined $result->{'objid'});  # This shouldn't have to be here, but somewhere in the load_objects routine it is sending back a blank one
	&debug("display_search_results: Building a row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug_hashref($result,  3, $site->{'debug_level'}, $site->{'debug_log'});
	$row .= &localize_template("row_search_result", "admin", 0, $site);
	$link_row = &localize_template("cell_link", "admin", 0, $site);
	$link_info = "$site->{'scriptpath'}/__script__\?&__options__";

	if (defined $result->{'title'}) {
	    $row =~ s/__title__/$result->{'title'}/gms;
	}
	# from contacts, so show user name
	else {
	    $row =~ s/__title__/$result->{'uname'}/gms;
	}

	$objid = $link_row;

	# Set up to use build_admin_object_link_info
	$result->{'objclass_inp'}=$result->{'objclass'};
	$result->{'objtype_inp'}=$result->{'objtype'};
	$result->{'objclass_pre'}=$result->{'objclass'};
	$result->{'objtype_pre'}=$result->{'objtype'};
	$result->{'came_from_search'} = "yes";
	&set_form_temporary($FORM, "objid,objclass_inp,objtype_inp,objclass_pre,objtype_pre,came_from_search", $result, $site);
	$link_info = &build_admin_object_link_info($FORM, $result->{'objid'}, "display_object", $encrypt_sid, $site);
	&debug("display_search_results: $link_info", 3, $site->{'debug_level'}, $site->{'debug_log'});

	$objid =~ s/__href__/$link_info/gms;
	$objid =~ s/__tag__/$result->{'objid'}/gms;
	$row =~ s/__objid__/$objid/gms;

	$prices = $link_row;
	$inventory = $link_row;
	if (($result->{'objclass'} ne "") && ($result->{'objclass'} eq $site->{'OBJCLASSES'}->{'product'}) ||
	    ($result->{'objtype'} ne "") && ($result->{'objclass'} eq $site->{'OBJTYPESS'}->{'product'})) {
	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editprice", $encrypt_sid, $site);
	    &reset_form_values($FORM, $saved, $site);
	    $prices =~ s/__href__/$link_info/gms;
	    $prices =~ s/__tag__/$site->{'script_msg'}{'price_tag'}/gms;

	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editsku", $encrypt_sid, $site);
	    &reset_form_values($FORM, $saved, $site);
	    $inventory =~ s/__href__/$link_info/gms;
	    $inventory =~ s/__tag__/$site->{'script_msg'}{'inventory_tag'}/gms;
	}
	else {
	    $prices =~ s/__href__//gms;
	    $prices =~ s/__tag__//gms;
	    $inventory =~ s/__href__//gms;
	    $inventory =~ s/__tag__//gms;

	}
	$row =~ s/__inventory__/$inventory/gms;
	$row =~ s/__prices__/$prices/gms;

	$linkage = $link_row;
	if ($result->{'objclass'} ne $site->{'OBJCLASSES'}->{'contact'}) {
	    $link_info = "$site->{'scriptpath'}/linkage.cgi\?&sid=$encrypt_sid&mode=editlink&loadfrom=db&obj1=$result->{'objid'}";
	    $linkage =~ s/__href__/$link_info/gms;
	    $linkage =~ s/__tag__/$site->{'script_msg'}{'linkage_tag'}/gms;
	}
	else {
	    $linkage =~ s/__href__//gms;
	    $linkage =~ s/__tag__//gms;
	}
	$row =~ s/__linkage__/$linkage/gms;

	$variants = $link_row;
	# is a master product
	if (($result->{'ref_from'} ne "") && ($result->{'ref_from'} eq $result->{'objid'})) {
	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_admin_object_link_info($FORM, $result->{'objid'}, "display_variants", $encrypt_sid,  $site);
	    $variants =~ s/__href__/$link_info/gms;
	    $variants =~ s/__tag__/$site->{'script_msg'}{'variant_tag'}/gms;
	    &reset_form_values($FORM, $saved, $site);
	}
	else {
	    $variants =~ s/__href__//gms;
	    $variants =~ s/__tag__//gms;
	}

	$row =~ s/__variants__/$variants/gms;
	$result = {};
    }

    if (($$FORM{'foundcount'} == 0) && ($$FORM{'came_from_next_link'})) {
	$$FORM{'page_error'} = $site->{'script_msg'}{'no_more_with_permissions'};
    }

    # Reset the form
    &reset_form_values($FORM, $saved_form, $site);

    $template =~ s/__row_search_result__/$row/gms;

    &debug("set_search_navigation: start: $previousRow,  last: $nextRow", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($FORM, 3, $site->{'debug_level'}, $site->{'debug_log'});
    $main->{'pre_row'} = $previousRow;
    $main->{'next_row'} = $nextRow;
    $main->{'this_row'} = $$FORM{'this_startrow'};

    $lang->{'pre_row'} = $$FORM{'pre_lang_startrow'};
    $lang->{'next_row'} = $$FORM{'next_lang_startrow'};
    $lang->{'this_row'} = $$FORM{'lang_startrow'};

    &debug("Just before set_search_navigation, lang info:  pre: $$FORM{'pre_lang_startrow'}, next: $$FORM{'next_lang_startrow'},  start: $$FORM{'lang_startrow'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    ($back, $next) = &set_search_navigation($link_details, $main, $lang, "$site->{'scriptpath'}/$site->{'scriptname'}", $site);
    &debug("set_search_navigation: returned back: $back, next: $next", 3,  $site->{'debug_level'}, $site->{'debug_log'});
    $object_inf = {};
    $object_inf->{'mode'} = $$FORM{'mode'};
    $template = &set_links($object_inf, $FORM, $template, $site);

    $template =~ s/__back__/$back/gms;
    $template =~ s/__next__/$next/gms;
    $template =~ s/__admin_title__/$site->{'script_msg'}{5}/gms;
    $template =~ s/__admin_descript__/$site->{'script_msg'}{6}/gms;
    $$FORM{'this_startrow'} = $$FORM{'startrow'};
    $$FORM{'startrow'} = $$FORM{'next_startrow'};  # get ready for the next start
    $template = &permutebit($template, $FORM);
    $template = &permutebit($template, $site);
    $template = &permuteclean($template, $site);
    $template = &set_headers("content", $template, $site);
    print $template;

}

##############################################################################
=pod

=head2 display_home

display_home - &display_home(\%FORM, $site);

=over 4

=item DESCRIPTION

To display the home page to the browser

=item INPUT

$FORM -	Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

The home page is displayed in the browser

=item NOTES

=back

=cut
###############################################################################
sub display_home {
    my ($FORM) = shift;
    my ($site) = shift;

    my (@pulldowns, $target, $template, @rows, $object_inf);


    &debug("IN DISPLAY_HOME",3, $site->{'debug_level'}, $site->{'debug_log'});
    @pulldowns = ('sel_status|2', 'sel_sku_series|4', 'sel_objtype|8');
    &loadselects($FORM, $FORM, $site->{'selskeletonpath'}, @pulldowns);

    $target = "os_allcommerce-admin_object-home";
    $template = &localize_template($target, "admin", 0, $site);

    @rows = ('row_spine','row_menu','row_headers','row_copyright','row_help','row_site_info','row_admin_object_home');
    $template = &permute_rows($template, $site, \@rows);
    $object_inf = {};
    $object_inf->{'mode'} = $$FORM{'mode'};
    $template = &set_links($object_inf, $FORM, $template, $site);
    $template = &permutebit($template, $FORM);
    $template = &permutebit($template, $site);
    $template =~ s/__admin__/$site->{'script_msg'}{4}/gms;
    $template =~ s/__admin_title__/$site->{'script_msg'}{5}/gms;
    $template =~ s/__admin_descript__/$site->{'script_msg'}{6}/gms;
    
    $template = &permuteclean($template, $site);

    &debug("printing $template", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $template = &set_headers("content", $template, $site);
    print $template;
}

######################### INITIALIZATION SUBROUTINES #######################
##############################################################################
=pod

=head2 setup_session

setup_session - &setup_session(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Check if the user is still logged in or if they have timed out.  If they
have timed out, redirect them to login.cgi.

If everything has passed, start the printing to the browser.  If there
is an error, then print the error to the browser.

=item INPUT

$FORM - Pointer to the FORM 

$site - A hashref containing information required throughout about the current
        site and session

$dbh - Current active database handle

=item OUTPUT

Error message printed to browser if encounter an error.

=item NOTES

=back

=cut
###############################################################################
sub setup_session {
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    $$FORM{'sid_ok'} = check_login($site, $dbh);
    if ( $$FORM{'sid_ok'} ) {
	if ( $$FORM{'sid_ok'} =~ /^ERROR/ ) {
	    ## hard error, redirected to login page
	    print "Location: /cgi-bin/om/admin/login.cgi\n\n";
	}
	elsif ( $$FORM{'sid_ok'} =~ /^Error/ ) {
	    ## soft error, send to default mode
	    $$FORM{'mode'}="home";
	    $$FORM{'__page_error__'} = $$FORM{'sid_ok'};
	} else {
	    ## welcome to the page...
	}
	print "Content-type: text/html\n\n";
   
    } 
    else {
	## unknown why sid_ok not populated with some string...
	#
	print "Content-type: text/html\n\n";
	print "<HTML><HEAD></HEAD><BODY>\n";
	print "$$FORM{'sid'} $site->{'script_msg'}{1} $$FORM{'sid_ok'} \n";
	exit 0;
    }
}

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);
    
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;
    
    @rpath=@temp;
    
    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
    
    $rundir= join "/", @rpath;
    chdir( $rundir );
    
    $relpath="";
    $siteshortname="";
    ## short circuit if $s does not contain cgi-bin
    if ( $s !~ /cgi-bin/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case
    
    while ( @temp ) {
        $u = pop @temp;
        
        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels
        
        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );
        
        if ( $u eq "cgi-bin" ) {
            $siteshortname = $temp[$#temp];
            @temp=();
        } # note - cgi-bin is off the site root, so the next up is site root
        
    } # done constructing relative path to store root
    
    $relpath .= "/../$siteshortname";
    ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    foreach $k (keys %FORM) {
        &debug("$k is $FORM{$k}",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables 
that are required throughout the script. 

=item INPUT

$scriptname - The name of the script (admin_object.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the 
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'} 
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    my ($select_hash, $q2);

    use DBI;
    # Version 0.9.7

    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/blowfisher.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/lib/validate.pm";
    require "$relpath/lib/objects.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/conf/html_def.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/generate.pm";
    require "$relpath/lib/generate_temps.pm";
    require "$relpath/lib/permission_layer.pm";
    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/login.pm";
    require "$relpath/lib/admin_object.pm";
    require "$relpath/lib/conversions.pm";

    require "$relpath/lib/language.pm";
    require "$relpath/lib/variant.pm";
#    require "$relpath/lib/search.pm";
    require "$relpath/lib/object_search.pm";

    
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## get form data, if any was submitted
    #
    $FORM = &ReadParse();
    $site->{'encrypted_sid'} = $$FORM{'sid'};
    $$FORM{'sid'} = &decrypt_sid($$FORM{'sid'}, $site);
    $site->{'sid'}=$$FORM{'sid'};
    
    $find=$$FORM{'skuskel'};
    if (!$find) {$find=$$FORM{'keyword'};}
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $interface = "";
    
    if ($$FORM{'sid'}) {
	$select_hash = {};
	$select_hash->{'TABLE'}->{'NAME'} = "sessions";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'language'} = "language";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'custid'} = "custid";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'interface'} = "interface";
	$select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid",  $$FORM{'sid'}, "and");
	$q = &build_select($select_hash, $site, $dbh, 1);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref('NAME_lc');
        $site->{'om_render_lang'}=$h->{'language'};
	$site->{'custid'} = $h->{'custid'};
        $session_lang = $h->{'language'};
        $interface=$h->{'interface'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );
    if ($site->{'custid'} ne "") {
	$select_hash = {};
	$select_hash->{'TABLE'}->{'NAME'} = "contacts";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'uname'} = "uname";
	$select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid",  $site->{'custid'}, "and");
	$q = &build_select($select_hash, $site, $dbh, 1);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref('NAME_lc');
	$site->{'uname'} = $h->{'uname'};
    }

    ## load messages
    $script=( split /\./, $scriptname )[0];
    #   &initClassAndType();
    
    ## done with pull-from-file bits, rest of global settings...
    #
    $x= (split /\?/, $ENV{'REQUEST_URI'})[0];   # strip get stuff
    @temp=split /\//, $x;
    @temp=("cgi-bin","test","admin_object.cgi") if (! $ENV{'REQUEST_URI'} );
    
    $script=( split /\./, $scriptname )[0];
    $n= $#temp - 1;

    # Configure all parameters to pass around in $site hash
    $site->{'interface'} = $interface;
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'attach_path'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/conf/attached";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/admin_object";
#    $site->{'sid'} = $site->{'sid'};
    ($site->{'OBJCLASSES'}, $site->{'OBJTYPES'}) = &getClassesAndTypes();
    $site->{'STATUS_TXT'} =  &loadseltomem("sel_status", $site->{'selskeletonpath'});
    $site->{'session_lang'} = $session_lang;
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'script_msg'} = &get_messages($site->{'om_render_lang'}, $site->{'scriptname'}, $site);
    $site->{'inherit_from_master'} = "brief";

    # Get the variable read in from conf/html_def.pm
    $site->{'html_def'} = &set_html_def();

    # All fields that are language sensitive
    $site->{'languageSensitive'} = &get_language_sensitive_fields($site, $dbh);

    $site->{'search_results_per_page'} = 10;
    # Flag, set to 0 to have default for Generate page? as no, and
    # set to 1 to have default as yes
    $site->{'generate_default'} = 0;

    # Set up spine image
    $site->{'spine_image'} = "/images/$site->{'om_render_lang'}/os_allcommerce/admin/spine/contentside.gif";
    $site->{'spine_desc'} = $site->{'script_msg'}{'content_manager_description'};

#    $site->{'permissions'} = &load_user_permissions($site->{'custid'}, $site, $dbh);
    $site->{'permission_path'} = "$site->{'relpath'}/lib/permissions";
    if (!$site->{'override_permissions'}) {
	$site->{'permissions'} = &load_user_cache($site);
    }
    $site->{'menu_gif'} = "content_menu.gif";
&debug("Debuggin admin_object.cgi..., spine image is $site->{'spine_image'}",3, $site->{'debug_level'}, ">/tmp/admin_object");
    &debug("Language Sensitive fields are $site->{'languageSensitive'}.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug("initsite:  Just set language to $session_lang, used $q", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($site->{'permissions'}, 3, $site->{'debug_level'}, $site->{'debug_log'});


    return ($site, $dbh, %{$FORM});
}

1;









