#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
###
#
# inventory.cgi
#
###

##############################################################################
=pod

=head1 inventory.cgi

inventory.cgi - Administrator Inventory & Pricing script

=head1 SYNOPSIS

http://YOURDOMAIN/cgi-bin/om/admin/inventory.cgi

=head1 DESCRIPTION

This script is responsible for displaying and submitting information for
inventory and pricing of objects

=head1 NOTES & BUGS

=cut
###############################################################################
use strict;

&main();

##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main driver is responsible for determining the flow of the inventory and
pricing script.  It displays all pages dependant on the mode that is passed
in by the FORM.  The templates are loaded dependant of the mode and the mode
is changed to prepare for submission after it is loaded.  Handles search, 
add/modify prices, add/modify inventory.

=item INPUT

NONE

=item OUTPUT

The FORM is populated and the display is posted according the the choices made
by the user.

=item NOTES

The main processing of the FORM data is done in os_allcommerce/lib/inventory.pm

=back

=cut
###############################################################################
sub main {
    my ($q, $numRows, @lstock, $inventory, $stock, @pulldowns, $v, $l, $spine);
    my (%FORM, %NVP, $site, $dbh, $select_hash);
    
    ($site, $dbh, %FORM) = &initscript();
    
    ## start the replacements hash
    #
    %NVP=();
    $NVP{'sitename'}=$site->{'sitename'};

    ######################### MODE-HANDLING ############################
    if (! $FORM{'mode'}) {
        $FORM{'mode'}="home";
    }
    
    if ($FORM{'mode'} eq "main") {
    $FORM{'mode'} = "home"; }
    
    ## login control
    #
    $FORM{'sid_ok'} = check_login($site, $dbh);
#    $FORM{'sid_ok'} = check_login($site->{'script'}, $FORM{'mode'}, $FORM{'sid'});
    
    if ( $FORM{'sid_ok'} ) {
        if ( $FORM{'sid_ok'} =~ /^ERROR/ ) {
            ## hard error, redirected to login page
            print "Location: /cgi-bin/om/admin/login.cgi\n\n";
            } elsif ( $FORM{'sid_ok'} =~ /^Error/ ) {
            ## soft error, send to default mode
            $FORM{'mode'}="home";
            $NVP{'__page_error_msg__'}=$FORM{'sid_ok'};
            } else {
            ## welcome to the page...
        }
        print "Content-type: text/html\n\n";
        } else {
        ## unknown why sid_ok not populated with some string...
        #
        print "Content-type: text/html\n\n";
        print "<HTML><HEAD></HEAD><BODY>\n";
        print "$FORM{'sid'} sid $site->{'script_msg'}{1} $FORM{'sid_ok'} \n";
        exit 0;
    }
    
    ## Define first entries in %NVP based on mode, sid, etc...
    #
    $NVP{'sid'}=$FORM{'sid'};
    $NVP{'lastmode'}=$FORM{'mode'};
    $NVP{'script'}="$site->{'scriptpath'}/$site->{'scriptname'}";
    $NVP{'scriptpath'}=$site->{'scriptpath'};
    
    # check to see if the homepage is simply clearing the form
    if (($FORM{'searchclear'}) and ($FORM{'mode'} eq "findsku")) {
        $FORM{'mode'}="home";
    }

    if ($FORM{'mode'} eq "search") {
	&inventory_search(\%FORM, $site, $dbh);
    }

    # real preprocessing
   
#    if ($FORM{'mode'} eq "findsku") {
#        &printsearch(\%FORM, \%NVP, $site, $dbh);
#        exit;
#    }
    
    ########################################################
    ## Primary Display Mode Handling
    &debug("MODE IS $FORM{'mode'}",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    if ($FORM{'mode'} eq "submitinv") {
        
        &debug("In submit inventory, master is $FORM{'master_objid'}!",3,$site->{'debug_level'}, $site->{'debug_log'});
        &submitInventory($FORM{objid},\%FORM, \%NVP, $site, $dbh);
        $FORM{'mode'} = "editsku";
    }
    elsif ( $FORM{'mode'} eq "submitprice" ) {
        
        &debug("In submitprice section, going into submitPrice",3,$site->{'debug_level'}, $site->{'debug_log'});
        &submitPrice($FORM{objid}, \%FORM, \%NVP, $site, $dbh);
    }
    
    if ( $FORM{'mode'} eq "editsku" ) {
        $NVP{'master_objid'} = $FORM{'master_objid'};
        &loadinventory($FORM{objid},\%FORM, \%NVP, $site, $dbh);
    }
    
    if ( $FORM{'mode'} eq "editprice" ) {
	$select_hash = {};
	$select_hash->{'TABLE'}->{'NAME'} = "inventory";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'lstock'} = "lstock";
	$select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $FORM{'objid'}, "and");
	$select_hash->{'statement'} = "SELECT";
	$q = &build_select($select_hash, $site, $dbh);
        $numRows = &query_getAll($q, $site, $dbh, \@lstock);
        &debug("$q\n$numRows returned",2,$site->{'debug_level'}, $site->{'debug_log'});
        $inventory = 0;
        foreach $stock (@lstock) {
            $inventory = $inventory + $stock->{'lstock'};
            
            &debug("$inventory is inventory!",3,$site->{'debug_level'}, $site->{'debug_log'});
        }
        
        $NVP{'objid'} = $FORM{'objid'};
        &loadprices($FORM{objid}, $inventory,\%FORM,\%NVP, $site, $dbh);
    }
    
    if ($FORM{'mode'} eq "home") {
        &loadhome(\%FORM,\%NVP, $site);
    }
    
    $dbh->disconnect if ( $dbh ) ;
}

##############################################################################
=pod

=head2 loadhome

loadhome - &loadhome(\%FORM, \%NVP, $site);

=over 4

=item DESCRIPTION

Set the browser to display the home page

=item INPUT

$FORM - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current site and session

=item OUTPUT

Displays the home page.

=item NOTES

=back

=cut
############################################################################
sub loadhome {
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    
    my (@pulldowns, $v, $l, $spine);
    $$FORM{'status_inp'} = "15000-A";
    @pulldowns = ('sel_status|2');
    &loadselects($FORM, $NVP, $site->{'selskeletonpath'},@pulldowns);
    
    $v="os_allcommerce-inventory-home";
    $l = loadtemplate ( $v, $site->{'skeletonpath'} );
    $l = permutebit( $l, $NVP );
    $l =~ s/__page_error__/$$FORM{'page_error'}/gms;
    $l =~ s/__admin_title__/$site->{'script_msg'}{'inventory_title'}/gms;
    $l =~ s/__admin_descript__/$site->{'script_msg'}{'inventory_manager_description'}/gms;
    $l=~ s/__page_error_msg__/$$NVP{'page_error_msg__'}/gms;
    $l=~ s/__script__/$site->{'scriptpath'}\/$site->{'script'}.cgi/gms;
    $l=~ s/__sitename__/$site->{'sitename'}/gms;
    $l=~ s/__sid__/$site->{'sid'}/gms;
    $l=~ s/__mode__/$$FORM{'mode'}/gms;
    $l = permuteclean($l);
    print $l;
}

######################### INVENTORY SUBROUTINES ############################

##############################################################################
=pod

=head2 loadinventory

loadinventory - &loadinventory($objid, \%FORM, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

Load all existing inventory for $objid from tables
Set links to variant products if $objid
is a master product.  Get the form ready to allow user to
edit the data.

=item INPUT

$objid - objid to get inventory for

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

Set $FORM{mode} to submitinv & displays the inventory that already exist

=item NOTES

=back

=cut
###############################################################################
sub loadinventory {
    my ($objid) = shift;
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($encrypt_sid);

    my ($price_link, $object_link);
    my (@existing_inventory, $numInventory, $numAxisNames);
    my ($isItMaster, $numWarehouses, $row_enter_button, $edit_target_headers, $header_template);
    my (@warehouse_inv, $pr, @rownames, $titles, $edit_target_varheaders);
    my ($manual_title, $manual_header, $auto_title, $auto_header, $new_title, $new_header);
    my ($v, $l, $varheader_template);
    
    &debug("IN LOAD INVENTORY!  Product is $objid!",3,$site->{'debug_level'}, $site->{'debug_log'});
    $encrypt_sid = &encrypt_sid($site->{'sid'}, $site);

    $$NVP{'sitename'}=$site->{'sitename'};
    $$NVP{'searchnum'}=1;
    $$NVP{'lastmode'}=$$FORM{'mode'};
    $$NVP{'scriptname'}="$site->{'scriptname'}";
    $$NVP{'scriptpath'}="$site->{'scriptpath'}";
    
    # Display the page
    $v="os_allcommerce-inventory-editsku";
    $l = loadtemplate ( $v, $site->{'skeletonpath'} );
    $numWarehouses = &getExistingInv($objid, \@warehouse_inv, $NVP, $site, $dbh);

    if ($numWarehouses == 0) {
	$l =~ s/__PAGE_ERROR_MSG__/$site->{'script_msg'}{'no_warehouse'}/gms;
    }
    else {
	$isItMaster = &isMasterProduct($objid, $site, $dbh, $NVP);
	# Set up a non-master product
	if (!$isItMaster) {
	    
	    $row_enter_button = loadtemplate("row_enter_button",$site->{'skeletonpath'});
	    $row_enter_button =~ s/__TDPROPS__/COLSPAN=4 ALIGN=RIGHT/gms;
	    $$NVP{'row_enter_button'} = $row_enter_button;
	    
	    $edit_target_headers = "row_inventory_headers";
	    $header_template=loadtemplate($edit_target_headers, $site->{'skeletonpath'});
	    
	    $edit_target_headers = "row_inventory_headers";
	    $header_template=loadtemplate($edit_target_headers, $site->{'skeletonpath'});
	    
	    if ($$FORM{'master_objid'} ne "") {
		&debug("Printing master_objid of $$FORM{'master_objid'}!",3,$site->{'debug_level'}, $site->{'debug_log'});
		$pr = &build_inventory_link_info($FORM, $$FORM{'master_objid'}, "editsku", $encrypt_sid, $site);
		&setBackToMaster($$FORM{'master_objid'}, $pr, "COLSPAN=4 ALIGN=RIGHT",$FORM, $NVP, $site);
	    }
	    
	    @rownames=( "row_inventory_editsku");
	    $l = &loadRows($numWarehouses, "inventory", \@warehouse_inv, @rownames, $l, $FORM, $site);
	}
	# Set up a master product
	else {
	    $$NVP{'master_objid'} = $$FORM{'master_objid'};
	    $titles = &get_fields_and_title(undef,$objid, $dbh, undef, $site);
	    $$NVP{'pagetitle'}="$site->{'script_msg'}{6} $site->{'script_msg'}{12} \($objid\) $titles->{'title'} \n";
	    $pr = &build_inventory_link_info($FORM, "", "editsku", $encrypt_sid, $site, $objid);
	    &setUpMasterLinks($objid,$pr, "inventory", $FORM, $NVP, $site, $dbh);
	    $edit_target_varheaders = "row_inventory_varheaders";
	    $varheader_template=loadtemplate($edit_target_varheaders, $site->{'skeletonpath'});
	}
	
    }

    $$FORM{'status_inp'} = '';
    &checkformbits($FORM);
    
    &writetonvp($FORM, $NVP);
    $$NVP{'mode'}="submitinv";
    
    &debug("Set to $$NVP{'mode'}!",3,$site->{'debug_level'}, $site->{'debug_log'});
    $l = permutebit( $l, $NVP );
    &debug("Replacing __STATUS_SELECT__ with $$NVP{'STATUS_SELECT'}",3,$site->{'debug_level'}, $site->{'debug_log'});

    $object_link = &build_admin_object_link_info($FORM, $$FORM{'objid'}, "display_object", $encrypt_sid, $site);
    if ($$FORM{'master_objid'} eq "") {
	$price_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editprice", $encrypt_sid, $site);
    }
    else {
	$price_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editprice", $encrypt_sid, $site, $$FORM{'master_objid'});
    }


    $l =~ s/__price_link__/$price_link/gms;
    $l =~ s/__price_tag__/$site->{'script_msg'}{'price_tag'}/gms;

    $l =~ s/__object_link__/$object_link/gms;
    $l =~ s/__object_tag__/$site->{'script_msg'}{'object_tag'}/gms;

    
    $l=~ s/__STATUS_SELECT__/$$NVP{'STATUS_SELECT'}/gms;
    $l=~ s/__row_inventory_header__/$header_template/gms;
    $l=~ s/__row_inventory_varheader__/$varheader_template/gms;
    $l=~ s/__page_error_msg__/$$NVP{'page_error_msg__'}/gms;
    $l=~ s/__script__/$site->{'scriptpath'}\/$site->{'script'}.cgi/gms;
    $l=~ s/__sitename__/$site->{'sitename'}/gms;
    $l=~ s/__sid__/$site->{'sid'}/gms;
    $l=~ s/__mode__/editprice/gms;
    $l=~ s/__objid__/$$FORM{'objid'}/gms;
    $l =~ s/__row_price_headers__/$header_template/gms;
    $l =~ s/__manual_title__/$manual_title/gms;
    $l =~ s/__manual_header__/$manual_header/gms;
    $l =~ s/__auto_title__/$auto_title/gms;
    $l =~ s/__auto_header__/$auto_header/gms;
    $l =~ s/__new_title__/$new_title/gms;
    $l =~ s/__new_header__/$new_header/gms;
    # Permute it to keep the information for seach
    $l = &permutebit($l, $FORM);
    $l = permuteclean($l);
    print $l;
}


##############################################################################
=pod

=head2 loadprices

loadprices - &loadprices($objid, $inventory, \%FORM, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

Load all existing prices for $objid from tables, display
the $inventory for the object.
Set links to variant products if $objid
is a master product.  Get the form ready to allow user to
edit the data.  Splits the form into existing manual prices,
existing auto generated prices, and a location to make
new manual prices

=item INPUT

$objid - objid to get prices for

$inventory - current inventory for the objid

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

Set $FORM{mode} to editprice

=item NOTES

=back

=cut
###############################################################################
sub loadprices {
    my ($objid) = shift;
    my ($inventory) = shift;
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    #load primary permute hash values
    #
    my ($inventory_link, $object_link);
    my (@existing_prices, $numPrices, $numAxisNames);
    my ($v, $l, $edit_target_headers, $header_template, $titles, $pr, $edit_target_varheaders);
    my ($varheader_template, $manual_header, $manual_title, @rownames, $numAutoPrices);
    my (@auto_prices, $auto_header, $auto_title, $i, $new_header, $new_title);
    my (@pulldowns, @junk, $regenerate_template, $enter_button, $where);
    my (@where_array);
    my ($encrypt_sid);

    $encrypt_sid = &encrypt_sid($site->{'sid'}, $site);
    
    $$NVP{'sitename'}=$site->{'sitename'};
    $$NVP{'searchnum'}=1;
    $$NVP{'lastmode'}=$$FORM{'mode'};
    $$NVP{'scriptname'}="$site->{'scriptname'}";
    $$NVP{'scriptpath'}="$site->{'scriptpath'}";
    
    # Display the page
    $v="os_allcommerce-inventory-editpriceP";
    $l = loadtemplate ( $v, $site->{'skeletonpath'} );
    
    $edit_target_headers = "row_price_headers";
    $header_template=loadtemplate($edit_target_headers, $site->{'skeletonpath'});
    $titles = &get_fields_and_title(undef,$objid, $dbh, undef, $site);
    
    # set up master product pricing
    if (&isMasterProduct($objid, $site, $dbh, $NVP)) {
        $$NVP{'master_objid'} = $$FORM{'master_objid'};
        $titles = &get_fields_and_title(undef, $objid, $dbh, undef, $site);
        $$NVP{'pagetitle'}="$site->{'script_msg'}{8} $site->{'script_msg'}{12} \($objid\) $titles->{'title'} \n";
#        $pr="&sid=$site->{'sid'}&mode=editprice&loadfrom=db&master_objid=$objid";
	$pr = &build_inventory_link_info($FORM, "", "editprice", $encrypt_sid, $site, $objid);
        &setUpMasterLinks($objid,$pr, "prices", $FORM, $NVP,$site, $dbh);
        
        $edit_target_varheaders = "row_price_varheaders";
        $varheader_template=loadtemplate($edit_target_varheaders, $site->{'skeletonpath'});
        
    }
    # Set up non-master product pricing
    else {
	$enter_button = &loadtemplate("cell_buttons", $site->{'skeletonpath'});
	$$NVP{'cell_buttons'} = $enter_button;
        $$NVP{'inventory'}="$site->{'script_msg'}{13} $inventory $site->{'script_msg'}{14}\n";
        $$NVP{'pagetitle'}="$site->{'script_msg'}{8} \($objid\) $titles->{'title'} \n";
        
        if ($$FORM{'master_objid'} ne "") {
            &debug("Printing master_objid of $$FORM{'master_objid'}!",3,$site->{'debug_level'}, $site->{'debug_log'});

#            $pr="&sid=$site->{'sid'}&mode=editprice&loadfrom=db";
	    $pr = &build_inventory_link_info($FORM, $$FORM{'master_objid'}, "editprice", $encrypt_sid, $site);
            &setBackToMaster($$FORM{'master_objid'}, $pr,"COLSPAN=5 ALIGN=RIGHT",$FORM, $NVP, $site);
        }
	&debug("Before getExisting Prices",3,$site->{'debug_level'}, $site->{'debug_log'});        
        # Get the existing prices
	$where_array[0] = &build_where_hash("autogen", "N", "and");
        $numPrices = &getExistingPrices($$FORM{'objid'},"unitct, unitpr, currency, status, autogen", \@where_array, \@existing_prices, $NVP, $site, $dbh);

        if ($numPrices >0 ) {
            $manual_header = $header_template;
	    $manual_title = loadtemplate("row_price_title", $site->{'skeletonpath'});
	    $manual_title =~ s/__title__/$site->{'script_msg'}{15}/gms;
            @rownames=( "row_manual_price_exists");
            $l = &loadRows($numPrices,"price", \@existing_prices, @rownames, $l, $FORM, $site);
        }
        
        # Get the existing prices
	$where_array[0] = &build_where_hash("autogen", "Y", "and");
        $numAutoPrices = &getExistingPrices($$FORM{'objid'},"unitct, unitpr, status, currency, autogen", \@where_array, \@auto_prices, $NVP, $site, $dbh);
        
        if ($numAutoPrices >0 ) {
            $auto_header = $header_template;
	    $auto_title = loadtemplate("row_price_title", $site->{'skeletonpath'});
	    $auto_title =~ s/__title__/$site->{'script_msg'}{16}/gms;
            for ($i=0; $i<$numAutoPrices; $i++) {
                $auto_prices[$i]->{'status_inp'} = $site->{'STATUS_TXT'}->{$auto_prices[$i]->{'status'}};
            }
            
            @rownames=( "row_autogen_price_exists");
            $l = &loadRows($numAutoPrices, "price", \@auto_prices, @rownames, $l, $FORM, $site);
            
        }
        
        @rownames=( "row_price_new");
        $l = &loadRows(1,"price", \@junk, @rownames, $l, $FORM, $site);
        
        $new_header = $header_template;
	$new_title = loadtemplate("row_price_title", $site->{'skeletonpath'});
	$new_title =~ s/__title__/$site->{'script_msg'}{17}/gms;
        
        #Load the status pulldown for the new information
        $$FORM{'status_inp'} = '';
        $$FORM{'currency_inp'} = '';
        @pulldowns = ( 'sel_status|2', 'sel_currency|2');
        &loadselects($FORM, $NVP, $site->{'selskeletonpath'},@pulldowns);
        &checkformbits($FORM);
        
        $regenerate_template = loadtemplate("row_regenerate", $site->{'skeletonpath'});
        if ($site->{'generate_default'} == 1) {
            $regenerate_template =~ s/__regenerate_inp__/ CHECKED/gms;
        }
        $regenerate_template =~ s/__COLSPAN__/COLSPAN=2/gms;
    }
    
    &writetonvp($FORM, $NVP);
    $$NVP{'mode'}="submitprice";
    $l = permutebit( $l, $NVP );


    $object_link = &build_admin_object_link_info($FORM, $$FORM{'objid'}, "display_object", $encrypt_sid, $site);
    if ($$FORM{'master_objid'} eq "") {
	$inventory_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editsku", $encrypt_sid, $site);
    }
    else {
	$inventory_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editsku", $encrypt_sid, $site, $$FORM{'master_objid'});
    }

    $l =~ s/__inventory_link__/$inventory_link/gms;
    $l =~ s/__object_link__/$object_link/gms;
    $l =~ s/__object_tag__/$site->{'script_msg'}{'object_tag'}/gms;
    $l =~ s/__inventory_tag__/$site->{'script_msg'}{'inventory_tag'}/gms;
    
    $l=~ s/__row_regenerate__/$regenerate_template/gms;
    $l=~ s/__unitct_pre__/1/gms;         # hardcode to 1 for this release
    $l=~ s/__STATUS_SELECT__/$$NVP{'STATUS_SELECT'}/gms;
    $l=~ s/__page_error_msg__/$$NVP{'page_error_msg__'}/gms;
    $l=~ s/__script__/$site->{'scriptpath'}\/$site->{'script'}.cgi/gms;
    $l=~ s/__sitename__/$site->{'sitename'}/gms;
    $l=~ s/__sid__/$site->{'sid'}/gms;
    $l=~ s/__mode__/editprice/gms;
    $l=~ s/__objid__/$$FORM{'objid'}/gms;
    $l =~ s/__row_price_headers__/$header_template/gms;
    $l=~ s/__row_price_varheader__/$varheader_template/gms;
    $l =~ s/__manual_title__/$manual_title/gms;
    $l =~ s/__manual_header__/$manual_header/gms;
    $l =~ s/__auto_title__/$auto_title/gms;
    $l =~ s/__auto_header__/$auto_header/gms;
    $l =~ s/__new_title__/$new_title/gms;
    $l =~ s/__new_header__/$new_header/gms;
    # Permute it to keep the information for seach
    $l = &permutebit($l, $FORM);
    $l = permuteclean($l);
    print $l;
}

############################ SEARCH SUBROUTINES ############################
##############################################################################
=pod

=head2 display_search_results

display_search_results - &display_search_results($link_details, $$FORM{'foundcount'}, $$FORM{'startrow'}, $$FORM{'next_startrow'}, $site, $FORM, \@results);

=over 4

=item DESCRIPTION

To display the search results to the browser

=item INPUT

$link_details - Details of the link to build the link to the next page with.

$found - Total number of rows found

$previousRow - The previous starting code

$nextRow - The next starting code

$site - A hashref containing information required throughout about the current
        site and session

$FORM -	Pointer to FORM hash

$p_results - Pointer to array of search results

=item OUTPUT

The search results are displayed

=item NOTES

=back

=cut
###############################################################################
sub display_search_results {
    my ($link_details) = shift;
    my ($previousRow) = shift;
    my ($nextRow) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($p_results) = shift;


    my ($template, $result, $row, $objid, $prices, $inventory, $linkage, $title, $variants, $main, $lang);
    my ($back, $next, $pages, $firstrecord, @rows, $start, $link_info, $link_row, $link_info, $search_info, $object_inf, $saved);

    my ($encrypt_sid);

    $encrypt_sid = &encrypt_sid($site->{'sid'}, $site);
    &debug("In display_search_results with $link_details", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &localize_template("os_allcommerce-admin_object-search", "admin", 0, $site);
    @rows = ('row_spine','row_menu','row_headers','row_copyright','row_help','row_site_info','row_search_results','row_search_navigation');
    $template = &permute_rows($template, $site, \@rows);

    # Save the objtype and objclass of the main form
    my $saved_form = &set_form_temporary($FORM, "came_from_search,objid,objclass_inp,objtype_inp,objtype_pre,objclass_pre", {}, $site);

    foreach $result (@$p_results) {
	next if (!defined $result->{'objid'});  # This shouldn't have to be here, but somewhere in the load_objects routine it is sending back a blank one
	&debug("display_search_results: Building a row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug_hashref($result,  3, $site->{'debug_level'}, $site->{'debug_log'});
	$row .= &localize_template("row_search_result", "admin", 0, $site);
	$link_row = &localize_template("cell_link", "admin", 0, $site);
	$link_info = "$site->{'scriptpath'}/__script__\?&__options__";

	if (defined $result->{'title'}) {
	    $row =~ s/__title__/$result->{'title'}/gms;
	}

	$objid = $link_row;

	# Set up to use build_admin_object_link_info
	$result->{'objclass_inp'}=$result->{'objclass'};
	$result->{'objtype_inp'}=$result->{'objtype'};
	$result->{'objclass_pre'}=$result->{'objclass'};
	$result->{'objtype_pre'}=$result->{'objtype'};
	$result->{'came_from_search'} = "yes";
	&set_form_temporary($FORM, "objid,objclass_inp,objtype_inp,objclass_pre,objtype_pre,came_from_search", $result, $site);
	$link_info = &build_admin_object_link_info($FORM, $result->{'objid'}, "display_object", $encrypt_sid, $site);
	&debug("display_search_results: $link_info", 3, $site->{'debug_level'}, $site->{'debug_log'});

	$objid =~ s/__href__/$link_info/gms;
	$objid =~ s/__tag__/$result->{'objid'}/gms;
	$row =~ s/__objid__/$objid/gms;

	$prices = $link_row;
	$inventory = $link_row;
	if (($result->{'objclass'} ne "") && ($result->{'objclass'} eq $site->{'OBJCLASSES'}->{'product'}) ||
	    ($result->{'objtype'} ne "") && ($result->{'objclass'} eq $site->{'OBJTYPESS'}->{'product'})) {

	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    if ($$FORM{'master_objid'} ne "") {
		$link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editprice", $encrypt_sid, $site, $$FORM{'master_objid'});
	    }
	    else {
		$link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editprice", $encrypt_sid, $site);
	    }
	    &reset_form_values($FORM, $saved, $site);
	    $prices =~ s/__href__/$link_info/gms;
	    $prices =~ s/__tag__/$site->{'script_msg'}{'price_tag'}/gms;

	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    if ($$FORM{'master_objid'} ne "") {
		$link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editsku", $encrypt_sid, $site, $$FORM{'master_objid'});
	    }
	    else {
		$link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editsku", $encrypt_sid, $site);
	    }
	    &reset_form_values($FORM, $saved, $site);
	    $inventory =~ s/__href__/$link_info/gms;
	    $inventory =~ s/__tag__/$site->{'script_msg'}{'inventory_tag'}/gms;
	}
	else {
	    $inventory =~ s/__href__//gms;
	    $inventory =~ s/__tag__//gms;
	    $prices =~ s/__href__//gms;
	    $prices =~ s/__tag__//gms;
	}
	$row =~ s/__inventory__/$inventory/gms;
	$row =~ s/__prices__/$prices/gms;


	$linkage = $link_row;
	$link_info = "$site->{'scriptpath'}/linkage.cgi\?&sid=$encrypt_sid&mode=editlink&loadfrom=db&obj1=$result->{'objid'}";
	$linkage =~ s/__href__/$link_info/gms;
	$linkage =~ s/__tag__/$site->{'script_msg'}{'linkage_tag'}/gms;

	$row =~ s/__linkage__/$linkage/gms;

	$variants = $link_row;
	# is a master product
	if (($result->{'ref_from'} ne "") && ($result->{'ref_from'} eq $result->{'objid'})) {
	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_admin_object_link_info($FORM, $result->{'objid'}, "display_variants", $encrypt_sid, $site);
	    $variants =~ s/__href__/$link_info/gms;
	    $variants =~ s/__tag__/$site->{'script_msg'}{'variant_tag'}/gms;
	    &reset_form_values($FORM, $saved, $site);
	}
	else {
	    $variants =~ s/__href__//gms;
	    $variants =~ s/__tag__//gms;
	}

	$row =~ s/__variants__/$variants/gms;
	$result = {};
    }

    # Reset the form
    &reset_form_values($FORM, $saved_form, $site);

    $template =~ s/__row_search_result__/$row/gms;

    &debug("set_search_navigation: start: $previousRow,  last: $nextRow", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($FORM, 3, $site->{'debug_level'}, $site->{'debug_log'});
    $main->{'pre_row'} = $previousRow;
    $main->{'next_row'} = $nextRow;
    $main->{'this_row'} = $$FORM{'this_startrow'};

    $lang->{'pre_row'} = $$FORM{'pre_lang_startrow'};
    $lang->{'next_row'} = $$FORM{'next_lang_startrow'};
    $lang->{'this_row'} = $$FORM{'lang_startrow'};

    &debug("Just before set_search_navigation, lang info:  pre: $$FORM{'pre_lang_startrow'}, next: $$FORM{'next_lang_startrow'},  start: $$FORM{'lang_startrow'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    ($back, $next) = &set_search_navigation($link_details, $main, $lang, "$site->{'scriptpath'}/$site->{'scriptname'}", $site);
    &debug("set_search_navigation: returned back: $back, next: $next", 3,  $site->{'debug_level'}, $site->{'debug_log'});
    $object_inf = {};
    $object_inf->{'mode'} = $$FORM{'mode'};
#    $template = &set_links($object_inf, $FORM, $template, $site);
    $template =~ s/__inventory_tag__//gms;
    $template =~ s/__price_tag__//gms;

    # Set which menu button on the top is highlighted
    $template = &set_headers("inventory", $template, $site);
    $template =~ s/__back__/$back/gms;
    $template =~ s/__next__/$next/gms;
    $$FORM{'this_startrow'} = $$FORM{'startrow'};
    $$FORM{'startrow'} = $$FORM{'next_startrow'};  # get ready for the next start
    $template =~ s/__admin_title__/$site->{'script_msg'}{'inventory_title'}/gms;
    $template =~ s/__admin_descript__/$site->{'script_msg'}{'inventory_manager_description'}/gms;
    $template = &permutebit($template, $FORM);
    $template = &permutebit($template, $site);
    $template = &permuteclean($template, $site);
    print $template;

}

######################### INITIALIZATION SUBROUTINES #######################

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);
    
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;
    
    @rpath=@temp;
    
    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
    
    $rundir= join "/", @rpath;
    chdir( $rundir );
    
    $relpath="";
    $siteshortname="";
    ## short circuit if $s does not contain cgi-bin
    if ( $s !~ /cgi-bin/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case
    
    while ( @temp ) {
        $u = pop @temp;
        
        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels
        
        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );
        
        if ( $u eq "cgi-bin" ) {
            $siteshortname = $temp[$#temp];
            @temp=();
        } # note - cgi-bin is off the site root, so the next up is site root
        
    } # done constructing relative path to store root
    
    $relpath .= "/../$siteshortname";
    ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    foreach $k (keys %FORM) {
        &debug("$k is $FORM{$k}",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables 
that are required throughout the script. 

=item INPUT

$scriptname - The name of the script (inventory.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the 
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'} 
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    my ($select_hash, $q2);

    use DBI;
    # Version 0.9.7
    require "$relpath/lib/variant.pm";
    require "$relpath/lib/blowfisher.pm";
    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/search.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/lib/validate.pm";
    require "$relpath/lib/objects.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/language.pm";
    require "$relpath/lib/generate.pm";
    require "$relpath/lib/permission_layer.pm";
    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/login.pm";
    require "$relpath/lib/inventory.pm";
    require "$relpath/lib/object_search.pm";
    
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## get form data, if any was submitted
    #
    $FORM = &ReadParse();
    $site->{'encrypted_sid'} = $$FORM{'sid'};
    $$FORM{'sid'} = &decrypt_sid($$FORM{'sid'}, $site);
    $site->{'sid'}=$$FORM{'sid'};
    
    $find=$$FORM{'skuskel'};
    if (!$find) {$find=$$FORM{'keyword'};}
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $interface = "";
    
    if ($$FORM{'sid'}) {
	$select_hash = {};
	$select_hash->{'TABLE'}->{'NAME'} = "sessions";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'language'} = "language";
	$select_hash->{'TABLE'}->{'FIELDS'}->{'interface'} = "interface";
	$select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid", $$FORM{'sid'}, "and");
	$q = &build_select($select_hash, $site, $dbh);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref('NAME_lc');
        $site->{'om_render_lang'}=$h->{'language'};
        $session_lang = $h->{'language'};
        $interface=$h->{'interface'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );
    
    ## load messages
    $script=( split /\./, $scriptname )[0];
    #   &initClassAndType();
    
    ## done with pull-from-file bits, rest of global settings...
    #
    $x= (split /\?/, $ENV{'REQUEST_URI'})[0];   # strip get stuff
    @temp=split /\//, $x;
    @temp=("cgi-bin","test","inventory.cgi") if (! $ENV{'REQUEST_URI'} );
    
    $script=( split /\./, $scriptname )[0];
    $n= $#temp - 1;

    # Configure all parameters to pass around in $site hash
    $site->{'interface'} = $interface;
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/inventory";
    $site->{'sid'} = $site->{'sid'};
    ($site->{'OBJCLASSES'}, $site->{'OBJTYPES'}) = &getClassesAndTypes();
    $site->{'STATUS_TXT'} =  &loadseltomem("sel_status", $site->{'selskeletonpath'});
    $site->{'session_lang'} = $session_lang;
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'script_msg'} = &get_messages($site->{'om_render_lang'}, $site->{'scriptname'}, $site);
    $site->{'search_results_per_page'} = 10;
    
    # Flag, set to 0 to have default for Generate page? as no, and
    # set to 1 to have default as yes
    $site->{'generate_default'} = 0;

    # Set up spine image
    $site->{'spine_image'} = "/images/$site->{'om_render_lang'}/os_allcommerce/admin/spine/inventoryside.gif";
    $site->{'spine_desc'} = $site->{'script_msg'}{'inventory_manager_description'};
    $site->{'menu_gif'} = "inventory_menu.gif";

    &debug("Debuggin inventory.cgi...",3, $site->{'debug_level'}, ">/tmp/inventory");
    return ($site, $dbh, %{$FORM});
}

1;


