#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# hint.pl
#
# retrieves a customer hint and displays it
#    written by GF 990521
#
#####
#
# Version 3 TG 20000918
#    Changed ReadParse2 to ReadParse
# Version 2 TG 20000913
#    Changed permute_include to permute_include2
# Version 1 TG 20000825
#    Retrofit for development standards
# Version 0.9.12 TG 20000802
#    Removed check for email of 8 characters at a minimum for valid email
# Version 0.9.11 TG 20000801
#    Added colon to your hint
# Version 0.8.10 IB 20000717
#    om_render_language->om_render_lang
# Version 0.9.9 IB 20000607
#    u_email -> uname
# Version 0.9.8 IB 20000606
#    language and interface from table 'sessions'
# Version 0.9.7 IB 20000531
#    added custom-lib.pm
# Version 0.9.6 GF 20000523
#    objclass changed to objtype
# Version 0.9.5 GF 20000515
#    added check for normal customer
# Version 0.9.4 GF 20000504
#    fixed error messages
# Version 0.9.3 GF 20000503
#    field names lowercase
# Version 0.9.2 GF 20000502
#    UID -> U_EMAIL
# Version 0.9.1 IB 20000501
#    introduced variable $script
# Version 0.7.28 JC 20000418
#    replaced loadtemplate with localize_template
#    included permute_include
#    removed $skeleton
# Version 0.7.27 GF 20000413
#    debug
# Version 0.7.26 GF 20000412
#    reformatted
#    changed UID to U_EMAIL, CUSTNUM to CUSTID
# Version 0.7.24 IB 20000411
#    default to language in site.pm
# Version 0.7.23 IB 20000410
#    multilingual messages
# Version 0.7.22 IB 20000402
#    new directory structure
# Version 0.7.21 IB 20000330
#    changed connect statement
# Version 0.7.20 JC 20000329
#    changed DBI prepare/connect
# Version 0.7.19 IB 20000327
#    corrected $relpath
# Version 0.7.18 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.17 JC 20000322
#    added colon to ssl_prefix
# Version 0.7.16 IB 20000317
#    renaming of templates
# Version 0.7.15 IB 20000302
#    updated copyright info
# Version 0.7.14 IB 20000226
#    changed store to site
# Version 0.7.13 JN 20000202
#    changed page redirection
# Version 0.7.12 JN 20000118
#    droped period from hint
# Version 0.7.11 JN 20000117
#    changed https string to store.pm's ssl_prefix
#    now using loadtemplate instead of old open()
#    got rid of !
# Version 0.7.10 JN 20000115
#    re-fixed ssl redirection
# Version 0.7.9 JN 20000107
#    fixed ssl redirect in nextcheckoutpg
# Version 0.7.8 RF 991203
#    added $site hashref pass via permutebit
# Version 0.7.7 IB 991203
#    added track
# Version 0.7.6 GF/IB 991202
#    added /site to /bits
# Version 0.7.5 IB 991126
#    changed bits/include to html/include
#    changed bits to bits/site
# Version 0.7.4 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.3 IB 991111
#    corrected the location of the DBI statement
# Version 0.7.2 IB 991109
#    removing of HACKS, and correcting of relpath
# Version 0.7.1 IB 991102
#    general cleanup
####

##############################################################################
=pod

=head1 hint.cgi

hint.cgi - Retreive a hint for a user

=head1 SYNOPSIS

http://YOURDOMAIN/cgi-bin/om/hint.cgi

=head1 DESCRIPTION

This script is responsible for submitting request for hint and displaying
the user's hint if they have entered valid information.

=head1 NOTES & BUGS

=cut
###############################################################################
use strict;

&main();
1;


##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main driver is reponsible for determining the flow of the hint
script.  It displays all pages dependant on the mode that is passed
in by the FORM.  The templates are loaded dependant of the mode and the mode
is changed to prepare for submission after it is loaded.

=item INPUT

NONE

=item OUTPUT

The FORM is populated and the display is posted according the the choices made
by the user.

=item NOTES

The main processing of the FORM data is done in os_allcommerce/lib/hint.pm

=back

=cut
###############################################################################
sub main {
    
    my (%FORM, %NVP, $site, $dbh, $select_hash, $error, $thispage);
    ($site, $dbh, %FORM) = &initscript();
    
    $select_hash->{'TABLE'}->{'NAME'} = "sessions";
    
    &checkIfLoggedIn($site, \%FORM);
    
    
    if ( $FORM{'email_inp'} ) {
        $FORM{'email_pre'}=$FORM{'email_inp'};  # keep the address populated
        # so the end user can see what they
        # entered
        # validate email address
        $error = "";
        $error = &validateHintInput(\%FORM, $site);
        if ($error eq "") {
            &retreiveHint(\%FORM, $site, $dbh);
        }
    } #/done with looking up hint
    
    ## start the response header
    #
    print "Content-type: text/html\n\n";
    
    ## take elements off page and store in $thispage
    #
    $thispage = localize_template("page_hint_lookup",$site->{'interface'},0, $site);
    
    ## fork and track the user
    #
    `$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/cgi-bin/om/track.cgi`;
    
    ## create the page stdout
    #
    $thispage = permutebit ( $thispage, \%FORM );
    $thispage = permutebit ( $thispage, $site );
    $thispage = permuteclean ( $thispage );
    $thispage = permute_include( $thispage, "html", 0, $site);
    
    print $thispage;
	$dbh->disconnect() if ($dbh);
}


######################### INITIALIZATION SUBROUTINES #######################

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);
    
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;
    
    @rpath=@temp;
    
    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
    
    $rundir= join "/", @rpath;
    chdir( $rundir );
    
    $relpath="";
    $siteshortname="";
    ## short circuit if $s does not contain cgi-bin
    if ( $s !~ /cgi-bin/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case
    
    while ( @temp ) {
        $u = pop @temp;
        
        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels
        
        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );
        
        if ( $u eq "cgi-bin" ) {
            $siteshortname = $temp[$#temp];
            @temp=();
        } # note - cgi-bin is off the site root, so the next up is site root
        
    } # done constructing relative path to store root
    
    $relpath .= "/../$siteshortname";
    ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    foreach $k (keys %FORM) {
        &debug("$k is $FORM{$k}",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables
that are required throughout the script.

=item INPUT

$scriptname - The name of the script (hint.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the
following values:

0 - no debug

1 - errors only

2 - all SQL statements, warnings & errors

3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}.
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'}
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    my ($select_hash, $q2);
    
    use DBI;
    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/lib/blowfisher.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/state.pm";
    require "$relpath/lib/validate.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/hint.pm";
    require "$relpath/lib/permission_layer.pm";
    require "$relpath/lib/db_layer.pm";
    
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## get form data, if any was submitted
    #
    $FORM = &ReadParse();
    $$FORM{'sid'} = &decrypt_sid($$FORM{'sid'}, $site);
    $site->{'sid'}=$$FORM{'sid'};
    
    $find=$$FORM{'skuskel'};
    if (!$find) {$find=$$FORM{'keyword'};}
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $interface = "";
    
    if ($$FORM{'sid'}) {
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "sessions";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'sid'} = "sid";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'custid'} = "custid";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'mdate'} = "mdate";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'interface'} = "interface";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'language'} = "language";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid",$$FORM{'sid'}, "and");
	$select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref;
        $site->{'om_render_lang'}=$h->{'language'};
        $session_lang = $h->{'language'};
        $interface=$h->{'interface'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );
    
    ## load messages
    $script=( split /\./, $scriptname )[0];
    #   &initClassAndType();
    
    ## done with pull-from-file bits, rest of global settings...
    #
    $x= (split /\?/, $ENV{'REQUEST_URI'})[0];   # strip get stuff
    @temp=split /\//, $x;
    @temp=("cgi-bin","test","hint.cgi") if (! $ENV{'REQUEST_URI'} );
    
    $script=( split /\./, $scriptname )[0];
    $n= $#temp - 1;
    
    # Configure all parameters to pass around in $site hash
    $site->{'interface'} = $interface;
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/hint";
    $site->{'sid'} = $site->{'sid'};
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'script_msg'} = &message($site);
    $site->{'sslpath'} ="$site->{'sitesslurl'}/cgi-bin/om/ssl";
    
    
    # Flag, set to 0 to have default for Generate page? as no, and
    # set to 1 to have default as yes
    $site->{'generate_default'} = 0;
    
    &debug("Debuggin hint.cgi...",3, $site->{'debug_level'}, ">/tmp/hint");
    return ($site, $dbh, %{$FORM});
}

##############################################################################
=pod

=head2 message

message - $site->{'script_msg'} = &message($site);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This routine loads in the
messages for the $site->{'om_render_lang'} that is currently active.

=item INPUT

$site - A hashref containing information required throughout about the current
site and session

=item OUTPUT

A reference to the %script_msg hash of messages that need to be used in the
script

=item NOTES

=back

=cut
##############################################################################
sub message {
    my ($site) = shift;
    # check the language that is being used and open appropriate
    # message language file or get messages below
    my (%script_msg, @temp);
    
    if ($site->{'om_render_lang'} eq "eng" ) {
        $script_msg{1}="Your hint is: ";
        $script_msg{2}="Error - Invalid email address!";
        $script_msg{3}="Error - No email address entered!";
        $script_msg{4}="Error - Wrong email address format, need \@";
        $script_msg{5}="No hint on file for this account!";
        
        } else {
        open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'script'}.messages");
        
        while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
        }
        
        close(EM);
    }
    return \%script_msg;
}

1;
