#!/usr/bin/perl 
#-d:ptkdb

sub BEGIN {
    $ENV{'DISPLAY'} = "10.1.1.92:0.0";
}
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# search.cgi
#
####

####
# Version 7 TG 20000918
#    Changed ReadParse2 to ReadParse
# Version 6 TG 20000913
#    Changed permute_include2 to permute_include
# Version 5 TG 20000831 
#     Grab cell_search to populate the search section of the page
# Version 4 TG 20000830
#     Added exact match
# Version 3 TG 20000829
#     Added check for search_type_inp
# Version 2 TG 20000828
#     Added 'and' 'or' searches
# Version 1 TG 20000825
#     Rewrote to meet development standards
###

###
# Overview:
#   Assemble search parameters
#   Obtain list of fields to query against by special term
#   Construct field-specific querries
#   Determine effected tables
#   Run querries by table
#   construct table-by-table result sets
#      NOTE - only tables searched now are:
#         things
#         variants
#         extraprop
#         prices
#         linkage?
#         thesaurus?
#   perform eliminative combinatorics
#   prepare final result set
#   handoff to display routines
#
###
##############################################################################
=pod

=head1 search.cgi

search.cgi - Site search for keywords

=head1 SYNOPSIS

http://YOURDOMAIN/cgi-bin/om/admin/search.cgi

=head1 DESCRIPTION

This script is responsible for displaying results from a site search

=head1 NOTES & BUGS

=cut
###############################################################################
use strict;
no strict ('refs');

&main();


##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main driver is responsible for determining the flow of the search
script.  It displays all pages dependant on the mode that is passed
in by the FORM.  The templates are loaded dependant of the mode and the mode
is changed to prepare for submission after it is loaded.

=item INPUT

NONE

=item OUTPUT

The FORM is populated and the display is posted according the the choices made
by the user.

=item NOTES

The main processing of the FORM data is done in os_allcommerce/lib/search.pm

=back

=cut
###############################################################################
sub main {
    my ($k, $sinput, $site, $dbh, %FORM, $ki, @search_array, $i, @words, $search_words, $key, $type);
    ($site, $dbh, %FORM) = &initscript();
    
    ######
    ### Assemble search parameters
    ######
    #   Syntax for form variables:
    #      search_fieldname_inp (value can contain range specifiers)
    #      search_parameter_inp (generally lacks range specifiers)
    #
    $sinput={};           # empty hashref of search terms
    foreach $k ( keys %FORM ) {
        next if ( $k !~ /^search_/ );       # skip if it isn't search
        next if ( $k !~ /_inp$/ );          # skip if it isn't an input
	next if ( $k =~ /search_type_inp/); # don't want to check on input type
        next if ( ! $FORM{$k} );            # skip blank parameters
        $ki = $k;
        $ki =~ s/^search_// ;               # generate new key
        $ki =~ s/_inp$// ;                  # trim "input" flag
        $sinput->{$ki}=$FORM{$k};           # create entry in search term hash
        
    } #/done constructing earch term list
    
    $i = 0;
    foreach $key (keys %{$sinput}) {
	@words[$i++] = $sinput->{$key};
    }

    $search_words = join " ", @words;
    @words = split / /, $search_words;

    # reduce the amount of words to the maximum number that we can search
    if ($#words >= $site->{'max_num_words_to_search'}) {
	@words = @words[0..$site->{'max_num_words_to_search'}-1];  
	&debug("$#words is new length of words", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$search_words = "";
	$search_words = join " ", @words;
    }

    &checkRedirect($site, $sinput);
    &debug("Search type is $FORM{'search_type_inp'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    @search_array = &do_site_search($dbh, $search_words, $FORM{'search_type_inp'}, $site, "weight_man");
    &update_search_stats($site, \%FORM);
    &display_search_results($#search_array+1, $site, \%FORM, $search_words, @search_array);
    $dbh->disconnect();
}

##############################################################################
=pod

=head2 update_search_stats

update_search_stats - &update_search_stats($site, \%FORM);

=over 4

=item DESCRIPTION

Add the search word to the tracking file, and call track.cgi to update the
information for the stats manager

=item INPUT

$site - A hashref containing information required throughout about the current
site and session

$FORM - pointer to the %FORM hash

=item OUTPUT

Updates the file os_allcommerce/data/site_search_keywords.txt with the information
requested by the search & updates the stats via track.cgi

=item NOTES

=back

=cut
###############################################################################
sub update_search_stats {
    my ($site) = shift;
    my ($FORM) = shift;

    `$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/cgi-bin/om/track.cgi`;
    
    # track the keywords
    open FH, ">>$site->{'relpath'}/data/site_search_keywords.txt";
    print FH "$$FORM{'search_keywords_inp'}\n";
    close FH;

}

##############################################################################
=pod

=head2 display_search_results

display_search_results - &display_search_results($resultcount, $site, \%FORM, @search_array);

=over 4

=item DESCRIPTION

Take the results and display them in the fashioned desired for the site search

=item INPUT

$num_records_matched - Number of rows found for the search

$site - A hashref containing information required throughout about the current
site and session

$FORM - pointer to the %FORM hash

@search_array - an array of hashrefs of rows which matched the search criteria

=item OUTPUT

Displays the search results page

=item NOTES

=back

=cut
###############################################################################
sub display_search_results {
    my ($num_records_matched) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($search_words) = shift;
    my (@search_array) = @_;
    
    my ($numRes, $startnum, $thisstartrow, $numLastPage, $lastrec);
    my ($nextstartnum, $lastrow, $nextflag, $pr, $back);
    my ($next, $pages, $firstrecord, $cell_np, $cell_pr, $thispage, $searchpage);
    my ($breaktot, $breakat1, $breakat2, $prevstartnum, $prevflag);
    my ($endrow, $i, $objid, $u, $h, $q, $expand1, $expand2);
    
    &debug("\nIN DISPLAY SEARCH!\n", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug("There are $num_records_matched entries that have been sorted!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
    $numRes = $num_records_matched;
    $num_records_matched=0 if ($num_records_matched < 0);
    
    if ( exists($$FORM{'startrow'} )) {
        $startnum = $$FORM{'startrow'};
    } # done checking on passed startnum
    else {
        $startnum = 0;
    }
    
    $thisstartrow = $startnum;
    
    $numLastPage = 0;
    $lastrec= $startnum + $site->{'max_recs_per_page'};
    if ($lastrec > $numRes) {
        $lastrec = $numRes;
        $numLastPage = $lastrec - $thisstartrow;
    }
    $lastrow = $lastrec;
    
    if ( $num_records_matched > $lastrec ) {
        $nextstartnum=$startnum+$site->{'max_recs_per_page'};
        $nextflag="yes";
    }
    
    if ($numRes > 0) {
        $$FORM{'start'} = $startnum + 1;
        $$FORM{'end'} = $lastrec;
        $$FORM{'startrow'} = $nextstartnum;
        $$FORM{'searchnum'} = $numRes;
    }
    else {
        $$FORM{'start'} = 0;
        $$FORM{'end'} = 0;
        $$FORM{'startrow'} = 0;
        $$FORM{'searchnum'} = 0;
    }
    
    
    &debug("Num_Records_Matched is $num_records_matched, lastrec is $lastrec, Startrow is $thisstartrow, next start is $nextstartnum!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
#    $pr = "sid=$site->{'sid'}&search_keywords_inp=$$FORM{'search_keywords_inp'}";
    $pr = "sid=$site->{'encrypted_sid'}&search_keywords_inp=$$FORM{'search_keywords_inp'}";
    ($back, $next, $pages, $firstrecord) = &setSearchNavigation($pr, $numRes,$site->{'max_recs_per_page'}, $thisstartrow, $lastrow, $site->{'scriptinfo'});
    $$FORM{'back'} = $back;
    $$FORM{'next'} = $next;
    $$FORM{'pages'} = $pages;
    
    $lastrec=$num_records_matched if ( $lastrec > $num_records_matched );
    @search_array = @search_array[$startnum..$lastrec];
    
    $num_records_matched++ if ( $num_records_matched>0 );
    
    ######
    ### Construct output page
    ######
    
    print "Content-type: text/html\n\n";
    
    $cell_np = &localize_template("cell_search_intermediate",$site->{'interface'},0, $site);
    $cell_pr = &localize_template("cell_search_product",$site->{'interface'},0, $site);
    $thispage = &localize_template("page_search_result",$site->{'interface'},0,$site);

    # if there are less than $resultsperpage entries, split the results in 1/2 to
    # find 2 columns
    if ($site->{'max_recs_per_page'} > $num_records_matched) {
        $breaktot=$num_records_matched;
    }
    # split up last page into 2 columns
    elsif ($numLastPage != 0) {
        $breaktot = $numLastPage + 1;   # add one for odd numbers, don't want to miss any
    }
    # split up into full pages
    else {
        $breaktot=$site->{'max_recs_per_page'};
    }
    
    $breakat1=int($breaktot/2);
    $breakat2=$breakat1+$breakat1;
    &debug("Set break total to be $breaktot\n Break points are $breakat1, $breakat2, $#search_array", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
    if ($breakat1 > $#search_array) {
        $breakat1=$#search_array;
    }
    if ($breakat2 > $#search_array) {
        $breakat2=$#search_array;
    }
    
    if ($startnum > 0) {
        $prevstartnum=$startnum-$site->{'max_recs_per_page'};
        $prevflag="yes";
    }
    
    
    &debug("Break points are $breakat1, $breakat2, $#search_array", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $lastrec = $thisstartrow + 10;
    &debug("Startrow is $thisstartrow!  End row is $lastrow! numsort is $#search_array", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($#search_array > 10) {
	$endrow = 10;
    }
    else {
	$endrow = $#search_array;
    }

    for ($i=0; $i < $endrow; $i++) {
        
        next if ($#search_array == 0);
        
        $objid=$search_array[$i]->{'objid'};
        &debug("In forloop with $objid!", 3, $site->{'debug_level'}, $site->{'debug_log'});
        
        next if (!$objid);
        
        if ($objid =~ /^x/ ) { # non-sellable objid
            $u=$cell_np;
            } else {
            $u=$cell_pr;
        }
        $h=$search_array[$i];
        
        $q=permutebit($u,$h);
        if ($i < $breakat1) {
            $expand1.=$q;
            } elsif ($i < $breakat2 && $i >= $breakat1) {
            $expand2.=$q;
        }
        
    }

    # get ready to permute the page
    $h={};
    $h->{'headergif'}="search_results.gif";
    $h->{'sitename'}=$site->{'sitename'};
    $h->{'title'}=$site->{'script_msg'}{2};
    $h->{'column_220_1'}=$expand1;
    $h->{'column_220_2'}=$expand2;
    $h->{'brief'}=$site->{'script_msg'}{1};
    
    # used to be num_records_matched
    if ($numRes) {
        $h->{'brief'}.=" $site->{'script_msg'}{5} $search_words $site->{'script_msg'}{6} $numRes $site->{'script_msg'}{3}!";
        } else {
	     $h->{'brief'}.=" $site->{'script_msg'}{5} $search_words $site->{'script_msg'}{6} $site->{'script_msg'}{4}!";
     }

     $h->{'site_background'}=$site->{'site_background'};
     $h->{'os_render_language'}=$site->{'om_render_lang'};
     $h->{'os_language'}=$site->{'om_render_lang'};

    $searchpage = &localize_template("cell_search",$site->{'interface'},0,$site);
    $thispage =~ s/__cell_search__/$searchpage/gms;
    $thispage =~ s/__exact_inp__/ CHECKED/gms;

     ## output the file to stdout
     #
     $thispage = permutebit( $thispage, $h);
     $thispage = permutebit( $thispage, $FORM );
     $thispage = permutebit( $thispage, $site );
     $thispage = permutebit( $thispage, \%ENV );
     $thispage = permuteclean ( $thispage  );
     $thispage = permute_include( $thispage, "html", 0, $site);

     print $thispage;
 }

##############################################################################
=pod

=head2 do_site_search

do_site_search - &do_site_search($dbh, $search_words, $type, $site, $sortfield);

=over 4

=item DESCRIPTION

Perform the search of the site for the keywords specified.  This is used by
search.cgi, the storefront search routine

=item INPUT

$dbh - current active database handle

$search_words - string of words, separated by a space

$type - type of search and, or, exact

$site - A hashref containing information required throughout about the current
site and session

$sortfield - the field which to sort by

=item OUTPUT

@search_array - an array of hashrefs of rows which matched the search criteria

=item NOTES

=back

=cut
###############################################################################
sub do_site_search {
    my ($dbh) = shift;
    my ($search_words) = shift;
    my ($type) = shift;
    my ($site) = shift;
    my ($sortfield) = shift;
    

    my ($search_results, $search_indx, @search_array, $objid, $filters);
    
    $filters->{'things'} = "title,keywords,brief";
    $filters->{'objtype'} = "$site->{'OBJTYPES'}->{'product'},$site->{'OBJTYPES'}->{'brand'},$site->{'OBJTYPES'}->{'product_line'},$site->{'OBJTYPES'}->{'article'}";
    $filters->{'visibility'} = "110000-A";


    $search_results = &search($search_words, $type, $site->{'max_search_results'}, $filters, \&build_site_search_queries, $site, $dbh);
    $search_indx=0;
    foreach $objid (keys %{$search_results->{'things'}}) {
	if (defined $search_results->{'things'}->{$objid}) {
	    &debug("Adding $objid to search results!", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $search_array[$search_indx++] = $search_results->{'things'}->{$objid};
	}
    }
    
    @search_array = sort { $a->{$sortfield} <=> $b->{$sortfield} } @search_array;
    @search_array = reverse @search_array;   # weight is to be sorted from highest to lowest, so reverse the list
    
    &printSearchArray($site, @search_array);
    return @search_array;
}

##############################################################################
=pod

=head2 checkRedirect

checkRedirect - &checkRedirect($site, $sinput);

=over 4

=item DESCRIPTION

*******************

=item INPUT

$site - A hashref containing information required throughout about the current
site and session

$sinput - hashref of all search words specified

=item OUTPUT

Prints out the redirected page if there is one that was specified

=item NOTES

Don't know when this would apply or how it would work.

=back

=cut
###############################################################################
sub checkRedirect {
    my $site = shift;
    my $sinput = shift;
    my ($rtarget, $k);
    ######
    ### Catch the "redirect" entries and act
    ######
    $rtarget="";                            # target for redirection
    foreach $k ( keys %{$sinput} ) {
        next if ( $k !~ /^redirect/ );
        $rtarget=$sinput->{$k} if ( $sinput->{$k} );
    } # done looking at $sinput->  keys for redirects
    
    if ( $rtarget ) {
        $rtarget="/html/$site->{'om_render_lang'}/$rtarget.shtml"
        if ( $rtarget !~ /(.shtml$|^\/cgi-bin|^http:)/ );
        print "Status: 302/Moved\n";
        print "Location: $rtarget\n";
        print "\n\n";
        
        exit 1;
    } #
}

######################### INITIALIZATION SUBROUTINES #######################

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);
    
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;
    
    @rpath=@temp;
    
    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
    
    $rundir= join "/", @rpath;
    chdir( $rundir );
    
    $relpath="";
    $siteshortname="";
    ## short circuit if $s does not contain cgi-bin
    if ( $s !~ /cgi-bin/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case
    
    while ( @temp ) {
        $u = pop @temp;
        
        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels
        
        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );
        
        if ( $u eq "cgi-bin" ) {
            $siteshortname = $temp[$#temp];
            @temp=();
        } # note - cgi-bin is off the site root, so the next up is site root
        
    } # done constructing relative path to store root
    
    $relpath .= "/../$siteshortname";
    ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    foreach $k (keys %FORM) {
        
        &debug("$k is $FORM{$k}",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables
that are required throughout the script.

=item INPUT

$scriptname - The name of the script (search.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the
following values:

0 - no debug

1 - errors only

2 - all SQL statements, warnings & errors

3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}.
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'}
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    my ($select_hash, $q2);
    
    use DBI;
    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/lib/blowfisher.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/state.pm";
    require "$relpath/lib/variant.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/objects.pm";
    require "$relpath/lib/permission_layer.pm";
    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/search.pm";
    
    # must be tha last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## get form data, if any was submitted
    #
    $FORM = &ReadParse();
    $site->{'encrypted_sid'} = $$FORM{'sid'};
    $$FORM{'sid'} = &decrypt_sid($$FORM{'sid'}, $site);
    $site->{'sid'}=$$FORM{'sid'};
    
    $find=$$FORM{'skuskel'};
    if (!$find) {$find=$$FORM{'keyword'};}
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $interface = "";
    
    if ($$FORM{'sid'}) {
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "sessions";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'language'} = "language";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'interface'} = "interface";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid", $$FORM{'sid'}, "and");
	$select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref;
        $site->{'om_render_lang'}=$h->{'language'};
        $session_lang = $h->{'language'};
        $interface=$h->{'interface'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );
    
    ## load messages
    $script=( split /\./, $scriptname )[0];
    
    ## done with pull-from-file bits, rest of global settings...
    #
    $x= (split /\?/, $ENV{'REQUEST_URI'})[0];   # strip get stuff
    @temp=split /\//, $x;
    @temp=("cgi-bin","test","search.cgi") if (! $ENV{'REQUEST_URI'} );
    
    $script=( split /\./, $scriptname )[0];
    $n= $#temp - 1;
    
    # Configure all parameters to pass around in $site hash
    $site->{'interface'} = $interface;
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    ($site->{'OBJCLASSES'}, $site->{'OBJTYPES'}) = &getClassesAndTypes();
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/search";
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'script_msg'} = &message($site);
    $site->{'max_search_results'} = 200;   # doesn't work yet!
    $site->{'search_results_per_page'} = 10;
    $site->{'max_recs_per_page'} = 10;

    # don't activate permissions for site searches
    $site->{'override_permissions'} = "yes";
    $site->{'max_num_words_to_search'} = 5;  # after 5 keywords, it cuts off the search keywords
#    &setsessionid();
    &debug("Debuggin search.cgi...",3, $site->{'debug_level'}, ">/tmp/search");
    return ($site, $dbh, %{$FORM});
}


##############################################################################
=pod

=head2 message

message - $site->{'script_msg'} = &message($site);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This routine loads in the
messages for the $site->{'om_render_lang'} that is currently active.

=item INPUT

$site - A hashref containing information required throughout about the current
site and session

=item OUTPUT

A reference to the %script_msg hash of messages that need to be used in the
script

=item NOTES

=back

=cut
###############################################################################
sub message {
    my ($site) = shift;
    
    my (@temp);
    my (%script_msg);
    
    # check the language that is being used and open appropriate
    # message language file or get messages below
    if ($site->{'om_render_lang'} eq "eng" ) {
        $script_msg{1}="Your search ";
        $script_msg{2}="Search Results!";
        $script_msg{3}="matches";
        $script_msg{4}="nothing! Please try again";
        $script_msg{5}="for";
        $script_msg{6}="returned";

        } else {
        open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'script'}.messages");
        while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
        }
        close(EM);
    }
    return \%script_msg;
}
1;
