#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

######
# sid.cgi
#    session id setting script
#    RF 981208 - based on welcome.cgi, version 4-981207
######

####
# Version 2 20000918 TG
#    Changed ReadParse2 to ReadParse
# Version 1 20000911 LCF
#	made strict work properly-- many bugfixes
# Version 0.7.18 NC 20000830
#    use strict, add headers
# Version 0.7.17 IB 20000717
#    $om_render_language=$om_render_lang
# Version 0.7.16 IB 20000627
#    added cgi-lib.pm
# Version 0.7.15 IB 20000606
#    language from table 'sessions'
# Version 0.7.14 IB 20000601
#    added custom-lib.pm
# Version 0.7.13 IB 20000411
#    default to language in site.pm
# Version 0.7.12 IB 20000410
#    multilingual messages
# Version 0.7.11 IB 20000401
#    new directory structure
# Version 0.7.10 IB 20000327
#    corrected $relpath
# Version 0.7.9 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.8 IB 20000302
#    updated copyright
# Version 0.7.7 IB 20000227
#    changed store to site
# Version 0.7.6 SV 20000113
#    corrected call to 'cat' in clear-image-replace code
#    added single quotes, added call to print
# Version 0.7.5 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.4 IB 991110
#    replacing with a clear image
# Version 0.7.3 IB 991109
#    removing of HACKS, and correcting of relpath
# Version 0.7.2 IB 991025
#    determine path from store root
#    adjust this script to work with track.cgi and inbound.cgi
# Version 0.7.1 GF 990720
#    debugging and testing
# Version 3 SH 990708
#    got rid of quick hack hardcode where storecode was set to W
# Version 2 - 981218
#    efficiency cleanup
# Version 1 - 981208
#
#### 


##############################################################################
=pod

=head1 sid.cgi

sid.cgi - give sid to each action at store side and admin side

=head1 SYNOPSIS

http://YOURDOMAIN/cgi-bin/om/sid.cgi

=head1 DESCRIPTION

This script is responsible for giving sid to each action at store side and admin side

=head1 NOTES & BUGS

=cut
###############################################################################
use strict;

&main();


##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main driver is responsible for determining the sid for each action at store side and admin side. Many functions are done based on sid. 

=item INPUT

NONE

=item OUTPUT

record sid into specific database tables

=item NOTES

=back

=cut
###############################################################################
sub main {
    my ($site, $dbh, %FORM) = &initscript();

    my $sid = &decrypt_sid($FORM{'sid'}, $site);
    ## just need to know if there is a sid
    #
    $sid = &setsessionid($site, $dbh, %FORM) if (!$ENV{'HTTP_COOKIE'});

    ## and now lets replace this with a clear image
    print "Content-type: image/gif\n\n";
    print `cat $site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$site->{'om_render_lang'}/general/clear.gif`;
	$dbh->disconnect() if ( $dbh );
}
######################### INITIALIZATION SUBROUTINES #######################

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with


=item NOTES

=back

=cut
###############################################################################
sub initscript {
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
	##
	use Cwd;
	my $separator = '/'; # What separates dirnames in this architecture?
	my ($scriptname, $args, $rundir);
	($scriptname, $args) = split /\?/, $0;
	if ($scriptname =~ /$separator/) {
		$scriptname =~ /^(.*)$separator(.*?)$/;
		$rundir = $1;
		$scriptname = $2;
	} else {
		$rundir = cwd();
	}
	my @path = split(/$separator/, $rundir);
	my $relpath;
	while (my $d = pop @path) {
		$relpath .= "..$separator";
		if ($d eq 'cgi-bin') {
			$relpath .= "..$separator";
			last;
		}
	}
	my $siteshortname = pop @path;
	$relpath .= $siteshortname;
	###########

    my ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables 
that are required throughout the script. 

=item INPUT

$scriptname - The name of the script (sid.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the 
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 

Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'} 
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    use DBI;
    # Version 0.9.7
    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/state.pm";    
    require "$relpath/lib/blowfisher.pm";
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    my $site=setsite();
    
    ## get form data, if any was submitted
    #
    my $form_ref = &ReadParse();
    my %FORM = %{$form_ref};
	
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    my $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    my $session_lang = '';
	
    if ($FORM{'sid'}) {
        my $q="SELECT language,interface FROM sessions WHERE sid=$FORM{'sid'} ";
        my $sts = $dbh->prepare( $q );
        $sts->execute;
        #my $h=$sts->fetchrow_hashref;
        my $h=$sts->fetchrow;
        $site->{'om_render_lang'}=$h->{'language'};
        $session_lang = $h->{'language'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" unless ( $session_lang );

    return ($site, $dbh, %FORM);
}

1;
