#!/usr/bin/perl -w
# This is a test
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#########################
# checkout.cgi          #
#     checkout srcipt   #
#########################

########
#
#  Version 1 NC 20001121
#
########

########################################################################
=pod

=head1 NAME

checkout.cgi

=head1 SYNOPSIS

This script go through all the checkout steps

=head1 DESCRIPTION

this script takes care of all the checkout steps, order recapture,
get shipping address, wrapping (if gift), get shipping method, 
get payment information, order preview, and order confirmation.

=head1 NOTES & BUGS

all the funtions that are only used by this srcipt except load_libs
are stored in checkout-lib.pm, other funtions are under the lib 
files that are included by this script. 

=cut
#######################################################################

use strict;

    ##
    # Determine where the script is called from and include the 
    # necessary library funtions.
    my ($scriptname, $relpath) = &load_libs();

    ##
    # Initialize the script
    my ($site, $form) = &init_script();    

    ##
    # For convenience, add scriptname and relpath to $site
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;

    ##
    # Load in error messages
    $site->{'msg'} = &checkout_message($site);

    ##
    # Check if there is a valid sid and custid 
    # if there is no valid sid, move to inbound.cgi
    # if there is no custid for current sid, move to welcome.cgi
    if (! $form->{'sid'}){

       # if there is no valid sid, then move to inbound.cgi
       print "Status: 302/Moved\n";
       print "Location: http://$site->{'siteurl'}/cgi-bin/om/inbound.cgi\n\n\n";
       exit 0;

    } else {

        # get custid for current sid
        $form->{'custid'} = &get_custid($site, $form);

        if (!$form->{'custid'}) {

           # if there is no custid for current sid,
           # move to welcome.cgi
           print "Status: 302/Moved\n";
           print "Location: http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi?checkout.cgi\n\n\n";
           exit 0;
       
        } # end of if (!$custid)

    } # end of else

    ## get currency symbol
    ($site->{'currency'}, $site->{'this_currency_symbol'}) = &get_currency_and_symbol($form, $site);

    ##
    # setup pagelist hashref
    my $pagelist = &build_pagelist();

    ##
    # Check if this is the first time entering first page 
    # of checkout
    # if so, set current page to ''
    # set next page to be orderrecap 

    if (!$form->{'current_page'} && !$form->{'next_page'}) {
        $form->{'current_page'} = ''; 
        $form->{'next_page'} = 'orderrecap';
    }

    ## 
    # add $form to page_hashref, easy to pass to sub function
    my $page_hashref = ();
    $page_hashref->{'form'} = $form;

    ##
    # add error message to page_hashref
#    $page_hashref->{'message'} = \%error_msg; 

    ##
    # Check if the current page and next page are the same,
    # if same, means that this page logic need to be run 
    # to check if it can go another page
    # if everything is ok, next page will be set to the
    # next step of checkout
    if ($page_hashref->{'form'}->{'current_page'} eq $page_hashref->{'form'}->{'next_page'}) {
        $page_hashref = &{$pagelist->{$page_hashref->{'form'}->{'current_page'}}}($page_hashref, $site);
    }

    ## 
    # Check the current page and next page, if they are not same,
    # means that we should load in next step of checkout
    # this if block cannot combined together with the right above
    # if block, since the execution of this if block depends on
    # the result from the above one
    if ( $page_hashref->{'form'}->{'current_page'} ne $page_hashref->{'form'}->{'next_page'} ) {
        $page_hashref = &{$pagelist->{$page_hashref->{'form'}->{'next_page'}}}($page_hashref, $site);
    }

    ## encrypt sid
    $form->{'sid'} = &encrypt_sid($form->{'sid'}, $site);

    ##
    # Load in page to be shown
    my $page = &localize_template($page_hashref->{'template'}, $site->{'interface'}, 0, $site);

    $page = &permutebit( $page, $page_hashref->{'form'} );
    $page = &permutebit( $page, $site );
    $page = &permuteclean( $page );
    $page = &permute_include( $page, $site->{'interface'}, 0, $site);  

    print "Content-type: text/html\n\n";
    print $page;

    $site->{'dbh'}->disconnect if ($site->{'dbh'});

1;

######################################################################
=pod

=head2 load_libs

load_libs - load_libs();

=over 4

=item DESCRIPTION

This function determines the path of from the site root

=item INPUT

NONE

=item OUTPUT

$scriptname - the name of the script

$relpath - the relative path to files that must be required

=item NOTES

=back

=cut
####################################################################
sub load_libs {
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #

    # Find out where we are, so we can load libs.
    ##
    use Cwd;
    my $separator = '/'; # What separates dirnames in this architecture?
    my ($scriptname, $args, $rundir);
    ($scriptname, $args) = split /\?/, $0;
    if ($scriptname =~ /$separator/) {
        $scriptname =~ /^(.*)$separator(.*?)$/;
        $rundir = $1;
        $scriptname = $2;
    } else {
        $rundir = cwd();
    }
    my @path = split(/$separator/, $rundir);
    my $relpath;
    while (my $d = pop @path) {
        $relpath .= "..$separator";
        if ($d eq 'cgi-bin') {
            $relpath .= "..$separator";
            last;
        }
    }
    my $siteshortname = pop @path;
    $relpath .= $siteshortname;
    ###########

    use DBI;
    require "$relpath/lib/cgi-lib.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/state.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/validate.pm";
    require "$relpath/lib/checkout-lib.pm";
    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/blowfisher.pm";    
    #require "$relpath/lib/payprocess.pm";

    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";

    ############

    return ($scriptname, $relpath);
}

