#!/usr/bin/perl 
 
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# wrapper.cgi
#    wrapperizer script that handles user request that don't accept cookies
#    HVC 20000327
#
####

#####
# Version 2 TG 20000918
#    Changed ReadParse2 to ReadParse
# Version 1 HVC 20000907
#    Hooks to handle wml
#    Coding Standards - use strict,pod
# Version 0.9.13 HVC 20000728
#    Handle http/https requests
# Version 0.9.12 HVC 20000725
#    Clean code,Handle multiple page search result and hint
# Version 0.9.11 HVC 20000720
#    Rewrite wrapper to be compatible with os_allcommerce
# Version 0.9.10 IB 20000717
#    $om_render_language->$om_render_lang
# Version 0.9.9 IB 20000606
#    commented out interface, because it is not needed
# Version 0.9.8 IB 20000606
#    language and interface from table 'sessions'
# Version 0.9.7 IB 20000531
#    added custom-lib.pm
# Version 0.9.6 IB 20000502
#    introduced $script variable
# Version 0.9.5 IB 20000421
#    clean up 
# Version 0.9.4 HVC 20000420
#    replace ERLOG with message subroutine
# Version 0.9.02 HVC 20000403
#    clean up code
#    modify FixLinks
# Version 0.9.01 HVC 20000327
#    handle both GET/POST method
#    compatible with openmerchant0_7
#
#####


################################################################################
=pod

=head1 NAME

wrapper.cgi

=head1 SYNOPSIS

This cgi script handles cookie less shopping


=head1 REQUIREMENTS

wrapper uses LWP::UserAgent 
To get it working the following packages have to be installed.
A. URI
B. HTML::Parser
C. MIME::Base64
D. Digest::MD5
E. libwww-perl
These packages can be obtained from http://www.linpro.no/lwp


=head1 DESCRIPTION

wrapper is invoked by inbound.cgi when the web browser
does not accept a cookie, so that shopping can continue.

wrapper gets the page contents from the apache server
and rewrites the links to pass through the wrapper and 
display on the contents on the browser.

The 3 major functions are Fix_HTML_Links,Fix_WML_Links and Load_Module
A. Fix_HTML_Links - Rewrite HTML links to pass through wrapper
B. Fix_WML_Links  - Rewrite WML  links to pass through wrapper
C. Load_Module - Load LWP::UserAgent and exit gracefully with a appropriate
   error message if not found

=head1 LIMITATION

A.  wrapper reads one line at a time and rewrites links to 
    pass through it, hence it arises a lot of limitations.
    Care is required while creating or modifying templates. 
    Shown below are classical examples, which are not handled by wrapper.

    a. HREF= "12-AA.shtml"    --- Extra space
    b. HREF = "12-AA.shtml"   --- Extra space 
    c. HREF="12AA.shtml"      --- Hardcoded in digit-char format
    d. HREF="12-AA.html"      --- Hardcoded to shtml
    e. HREF=12-AA.shml        --- NO double codes
    f. FORM METHOD=POST ACTION="/cgi-bin/om/search.cgi"  
                              --- Need to be on the same line
    g. FORM METHOD=POST  ACTION="/cgi-bin/om/search.cgi"
                              --- Extra space between POST and ACTION

B. HTTPS requests are handled as HTTP

=head1 AUTHOR

Harish V.C. (harish@opensales.org)

=cut
####################################################################################



use strict;
&main();
1;





###############################################################################
########################   WRAPPER SUBROUTINES   ##############################
###############################################################################
sub main {
  my($sid,$sts,$om_render_lang,$interface); 
  my($pg_targ,$target,$target2,$pattern_xyz);
  my($argument,$new_target,$dbh,$q,$h); 
  my($postdata,$ua,$request,$response,$content,$keyval);

  ## determine path from store (site) root and store db name...
  #
  #  note - this script runs at server root or in cgi-bin/om,
  #    generally. Test these conditions first.
  #  $0 - may not contain a full path, bootstrap with "pwd"
  #
  my $s;
  if ( $0 !~ /^\// ) {  # $0 is not an absolute path
    $s=`pwd`;          # hope pwd on path...
    chomp $s;
    $s.="/$0";
  } else {              # $0 IS an absolute path
    $s=$0;
  } # done determining if $0 is relative or absolute
  my @temp=split /\//, $s;
  
  my @rpath=@temp;
  
  my $scriptname = pop @rpath;
  $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
  
  my $rundir= join "/", @rpath;
  chdir( $rundir );
  my $relpath="";
  my $siteshortname="";
  ## short circuit if $s does not contain cgi-bin
  if ( $s !~ /cgi-bin/ ) {
    $siteshortname = pop @rpath;
    $relpath=".";
    @temp=();
  } # done testing for degenerate case
  
  while ( @temp ) {
    my $u = pop @temp;
    
    ## skip references to the present directory
    next if ( $u eq '.' );
    next if ( length($u)==0 );        # skip null directory levels
    
    $relpath.="/.." if ( $relpath);
    $relpath.="." if ( ! $relpath );
    
    if ( $u eq "cgi-bin" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
    } # note - cgi-bin is off the site root, so the next up is site root
    
  } # done constructing relative path to store root
  
  $relpath .= "/../$siteshortname";
  
  use DBI;
  require "$relpath/conf/site.pm";
  require "$relpath/lib/state.pm";
  require "$relpath/lib/localization.pm";
  require "$relpath/lib/cgi-lib.pm";
  require "$relpath/lib/blowfisher.pm";
  # load last
  require "$relpath/lib/custom-lib.pm";
  
  ## establish site-specific parameters
  my $site=&setsite();
  
  ## load messages
  my $om_render_lang="eng";    # hard coded before reading from the tables
  my $script=( split /\./, $scriptname )[0];
  my %script_msg = &message($om_render_lang,$relpath,$script); # user interaction messages
  
  
  ## HVC - Make wrapper.cgi act like a browser.
  ## Here wrapper.cgi asks for the contents of the target page 
  ## from the apache server, and then parses the returned contents
  ## to change links, so that user is always 
  ## within wrapper.cgi, it also handle GET/POST method accordingly.
  
  
  # Load Module - Checks to see if the required module is present
  Load_Module("LWP::UserAgent",$site);
  ## Get the CGI bits
  #
  # $length is what ReadParse returns first by default. 
  # We don't need it, but have to take the data anyway.
  my ($form_ref) = &ReadParse();
  my %FORM = %{$form_ref};

  # Check for SID
  if ($FORM{'sid'}) {
    $sid=$FORM{'sid'};
    # Open database connection
    $q="$site->{'db_site_config'}";
    $dbh=DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    $FORM{'sid'}=&decrypt_sid($FORM{'sid'},$site);
    $q="SELECT language,interface from sessions where sid=$FORM{'sid'} ";
    $FORM{'sid'}=&encrypt_sid($FORM{'sid'},$site);
    $sts = $dbh->prepare( $q );
    $sts->execute;
    $h=$sts->fetchrow_hashref;
    $sts->finish();
    $om_render_lang=$h->{'language'};
    $interface=$h->{'interface'}; 
    $om_render_lang=$site->{'om_locale_lang_def'} if ( ! $om_render_lang ); 
    $om_render_lang="eng" if ( ! $om_render_lang );
    $interface="html" if ( ! $interface ); 
    # Set content type
    if ($interface =~ "html"){
      print "Content-type: text/html\n\n";}
    elsif ($interface =~ "wml"){
      print "Content-type: text/vnd.wap.wml \n";}} 
  else {
    print "Content-type: text/html\n\n"; 
    print "<HTML><HEAD><TITLE>wrapper.cgi messages</TITLE></HEAD><BODY><BR><BR>$script_msg{1}</BODY></HTML>";
    print "\n\n";
    exit -1 ; }
  
  # Define path
  my $wrapper_path=sprintf("http://%s/cgi-bin/om/wrapper.cgi",$ENV{'HTTP_HOST'});
  my $default_path=sprintf("http\:\/\/%s\/cgi\-bin\/om\/wrapper.cgi\?sid=%s\&target=\/html\/eng\/home.shtml",$ENV{'HTTP_HOST'},$sid);
  my $old_home_path=sprintf("http\:\/\/%s\/$interface\/$om_render_lang\/home.shtml",$ENV{'HTTP_HOST'});
  my $new_home_path_end=sprintf("\/cgi-bin\/om\/wrapper.cgi\?sid=%s\&target=\/$interface\/$om_render_lang\/home.shtml",$sid);
  my $welcome_path=sprintf("http://%s/cgi-bin/om/welcome.cgi",$ENV{'HTTP_HOST'});
  my $new_welcome_path=sprintf("http://%s/cgi-bin/om/wrapper.cgi\?sid=%s&target=/cgi-bin/om/welcome.cgi",$ENV{'HTTP_HOST'},$sid);
  my $checkout_path=sprintf("http://%s/cgi-bin/om/wrapper.cgi\?sid=%s&target=/cgi-bin/om/ssl/checkout.cgi",$ENV{'HTTP_HOST'},$sid);
  
  
  # Check for target
  if ($FORM{'target1'}) {
    $pg_targ=$FORM{'pg_targ'} ;
    $pg_targ.=$FORM{'target1'};
    $pg_targ=sprintf("%s&target=%s",$FORM{'pg_targ'},$FORM{'target1'});
    $FORM{'pg_targ'}=$pg_targ;
    $target=$FORM{'target'}; }
  elsif ($FORM{'target'}) {
    $target=$FORM{'target'} ;
    if(($FORM{'search_keywords_inp'}) && (($FORM{'startrow'} eq "0") || ($FORM{'startrow'}))) {
      # Searching for multiple pages
      $target2="&search_keywords_inp=";
      $target2.=$FORM{'search_keywords_inp'};
      $target2.="&startrow=";
      $target2.=$FORM{'startrow'};}
    if ($ENV{'HTTP_REFERER'} ne $wrapper_path) { 
      if ( ($ENV{'HTTP_REFERER'} =~ /target1/) || ($ENV{'HTTP_REFERER'} eq $welcome_path) ) {
	$FORM{'pg_targ'}=$checkout_path;
	$pg_targ=$FORM{'pg_targ'}; } 
      else {
	$FORM{'pg_targ'}=$ENV{'HTTP_REFERER'} ;
	$pg_targ=$FORM{'pg_targ'}; }}
    elsif (! $FORM{'pg_targ'}) {
      $FORM{'pg_targ'}=$default_path;
      $pg_targ=$FORM{'pg_targ'} ; 
      $pattern_xyz="\/html\/eng\/http\:\/\/$ENV{'HTTP_HOST'}\/cgi\-bin\/om\/wrapper.cgi\?sid=$sid";
      $target =~ s/$pattern_xyz//gi;}}
  else {
    print "Content-type: text/html\n\n"; 
    print "<HTML><HEAD><TITLE>wrapper.cgi messages</TITLE></HEAD><BODY><BR><BR>$script_msg{2}</BODY></HTML>";
    print "\n\n";
    exit -1; }
    
  ## Handle diffrent conditions
  if (($FORM{'mode'} eq "APPROVE")&& ($FORM{'abc'})) {
    $argument=sprintf("sid=%s\&mode=%s\&addrid=%s\&addressbook=%s\&nextage=%s",$sid,$FORM{'mode'},$FORM{'addrid'},$FORM{'addressbook'},$FORM{'nextage'});
    $new_target=sprintf("http://%s%s\?%s",$ENV{'HTTP_HOST'},$target,$argument);
    $FORM{'mode'}=" ";
    $target=" ";}
  elsif ($FORM{'mode'} eq "SHOW") {
    $argument=sprintf("sid=%s\&mode=%s\&load=%s\&cur_page=%s\&nextpage=%s\&addrid=%s\&func=%s",$sid,$FORM{'mode'},$FORM{'load'},$FORM{'cur_page'},$FORM{'nextpage'},$FORM{'addrid'},$FORM{'func'});
    $new_target=sprintf("http://%s%s\?%s",$ENV{'HTTP_HOST'},$target,$argument);
    $FORM{'mode'}=" ";}
  else {$new_target=sprintf("http://%s%s\?sid=%s%s",$ENV{'HTTP_HOST'},$target,$sid,$target2);}
  
  # GET Method
  if ($ENV{REQUEST_METHOD} eq 'GET' ) { 
    $ua = new LWP::UserAgent;
    $ua->agent("Mozilla/4.0" . $ua->agent);
    $request = new HTTP::Request GET=>$new_target;
    $response = $ua->request($request);
    if ($response->is_error()) { 
      print "Content-type: text/html\n\n"; 
      print "<HTML><HEAD><TITLE>wrapper.cgi messages</TITLE></HEAD><BODY><BR><BR>$script_msg{3}</BODY></HTML>";
      print "\n\n";
      exit -1;} 
    else { 
      $content = $response->content();
      if ($interface=~"html"){
	&Fix_HTML_Links($content,$sid,$interface,$om_render_lang);}
      elsif ($interface=~"wml"){
	&Fix_WML_Links($content,$sid,$interface,$om_render_lang);}
      &InsertEntry($sid,$site->{'clusterid'},$dbh);  }}
  # POST Method
  elsif ($ENV{REQUEST_METHOD} eq 'POST' ) {
    $ua = new LWP::UserAgent;
    $ua->agent("Mozilla/4.0" . $ua->agent);
    $request = new HTTP::Request POST=>$new_target;
    $request->content_type('application/x-www-form-urlencoded');
    foreach $keyval (keys(%FORM)) {
      # SAM's magic number
      $FORM{$keyval} =~ s/\&amp\;/\%26/g;
      $FORM{$keyval} =~ s/\&/\%26/g;
      if($interface=~"html"){
	$postdata .= ("$keyval=$FORM{$keyval}&");}
      elsif(($interface=~"wml") && ($keyval ne "mode") && ($keyval ne "addrid")) {
	$postdata .= ("$keyval=$FORM{$keyval}&");}}
    chomp($postdata);
    $request->content($postdata);
    $response = $ua->request($request);
    if ($response->is_error()) { 
      print "Content-type: text/html\n\n"; 
      print "<HTML><HEAD><TITLE>wrapper.cgi messages</TITLE></HEAD><BODY><BR><BR>$script_msg{4}</BODY></HTML>";
      print "\n\n";
      exit -1;} 
    else { 
      $content = $response->content();
      if ($interface=~"html"){
	&Fix_HTML_Links($content,$sid,$interface,$om_render_lang);}
      elsif ($interface=~"wml"){
	&Fix_WML_Links($content,$sid,$interface,$om_render_lang);}
  &InsertEntry($sid,$site->{'clusterid'},$dbh);  }}
  # Neither GET or POST - Error
  else {
    print "Content-type: text/html\n\n"; 
    print "<HTML><HEAD><TITLE>wrapper.cgi messages</TITLE></HEAD><BODY><BR><BR>$script_msg{5}</BODY></HTML>";
    print "\n\n";
    exit -1;}

   $dbh->disconnect if ( $dbh );}




################################################################################
=pod

=head2 Fix_HTML_Links

 Fix_HTML_Links  - &Fix_HTML_Links($content,$sid,$interface,$om_render_lang);
 $contents       - contents of the page
 $sid            - session id
 $interface      - html/wml/xml
 $om_render_lang - eng (english)

=over 4

=item DESCRIPTION

 Fix HTML links to pass through wrapper 


=back

=cut
################################################################################
sub Fix_HTML_Links {
my @entirepage = split(/\n/,$_[0]);
my $sid=$_[1];
my $interface=$_[2];
my $om_render_lang=$_[3];

my $old_home_path=sprintf("http\:\/\/%s\/$interface\/$om_render_lang\/home.shtml",$ENV{'HTTP_HOST'});
my $new_home_path_end=sprintf("\/cgi-bin\/om\/wrapper.cgi\?sid=%s\&target=\/$interface\/$om_render_lang\/home.shtml",$sid);
my $home_path_wc="HREF=\"\/$interface\/$om_render_lang\/home\.shtml\"";
my $home_path_nc="HREF=\/$interface\/$om_render_lang\/home\.shtml";
my $new_home_path="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/$interface\/$om_render_lang\/home.shtml";
my $cart_path="HREF=\"\/cgi-bin\/om\/cart.cgi\"";
my $new_cart_path="HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/cart.cgi\"";
my $account_path="HREF=\"\/cgi-bin\/om\/account.cgi\"";
my $new_account_path="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/account.cgi";
my $wishlist_path="HREF=\"\/cgi-bin\/om\/wishlist.cgi\"";
my $new_wishlist_path="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/wishlist.cgi";
my $registry_path="HREF=\"\/cgi-bin\/om\/registry.cgi\"";
my $new_registry_path="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/registry.cgi";
my $hint_path="\"\/cgi-bin\/om\/hint.cgi\\?pg_targ=https:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/ssl\/checkout\.cgi\\?sid=$sid\"";
my $new_hint_path="/cgi-bin/om/wrapper.cgi?sid=$sid&target=/cgi-bin/om/hint.cgi&pg_targ=/cgi-bin/om/wrapper.cgi?sid=$sid&target1=/cgi-bin/om/ssl/checkout.cgi";
my $pattern2="method=post action=";
my $pattern3="name=\"sid\" value=\"\"" ;
my $pattern4="name=\"sid\" value=\"$sid\"" ;
my $pattern5=sprintf("http://%s/%s/%s/home.shtml",$ENV{'HTTP_HOST'},$interface,$om_render_lang);
my $pattern6="/cgi-bin/om/wrapper.cgi?sid=$sid&target=/$interface/$om_render_lang/home.shtml";
my $pattern7=sprintf("http://%s/cgi-bin/om/welcome.cgi\\?sid=$sid\&pg_targ=https://%s/cgi-bin/om/ssl/checkout.cgi\\?sid=$sid",$ENV{'HTTP_HOST'},$ENV{'HTTP_HOST'});
my $pattern8="/cgi-bin/om/wrapper.cgi?sid=$sid&target=/cgi-bin/om/welcome.cgi&pg_targ=/cgi-bin/om/wrapper.cgi?sid=$sid&target1=/cgi-bin/om/ssl/checkout.cgi";
my $pattern9="HREF\=\"http\:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/ssl\/checkout.cgi\\?sid\=$sid\"";
my $pattern10="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/cgi-bin\/om\/ssl\/checkout.cgi";
my $pattern11="VALUE\=\"checkout.cgi\"";
my $pattern12="VALUE\=\"\/cgi-bin\/om\/ssl\/checkout.cgi\"";
my $pattern13="HREF=\"checkout.cgi\\?";
my $pattern14="HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/cgi-bin\/om\/ssl\/checkout.cgi\&abc=abc\&";
my $pattern15="HREF=\"https:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/ssl\/checkout\.cgi\\?sid=$sid\"";
my $pattern16=sprintf("http://%s/cgi-bin/om/welcome.cgi\\?sid=$sid\&pg_targ=http://%s/cgi-bin/om/ssl/checkout.cgi\\?sid=$sid",$ENV{'HTTP_HOST'},$ENV{'HTTP_HOST'});
my $pattern17="HREF=http:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/welcome.cgi\\?sid=$sid";
my $pattern18="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/welcome.cgi";
my $pattern19="target=\/cgi-bin\/om\/ssl\/checkout.cgi";
my $pattern20="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/ss/\/checkout.cgi";
my $pattern22="METHOD=\"POST\" action=";
my $pattern23="HREF=\"\/cgi-bin\/om\/search.cgi\\?sid=\&search_keywords_inp";
my $pattern24="HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/cgi-bin\/om\/search.cgi\&search_keywords_inp";
my $pattern25="HREF=http:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/welcome.cgi\\?sid=$sid&pg_targ=\>";
my $pattern26="HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target=\/cgi-bin\/om\/welcome.cgi&pg_targ=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target1=\/cgi-bin\/om\/ssl\/checkout.cgi\>";
my $pattern27="pg_targ=http://$ENV{'HTTP_HOST'}/cgi-bin/om/wrapper.cgi\\?sid=$sid\&";
my $pattern28="pg_targ=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid&target1=\/cgi-bin\/om\/ssl\/checkout.cgi";
my $pattern29="\"\/cgi-bin\/om\/hint.cgi\\?pg_targ=http:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/ssl\/checkout\.cgi\\?sid=$sid\"";

foreach my $eachline (@entirepage) {
  if ($eachline =~ /$home_path_wc/i) {
    $eachline =~ s/$home_path_wc/$new_home_path/gi ;}
  if ($eachline =~ /$home_path_nc/i) {
    $eachline =~ s/$home_path_nc/$new_home_path/gi ;}
  if ($eachline =~ /$cart_path/i) {
    $eachline =~ s/$cart_path/$new_cart_path/gi ;}
  if ($eachline =~ /$account_path/i) {
    $eachline =~ s/$account_path/$new_account_path/gi ;}
  if ($eachline =~ /$wishlist_path/i) {
    $eachline =~ s/$wishlist_path/$new_wishlist_path/gi ;}
  if ($eachline =~ /$registry_path/i) {
    $eachline =~ s/$registry_path/$new_registry_path/gi ;}
  if ($eachline =~ /$hint_path/i) {
    $eachline =~ s/$hint_path/$new_hint_path/gi ;}
  if ($eachline =~ /$pattern29/i) {
    $eachline =~ s/$pattern29/$new_hint_path/gi ;}
  if ($eachline =~ /$old_home_path/i) {
    $eachline =~ s/$old_home_path/$new_home_path_end/ig ;}
  if((($eachline !~ /HREF=\"\//) || ($eachline !~ /HREF=\//)) && ($eachline !~ /HREF=\"http/) && ($eachline =~ /HREF=\"[a-zA-Z0-9].*html\"/)){
    my @junk=split(/\"/,$&);
    $eachline =~ s/HREF=\"[a-zA-Z0-9].*$interface\"/HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/$interface\/$om_render_lang\/$junk[1]\"/gi;}
  if($eachline =~ /HREF=\"\/$interface\/$om_render_lang\/[0-9]+\-[a-zA-Z]+\.shtml\"/){
    $eachline =~ s/HREF=\"\/$interface\/$om_render_lang/HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/$interface\/$om_render_lang/gi;}
  if($eachline =~ /href=\"\/$interface\/$om_render_lang\/[0-9]+\-[a-zA-Z]+\.shtml\"/){
    $eachline =~ s/href=\"\/$interface\/$om_render_lang/HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/$interface\/$om_render_lang/gi;}
  if($eachline =~ /HREF=\"\/$interface\/$om_render_lang\/auto\/[0-9]+\-[a-zA-Z]+\.shtml\"/){
    $eachline =~ s/HREF=\"\/$interface\/$om_render_lang\/auto/HREF=\"\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/$interface\/$om_render_lang\/auto/gi;}
  if($eachline =~ /HREF=\/$interface\/$om_render_lang\/[0-9]+\-[a-zA-Z]+\.shtml/){
    $eachline =~ s/HREF=\/$interface\/$om_render_lang/HREF=\/cgi-bin\/om\/wrapper.cgi\?sid=$sid\&target=\/$interface\/$om_render_lang/;}
  if ($eachline =~ /$pattern2/i)  {
    $eachline =~ s/$pattern2/METHOD=POST ACTION=\"\/cgi\-bin\/om\/wrapper.cgi\"\>\<INPUT TYPE=\"HIDDEN\" NAME=\"sid\" VALUE=\"$sid\"\>\<INPUT TYPE=\"HIDDEN\" NAME=\"target\" VALUE=/gi; }
  if ($eachline =~ /$pattern22/i) {
    $eachline =~ s/$pattern22/METHOD=\"POST\" ACTION=\"\/cgi\-bin\/om\/wrapper.cgi\"\>\<INPUT TYPE=\"HIDDEN\" NAME=\"sid\" VALUE=\"$sid\"\>\<INPUT TYPE=\"HIDDEN\" NAME=\"target\" VALUE=/gi; }
  if ($eachline =~ /$pattern3/i) {
    $eachline =~ s/$pattern3/$pattern4/gi ;}
  if ($eachline =~ /$pattern5/i) {
    $eachline =~ s/$pattern5/$pattern6/gi ;}
  if ($eachline =~ /$pattern7/i) {
    $eachline =~ s/$pattern7/$pattern8/gi ;}
  if ($eachline =~ /$pattern16/i) {
    $eachline =~ s/$pattern16/$pattern8/gi ;}
  if ($eachline =~ /$pattern9/i) {
    $eachline =~ s/$pattern9/$pattern10/gi ;}
  if ($eachline =~ $pattern15 ) {
    $eachline =~ s/$pattern15/$pattern10/gi ;}
  if ($eachline =~ /$pattern11/i) {
    $eachline =~ s/$pattern11/$pattern12/gi ;}
  if ($eachline =~ /$pattern13/i) {
    $eachline =~ s/$pattern13/$pattern14/gi ;}
  if ($eachline =~ /$pattern25/i) {
    $eachline =~ s/$pattern25/$pattern26/gi ;}
  if ($eachline =~ /$pattern27/){
    $eachline =~ s/$pattern27/$pattern28/gi ;}
  if (($eachline =~ /$pattern17/i) && ($eachline !~ /$pattern19/)) {
    $eachline =~ s/$pattern17/$pattern18/gi ;}
  if (($eachline =~ /$pattern17/i) && ($eachline =~ /$pattern19/)) {
    $eachline =~ s/$pattern19//gi;
    $eachline =~ s/$pattern17/$pattern18/gi ;}
 if ($eachline =~ /$pattern23/i) {
    $eachline =~ s/$pattern23/$pattern24/gi ;}
  # Print contents
  print $eachline;
  print "\n"; }}





################################################################################
=pod

=head2 Fix_WML_Links

 Fix_WML_Links   - &Fix_WML_Links($content,$sid,$interface,$om_render_lang);
 $contents       - contents of the page
 $sid            - session id
 $interface      - html/wml/xml
 $om_render_lang - eng (english)

=over 4

=item DESCRIPTION

 Fix WML links to pass through wrapper 


=back

=cut
################################################################################
sub Fix_WML_Links {
my @entire_wml_page = split(/\n/,$_[0]);
my $sid=$_[1];
my $interface=$_[2];
my $om_render_lang=$_[3];

open WAPOUTPUT, "> ../../$interface/$om_render_lang/page.wml";

foreach my $each_wml_line (@entire_wml_page) {

  # HVC - TODO 
  # Handle WML templates
  
  # Write each line to a file
  print WAPOUTPUT "$each_wml_line \n";}


# Print the contents
print "Location: http://$ENV{'HTTP_HOST'}/$interface/$om_render_lang/page.wml\n\n";
print "Content-type: text/vnd.wap.wml \n";
close(WAPOUTPUT);}






##############################################################################
=pod

=head2 message

 message         - &message($om_render_lang,$relpath,$script);
 $om_render_lang - eng (english)
 $relpath        - relative path
 $script         - error message type

=over 4

=item DESCRIPTION

 To support multiple languages, all error messages need to be stripped out of
 the script and translated to different languages.  This routine loads in the
 messages for the $site->{'om_render_lang'} that is currently active.

=item INPUT

 $site, A hashref containing information required throughout about the current
         site and session

=item OUTPUT

 A reference to the %script_msg hash of messages that need to be used in the
 script

=back

=cut
###############################################################################
sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

  my %script_msg;
  my $om_render_lang =$_[0];
  my $relpath=$_[1];
  my $script=$_[2];

  if ($om_render_lang eq "eng" ) {
    $script_msg{1}="sid not found, wrapper.cgi fails.";
    $script_msg{2}="target not found, wrapper.cgi fails."; 
    $script_msg{3}="Error in GET Method, wrapper.cgi fails."; 
    $script_msg{4}="Error in POST Method, wrapper.cgi fails."; 
    $script_msg{5}="Neither GET/POST Method, wrapper.cgi fails."; } 
  else {
    open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
    while(<EM>) {
      chomp;
      next if /^\#/;
      next if (! $_);
      my @temp=split /\t/;
      $script_msg{$temp[0]}=$temp[1]; }
    close(EM); }
  return %script_msg;}






################################################################################
=pod

=head2 Load_Module

Load_Module - &Load_Module(LWP::UserAgent);

=over 4

=item DESCRIPTION

Check to see if the required module is present

=item OUTPUT

Display error message on screen

=back

=cut
################################################################################
sub Load_Module{
  my ($site)=$_[1];
  eval "require $_[0]";
  if($@) { 
    print "Content-type: text/html\n\n";

    print <<EOM;
<html>
<head> <title> No cookiless shopping  </title> </head>
<body marginwidth="0" marginheight="0" leftmargin="0" topmargin="0" bgcolor="#ffffff">

<!-- Header -->
<table cellpadding="0" cellspacing="0" border="0" width="100%">
   <tr> <td>
&nbsp;
      </td>
      <td width="100%" bgcolor="#869dc2">
         <table cellpadding="8" cellspacing="0" border="0" width="100%">
            <tr>
               <td width="100%" valign="bottom">
                  <font face="verdana, helvetica, arial" size="3" color="#ffffff" style="font-size: 14pt;"><b>$site->{'sitename'}</b></font><br>
               </td>
               <td align="right">
                 &nbsp;
               </td>
            </tr>
         </table>
      </td>
   </tr>
</table>
<table cellpadding="0" cellspacing="0" border="0" width="100%">
   <tr>
      <td width="100%" bgcolor="455d83" background="/images/eng/os_allcommerce/admin/general/table_back.gif">
         <img src="/images/eng/os_allcommerce/admin/general/clear.gif" width="1" height="27" border="0" alt=""><br>
      </td>
   </tr>
</table>

    <P>Problem with $_[0] Module
    <P>Got the following error message
    <P>$@<BR><BR>
    <P>This store does not support cookieless shopping.
    <BR> If you require cookieless shopping, please contact
    <a href="mailto:$site->{'customer_service_email_address'}">
    customer services</a>.
    </BODY></HTML>


EOM
    exit -1 ; }}
 


########################REMOVE###################################################
## Make entry in clicks database
sub InsertEntry {
my $sid=$_[0];
my $clusterid=$_[1];
my $dbh=$_[2];

my $now=time();
my $value;

# insert values in clicks
my $q="INSERT INTO clicks VALUES ";

if ($ENV{'HTTP_REFERER'}) {
  $value = $ENV{'HTTP_REFERER'};} 
else {
   $value = "";}

$q.="('$sid','$now','$ENV{'REQUEST_URI'}','$value','$clusterid')";
my $sts = $dbh->prepare( $q );
$sts->execute;
}


