#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# po.cgi
#	PO manager for Zelerate WMS
#	LCF 20001125
####

############################################################
# Begin standard init
############################################################

use strict;
my ($scriptname, $relpath) = &load_libs();
my ($site, $form, $dbh)    = &init_script($scriptname, $relpath);

# Extra site data
$site->{'sid'}      = &setsessionid($site, $dbh, %{$form});
$site->{'home'}     = "http://$site->{siteurl}/html/eng/wms/";
$site->{'subtitle'} = '[PO]';

# Page and nav defaults
my $current_page = 'wm_page_wms_login';
my $nav          = qq{| PO};

############################################################
# End standard init 
############################################################


############################################################
# Begin main script operation
############################################################

####
# Check for a valid logged-in admin session.
#    First checks for a new login, via form data
#    If that's not found, it looks for a session with a 
#    valid custid. Returns the user's objid, or null (if 
#    not logged in).
####

my $admin_id = &check_for_login($form, $site);


####
# Short-circuit
#    Leave right away if there's no admin_id
####

#&page_out($form, $site, $current_page, $nav) unless ($admin_id);

# We're going back to the allcommerce login script if not logged in.
print "Location: http://$site->{'siteurl'}/cgi-bin/om/admin/login.cgi\n\n" unless ($admin_id);




####	
# Main operational switch 
#    See if we need the edit screen, the list screen, or the 
#    main screen (in that order).
####

if ($form->{'edit'}) {

	$form->{'ponum'} = $form->{'ponum_save'} unless ($form->{'ponum'});
	$form->{'ponum_save'} = $form->{'ponum'} unless ($form->{'ponum_save'});
	warn "PONUM is <<$form->{'ponum'}>>\n";

	$form->{'inp_err'} .= &create_po_main($site, $form);
	
	if ($form->{'inp_err'}) {
		$form->{'save.ponum'} = $form->{ponum};
		undef $form->{'ponum'};
	}
	
	$form->{'inp_err'} .= &create_po_item($site, $form);
	$form->{'inp_err'} .= &po_finalize($site, $form);
	
	$form->{'po_current_disp'} = &current_po($site, $form);
	$form->{'po_edit_cell'   } = &po_edit_cell($site, $form);	

	$current_page = 'wm_page_po_edit';
	$nav = qq{| <A HREF="po.cgi">PO</A> | PO UI};

} elsif ($form->{'list'}) {

	$form->{'open_po_list'   } = &list_open_pos($site, $form);

	$current_page = 'wm_page_po_open';
	$nav = qq{| <A HREF="po.cgi">PO</A> | Open PO Report};

} else {

	$current_page = 'wm_page_po_main';

}


####
# Finish up
#    Now do the standard output stuff, with the data as set above.
####

&page_out($form, $site, $current_page, $nav);


############################################################
# End of main script.
############################################################


############################################################
# Begin local libraries
############################################################

sub load_libs {
	use Cwd;
	my $separator = '/'; # What separates dirnames in this architecture?
	my ($scriptname, $args, $rundir);
	($scriptname, $args) = split /\?/, $0;
	if ($scriptname =~ /$separator/) {
		$scriptname =~ /^(.*)$separator(.*?)$/;
		$rundir = $1;
		$scriptname = $2;
	} else {
		$rundir = cwd();
	}
	my @path = split(/$separator/, $rundir);
	my $relpath;
	while (my $d = pop @path) {
		$relpath .= "..$separator";
		if ($d eq 'cgi-bin') {
			$relpath .= "..$separator";
			last;
		}
	}
	my $siteshortname = pop @path;
	$relpath .= $siteshortname;

	use DBI;
	require "$relpath/lib/wms_funcs.pm";
	# Load standard lib
	&load_standard_libs($relpath);
	
	# Load custom lib
	# must be the last require statement
	require "$relpath/lib/custom-lib.pm";

	
	return ($scriptname, $relpath);
}
	


