#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# 
#	Problem Orders & Inventory Mgmt for Zelerate WMS
#	LCF 20001221
####


############################################################
# Begin standard init
############################################################

use strict;
my ($scriptname, $relpath) = &load_libs();
my ($site, $form, $dbh)    = &init_script($scriptname, $relpath);

# Extra site data
$site->{'sid'}      = &setsessionid($site, $dbh, %{$form});
$site->{'home'}     = "http://$site->{siteurl}/html/eng/wms/";
$site->{'subtitle'} = '[Problem Orders & Inventory Management]';

# Page and nav defaults
my $current_page = 'wm_page_probleminv_main';
my $nav          = qq{| Problem Orders & Inventory Management};

############################################################
# End standard init 
############################################################


############################################################
# Begin main script operation
############################################################
if ($form->{'edit_box'} && $form->{'boxid'}) {
	my $box = &get_one_box($site, $form->{'boxid'});
	
	unless ($box->{'objid'}) {
		$form->{'inp_err'} = "Can't find box $form->{'boxid'}\n";
		$form->{'boxid'} = '';
	}
}
	

if ($form->{'move'} && $form->{'save'}) {
	
	$form->{'inp_err'} = &move_box($site, $form->{'objid'}, $form->{'uid'}, $form->{'location'}, 'active', 'reserve');
	$current_page = 'wm_page_pfpick_move';
	$nav          = qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | Move Box};
	
} elsif ($form->{'move'}) {
	
	$current_page = 'wm_page_pfpick_move';
	$nav          = qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | Move Box};

} elsif ($form->{'find_bin'}) {
	
	my $find = &localize_template('wm_cell_find_bin_input', $site->{'current_interface'},0,$site);
	$form->{'report'} = &permutebit($find, $form);
	
	if ($form->{'bin'}) {
		my $stat = $form->{'status'} || 'reserve';
		$form->{'report'} .= &list_bin_contents($site, $form, $stat);
	}
	
	$form->{'head_title'} = "List Inventory by Bin Location: $form->{'status'}";
	$current_page = 'wm_page_receiving_reports';
	$nav		  =	qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | $form->{'head_title'}};
	
} elsif ($form->{'find_sku'}) {

	$form->{'report'} = &localize_template('wm_cell_find_sku_input', $site->{'current_interface'},0,$site);
	
	if ($form->{'sku'}) {
		$form->{'report'} .= &find_boxes_by_sku($site, $form);
	}
	
	$form->{'head_title'} = "Find SKU in Active and Reserve Locations";
	$current_page = 'wm_page_receiving_reports';
	$nav		  =	qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | $form->{'head_title'}};

} elsif ($form->{'edit_box'} && $form->{'boxid'}) {

	$form->{'inp_err'} = &edit_box_save($site, $form);
	$form->{'box_details'} = &display_box($site, $form->{'boxid'});
	$form->{'edit_details'} = &box_edit_form($site, $form->{'boxid'});
	$current_page = 'wm_page_box_edit';
	$nav		  =	qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | Edit Box Details};
	
} elsif ($form->{'edit_box'}) {

	$form->{'report'} = &enter_boxid($site, $form);
	$form->{'head_title'} = "Edit Box Details";
	$current_page = 'wm_page_receiving_reports';
	$nav		  =	qq{| <A HREF="probleminv.cgi">Problem Orders & Inventory Management</A> | $form->{'head_title'}};
	
}

&page_out($form, $site, $current_page, $nav);

############################################################
# Begin local libraries
############################################################

sub load_libs {
	use Cwd;
	my $separator = '/'; # What separates dirnames in this architecture?
	my ($scriptname, $args, $rundir);
	($scriptname, $args) = split /\?/, $0;
	if ($scriptname =~ /$separator/) {
		$scriptname =~ /^(.*)$separator(.*?)$/;
		$rundir = $1;
		$scriptname = $2;
	} else {
		$rundir = cwd();
	}
	my @path = split(/$separator/, $rundir);
	my $relpath;
	while (my $d = pop @path) {
		$relpath .= "..$separator";
		if ($d eq 'cgi-bin') {
			$relpath .= "..$separator";
			last;
		}
	}
	my $siteshortname = pop @path;
	$relpath .= $siteshortname;

	use DBI;
	require "$relpath/lib/wms_funcs.pm";
	# Load standard lib
	&load_standard_libs($relpath);
	
	# Load custom lib
	# must be the last require statement
	require "$relpath/lib/custom-lib.pm";

	
	return ($scriptname, $relpath);
}
	
