#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
# 
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
#
# configure.pl
#    installation routine that will check Perl version,
#    DBI/DBD functionality, database creation/recreation,
#    data structure input, write to conf files, and inital dataload
#
#    Version 21 RF 20001015
#       new company name
#
#    Version 20 RF 20001009
#       reinserted lost Apache "DirectoryIndex" entry for virtual host
#       rename -> copy routine for httpd conf file
#
#    Version 19 JC 20001003
#       added default localhost
#
#    Version 18 JC 20001002
#       cosmetic changes part 2
#
#    Version 17 JC 20000929
#       cosmetic changes to script
#                                   
#    Version 16 JC 20000920
#       bug fixes to issues in version 14
#
#    Version 15 GF 20000914
#       Removed lt from Data Source
#
#    Version 13 NC 20000911
#       SSLDisable
#
#    Version 12 JC 20000911
#       $driver needed to recreate database
#
#    Version 11 JC 20000911
#       bug fixes
#
#    Version 10 JC 20000906
#       bug fixes
#
#    Version 9 JC 20000829
#       change of wwwlib-perl library error message
#
#    Version 8 JC 20000803
#       bug fix to conditional
#
#    Version 7 IB 20000730
#       fixed a typo
#
#    Version 6 JC 20000730
#       ssl revision
#
#    Version 5 JC 20000728
#       bug fixes and separation of database choices
#
#    Version 4 JC 20000727
#       added help on command line
#
#    Version 3 JC 20000726
#       put contacts insert line in
#       (dropped between version 1 and 2) 
#
#    Version 2 JC 20000726
#       bug fixes
#       added httpd subroutine
#       added arguments on the command line
#
#    Version 1 JC 20000723
#       creation
#
#

use strict;

###############################################################################
=pod

=head1 configure.pl 

configure.pl - configuration script

=head1 SYNOPSIS


=head1 DESCRIPTION

This script is responsible for configurating AllCommerce.

=head1 NOTES & BUGS

=cut
###############################################################################  

&main();

##############################################################################
=pod

=head2 libs

libs

=over 4

=item DESCRIPTION

Initial library load.

=item INPUT

None

=item OUTPUT

None

=item NOTES

=back

=cut
############################################################################### 

sub libs {

    require "./lib/dataload.pm";
    require "./lib/update_conf.pm";
    require "./lib/admin_object.pm";
    require "./lib/db_layer.pm";
    require "./lib/permission_layer.pm";
    require "./lib/filetools.pm";
    require "./lib/conversions.pm";
    require "./lib/validate.pm";
    require "./lib/variant.pm";
    require "./lib/formtools.pm"; 
    require "./lib/configure.pm";
    require "./lib/language.pm";
    require "./lib/custom-lib.pm";
    require "./lib/generate_temps.pm";
}

sub main {
    my ($lang,$scriptname,$site,$new,$param,$sth,$dbh,$remote);

    $scriptname="configure.pl";
    ## give $site relpath for get_messages subroutine
    $site->{'relpath'}=".";

    ## hardcode for english for now
    $lang="eng";

    &libs;

    $site->{'script_msg'} = &get_messages($lang,$scriptname,$site);

    if ( @ARGV ) {

        if ( $ARGV[0] eq "database" ) {
            &version($site);
            ($remote) = &dbi($site); 
            ($new,$dbh,$site)=&db($site,$remote);
            &tables($site,$dbh);
        } elsif ( $ARGV[0] eq "http" ) {
            &version($site);
            ($param)=&httpd($site);
        } elsif ( $ARGV[0] eq "conf" ) {
            &version($site);
            ( $site ) = &conf($site,$new,$param);
            `../crons/generate_temps.pl 47004-A`;
            `../crons/generate_temps.pl 74003-A`;
        } elsif ( $ARGV[0] eq "idl" ) {
            &version($site);
            ($remote) = &dbi($site);
            ($new,$dbh,$site)=&db($site,$remote);
            &tables($site,$dbh);
            &idl($site,$dbh);
        } elsif ( $ARGV[0] eq "ship" ) {
            &version($site);
            ($remote) = &dbi($site);
            ($new)=&db($site,$remote);
            &tables($site,$dbh);
            &idl($site,$dbh);
            &ship($site);
        } elsif ( $ARGV[0] eq "help" ) {
            print "Options:  database, http, conf, idl, ship\n";
            print "Please refer to the install notes for more information.\n";
            exit;
        } else {
            print "Bad Argument.\n";
            exit;
        }       
    } else {

        &header($site);
        &version($site);
        ($remote) = &dbi($site);
        ($new,$dbh,$site) = &db($site,$remote);
        ($param) = &httpd($site);
        ($site) = &conf($site,$new,$param); 
        &tables($site,$dbh);
        &idl($site,$dbh);
        &ship($site,$dbh);

    } 

    $dbh->disconnect if ($dbh);

}

