#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

###############################################################################=pod

=head1 generate_site.pl

generate_site.pl - Content Generation Script

=head1 SYNOPSIS


=head1 DESCRIPTION

This script is responsible for constructing html, wml, and other content files
from admin_object.cgi.

=head1 NOTES & BUGS

=cut
###############################################################################

&main();

##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

=item INPUT

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################

sub main {

    my ($site,$dbh,$objid,$index,$linkup,$linkdown);

    ($site, $dbh) = &initscript();
    ($objid,$index)=&query_database($site,$dbh);
    ($objid,$index)=&query_things($site,$dbh,$objid,$index);
    ($objid,$index)=&query_prices($site,$dbh,$objid,$index);
    ($objid,$index)=&query_extraprop($site,$dbh,$objid,$index);
    ($objid,$index)=&query_inventory($site,$dbh,$objid,$index);
    ($linkup,$linkdown)=&query_linkage($site,$dbh);  
    &build_pages($site,$dbh,$objid,$index,$linkup,$linkdown);

    $dbh->disconnect;
}

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);

    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;

    @rpath=@temp;

    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

    $rundir= join "/", @rpath;
    chdir( $rundir );

    $relpath="";
    $siteshortname="";
    ## short circuit if $s does not contain cgi-bin
    if ( $s !~ /crons/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case

    while ( @temp ) {
        $u = pop @temp;

        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels

        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );

        if ( $u eq "crons" ) {
            $siteshortname = $temp[$#temp];
            @temp=();
        } # note - crons is off the site root, so the next up is site root

    } # done constructing relative path to store root

    $relpath .= "/../$siteshortname";
    ($site, $dbh) = &initsite($scriptname, $relpath);

    return ($site, $dbh);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables
that are required throughout the script.

=item INPUT

$scriptname - The name of the script (admin_object.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'}
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;

    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    my ($select_hash, $q2);

    use DBI;

    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/permission_layer.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/generate.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/generate.pm";

    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";

    ## establish site-specific parameters
    #
    $site=setsite();

    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});


    $site->{'om_render_lang'} = "";
    $interface = "";

    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );

    ## load messages
    $script=( split /\./, $scriptname )[0];
    #   &initClassAndType();

    $n= $#temp - 1;
    # Configure all parameters to pass around in $site hash
    $site->{'interface'} = $interface;
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/generate_site";
    $site->{'sid'} = $site->{'sid'};
#    ($site->{'OBJCLASSES'}, $site->{'OBJTYPES'}) = &getClassesAndTypes();
#    $site->{'STATUS_TXT'} =  &loadseltomem("sel_status", $site->{'selskeletonpath'});
    $site->{'session_lang'} = $session_lang;
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
#    $site->{'script_msg'} = &get_messages($site->{'om_render_lang'}, $site->{'scriptname'}, $site);

    return ($site, $dbh);
}

sub debug { };
1;
