#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
# make_log_digest.pl
# RF 990416
#
# Version 11 IB 20000502
#    introduced $script variable
# Version 10 IB 20000411
#    $def_lang -> $om_render_lang
# Version 9 JC 20000411
#    default to eng if no default lang 
# Version 8 JC 20000410
#    multilingual messages 
# Version 7 IB 20000302
#    updated copyright
# Version 6 RF 991015
#    converted to new directory structures
#       -> use of transit/export/log
#       -> use of transit/import/trigger
# Version 5 RF 990419
#    added cleanup of outboundlogs based on contents of /www/readback
# Version 4 RF 990419
#    added compression of logs and movement into /www/outboundlogs
# Version 3 RF 990419
#    fixed bug in pageview counter
# Version 2 RF 990416
#    excluded error logs from processing
#    corrected wording of hits/pageviews
#    TBD - 
#       zip primary logs when done
#       move processed files to an archive or upload point?
# Version 1 RF 990416
#
#
##############################################################################
use strict;
use File::Copy;
##############################################################################
=pod

=head1 make_log_digest.pl

make_log_digest.pl -

=over 4

=item DESCRIPTION
 
 script to take a logfile and prepare digest of it:
    hits
    pageviews
    session ids
    pageviews by pagename
     

=item INPUT

=item OUTPUT

=item NOTES

 BUGS and TO-DO
   1. directory hardcoded
   2. looks for logfiles ending in .YYMMDD_HHMM for processing
          then renames it to host.filename, moves it to the
          /usr/local/home/sysproc/movelogs/outbound directory
          and puts the digest there as well
   3. referer data and proper sessioning


=back

=cut
###############################################################################

my $server_root="/www/stronghold";
my $server_logs="$server_root/logs";
my $pickuppoint="/www/outboundlogs";
my $confirmdrop="/www/readback";
my $server_log_archive="/www/logarchive";

## construct just the host portion of the fully qualified hostname
#

use Sys::Hostname;
my $hostname = hostname;
$hostname = ( split /\./, $hostname )[0];


# Find out where we are, so we can load libs.
##
use Cwd;
my $separator = '/'; # What separates dirnames in this architecture?
my ($scriptname, $args, $rundir);
($scriptname, $args) = split /\?/, $0;
if ($scriptname =~ /$separator/) {
	$scriptname =~ /^(.*)$separator(.*)$/;
	$rundir = $1;
	$scriptname = $2;
} else {
	$rundir = cwd();
}
chdir( $rundir );
my @path = split(/$separator/, $rundir);
my $relpath='./';
while (my $d = pop @path) {
	$relpath .= "..$separator";
	if ($d eq 'crons') {
		$relpath .= "..$separator";
		last;
	}
}
my $siteshortname = pop @path;
$relpath .= $siteshortname;
###########

 require "$relpath/conf/site.pm"; 

## multilingual messages
#
my $site = setsite();
my $om_render_lang="$site->{'om_locale_lang_def'}";
my $om_render_lang="eng" if ( ! $om_render_lang );
 
## load messages
#
my %script_msg=();
my $script=( split /\./, $scriptname )[0];
&message; 

## cleanup of transit directory
#
my @files=();                  # empty list of logfiles to move
opendir LOGS, $confirmdrop;
my @allfiles = readdir LOGS;
closedir LOGS;

my $f;
foreach $f ( @allfiles ) {
   next if ( $f !~ /^ok\./ );     # if not an "ok" file ...
	unlink $confirmdrop/$f;
   $f =~ s/^ok\.// ;              # remove the leading "ok"
	move("$pickuppoint/$f","$server_log_archive"); # to the archive...
} # done digging through the readbacks

## move into logs directory and move new files 
#
@files=();                  # empty list of logfiles to move
opendir LOGS, $server_logs;
@allfiles = readdir LOGS;
closedir LOGS;

while (@allfiles) {
   $f = pop @allfiles;
   next if ( $f !~ /^sh_/ );                   # all logs start with this
   next if ( $f !~ /\d{6}_\d{4}/ );  # skip non-timestamped logs
   next if ( $f !~ /transfers/ );              # avoid the error logs
   push @files, $f;
} # done looking through file list


## hack for testing
# @files= ( "sh_homepage_transfers.990316_0100" );

while (@files) {               # start moving files
   my ($fstart,$fend,$fdigest,$hits,$pgvw,$l,$pr,$st,$k);
   my (%refer_p,%view_p,%search_t);
   my (@pn,@temp);
   $fstart = pop @files;
   $fend = "$hostname.$fstart";
   $fdigest="$fend".".dig";

   ## initialize hashes
   #
   %refer_p=();             # referers
   %view_p=();              # views by pagename
   %search_t=();            # searches by searchterm
   ## scalars
   #
   $hits=0;
   $pgvw=0;

   open(FH, "<$server_logs/$fstart" );
   while (<FH>) {
      $hits++;            # increment the hits counter
      $l = $_ ;
      @pn =  ( $l=~/"GET.+?"/g ) ;
      @pn =  ( $l=~/"POST.+?"/g ) if ( ! @pn );
      # now get the requested page name 
      $pr = ( split /\s+/, $pn[0] )[1];

      ## what kind of request?
      #
      # $pr = "z_image_hits" if ( $pr =~ /(gif|jpg)$/ ) ;
      if ( $pr =~ /(gif|jpg)$/ ) {
         $pr = "z_image_hits" ;
      } else {
         $pgvw++;                    # it is a pageview!
      } # done determining if an image hit or pageview
      $st="";
      ( $pr,$st ) = ( split /\?/, $pr );
      if ( $st ) {
         ( $k,$st ) = split /keywords=/, $st;
         ( $st,$k ) = split /\&/, $st;
         $st="-" if ( ! $st );
         # print " $st \n";
         $search_t{$st}++;
      }
      $view_p{$pr}++;

   } # end of reading through lines of logfile
   close(FH);

   open(FO , ">$server_logs/$fdigest");
   print FO "$hostname \n";

   ## now dump statistics
   #
   print FO "$script_msg{1}:\n";
   print FO "   $script_msg{2}:                $hits \n";
   print FO "   $script_msg{3}:          $pgvw \n";
   $pgvw = $view_p{'/'};
   print FO "   $script_msg{4}: $pgvw \n";
   $pgvw = $view_p{'/cgi-bin/register.cgi'} ;
   print FO "   $script_msg{5}: $pgvw \n";
   print FO "\n\n";

   print FO "$script_msg{6}:\n";
   @temp = sort keys %view_p;
   foreach $k ( @temp ) {
      print FO "   $view_p{$k} views $k \n";
   } #

   print FO "$script_msg{7}:\n";
   @temp = sort keys %search_t;
   foreach $k ( @temp ) {
      print FO "   $search_t{$k} searches: $k \n";
   } #

   ## cleanup
   #
   close(FO);

   ## move logfile
	my $gid = getgrnam("nobody");
	my $uid = getpwnam("logmove") || die "no \"logmove\" user. Use adduser to add logmove. Make sure to add it nologin.\n";
   next if ( -e "$server_logs/$fend" );
	move("$server_logs/$fstart","$server_logs/$fend");  # rename host.
   `/bin/gzip $server_logs/$fend`;                     # compress log
   $fend.=".gz";
	move("$server_logs/$fend","$pickuppoint/$fend");  # pickup
	chown $uid, $gid, "$pickuppoint/$fend";  # logmove owns it
	move("$server_logs/$fdigest","$pickuppoint/$fdigest");  # pickup
	chown $uid, $gid, "$pickuppoint/$fdigest";  # logmove owns it

}                              # done moving files

##############################################################################

##############################################################################
=pod

=head2 message

message -

=over 4

=item DESCRIPTION

checks the language that is being used and open appropriate
message language file or get messages below


=item INPUT

=item OUTPUT

A script message hashref containing appropriate messages.

=item NOTES


=back

=cut
###############################################################################
sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

   if ($om_render_lang eq "eng" ) {
      $script_msg{1}="Basic Server Stats";
      $script_msg{2}="Hits";
      $script_msg{3}="Page Views";
      $script_msg{4}="Root Homepage Views";
      $script_msg{5}="Signup Thanks/Error";
      $script_msg{6}="Pageviews by pagename";
      $script_msg{7}="Search Terms";
   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         my @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }
      close(EM);
   }
}

1;
