#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# run_stats.pl
# 
# Create daily summary report on core statistics
# Version 2 NC 20000929
#    space between titles of variants
# Version 1 NC 20000920
#    mark voided orders using orderid instead of sid
# Version 0.7.36 NC 20000829
#    get rid of voided orders
# Version 0.7.35 NC 20000825
#    fixed report and add number and percentage for visitors left 
# Version 0.7.34 NC 20000822
#    header document, use strict, exclude employees' stuff
# Version 0.7.33 NC 20000818
#    fixed reports by adding back the missing contents and editing some
#    field.
# Version 0.7.32 IB 20000729
#    fixed reports
# Version 0.7.32 IB 20000727
#    correction to order totatls (exclude incomplete orders)
# Version 0.7.31 IB 20000727
#    correction to shipping cost
# Version 0.7.30 IB 20000726
#    shipping costs
# Version 0.7.29 IB 20000724
#    changed the negative value for no ordering
# Version 0.7.28 IB 20000719
#    FORM variables were not being filled before the second pass
# Version 0.7.27 IB 20000717
#    correct the location of the report for cron jobs
# Version 0.7.26 IB 20000711
#    TOP20->TOP25
# Version 0.7.25 IB 20000710
#    objid -> lineobjid
# Version 0.7.24 IB 20000707
#    msku -> objid
# Version 0.7.23 IB 20000706
#    removed EE1 
#    corrected blank line in the report
# Version 0.7.22 IB 20000702
#    corrected report
# Version 0.7.21 IB 20000606
#    deleted redundant comment lines
# Version 0.7.20 IB 20000603
#    further conversion
# Version 0.7.19 IB 20000602
#    converted to Rel. 1.0.1
# Version 0.7.18 IB 20000531
#    added custom-lib.pm
# Version 0.7.17 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.16 JC 20000411
#    default to eng if no default lang
# Version 0.7.15 JC 20000330
#    changed connect statement
# Version 0.7.14 JC 20000328
#    changed DBI prepare/connect
# Version 0.7.13 IB 20000327
#    corrected $relpath
# Version 0.7.12 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.11 IB 20000316
#    new directory structure
# Version 0.7.10 IB 20000302
#    removed -w
# Version 0.7.9 IB 20000301
#    updated copyright
# Version 0.7.8 IB 20000229
#    changed table to database
# Version 0.7.7 IB 20000223
#    changed the name store.pm to site.pm
# Version 0.7.6 JZ 20000215
#    display actual checked out orders, use templates
# Version 0.7.5 JZ 20000207
#    corrected the numbers
# Version 0.7.4 IB 20000204
#    changed "Snapshot view" to "Snapshot View", deleted obsolete summary
# Version 0.7.3 JZ 20000126
#    print to file
# Version 0.7.2 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    removing of HACKS, and correcting of relpath
# Version 2 RF 991108
#    rearangement of hash ordering and prelim dump
# Version 1 RF 991104
#    creation
###############################################################################

=pod

=head1 run_stats.pl

run_stats.pl - Administrator Statistic 

=head1 SYNOPSIS

http://domain/os_allcommerce/cron/run_stats.pl

=head1 DESCRIPTION

This script is for generating statistics report which can be called by statsman.cgi or be run by crontab. 

=head1 NOTES & BUGS

=cut

###############################################################################

use strict;

&main();

##############################################################################

=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

The main subroutine is resposible for determining the flow of the run_stats.pl script. 

=item INPUT

AVG[0] - user who generate the report 

AVG[1] - start time to generate the report, which is the machine system time

AVG[2] - the length of days for generating the report (int)

=item OUTPUT

set the data for the home page of Stats Manager

if user=cron, create detailed report /os_allcommerce/data/stats/cron/yyyymmdd.html 

if user is anything else other than cron, create detailed report /os_allcommerce/data/stats/include/yyyymmdd.html 

=item NOTES

=back

=cut

###############################################################################

sub main {

    my ($site, %FORM, %NVP, $dbh);

    ## Initialization
    ($site, $dbh, %FORM) = &initscript();
    %NVP=();

    ########### general flow ###########
    #
    # validation of input args ( can be order list )
    #    fill in defaults for store/date
    #
    # select all sessions for date
    #    determine sessions per referer
    #    determine sessions per adbuy
    #    determine sessions per artwork
    #    determine sessions per affiliate
    #    determine sessions per browser
    # 
    # select all pageviews
    #    determine pagedepth per session
    #    determine total number of page views
    #
    # select all orders
    #    determine total revenues
    #    determine total line items
    #    determine total items
    #    determine total shipping
    #    determine total tax
    #
    # Derivative Values
    #    average pagedepth = pageviews/sessions
    #    conversion rate = sessions/orders
    ########################################################    


    ## initialize tallying hash
    #
    my ($days, $statuser, $time_n, $time_i, $time_f);
    my ($sth, $filedate, $snap, $fromtime, $totime, $snapflag);
   
    $days = 0;
    $NVP{'snap_visleft'} = 0;
    $NVP{'snap_checkout'} = 0;
    $NVP{'snap_left'} = 0;
    $NVP{'snap_leftorder'} = 0;
    $NVP{'topitems'} = "";
    $NVP{'snap_newjoin'} = 0;
    $NVP{'snap_noorder'}=0;
    

    ## construct timestamps
    #
    if ( $#ARGV >= 2 ) {
    # there was a second argument, see if it is numeric...
    # TBD - what formats are acceptable?
    #	$test_str = $ARGV[1];
    #	$year  = $test_str =~ m/.{4}$/  ;
	
    #	if (($year < 1970) or ($year > 2100)) {
    #			print "ERROR: invalid year format yyyymmdd\n";
    #			exit 0;
    #		}
    ### for now the number is the int days back
	$days = $ARGV[2];
    }

    # figure out the user who generated this file, default is cron
    if ($ARGV[0]) {
 	$statuser = $ARGV[0];
    } else {
	$statuser = 'cron';
    }

    # by default, this will run by cron and it does not have any argument
    if ($ARGV[1]) {
	$time_n=$ARGV[1];
    } else {
	$time_n=time();
    }

    # we do the stats every 24 hours in default 3:00am in the morning
    # construct timestamp of midnight last
   
    for (0..$days){

	if ($time_i) {$time_n = $time_i - (24 * 60 * 60);}
	$time_f=$time_n ; 
        $time_i=$time_f - (24 * 60 * 60); # midnights are always 24hours apart

        my ($q, $num, $h, %employee, %tot, @results);
        %employee=();
        %tot=();

        ##search for contacts (NC)
        #$q="SELECT objid, objtype, cdate FROM contacts"; 
        my $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "contacts";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objtype'} = "objtype";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'cdate'} = "cdate";
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);
#        $sth= $dbh->prepare ( $q );
#        $sth->execute;
#        $num=$sth->rows;
        foreach $h (@results) {

            if ($h->{'objtype'} eq '39004-A'){
               $employee{$h->{'objid'}}=1;  # Mark the employees

            } elsif ( $h->{'cdate'} >= $time_i && $h->{'cdate'} < $time_f ) {
               $NVP{'snap_newjoin'}++; # count new customers
            }            
        }

        ##search for sid with clicks (NC)
        my %clicks=();
        #$q="SELECT sid FROM clicks where cdate>=$time_i and cdate<$time_f";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "clicks";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'sid'} = "sid";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);	
#        $sth= $dbh->prepare ( $q );
#        $sth->execute;
#        $num=$sth->rows;
	foreach $h (@results) {
	    my $sid = $h->{'sid'};
#            my $sid = $sth->fetchrow_array;
            $clicks{$sid}=1;
        }     

        ## sessions
        #
        # initialize session hash
	my %session=();
        # initialize counter hashes
        my %ses_vis=();               # by visitor
	my %ses_ref=();               # by referer
	my %ses_adb=();               # by adbuy
	my %ses_adb_pur=();           # made purchase
	my %ses_art=();               # by banner artwork
	my %ses_art_pur=();           # made purchase
	my %ses_bro=();               # by browser
        #my %ses_aff=();               # by affiliate code
                                           
        $ses_vis{'log'} = 0;          # total logged in visitors (NC)
        $ses_vis{'nolog'} = 0;        # total not logged in visitors (NC)
        my $totvisitors = 0;          # total visitors (NC)
        my $tot_bro=0;                # total browsers (NC)

        my %visitors=();              # hash holds visitors sid

	 #$q="SELECT \* from sessions where cdate>=$time_i and cdate<$time_f ";
   $select_hash = {};
   $select_hash->{'TABLE'}->{'NAME'} = "sessions";
   $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
   $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
   $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
   $select_hash->{'statement'} = "SELECT";
   $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);	
#	$sth = $dbh->prepare( $q );
#	$sth->execute;
#	$num=$sth->rows;
	$tot{'sessions_raw'}=$num;       # total sids burned through
	$tot{'sessions_ok'}=0;           # ignoring sessions from within site
	foreach $h (@results) {
	    #foreach ( 1..$num ) {
#		$h = $sth->fetchrow_hashref;
 
                # Get rid of employees' stuff and mark sid which is run by employees
                if ($employee{$h->{'custid'}}) {
                    $employee{$h->{'sid'}}=1; #employees' sid
                    next;
                }

		## filter out internally referred sessions
		#
		$tot{'sessions_ok'}++;
		
		## store session minimum data
		#
		$session{$h->{'sid'}}=$h;   # HACK - this will explode for large sessions...
		
		## test for existence of counters
		#
		#if ( $h->{'sid'} && $clicks{$h->{'sid'}} ){ # count customers with at least one click (NC) 
                if ( $h->{'sid'} ) {

                    $visitors{$h->{'sid'}}=1; # mark the visitors sid       
      
                    #Count toal visitors, login, and not login visitors (NC)     
                    $totvisitors ++;      # Total visitors
                  
                    if ($h->{'custid'}){
                        $ses_vis{'log'} ++;   # Login visitors
                    } else {
                        $ses_vis{'nolog'} ++; # Not login visitors
                    }  

	       	$ses_ref{$h->{'referer'}}++ if ( $h->{'referer'} );     # by referer
		$ses_adb{$h->{'adbuy'}}++   if ( $h->{'adbuy'} );         # by adbuy code
		$ses_art{$h->{'artwork'}}++ if ( $h->{'artwork'} ) ;    # by banner ad artwork
		$ses_bro{$h->{'browser'}}++ if ( $h->{'browser'} ) ;    # by customer browser
                $tot_bro++ if ( $h->{'browser'} );   # Count total browsers (NC)
                } 
                ## TBD - add affiliate counting
	} # done with sessions

       
        ## start in on order lines (search ocart)

        # login orders are placed in orders
        # checked out orders are placed in ocart
        # orders that are not checked out are placed in tcart

        # search for voided orders
        my %voided=();

        #$q="SELECT objid from orders where cdate>=$time_i and cdate<$time_f and (status = '15028-A' or status = '15029-A')";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "orders";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[2] = build_where_hash("status","15028-A", "or");
        $select_hash->{'TABLE'}->{'WHERE'}[3] = build_where_hash("status","15029-A", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);	
#        $sth = $dbh->prepare( $q );
#        $sth->execute;
#        $num=$sth->rows;
     
        foreach $h (@results) {
	    #foreach (1..$num) {
#           my $objid = $sth->fetchrow_array;
	    my $objid = $h->{'objid'};
           $voided{$objid}=1;
        }
        

	my %tot_sku=();
	my %sku_price=();
        my %tot_objid=();
        my %objid_price=();
        my %tot_lineobjid=();
        my %lineobjid_price=();
        my %checkedout=();
        my %tot_adb=();
        my %inocart=();             # hash holds the order sid in ocart

       
	#$q="SELECT \* from ocart where cdate>=$time_i and cdate<$time_f and objclass != '140-A'";
   $select_hash = {};
   $select_hash->{'TABLE'}->{'NAME'} = "ocart";
   $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
   $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
   $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
#   $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("objclass","<!140-A", "and");
   $select_hash->{'TABLE'}->{'WHERE'}[2] = build_where_hash("objclass","81001-A", "and");
   $select_hash->{'statement'} = "SELECT";
   $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);	
#	$sth = $dbh->prepare( $q );
#	$sth->execute;
#	$num=$sth->rows;
        $tot{'orders_shp'}=0;          # total shipping paid
	$tot{'orders_raw'}=$num;       # total pageviews burned through
#	foreach ( 1..$num ) {
#           $h = $sth->fetchrow_hashref;
	foreach $h (@results) {

           # Get rid of employees' stuff
           next if $employee{$h->{'sid'}};

           $inocart{$h->{'sid'}}=1;   #hash holds orders sid in ocart

           # get rid of voided orders
           next if $voided{$h->{'orderid'}};

           $checkedout{$h->{'orderid'}} =1;
           $tot_lineobjid{$h->{'lineobjid'}} = 0 if ( ! $tot_lineobjid{$h->{'lineobjid'}});
           $tot_lineobjid{$h->{'lineobjid'}} += $h->{'quant'} ;

           $lineobjid_price{$h->{'lineobjid'}} = $h->{'unitpr'} if ( ! $lineobjid_price{$h->{'lineobjid'}});

	   # next if (! $orders{$h->{'sid'}});  #order not checked out
	   $tot_sku{$h->{'msku'}} = 0 if ( ! $tot_sku{$h->{'msku'}});
           $tot_sku{$h->{'msku'}} += $h->{'quant'} ;
           $tot_objid{$h->{'objid'}} = 0 if ( ! $tot_objid{$h->{'objid'}});
           $tot_objid{$h->{'objid'}} += $h->{'quant'} ;

           $sku_price{$h->{'msku'}} = $h->{'unitpr'} if ( ! $sku_price{$h->{'msku'}});
           $objid_price{$h->{'objid'}} = $h->{'unitpr'} if ( ! $objid_price{$h->{'objid'}});

	   next if ( ! $h->{'sid'} );
	   if ($session{$h->{'sid'}}->{'adbuy'}) {
               if (! $ses_adb_pur{ $session{$h->{'sid'}}->{'adbuy'} }) {
	           $ses_adb_pur{ $session{ $h->{'sid'} }->{'adbuy'} } =0;
	       } 
	      
               $ses_adb_pur{$session{$h->{'sid'}}->{'adbuy'}}++; 
              
               if (!$tot_adb{$session{$h->{'sid'}}->{'adbuy'}}) {
	           $tot_adb{$session{$h->{'sid'}}->{'adbuy'}} = 0 ;
	       }
	      
               $tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'unitpr'}
	   }	
	   if ($session{$h->{'sid'}}->{'artwork'}) {
	        if (! $ses_art_pur{ $session{$h->{'sid'}}->{'artwork'} }) {
		     $ses_art_pur{ $session{ $h->{'sid'} }->{'artwork'} } =0;
	        } 
		$ses_art_pur{$session{$h->{'sid'}}->{'artwork'}}++; 
	   }	
	} # done with order line items

        #$q="SELECT \* from ocart where cdate>=$time_i and cdate<$time_f and objclass = '140-A'";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "ocart";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[2] = build_where_hash("objclass","140-A", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
#        $sth = $dbh->prepare( $q );
#        $sth->execute;
#        $num=$sth->rows;
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);		
        $tot{'orders_shp'}=0;          # total shipping paid
        #foreach ( 1..$num ) {
         #  $h = $sth->fetchrow_hashref;
	foreach $h (@results) {
           # Get rid of employees' stuff
           next if $employee{$h->{'sid'}};

           # get rid of voided orders
           next if $voided{$h->{'orderid'}};

           $tot{'orders_shp'} += $h->{'linetotal'};
        }

        # start in tcart
        # number of people left stuff in cart and left site, search tcart
        my %intcart=();

        #Just count orders without shippings (NC)
        #$q = "SELECT sid from tcart where objclass!='140-A' and cdate>=$time_i and cdate<$time_f";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "tcart";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'sid'} = "sid";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("objclass","81001-A", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[2] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);		
#        $sth = $dbh->prepare( $q );
#        $sth->execute;
#        $num=$sth->rows;
#        foreach (1..$num) {
#           $h = $sth->fetchrow_hashref;
	foreach $h (@results) {
           # Get rid of employees' stuff
           next if $employee{$h->{'sid'}};

           $intcart{$h->{'sid'}} =1;
        }

        #start on orders
        #
        my %tot_art=();
        my %inorders=();   # hash holds order sid in orders
        my %orders=();

        #$q="SELECT \* from orders where cdate>=$time_i and cdate<$time_f";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "orders";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);		
#        $sth = $dbh->prepare( $q );
#        $sth->execute;
#        $num=$sth->rows;
        
        $tot{'orders_act'}=0;          # actual count of real orders
        $tot{'orders_tax'}=0;          # total sales tax collected
        $tot{'orders_mds'}=0;          # total merchandise sold
        $tot{'orders_tot'}=0;       # total charged for orders

#        foreach ( 1..$num ) {

#             $h = $sth->fetchrow_hashref;
	foreach $h (@results) {

             # Get rid of employees' stuff
             next if $employee{$h->{'sid'}};

             $inorders{$h->{'id'}}=1;     # mark the order sid in orders

             # get rid of voided orders
             next if $voided{$h->{'objid'}};  # voided prior to handling

             if ($h->{'status'} eq '15018-A' || $h->{'status'} eq '15020-A' ) {

                $tot{'orders_num'}++;  # total orders with checkout and not checkout
                $orders{$h->{'sid'}}=1;

                ## TBD - exclude uncompleted orders
 
                ## count revenue for the session
                #
                $session{$h->{'sid'}}->{'revenue'}=$h->{'ordtotal'};

                if ($session{$h->{'sid'}}->{'adbuy'}) {
                        $tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'ordtotal'};
                }
                if ($session{$h->{'sid'}}->{'artwork'}) {
                        $tot_art{$session{$h->{'sid'}}->{'artwork'}} += $h->{'ordtotal'};
                }
                $tot{'orders_tax'}+= $h->{'st_tax'};
                $tot{'orders_mds'}+= $h->{'st_taxable'} + $h->{'st_nontax'};
                #Order total include product total and tax total (NC)
                $tot{'orders_tot'}+= $h->{'st_taxable'}; 
            }
        } # done with orders     

        ## Start to count visitors left status -- with empty cart, with stuff in cart, just before completing order, and purchased.       
        foreach (keys %visitors){
                 if ($inocart{$_}) {
                     $NVP{'snap_checkout'}++;
                 } else {
                     $NVP{'snap_visleft'}++;
                 }

                 if (!$intcart{$_} && !$inocart{$_}){
                     $NVP{'snap_noorder'}++;
                 }

                 if ($intcart{$_} && !$inorders{$_}) {
                     $NVP{'snap_left'}++;
                 } elsif ($intcart{$_} && $inorders{$_}) {
                     $NVP{'snap_leftorder'}++;
                 }
        }

        ## start in on clicks

        my %tot_ref=();
        my %tot_req=();
        my %geom=();
        $tot{'pageviews_raw'}=0;
        #$q="SELECT \* from clicks where cdate>=$time_i and cdate<$time_f and request like '%.shtml'";  # Count pages at store side (NC)
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "clicks";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("cdate",">=$time_i", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("cdate","<$time_f", "and");
        $select_hash->{'TABLE'}->{'WHERE'}[2] = build_where_hash("request","%.shtml", "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);		
#        $sth = $dbh->prepare( $q );
#        $sth->execute;
#        $num=$sth->rows;
        $tot{'pageviews_sid'}=0;          # pageviews with session id known
        $tot{'homepage'}=0;
        $tot{'refer'}=0;
        $tot{'request'}=0;

#        foreach ( 1..$num ) {
#                $h = $sth->fetchrow_hashref;

	foreach $h (@results) {

                # Get rid of employees' stuff
                next if $employee{$h->{'sid'}};

                $tot{'pageviews_raw'}++;  # total pageviews burned through
                $tot{'homepage'}++ if ( $h->{'request'} =~ /html\/$site->{'om_render_lang'}\/home.shtml/); 
                my @tmp = split(/\?/, $h->{'request'});
                $tot_req{$tmp[0]} ++;
                $tot{'request'}++;

                if( $h->{'referer'} !~ /$site->{'siteurl'}/ && $h->{'referer'} !~ /$site->{'sitesslurl'}/ ) {
                        @tmp = split(/\?/, $h->{'referer'});
                        $tot_ref{$tmp[0]} ++ if $h->{'referer'};
                        $tot{'refer'}++;
                }
                next if ( ! $h->{'sid'} );

                #figure out the geometery
                if ($orders{$h->{'sid'}}) {  #this click has at least an order, retrieve product
                        $geom{$h->{'sid'}}->{$h->{'cdate'}}->{'referer'}= $h->{'referer'};
                        $geom{$h->{'sid'}}->{$h->{'cdate'}}->{'request'}= $h->{'request'};
                }

                $tot{'pageviews_sid'}++;
                $session{$h->{'sid'}}->{'clicks'}=0 if (!$session{$h->{'sid'}}->{'clicks'});

                $session{$h->{'sid'}}->{'clicks'}++;
        } # done with pageviews

        
        ##Get title for each variant from extraprop (NC)
#        my %vtitle=();

        #$q="SELECT objid, body from extraprop WHERE fieldname='encoding' and body like '%:%'";
#        $select_hash = {};
#        $select_hash->{'TABLE'}->{'NAME'} = "extraprop";
#        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
#        $select_hash->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
#        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("fieldname","encoding", "and");
#        $select_hash->{'TABLE'}->{'WHERE'}[1] = build_where_hash("body","%:%", "and");
#        $select_hash->{'statement'} = "SELECT";
#        $q = &build_select($select_hash, $site, $dbh, 1);
#	@results = undef @results;
#	$num = &query_getAll($q, $site, $dbh, \@results);		
#        $sth = $dbh->prepare($q);
#        $sth->execute;
#        $num=$sth->rows;
#        foreach (1..$num) {
#	foreach $h (@results) {
#               my ($objid, $body) = $sth->fetchrow_array;
#	    my $objid = $h->{'objid'};
#	    my $body = $h->{'body'};

#               my @titles = split (/\n/, $body);
#               foreach (@titles) {
#                   my ($k, $v) = split (/:/, $_);
#                   $vtitle{$objid}->{$k}=$v;
#               }
#        }

        #search for titles for all pages
	my %title=();
#        my %axisnames=();

        #$q="SELECT objid, msku, title, axisnames, vcode from things";
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "things";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
        #$select_hash->{'TABLE'}->{'FIELDS'}->{'msku'} = "msku";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'title'} = "title";
        #$select_hash->{'TABLE'}->{'FIELDS'}->{'axisnames'} = "axisnames";
        #$select_hash->{'TABLE'}->{'FIELDS'}->{'vcode'} = "vcode";
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
	@results = undef @results;
	$num = &query_getAll($q, $site, $dbh, \@results);		

#	$sth = $dbh->prepare( $q );
#	$sth->execute;
#        $num=$sth->rows;
#	foreach (1..$num) {
#		$h = $sth->fetchrow_hashref;
	foreach $h (@results) {
                
                 # Load titles for each page                 
#                 $title{$h->{'msku'}} = $h->{'title'} if ($h->{'title'});
#
#                 if ($h->{'title'}) {
                      $title{$h->{'objid'}} = $h->{'title'};
#                 }
#                 else {
#                      $title{$h->{'objid'}} = $title{$h->{'msku'}};}

#                 if ($h->{'axisnames'}){
#                    $axisnames{$h->{'msku'}}=$h->{'axisnames'};  
#                 }

#                 if ($h->{'vcode'}) {
#                         my @axis = split (/:/, $axisnames{$h->{'msku'}});
#                         my $n = (my @vcode = split (/:/, $h->{'vcode'}));
#                         $title{$h->{'objid'}} .= " --"; 
#                         for (my $i=0; $i<$n; $i++){
#                             $title{$h->{'objid'}} .= " $vtitle{$axis[$i]}->{$vcode[$i]}";
#                         }                 
#                 }
         }	

         ## Output something...
         #
         my (@fd, $filemonth);
         $fromtime = scalar localtime($time_i);
         $totime = scalar localtime($time_f);
         @fd = localtime($time_f);
         $fd[4] += 1;
         if ($fd[4] <10) { $filemonth = "0".$fd[4]; }
         else { $filemonth = $fd[4] ;}

         $fd[3] = "0".$fd[3] if $fd[3] <10;
         $filedate = (1900 + $fd[5]).$filemonth.$fd[3];

         ## check if the file for the day already exist
         my @dirfiles;

         opendir (DIR, $site->{'targetpath'});
         @dirfiles = readdir (DIR);
         close (DIR);

         #check if cron and include directory exist
         my ($cronexist, $includeexist);
         foreach my $d (@dirfiles) {
		if ($d eq 'cron') {
			$cronexist = 1;
		} elsif ($d eq 'include') {
			$includeexist = 1;
		}
	 }

       	 if (! $cronexist) { system("mkdir $site->{'targetpath'}/cron"); }
	 if (! $includeexist) { system("mkdir $site->{'targetpath'}/include"); }

        $snapflag = 1;
        if ($statuser eq "cron") { # cron time
                $snapflag = 0;
                $site->{'targetpath'} .= "/cron";
        } else {
                $site->{'targetpath'} .= "/include";
        }

	##snapshot
	$NVP{'fromtime'} = $fromtime;
	$NVP{'totime'} =  $totime;
	$NVP{'statsuser'} = $statuser;
        $NVP{'snap_page'} = $tot{'pageviews_raw'};
        $NVP{'snap_homepage'} = $tot{'homepage'};
        $NVP{'homepagep'} = 0;
        $NVP{'homepagep'} = cutnumber($tot{'homepage'} / $tot{'pageviews_raw'} *100) if $tot{'pageviews_raw'};
	$NVP{'snap_visitor'} = $totvisitors; 
        $NVP{'snap_log'} = $ses_vis{'log'}; 
        $NVP{'logp'} = 0;
        $NVP{'logp'} = cutnumber($ses_vis{'log'} / $totvisitors *100) if $totvisitors;
        $NVP{'snap_nolog'} = $ses_vis{'nolog'};
        $NVP{'nologp'} = 0;
        $NVP{'nologp'} = cutnumber($ses_vis{'nolog'} / $totvisitors *100) if $totvisitors;
        $NVP{'visleftp'}=0;
        $NVP{'visleftp'} =cutnumber ( $NVP{'snap_visleft'} / $totvisitors*100) if
$totvisitors;
        # (NC)

        $NVP{'noorderp'} = 0;
        $NVP{'noorderp'} = cutnumber($NVP{'snap_noorder'} / $totvisitors *100) if $totvisitors;

        $NVP{'leftp'} = 0;
        $NVP{'leftp'} = cutnumber($NVP{'snap_left'} / $totvisitors *100) if $totvisitors;

        $NVP{'leftorderp'} = 0;
        $NVP{'leftorderp'} = cutnumber($NVP{'snap_leftorder'} / $totvisitors *100) if $totvisitors;

        $NVP{'checkoutp'} = 0;
        $NVP{'checkoutp'} = cutnumber($NVP{'snap_checkout'} / $totvisitors *100) if $totvisitors;

        $NVP{'newjoinp'} = 0;
        $NVP{'newjoinp'} = cutnumber($NVP{'snap_newjoin'} / $totvisitors *100) if $totvisitors;

        my $con_rate;
	if ($totvisitors) {
                $con_rate = cutnumber($tot{'pageviews_raw'}/$totvisitors);
	} else {
		$con_rate = 'N/A';
	}
	$NVP{'snap_per_visitor'} = $con_rate;

        if ($totvisitors) {
                $con_rate = $tot{'orders_num'}/$totvisitors;
                $con_rate = substr($con_rate, 0, 6);
        } else {
                $con_rate = 'N/A';
        }	
        $NVP{'snap_convert'} = $con_rate;

        ##report
        my ($targ, $t, $i, $pageurl, $TOP25, $viewpercent, @page);
        my (@objid, $pagetitle, $pagerefurl);
        my %REP=();
        $TOP25=25;

        $targ = "row_stats_report";
        $t = loadtemplate($targ, $site->{'skeletonpath'});
        $t =~ s/<!--.*-->\n//s;

	$i = 0;
	foreach $pageurl (sort {$tot_req{$b} <=> $tot_req{$a};} keys %tot_req) {
		$i++;
                last if $i>$TOP25;
                $viewpercent = cutnumber($tot_req{$pageurl}/$tot{'request'} *100);
		@page = split(/\//, $pageurl);
                @objid = split(/\./, $page[$#page]);

                    if ($objid[0] eq 'home') {
                           $pagetitle = 'Homepage';
                    }
                    else{
                         $pagetitle = ucfirst($title{$objid[0]});
                    }
	   
            
            $pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

	    $REP{'NUM'} = $i;
            $REP{'TITLE'} = "$pagetitle<br>";
	    $REP{'URL'} = $pagerefurl;
	    $REP{'NUMBER'} = $tot_req{$pageurl};
	    $REP{'PERCENT'} = $viewpercent;

	    $NVP{'top25page'} .= permutebit($t, \%REP);
	}
	
	$i = 0;
	foreach $pageurl (sort {$tot_req{$a} <=> $tot_req{$b};} keys %tot_req) {
		$i++;
		last if $i>$TOP25;
                $viewpercent = cutnumber($tot_req{$pageurl}/$tot{'request'} *100);
		@page = split(/\//, $pageurl);
                @objid = split(/\./, $page[$#page]);

                if ($objid[0] eq 'home') {
                       $pagetitle = 'Homepage';
                }
                else{
                       $pagetitle = ucfirst($title{$objid[0]});
                }

		$pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

		$REP{'NUM'} = $i;
                $REP{'TITLE'} = "$pagetitle<br>";
                $REP{'URL'} = $pagerefurl;
                $REP{'NUMBER'} = $tot_req{$pageurl};
                $REP{'PERCENT'} = $viewpercent;   

                $NVP{'least25page'} .= permutebit($t, \%REP); 
	}

        my ($tt, %BR);
        %BR=();
	$i = 0;
        $targ="row_stats_topbrowser";
        $tt = loadtemplate($targ, $site->{'skeletonpath'});
        $tt =~ s/<!--.*-->\n//s;

	foreach $pageurl (sort {$ses_bro{$b} <=> $ses_bro{$a};} keys %ses_bro) {
		$i++;
		last if $i>$TOP25;

                $viewpercent = cutnumber($ses_bro{$pageurl}/$tot_bro *100) if ($tot_bro); #(NC)
                $BR{'NUM'} = $i;
                $BR{'BROW'} = $pageurl;
                $BR{'NUMBER'} = $ses_bro{$pageurl};
                $BR{'PERCENT'} = $viewpercent;

                $NVP{'top25browser'} .= permutebit($tt, \%BR);
	
        }

        $i = 0;
        foreach $pageurl (sort {$tot_ref{$b} <=> $tot_ref{$a};} keys %tot_ref) {
		$i++;
		last if $i>$TOP25;
                $viewpercent = cutnumber($tot_ref{$pageurl}/$tot{'refer'} *100);

                $REP{'NUM'} = $i;
                $REP{'TITLE'} = "";
                $REP{'URL'} = $pageurl; 
                $REP{'NUMBER'} = $tot_ref{$pageurl};
                $REP{'PERCENT'} = $viewpercent; 

		$NVP{'top25refer'} .= permutebit($t, \%REP);
	}

        #order information
        my %ITEM=();
        $NVP{'orders_num'}=0;       
 
	$NVP{'order_num'} = $tot{'orders_num'};
	my ($tmp_order_total) = cutnumber($tot{'orders_tot'}); 
	$NVP{'order_total'} = $site->{'currency_symbol'} . "$tmp_order_total";
        $NVP{'order_ship'} = cutnumber( $tot{'orders_shp'});
        $NVP{'order_tax'} = cutnumber( $tot{'orders_tax'});

        $targ="row_stats_topitem";
        $t = loadtemplate($targ, $site->{'skeletonpath'});
        $t =~ s/<!--.*-->\n//s;
	$i = 0;
        $snap .= "&"; 

        foreach my $lineobjid (sort {$tot_lineobjid{$b} <=> $tot_lineobjid{$a};} keys %tot_lineobjid) {
           $i++;
           last if $i>$TOP25;
           my $lineobjid_sale = $lineobjid_price{$lineobjid}*$tot_lineobjid{$lineobjid};
           my $lineobjidurl = "http://$site->{'siteserver'}$site->{'sitedomain'}/html/$site->{'om_locale_lang_def'}/$lineobjid\.shtml";
           $ITEM{'NUMBER'} = $i;
           $ITEM{'LINEOBJIDURL'} = $lineobjidurl;
           $ITEM{'LINEOBJID'} = $lineobjid;
           $ITEM{'TOT_LINEOBJID'} = $tot_lineobjid{$lineobjid};
           $ITEM{'PRICE'} = $lineobjid_price{$lineobjid};
           $ITEM{'SALE'} = cutnumber($lineobjid_sale);
	   $ITEM{'currency_symbol'} = $site->{'currency_symbol'};

           $NVP{'topitems'} .= permutebit ($t, \%ITEM);
        }

        #Advertising Summary

	$targ="row_stats_ad";
	$t = loadtemplate($targ, $site->{'skeletonpath'}); 
        $t =~ s/<!--.*-->\n//s;
       
        #adbuy
        my %AD=();
	foreach my $adbuy (sort {$ses_adb{$b} <=>  $ses_adb{$a};} keys %ses_adb) {
		$con_rate = $ses_adb_pur{$adbuy} / $ses_adb{$adbuy} if $ses_adb{$adbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_adb{$adbuy} / $ses_adb_pur{$adbuy} if ($ses_adb_pur{$adbuy});
		$AD{'AD'} = $adbuy ;
		$AD{'VISITOR'} = $ses_adb{$adbuy};
		$AD{'ORDER'} = $ses_adb_pur{$adbuy};
		$AD{'TOTAL'} = $tot_adb{$adbuy};
		$AD{'AVESALE'} = $avesale;
		$AD{'CONVERSION'} = $con_rate;
		
		$NVP{'adbuy_row'} .= permutebit($t, \%AD);
	}

        #artwork
	foreach my $artbuy (sort {$ses_art{$b} <=>  $ses_art{$a};} keys %ses_art) {
		$con_rate = $ses_art_pur{$artbuy} / $ses_art{$artbuy} if $ses_art{$artbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_art{$artbuy} / $ses_art_pur{$artbuy} if ($ses_art_pur{$artbuy});

		$AD{'AD'} = $artbuy;
                $AD{'VISITOR'} = $ses_art{$artbuy};
                $AD{'ORDER'} = $ses_art_pur{$artbuy};
                $AD{'TOTAL'} = $tot_art{$artbuy}; 
                $AD{'AVESALE'} = $avesale;
                $AD{'CONVERSION'} = $con_rate;
 
                $NVP{'artwork_row'} .= permutebit($t, \%AD);
	}
     }
     
     #write to the stats file
     my $targ = "os_allcommerce-runstats";
     my $l = loadtemplate($targ, $site->{'skeletonpath'});
     $NVP{'currency_symbol'} = $site->{'currency_symbol'};
     $l = permutebit($l, \%NVP);
     open(STAT, ">$site->{'targetpath'}/$filedate.html");
     print STAT $l;
     close(STAT);

     $snap = "$fromtime"."&"."$totime"."&"."$statuser";
     $snap .= "\&$NVP{'snap_page'}";
     $snap .= "\&$NVP{'snap_visitor'}";
     $snap .= "\&$NVP{'order_num'}";
     $snap .= "\&$NVP{'order_total'}";
     $snap .= "\&$NVP{'topitems'}";
     $snap .= "&"."$filedate";

     print STDOUT $snap if ($snapflag);

     $sth->finish if ( $sth );
     $dbh->disconnect if ( $dbh );

}


######################### INITIALIZATION SUBROUTINES #########################

##############################################################################
=pod

=head2 initscript

initscript - (%FORM, $site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and set up the script's environment and variables

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my (%FORM, $k, $dbh, $site);
    

    # Find out where we are, so we can load libs.
    ##
    use Cwd;
    my $separator = '/'; # What separates dirnames in this architecture?
    my ($scriptname, $args, $rundir);
    ($scriptname, $args) = split /\?/, $0;
    if ($scriptname =~ /$separator/) {
	    $scriptname =~ /^(.*)$separator(.*)$/;
	    $rundir = $1;
	    $scriptname = $2;
    } else {
	    $rundir = cwd();
    }
    chdir( $rundir );
    my @path = split(/$separator/, $rundir);
    my $relpath='./';
    while (my $d = pop @path) {
	    $relpath .= "..$separator";
	    if ($d eq 'crons') {
		    $relpath .= "..$separator";
		    last;
	    }
    }
    my $siteshortname = pop @path;
    $relpath .= $siteshortname;
    ###########

    ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);
    foreach $k (keys %FORM) {
        &debug("$k is $FORM{$k}",3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    return ($site, $dbh, %FORM);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh, %FORM) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables 
that are required throughout the script. 

=item INPUT

$scriptname - The name of the script 

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

%FORM  - A ReadParse version of the FORM to start with

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the 
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 
Be sure to start it with >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'} 
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $FORM, $site, $dbh);
    
    use DBI;
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/conf/site.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/db_layer.pm";
    require "$relpath/lib/permission_layer.pm";
    
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $interface = "";
    
    if ($$FORM{'sid'}) {
        #$q="SELECT language,interface FROM sessions WHERE sid=$$FORM{'sid'} ";
        my $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "sessions";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'interface'} = "interface";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'language'} = "language";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = build_where_hash("sid",$$FORM{'sid'}, "and");
        $select_hash->{'statement'} = "SELECT";
        $q = &build_select($select_hash, $site, $dbh, 1);
        $sts = $dbh->prepare( $q );
        $sts->execute;
        $h=$sts->fetchrow_hashref;
        $site->{'om_render_lang'}=$h->{'language'};
        $session_lang = $h->{'language'};
        $interface=$h->{'interface'};
    }
    
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $session_lang="eng" if ( ! $session_lang );
    $interface="html" if ( ! $interface );
    
    ## load messages
    $script=( split /\./, $scriptname )[0];
    
    ## done with pull-from-file bits, rest of global settings...
    #
    $x= (split /\?/, $ENV{'REQUEST_URI'})[0];   # strip get stuff
    @temp=split /\//, $x;
    @temp=("cgi-bin","test","inventory.cgi") if (! $ENV{'REQUEST_URI'} );
    
    $script=( split /\./, $scriptname )[0];
    $n= $#temp - 1;

    # Configure all parameters to pass around in $site hash
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/run-stats";
    $site->{'sid'} = $site->{'sid'};
    $site->{'session_lang'} = $session_lang;
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'targetpath'}="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/data/stats";    
    # Flag, set to 0 to have default for Generate page? as no, and
    # set to 1 to have default as yes
    $site->{'generate_default'} = 0;

    #&debug("Debuggin inventory.cgi...",3, $site->{'debug_level'}, ">/tmp/stats");
    return ($site, $dbh, %{$FORM});
}

###############################################################################

=pod

=head2 cutnumber

cutnumber - $_ = cutnumber($_);

=over 4

=item DESCRIPTION

Round the number to the hundreth (1/100)

=item INPUT

$_ - any number need to be rounded

=item OUTPUT

Returns a rounded 2 digits decimal

=item NOTES

None

=back

=cut

###########################################################################
sub cutnumber {
   my $temp =  shift @_;
   $temp += 0.005 ;
   my ($t1, $t2) = split /\./, $temp;
   $t2 .= "00";
   $t1 .= "\.".substr($t2, 0, 2);
   return $t1;
}

###########################################################################

1;
