#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

##############################################################################
=pod

=head1 xml_order_export.pl

xml_order_export.pl - export order

=head1 SYNOPSIS

perl xml_order_export.pl ORDER-ID1 .. ORDER-IDN

=head1 DESCRIPTION

Program to create xml order files for transfer to warehouse
arguments: storecode, orderid

=head1 NOTES & BUGS

=cut
###############################################################################
#
# Version 1.0 TG 20000919
#    Wrote for version 1.1

&main(@ARGV);


##############################################################################
=pod

=head2 main

main - &main();

=over 4

=item DESCRIPTION

For each order that is passed in, export the information and create the .xml
file.

=item INPUT

@order_ids = Array of order IDs

=item OUTPUT

xml data files which are in the format $objid.xml and are stored in $site->{'targetpath'}
directory.

=item NOTES

May have issues with multiple languages.

=back

=cut
###############################################################################
sub main {
    my (@orders) = @_;
    
    my ($site, $dbh, %template, $order, $order_info);
    
    ($site, $dbh) = &initscript();
    foreach $order (@orders) {
	$order_info = &getXMLData($order, $dbh, $site);
	&printXMLdata($order_info, $site);
    }

    $dbh->disconnect() if ($dbh);
    
}
    

######################### INITIALIZATION SUBROUTINES #######################

##############################################################################
=pod

=head2 initscript

initscript - ($site, $dbh) = &initscript();

=over 4

=item DESCRIPTION

Determine the relative path, and connect to the database

=item INPUT

none

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item NOTES

=back

=cut
###############################################################################
sub initscript {
    my ($s, @temp, @rpath, $rundir, $siteshortname, $u, $scriptname, $relpath);
    my ($k, $dbh, $site);
    
    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in cgi-bin/om/admin,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #
    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
        } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute
    @temp=split /\//, $s;
    
    @rpath=@temp;
    
    $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff
    
    $rundir= join "/", @rpath;
    chdir( $rundir );
    
    $relpath="";
    $siteshortname="";

    ## short circuit if $s does not contain crons
    if ( $s !~ /crons/ ) {
	$siteshortname = pop @rpath;
	$relpath=".";
	@temp=();
    } # done testing for degenerate case
    
    while ( @temp ) {
        $u = pop @temp;
        
        ## skip references to the present directory
        next if ( $u eq '.' );
        next if ( length($u)==0 );        # skip null directory levels
        
        $relpath.="/.." if ( $relpath);
        $relpath.="." if ( ! $relpath );
        
	if ( $u eq "crons" ) {
	    $siteshortname = $temp[$#temp];
	    @temp=();
	} # note - crons is off the site root, so the next up is site root        
    } # done constructing relative path to store root
    
    $relpath .= "/../$siteshortname";
    ($site, $dbh) = &initsite($scriptname, $relpath);
    $site->{'siteshortname'} = $siteshortname;
    
    return ($site, $dbh);
}

##############################################################################
=pod

=head2 initsite

initsite - ($site, $dbh) = &initsite($scriptname, $relpath);

=over 4

=item DESCRIPTION

The init site loads in all required library modules and creates the variables 
that are required throughout the script. 

=item INPUT

$scriptname - The name of the script (inventory.cgi)

$relpath - The relative path to files that must be required

=item OUTPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item NOTES

To add parameters that need to be passed around to a lot of locations, add them
to the $site hashref.

To change the debug of the script, change $site->{'debug_level'} to one of the 
following values:

    0 - no debug

    1 - errors only

    2 - all SQL statements, warnings & errors

    3 - all of the above

To change the location of the debug log, change the value of $site->{'debug_log'}. 
Be sure to start it will >> to allow appending to the file.  To set the Generate
Page? button to be selected by default, change the $site->{'generate_default'} to
1.  To have it non-selected by default, change the $site->{'generate_default'} 
to 0.

=back

=cut
###############################################################################
sub initsite {
    my ($scriptname) = shift;
    my ($relpath) = shift;
    
    my ($find, $q, $interface, $sts, $h, $x, @temp, $n, $k);
    my ($session_lang, $script, $site, $dbh);
    my ($select_hash, $q2);

    use DBI;
    require "$relpath/conf/site.pm";
    require "$relpath/lib/xml.pm";
    require "$relpath/lib/conversions.pm";
    require "$relpath/lib/filetools.pm";
    require "$relpath/lib/formtools.pm";
    require "$relpath/lib/variant.pm";
    require "$relpath/lib/localization.pm";
    require "$relpath/lib/eapi.pm";
    
    # must be the last require statement
    require "$relpath/lib/custom-lib.pm";
    
    ## establish site-specific parameters
    #
    $site=setsite();
    
    ## connect to database
    # system 6 database state, products, and customer tables are collapsed
    #
    $q = "$site->{'db_site_config'}";
    $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
    
    $site->{'om_render_lang'} = "";
    $site->{'om_render_lang'}=$site->{'om_locale_lang_def'} if ( ! $site->{'om_render_lang'} );
    $site->{'om_render_lang'}="eng" if ( ! $site->{'om_render_lang'} );
    $interface="html" if ( ! $interface );
    
    ## load messages
    $script=( split /\./, $scriptname )[0];

    # Configure all parameters to pass around in $site hash
    $site->{'selskeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/html/sel";
    $site->{'skeletonpath'} = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/xml";
    $site->{'scriptpath'} = join "/", ( @temp[0..$n] );
    $site->{'script'} = ( split /\./, $scriptname )[0];
    $site->{'scriptname'} = $scriptname;
    $site->{'relpath'} = $relpath;
    $site->{'debug_level'} = 3;
    $site->{'debug_log'} = ">>/tmp/xml_order_export";
    $site->{'sid'} = $site->{'sid'};
    $site->{'session_lang'} = $session_lang;
    $site->{'scriptinfo'} = "$site->{'scriptpath'}/$site->{'scriptname'}";
    $site->{'script_msg'} = &message($site);
    $site->{'targetpath'}="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/transit/export/order";    
    # Flag, set to 0 to have default for Generate page? as no, and
    # set to 1 to have default as yes
    $site->{'generate_default'} = 0;

    &debug("Debuggin inventory.cgi...",3, $site->{'debug_level'}, ">/tmp/inventory");
    return ($site, $dbh);
}

##############################################################################
=pod

=head2 message

message - $site->{'script_msg'} = &message($site);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This routine loads in the
messages for the $site->{'om_render_lang'} that is currently active.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

A reference to the %script_msg hash of messages that need to be used in the
script

=item NOTES

=back

=cut
###############################################################################
sub message {
    my ($site) = shift;
    
    my (@temp);
    my (%script_msg);
    
    # check the language that is being used and open appropriate
    # message language file or get messages below
    if ($site->{'om_render_lang'} eq "eng" ) {
	$script_msg{1}="('second','minute','hour','day_om','month','year')";
    } else {
        open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'script'}.messages");
        while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
        }
        close(EM);
    }
    return \%script_msg;
}

1;
