#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#####
# inbound.cgi
#    session id setting and redirecting script
#    RF 981208 - based on sid.cgi, version 1-981208
#
#####

####
#
# Version 1.2.1 Yani Copas 20010109
#    fixed path split in redirect_no_cookies 
# Version 1 LCF 20000912
#	 Conform to coding standards, abstract & simplify.
# Version 0.7.41 HVC 20000829
#    Further changes to support WML
# Version 0.7.40 JC 20000712
#    wml support 
# Version 0.7.39 JC 20000707
#    put coma in update statement for sessions
# Version 0.7.38 IB 20000605
#    corrected typos
# Version 0.7.37 IB 20000531
#    added custom-lib.pm
# Version 0.7.35 GF 20000519
#    debugging for datastructure 0.9.18
#    script cleanup and lowercase fieldnames
# Version 0.7.34 JC 20000428
#    changed logging of om_render_lang from cookie to sessions table
# Version 0.7.33 IB 20000421
#    removed loading of messages
# Version 0.7.32 HVC 20000420
#    call wrapper.cgi for no cookie request
# Version 0.7.31 IB 20000411
#    default to language in site.pm
# Version 0.7.30 IB 20000410
#    corrected =~ to eq
# Version 0.7.29 IB 20000409
#    declared the array %script_msg().
# Version 0.7.28 IB 20000408
#    multilingual messages
# Version 0.7.27 RF 20000406
#    setup of user interface language and setting cookie OM_UI_LANGUAGE
# Version 0.7.26 IB 20000406
#    corrections
# Version 0.7.25 JC 20000405
#    localization message support
# Version 0.7.24 JC 20000405
#    check browser lang settings
# Version 0.7.23 JC 20000405
#    localization message support
# Version 0.7.22 JC 20000404
#    check to see if default language is valid
# Version 0.7.21 JC 20000403
#    localization support
# Version 0.7.20 JC 20000330
#    changed connect statement
# Version 0.7.19 JC 20000329
#    changed DBI prepare/connect
# Version 0.7.18 IB 20000327
#    corrected $relpath
# Version 0.7.17 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.16 IB 20000308
#    changed REFERER to HTTP_REFERER
#    initialized uninitialized variables to avoid error messages
# Version 0.7.15 IB 20000302
#    updated copyright
# Version 0.7.14 IB 20000226
#    changed store to site
# Version 0.7.13 GF 991203
#    commented out getpage redirect
#    redirected to cookie warning
# Version 0.7.12 IB 991116
#    corrected the number of arguments in a database call
# Version 0.7.11 IB 991113
#    removed cgi-lib.pm because it is not needed
# Version 0.7.10 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.9 IB 991111
#    fixed list of libraries
# Version 0.7.8 IB 991110
#    correction of a database call
# Version 0.7.7 IB 991109
#    removing of HACKS, and correcting of relpath
# Version 0.7.6 IB 991025
#    the UID field has been dropped from table clicks
# Version 0.7.5 RF 991022
#    fixed path failure to static pages
# Version 0.7.4 RF 991022
#    fixed redirection failure to getpage.cgi
# Version 0.7.3 RF 991022
#    added patch to older store.pm - assume database name is
#    also the name of the directory below ./cgi-bin in the 
#    tree, and provide that as an argument to setstore()
#    Also derefed the $store hashref to satisfy older libraries
# Version 0.7.2 RF 991022
#    near total rewrite
#    added relative path detection for requires statements
#    trimmed diagnostic printing
#    added logic to spot non-cookie taking customers
# Version 0.7.1 GF 990830
#    trying to revitalize this script
# Version 4 - 981218 
#    efficiency improvements in prep for eWallet hit
# Version 2 - 981213
#    added logging of inbounds...
# Version 1 - 981208
#
#####
use strict;

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "cwd"
#	note: The cwd section will fail if the site is symlinked.
#
use Cwd;
my $separator = '/'; # What separates dirnames in this architecture?
my ($scriptname, $args, $rundir);
($scriptname, $args) = split /\?/, $0;
if ($scriptname =~ /$separator/) {
	$scriptname =~ /^(.*)$separator(.*?)$/;
	$rundir = $1;
	$scriptname = $2;
} else {
	$rundir = cwd();
}
my @path = split(/$separator/, $rundir);
my ($relpath, $siteshortname);

## short circuit if $s does not contain cgi-bin
# This happens in scripts linked from the top-level
if ( $rundir !~ /cgi-bin/ ) {
   $siteshortname = pop @path;
   $relpath=".";
   @path=();
} 

while (my $d = pop @path) {
	$relpath .= "..$separator";
	if ($d eq 'cgi-bin') {
		$relpath .= "..$separator";
		last;
	}
}
$siteshortname = pop @path;
$relpath .= $siteshortname;

use DBI;
require "$relpath/conf/site.pm";
require "$relpath/lib/state.pm";
require "$relpath/lib/blowfisher.pm";
require "$relpath/lib/localization.pm";
require "$relpath/lib/custom-lib.pm";

my $sid=$ENV{'HTTP_COOKIE'};
$sid=( split /sid=/, $sid )[1] if ( $sid );
$sid=( split /&/, $sid )[0] if ( $sid );

## WML support
my $http_accept=$ENV{"HTTP_ACCEPT"};
my $http_user_agent=$ENV{"HTTP_USER_AGENT"};

my $interface;
# Determine the device from where request is coming from
if (($http_accept =~ "text\/vnd\.wap\.wml") ||
    ($http_user_agent =~ "wap") ||
    ($http_user_agent =~ "7110") )
{
                $interface="wml";
} 

## establish site-specific parameters
#
my $site = &setsite();

my $now=time();

## set up localization
#
my ($om_render_lang) = &get_render_lang($site);

# Make sure we have an interface
$interface="html" if ( ! $interface);

## open database connection
#
my $dbh = &get_dbh($site);

# Flag to know if we've already output the redirect or not
my $done = 0;

## Handle request across WAP
if ($interface =~ "wml") {
	$sid = &wml_redirect($sid, $om_render_lang, $interface);
	$done = 1;
}

## now either have a sid or not...
#
if ( $sid && !$done) {                # session id cookie present
                                      # have a cookie, just send them 
                                      # to their requested page with 
                                      # simple logging
 	# Do the actual redirect
	&have_sid_redirect($interface, $om_render_lang);
	$done = 1;

} elsif (!$done) {						# session id cookie absent
										# do not have a cookie, redirect 
										# user request to wrapper.cgi

	# initialize REQUEST_URI so tests won't error
	$ENV{'REQUEST_URI'}="" unless ( $ENV{'REQUEST_URI'} ); 

	# Look for an sid embedded in the URI
	$sid = &find_embedded_sid();

	if ( $ENV{'REQUEST_URI'} =~ /cookietest=on/ )  {   # we just tried to give 
													   # them a cookie, and failed

		&redirect_no_cookies($interface, $om_render_lang, $sid);
		$done = 1;

	} else { 									# give them a cookie and 
												# redirect them back here
		
		$sid = &redirect_with_cookie($sid, $interface, $site, $dbh);
	}
} #/done with cookie acceptance testing for cookieless sessions 

## cleanup statement handles, log event in 
## sessions & clicks, and close connection
#

# Log to sessions
&log_to_session($sid, $dbh, $om_render_lang, $interface, $site);
# Log to clicks.
&log_to_clicks($sid, $dbh, $site);

# we're done.
$dbh->disconnect if ( $dbh );
exit 1;

#######################################
##### End of script, start of functions
#######################################

sub log_to_session {
	my ($sid, $dbh, $om_render_lang, $interface, $site) = @_;
        $sid = &decrypt_sid($sid,$site);
	# Log entries in sessions
	my $q="UPDATE sessions SET language='$om_render_lang',interface='$interface' WHERE sid='$sid'";
	$dbh->do( $q ); 
	return 1;
}


sub log_to_clicks {
	# Log entries in clicks
	my ($sid, $dbh, $site) = @_;
	my $now = time();
        	
	my $http_ref = '';

        $sid = &decrypt_sid($sid,$site);
	if ($ENV{'HTTP_REFERER'}) {
	  my $http_ref = $ENV{'HTTP_REFERER'}; 
	} 
	
	my $q = "INSERT INTO clicks VALUES ('$sid','$now','$ENV{'REQUEST_URI'}','$http_ref','$site->{'clusterid'}')";
	$dbh->do( $q );
	return 1;
}



sub redirect_with_cookie {
	my ($sid, $interface, $site, $dbh) = @_;
	
	my $target;
	$target = $ENV{'REQUEST_URI'} if ($ENV{'REQUEST_URI'} && $ENV{'REQUEST_URI'} ne '/');
	
	if ( ! $sid ) { 							   # get a session id

	   $sid = &setsessionid($site, $dbh); 					

	} else {									   # set a cookie with sid 
												   # salvaged from URI

	  print "Set-Cookie: sid=$sid; path=/; domain=$ENV{'SERVER_NAME'}\n" if ( $interface eq "html");
	  print "Set-Cookie: sid=$sid Comment=setting sid \n" if ( $interface eq "wml");
	} #/done with selection of how to set the cookie 

	print "Status: 302/Moved\n";
	print "Location: /cgi-bin/om/inbound.cgi?sid=$sid&cookietest=on&target=$target\n";
	print "\n\n";

	return $sid;
}

	
sub redirect_no_cookies {
	my ($interface, $om_render_lang, $sid) = @_;

	my $target = ( split /page=/,$ENV{'REQUEST_URI'},2 )[1];
	$target="/$interface/$om_render_lang/home.shtml" unless ( $target ); 

	print "Status: 302/Moved\n";
   $sid = &encrypt_sid($sid,$site);
	print "Location: /cgi-bin/om/wrapper.cgi?sid=$sid&target=$target\n";
	print "\n\n";

	return;
}


sub find_embedded_sid {
	my $sid;
	if ( $ENV{'REQUEST_URI'}=~/sid=/ ) {			   # sid embeded in URI
	   $sid=$ENV{'REQUEST_URI'};
	   $sid=( split /sid=/, $sid )[1];
	   $sid=( split /&/, $sid )[0] if ( $sid );
	} # done looking for embeded sid

	return $sid;
}

	
sub have_sid_redirect {
   my ($interface, $om_render_lang) = @_;
   
   my $target = ( split /page=/,$ENV{'REQUEST_URI'},1 )[1];
   $target = "" if ( ! $target );     # default to homepage
   $target =~ s/&cookietest=on// ;

   $target="/$interface/$om_render_lang/home.shtml" if ( ! $target ); 
   print "Status: 302/Moved\n";
   print "Location: $target\n";    
   print "\n\n";
   return;
}


# Determine the language to use...
sub get_render_lang {
	my $site = shift;
	
	my $lang=$ENV{'HTTP_ACCEPT_LANGUAGE'};
	my @langs=split(/,/, $lang);

	my ($om_render_lang);
	foreach my $k (@langs) {
	   $k =~ s/\s+//g;                     # kill spaces

		my $langcomp;
	   if ( $k =~ /-/ ) {
    	  $langcomp=(split(/-/, $k))[0];   # throw away bits after dash
	   } else {
    	  $langcomp=$k;
	   }

	   # now have a language code, get the 3-digit code used by OM
	   # and see if this is on the list of active languages
	   my $lang_three=(split /\t/, &om_iso_639($langcomp))[0];
	   next unless ( ($lang_three) && 
	   				 ($lang_three =~ /$site->{'om_locale_lang_act'}/));

	   $om_render_lang=$lang_three;
	   last;	# If we've set $om_render_lang, we're done.	   
	} # done running through list of browser preferred languages

	# if null render language, select first of the defined active languages
	$om_render_lang=( split /\W+/, $site->{'om_locale_lang_def'} )[0]
	   unless ( $om_render_lang );

	# Still nothing? Well, try for om_locale_lang_def site setting
	$om_render_lang=$site->{'om_locale_lang_def'} unless ( $om_render_lang );

	# if *still* null, hardcode to english just to get something set
	$om_render_lang="eng" unless ( $om_render_lang );

	return $om_render_lang;
}


sub get_dbh {
	my $site = shift;
	
	my $dbh=DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
	return $dbh;
}


sub wml_redirect {
	my ($sid, $om_render_lang, $interface) = shift;

	my $now = time;
	# Define content type
	print "Content-type: text/vnd.wap.wml \n";

	$sid = setsessionid() unless ($sid);
	
	# Is there a target ?
	my $target="http:\/\/$ENV{'HTTP_HOST'}\/cgi-bin\/om\/wrapper.cgi?sid=$sid&target=\/$interface\/$om_render_lang\/home.wml";

	# Redirect the request
	print "Location: $target \n\n";
	print "Content-type: text/vnd.wap.wml \n";

	return $sid;
}

1;
