#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

use strict;

##############################################################################
=pod

=head1 admin_object.pm

admin_object.pm - Subroutines for processing done within admin_object.cgi

=head1 DESCRIPTION

This library is used for all subroutines that are used exclusively by admin_object.cgi.
This module allows removal of data processing from the main cgi script, allowing
it to only be responsible for displaying the information gathered in this module.

=head1 NOTES & BUGS

The calling script must require db_layer.pm

=cut

##############################################################################

###
#
# admin_object.pm
#



################## DATABASE CONFIGURATION INFORMATION ########################

##############################################################################
=pod

=head2 create_object

create_object - &create_object(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

To take all of the input from the $FORM and insert the data into the database.  
The data will first go into the primary table (from get_primary_table_name, things 
table (or contact table for user records).  Then, whatever is left over will be
inserted into the extraprop table (secondary tables).  This can easily 
be extended to check pricing and inventory prior to inserting into things!  If 
the insert into things fails, then we insert the record in multiple pieces to the 
extraprop table, as it is a different language.

=item INPUT

$input_hashref - Pointer to the hash to use to find all of the input from the user.
                 Normally, this is a reference to the FORM hash.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message (by mapping into $site->{'script_msg'}) if an error
         occured, undef if no error occurred.

If no errors, a new object is created, and $$input_hashref{'objid'} is set with the newly
created object id.

=item NOTES

=back

=cut
###############################################################################
sub create_object {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, $main_hashref, $extra_hashref, $extra_q, $q, @extra_inserts, $main_fields);
    my ($field, $key, $objid, $master_info, $error_flags, $flag, $vcode, $extras);

    &debug("create_object: In here with $$input_hashref{'objtype_inp'}, $$input_hashref{'objclass_inp'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $$input_hashref{'new_object'} = "yes";

    if ($$input_hashref{'axisnames_inp'} ne "") {
	$$input_hashref{'has_axisnames'} = "yes";
    }

    $error_flags = &validate_attributes($input_hashref, $extras, $site, $dbh);

    if ((defined $error_flags->{'new_row_has_input'}) && (!$error_flags->{'validationcheck'})) {
	&debug("create_object:  Error occured in validate_attributes", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{7};
    }

    if ((!$error_flags->{'validationcheck'}) && (!defined $$input_hashref{'is_a_new_row'})) {
	&debug("create_object:  Error occured in validate_attributes", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{7};
    }

    if ($error_flags->{'changestowrite'} eq "no") {
	&debug("create_object:  Nothing to write, just return", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    &debug_hashref($error_flags, 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($error = &validate_object($input_hashref, $site, $dbh)) {
	&debug("create_object:  $error from validate_object - $site->{'script_msg'}{$error}", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }

    $vcode = "";
    $main_hashref->{'TABLE'}->{'NAME'} = &get_primary_table_name($$input_hashref{'objtype_inp'}, $$input_hashref{'objclass_inp'}, $site);
    $main_hashref->{'statement'} = "insert";

    $main_fields = &get_field_names($main_hashref->{'TABLE'}->{'NAME'}, $site, $dbh);

    foreach $key (keys %{$input_hashref}) {
	next if ($key !~ /_inp/);
	if (($key =~ /axis/) && ($key ne "axisnames_inp")) {
	    $vcode = "yes";
	    &debug("create_object: Need to build the vcode", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    next;  # don't insert this info into extraprop
	}
	next if ($$input_hashref{$key} eq "");

	$field = $key;
	$field =~ s/_inp//gms;

	# If it is the generate_inp field
#	if (($field eq "generate") && ($$input_hashref{'generate_inp'} == 1)) {
	if (($field eq "gen") && ($$input_hashref{'gen_inp'} == 1)) {
	    &debug("create_object: This is the que to regenerate the page $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $flag->{'objid'} = $objid;
	    $flag->{'generate_page'} = 1;
	    next;
	}


	if ($main_fields =~ /$field/) {
	    if ($main_hashref->{'TABLE'}->{'NAME'} eq "things") {

		# take care of the master products
		if (($field eq "axisnames") && ($$input_hashref{$key} ne "")) {
		    $main_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = "__objid__";
		}

	    }
	    $main_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $$input_hashref{$key};
	}
	else {
	    $extra_hashref = {};
	    $extra_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	    $extra_hashref->{'statement'} = "insert";
	    $extra_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = $field;	    
	    $extra_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $$input_hashref{$key};	    
	    if ($site->{'languageSensitive'} =~ /$field/) {
		$extra_hashref->{'TABLE'}->{'FIELDS'}->{'language'} = $site->{'om_render_lang'};
	    }
	    else {
		$extra_hashref->{'TABLE'}->{'FIELDS'}->{'language'} = "";
	    }
	    push @extra_inserts, $extra_hashref;
	}

    }

    # Add the variant product information
    if ($vcode ne "") {
	$vcode = &build_variant_product_vcode($input_hashref, $site, $dbh);
	&debug("create_object: Just build vcode to be $vcode", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$$input_hashref{'vcode_inp'} = $vcode;
	# There is a problem with the variant product generation
	if (&validate_variant_product($input_hashref, $site, $dbh)) {
	    &debug("create_object:  Error occured in validate_attributes", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{7};
	}
	$master_info = &inherit_master_fields($$input_hashref{'master_objid'}, "msku,title,objclass,objtype", $input_hashref, $site, $dbh);
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = $$input_hashref{'master_objid'};
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'msku'} = $master_info->{'msku'};
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'title'} = $master_info->{'title'};
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'objtype'} = $master_info->{'objtype'};
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'objclass'} = $master_info->{'objclass'};
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'vcode'} = $vcode;
	&debug("create_object: Flagging to update masters varaints.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	$flag->{'objid'} = $$input_hashref{'master_objid'};
	$flag->{'objtype'} = $site->{'OBJTYPES'}->{'product'};
    }

    # Make sure that the vcode was specified for this object, all of the vcode!
    if (($$input_hashref{'master_objid'} ne "") && ((!defined $$input_hashref{'vcode_inp'}) || ($$input_hashref{'vcode_inp'} eq ""))) {
	&debug("validate_object: Variant product, need to specify all of vcodes!", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$$input_hashref{'row.row0.axis0_err'} = $site->{'script_msg'}{'set_variant_pulldowns'};
	return ($site->{'script_msg'}{7});
    }


    $main_hashref->{'append_objid'} = "yes";
    $main_hashref->{'TABLE'}->{'FIELDS'}->{'language'} = $site->{'om_render_lang'};
    # if status is not part of the definition, then default it to active
    if (!defined $main_hashref->{'TABLE'}->{'FIELDS'}->{'status'}) {
	$main_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15000-A"; 
    }

    # if we are supposed to generate a sku for a brand, then do
    if (($$input_hashref{'objtype_inp'} eq $site->{'OBJTYPES'}->{'brand'}) && (defined $site->{'autosku'})) {
	if (!defined $main_hashref->{'TABLE'}->{'FIELDS'}->{'msku'}) {
	    # the __objid__ doesn't matter, as it will take the brand sku_skel
	    $main_hashref->{'TABLE'}->{'FIELDS'}->{'msku'} = &create_msku("__objid__", $site, $dbh, $$input_hashref{'sku_skel_inp'});
	}
    }

    # if we are supposed to generate a sku for products, then do so
    if (($$input_hashref{'objtype_inp'} eq $site->{'OBJTYPES'}->{'product'}) && (defined $site->{'autosku'})) {
	if (!defined $main_hashref->{'TABLE'}->{'FIELDS'}->{'msku'}) {
	    $main_hashref->{'TABLE'}->{'FIELDS'}->{'msku'} = &create_msku($$input_hashref{'sku_series_inp'}, $site, $dbh);
	}
    }

    $q = &build_sql($main_hashref, $site, $dbh, $site->{'override_permssions'});
    ($q, $objid) = split /\:\:/, $q;

    if (!$q) {
	&debug("create_object: Insufficient permissions to insert!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{2};
    }

    if (!&query_execute($q, $site, $dbh)) {
	&debug("create_object: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }

    foreach $extra_hashref (@extra_inserts) {
	$extra_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = $objid;
	$q = &build_sql($extra_hashref, $site, $dbh, $site->{'override_permissions'});
	if (!$q) {
	    &debug("create_object: Insufficient permissions to insert!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{2};
	}	
	if (!&query_execute($q, $site, $dbh)) {
	    &debug("create_object: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $error;
	}
    }
    
    # See if we need to update variant titles
    $error = &check_variant_titles($flag, $site, $dbh);
    if (defined $error) {
	&debug("create_object: Could not check for variant titles", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }

    &generate_text_files($$input_hashref{'objtype_inp'}, $objid, $site, $dbh, $flag->{'generate_page'});

    # See if we need to add information to the html defintions file
    if ($$input_hashref{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'attribute'}) {
	&update_attribute_html_list($input_hashref, $site);
	&check_language_sensitive_fields($input_hashref, $site, $dbh);
    }

    &debug("create_object: Successfully completed.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $$input_hashref{'objid'} = $objid;
    $$input_hashref{'new_object'} = undef $$input_hashref{'new_object'};
    return;
}

##############################################################################
=pod

=head2 update_object

update_object - &update_object("2-AA", \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

To take all of the input from the $input_hashref and udpate the data into the 
database.  The data will first go into the primary table (from get_primary_table,
things for most items, (or contact table for user records)) first, and whatever 
fails, will be updated or inserted into the extraprop table.  The updates are done
to the primary table on a language by language basis.  If the update fails for the
particular language, the non-language sensitive fields are again updated, and all
other language sensitive data is either updated or inserted into the extraprop 
table.

This can easily be extended to check pricing and inventory prior to updating things!

=item INPUT

$objid - Object ID of the object to modify.

$input_hashref - Pointer to the hash to use to find all of the input from the user.
                 Normally, this is a reference to the FORM hash.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message (by mapping into $site->{'script_msg'}) if an error
         occured, undef if no error occurred.

=item NOTES

=back

=cut
###############################################################################
sub update_object {
    my ($objid) = shift;
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, $main_hashref, $extra_hashref, $extra_q, $q, @extra_updates, $main_fields);
    my ($field, $key, $previous, $extra_indx, $flag, $error_flags, $where_indx, $extras);

    $extras = {};
    if ($$input_hashref{'axisnames_inp'} ne "") {
	$$input_hashref{'has_axisnames'} = "yes";
    }
    $error_flags = &validate_attributes($input_hashref, $extras, $site, $dbh);
    if (!$error_flags->{'validationcheck'}) {
	&debug("update_object:  Error occured in validate_attributes", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{7};
    }

    if ($error_flags->{'changestowrite'} eq "no") {
	&debug("update_object:  Nothing to write, just return", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    if ($error = &validate_object($input_hashref, $site, $dbh)) {
	&debug("update_object:  $error from validate_object - $site->{'script_msg'}{$error}", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }

    $main_hashref->{'statement'} = "update";
    $main_hashref->{'TABLE'}->{'NAME'} = &get_primary_table_name($$input_hashref{'objtype_inp'}, $$input_hashref{'objclass_inp'}, $site);
    &debug("update_object: Primary table is $main_hashref->{'TABLE'}->{'NAME'}, $$input_hashref{'objtype_inp'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $main_fields = &get_field_names($main_hashref->{'TABLE'}->{'NAME'}, $site, $dbh);

    foreach $key (keys %{$input_hashref}) {
	next if ($key !~ /_inp/);
	$field = $key;
	$field =~ s/_inp//gms;

	# If it is the generate_inp field
#	if (($field eq "generate") && ($$input_hashref{'generate_inp'} == 1)) {
	if (($field eq "gen") && ($$input_hashref{'gen_inp'} == 1)) {
	    &debug("update_object: This is the que to regenerate the page $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $flag->{'objid'} = $objid;
	    $flag->{'generate_page'} = 1;
	    next;
	}

	if ($main_hashref->{'TABLE'}->{'NAME'} eq "things") {

	    # The variant title changed, and we need to flag it to update variant product's titles
	    if (($field eq "title") && ($$input_hashref{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'variant'}) && 
		($$input_hashref{'title_inp'} ne $$input_hashref{'title_pre'})) {
		&debug("update_object:  We just changed the title of variant, so flag to check products!", 3, $site->{'debug_level'}, $site->{'debug_log'});
		$flag->{'objid'} = $objid;
		$flag->{'objclass'} = $$input_hashref{'objclass_inp'};
		$flag->{'objtype'} = $$input_hashref{'objtype_inp'};
	    }

	    # This is a master product who's 
	    if ($field eq "axisnames") {
		# title changed, therefore we have to flag change in variant product titles
		if ($$input_hashref{'title_inp'} ne $$input_hashref{'title_pre'}) {
		    $flag->{'objid'} = $objid;
		    $flag->{'objclass'} = $$input_hashref{'objclass_inp'};
		    $flag->{'objtype'} = $$input_hashref{'objtype_inp'};
		}

		if ($$input_hashref{'axisnames_inp'} ne "") {
		    $main_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = $objid;
		}
	    }

	    # The variant set changed, so update all variant products which use this set
	    if (($field eq "variant_set") && ($$input_hashref{'variant_set_inp'} ne $$input_hashref{'variant_set_pre'})) {
		$flag->{'objid'} = $objid;
		$flag->{'objclass'} = $$input_hashref{'objclass_inp'};
		$flag->{'objtype'} = $$input_hashref{'objtype_inp'};
	    }
    
	}	    

	$previous = $field . "_pre";
	if (defined $$input_hashref{$previous}) {
	    next if ($$input_hashref{$key} eq $$input_hashref{$previous});  # If the value didn't change, then don't update
	}

	if ($main_fields =~ /$field/) {
	    $main_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $$input_hashref{$key};
	}
	else {
	    $extra_hashref = {};
	    $extra_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	    $extra_hashref->{'statement'} = "update";
	    $extra_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $$input_hashref{$key};	    
	    $where_indx = 0;
	    $extra_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
	    $extra_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", $field, "and");
	    if ($site->{'languageSensitive'} =~ /$field/) {
		$extra_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("language", $site->{'om_render_lang'}, "and");
	    }
	    push @extra_updates, $extra_hashref;
	}
    }

    $where_indx = 0;
    $main_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    $main_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("language", $site->{'om_render_lang'}, "and");

    $q = &build_sql($main_hashref, $site, $dbh, $site->{'override_permssions'});

    if (!$q) {
	&debug("update_object: Insufficient permissions to insert!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{2};
    }

    if (!&query_execute($q, $site, $dbh)) {
	$error = &update_language_object($objid, $main_hashref, $site, $dbh);
	if (defined $error) {
	    &debug("update_object: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $error;
	}
    }

    $error = &alter_extraprop($site, $dbh, \@extra_updates);

    if (defined $error) {
	&debug("update_object: Could not update extraprop rows", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }


    # See if we need to update variant titles
    $error = &check_variant_titles($flag, $site, $dbh);
    if (defined $error) {
	&debug("update_object: Could not check for variant titles", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $error;
    }

    # See if we need to add information to the html defintions file
    if ($$input_hashref{'objclass_inp'} eq $site->{'OBJCLASSES'}->{'attribute'}) {
	&update_attribute_html_list($input_hashref, $site);
	&check_language_sensitive_fields($input_hashref, $site, $dbh);
    }

    &generate_text_files($$input_hashref{'objtype_inp'}, $objid, $site, $dbh, $flag->{'generate_page'});
    &debug("update_object: Successfully completed.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return;
}


##############################################################################
=pod

=head2 update_language_object

update_language_object - &update_language_object("3-AA", $main_hashref, $site, $dbh);

=over 4

=item DESCRIPTION

When defining objects in multiple languages, it is necessary to update and 
possible insert the additional language rows into the extraprop table.  Therefore, 
if we try to update into one language, and it returns a failure, then we must 
update/insert multiple rows, one for each language sensitive field which was modified
into the extraprop table.  We must also be sure to update the non-language sensitive 
fields in the primary table.

=item INPUT

$objid - Object ID of the object to modify.

$main_hashref - The hashref that was used to define the query which failed due
                to a language restriction.  This is used to build the next set
                of queries which will complete the update requests through 
                multiple languages.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message (by mapping into $site->{'script_msg'}) if an error
         occured, undef if no error occurred.

=item NOTES

=back

=cut
###############################################################################
sub update_language_object {
    my ($objid) = shift;
    my ($main_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($extra_hashref, $field, $q, $update_main, $extra_insert, $where_indx, @where_array);
    my ($where_indx, $while_indx, $where_hashref);

    &debug("In update_language_object", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $extra_hashref->{'statement'} = "update";
    $extra_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $extra_hashref->{'TABLE'}->{'FIELD'}->{'language'} = $site->{'om_render_lang'};
    foreach $field (keys %{$main_hashref->{'TABLE'}->{'FIELDS'}}) {
	if ($site->{'languageSensitive'} =~ /$field/) {
	    $extra_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $main_hashref->{'TABLE'}->{'FIELDS'}->{$field};
	    $extra_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = $field;
	    $where_indx = $while_indx = 0;
	    while ($where_hashref = $main_hashref->{'TABLE'}->{'WHERE'}[$while_indx++]) {
		$extra_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
	    }

	    $extra_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", $field, "and");

	    $q = &build_sql($extra_hashref, $site, $dbh, $site->{'override_permissions'});
	    if (!$q) {
		&debug("update_language_object: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
		return $site->{'script_msg'}{2};
	    }	
	    
	    if (!&query_execute($q, $site, $dbh)) {
		$extra_insert = &build_insert_from_update($extra_hashref, $site, $dbh);
		if (!$extra_insert) {
		    &debug("update_language_object: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
		    return $site->{'script_msg'}{2};
		}	

		if (!&query_execute($extra_insert, $site, $dbh)) {
		    &debug("update_language_object: $q failed", 1, $site->{'debug_level'}, $site->{'debug_log'});
		    return $site->{'script_msg'}{3};		    
		}
	    }
	}
	else {
	    $update_main->{'TABLE'}->{'FIELDS'}->{$field} = $main_hashref->{'TABLE'}->{'FIELDS'}->{$field};
	}

    }

    # If there is something to update in the main table, build the query and execute
    if (defined $update_main) {
	$update_main->{'TABLE'}->{'NAME'} = $main_hashref->{'TABLE'}->{'NAME'};
	$update_main->{'statement'} = "update";
	$where_indx = 0;
	$update_main->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    
	$q = &build_sql($update_main, $site, $dbh, $site->{'override_permissions'});
	if (!$q) {
	    &debug("update_language_object: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{2};
	}	
	
	if (!&query_execute($q, $site, $dbh)) {
	    &debug("update_language_object: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{3};
	}
    }

    &debug("update_language_object: Successfully completed.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return;
}

##############################################################################
=pod

=head2 alter_extraprop

alter_extraprop - &alter_extraprop($site, $dbh, \@queries);

=over 4

=item DESCRIPTION

Extraprop has a lot of action.  It is sometimes necessary to do a variety of 
extraprop inserts, selects or updates all at once.  This routine takes an 
array of hashrefs describing the SQL statements to be performed on extraprop 
and executes them all, until it comes to an error or completes.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$extra_sql - Pointer to an array of extraprop queries definition hashrefs.

=item OUTPUT

$error - Error message (by mapping into $site->{'script_msg'}) if an error
         occured, undef if no error occurred.

=item NOTES

=back

=cut
###############################################################################
sub alter_extraprop {
    my ($site) = shift;
    my ($dbh) = shift;
    my ($extra_sql) = shift;

    my ($extra_hashref, $q, $extra_fields);

    foreach $extra_hashref (@$extra_sql) {
	$q = &build_sql($extra_hashref, $site, $dbh, $site->{'override_permssions'});

	if (!$q) {
	    &debug("alter_extraprop: Insufficient permissions to insert!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{2};
	}

	if (!&query_execute($q, $site, $dbh)) {
	    # if update failed, then we need to insert it, as it wasn't already put into extraprop
	    $q = &build_insert_from_update($extra_hashref, $site, $dbh);
	    if (!$q) {
		&debug("alter_extraprop: Insufficient permissions to insert!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
		return $site->{'script_msg'}{2};
	    }
	    if (!&query_execute($q, $site, $dbh)) {  
		&debug("alter_extraprop: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
		return $site->{'script_msg'}{3};
	    }
	}
    }

    &debug("alter_extraprop: Successfully completed.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return;
}

##############################################################################
=pod

=head2 get_primary_table_name

get_primary_table_name - &get_primary_table_name("45000-A", "134-AA", $site);

=over 4

=item DESCRIPTION

To find which table that the object would best belong in based on it's object
type.  This will eventually be replaced with mapping functions.  For now, this
is the mapping function.

=item INPUT

$objtype - Type of object to find the primary table for

$objclass - Type of class to find the primary table for

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

A name of an os_allcommerce table which is the primary table for the objtype passed
in.

=item NOTES

It is necessary to allow for either the objclass or the objtype to be compared to, as
some objects are instances of a class, but do NOT have a type (ex Variants).  Therefore
we use whichever one is defined, looking at the type first.

=back

=cut
##############################################################################
sub get_primary_table_name {
    my ($objtype) = shift;
    my ($objclass) = shift;
    my ($site) = shift;

    if ($objtype ne "") {
	if ($objtype eq $site->{'OBJTYPES'}->{'contact'}) {
	    return "contacts";
	}
	else {
	    return "things";
	}
    }
    elsif ($objclass ne "") {
	if ($objclass eq $site->{'OBJCLASSES'}->{'contact'}) {
	    return "contacts";
	}
	else {
	    return "things";
	}

    }
    else {
	&debug("get_primary_table_name:  Not sure what table is primary, return things.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return "things";
    }
   
}


##############################################################################
=pod

=head2 generate_text_files

generate_text_files - &generate_text_files("47004-A", "", $site, $dbh);

=over 4

=item DESCRIPTION

Various objects in the all commerce system rely on both data in the database
and data on the file system.  Most items that are stored in both locations are
required for fast access.  Therefore, it is necessary to update the file system
files when certain events occur in the database.  This routine looks for the
traces that are left behind when these events occur, and generates the appropriate
text files.

It concerns itself with creating the sel files for brands, warehouses.

=item INPUT

$objtype - Type of object to check if need to generate files for

$objid - Objid of the object to check if need to generate files for

$site - A hashref containing information required throughout about the current
        site and session

$dbh - Active database handler

$generate_pages - Flag to generate the page, if defined. OPTIONAL parameter

=item OUTPUT

=item NOTES

If you ever note that these files are NOT being updated, this means that there
is a permissions problem.  The entire os_allcommerce tree should have user and
group names that are the same as the web server user and group.  If they are not,
the web server user will not have permissions to write these files to the disk.

=back

=cut
###############################################################################
sub generate_text_files {
    my ($objtype) = shift;
    my ($objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($generate_page) = shift;

    &debug("generate_text_files: In with $objtype, $objid, $generate_page", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($objtype eq $site->{'OBJTYPES'}->{'brand'}) {
	&debug("generate_text_files: Generated brand info", 3, $site->{'debug_level'}, $site->{'debug_log'});
#	`$site->{'fsrootcron'}/generate_temps.pl $objtype`;
	&generate_temps($dbh, $site, $objtype);
    }

    if ($objtype eq $site->{'OBJTYPES'}->{'warehouse'}) {
	&debug("generate_text_files: Generated warehouse info", 3, $site->{'debug_level'}, $site->{'debug_log'});
	`$site->{'fsrootcron'}/generate_temps.pl $objtype`;
#	&generate_temps($dbh, $site, $objtype);
    }

    if ($objtype eq $site->{'OBJTYPES'}->{'encoding'}) {
	&debug("generate_text_files: Generated encoding for $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
#	`$site->{'fsrootcron'}/generate_temps.pl $objid`;
	&generate_temps($dbh, $site, $objid);	
    }

    # GUI definitions for objects
    if ($objtype eq $site->{'OBJTYPES'}->{'object'}) {
	&debug("generate_text_files: Generated gui for $objtype $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
#	`$site->{'fsrootcron'}/generate_temps.pl $objtype $objid`;
	&generate_temps($dbh, $site, $objtype, $objid);	
    }

    # Generate triggers, generate the page
    if ((defined $generate_page) && ($generate_page == 1)) {
	&debug("generate_text_files: Going into generate triggers for $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&generate_specific($objid, $site, $dbh);
    }

}

##############################################################################
=pod

=head2 create_msku

create_msku - &create_msku(2-AA, $site, $dbh);
            - &create_msku("__objid__", $site, $dbh, 2-AA);

=over 4

=item DESCRIPTION

Determines the next sku for automated sku generations.  Works for both products
and brands.  If it is a brand that called this routine, the sku_skel is populated
and the sku_series is not.  If it is a product that called this routine, the sku_series
is populated and the sku_skel is not.

Find the next sku in the series and return it to the calling routine.

=item INPUT

$sku_series - Object ID of the sku_series (the objid pointing to the brand that
              is selected for the product).  This is not known for a brand generation,
              as the sku_series will eventually become the objid of the brand created.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$brand_skel - If this is a brand generation, this field will be supplied.  Therefore this
              is OPTIONAL.  This is the 3 letter unique code entered when defining the 
              brand.  When a product generation is the reason of the call, this is not 
              supplied.

=item OUTPUT

$msku - The next sku in the series, built on the sku_skel, store code, and the next
        number in the series.

=item NOTES

This is only called for automatic sku generation. If the store is not set up for
automatic sku generation ($site->{'autosku'} not defined), then this will never be
called.

=back

=cut
###############################################################################
sub create_msku {
    my ($sku_series) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($brand_skel) = shift;  # only if it brand


    my ($sql_hashref, $q, $numRows, $thisrow, $skel, $msku, $is_brand, $where_indx);

    # for a product
    if (!defined $brand_skel) {
	$sql_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	$sql_hashref->{'statement'} = "SELECT";
	$sql_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
	$sql_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
	$sql_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "fieldname";
	$where_indx = 0;
	$sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $sku_series, "and");
	$sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "sku_skel", "and");

	# override permissions, as if can create a product, must be able to create this
	$q = &build_sql($sql_hashref, $site, $dbh, 1);
	if (!$q) {
	    &debug("create_msku: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{2};
	}	
	
	($numRows, $thisrow)  = &query_getNext($q, $site, $dbh);
	if (defined $thisrow) {
	    $skel = $thisrow->{'body'};
	}	

    }
    else {  # for a brand
	$skel = $brand_skel;
	$numRows = 1;
	$is_brand = "yes";
    }

    if ( $numRows ) {   # got at least one row
	$skel =~ tr/ //d ;
	if (defined $is_brand) {
	    $msku = &get_next_sku_series($skel, $site, $dbh, $is_brand);
	}
	else {
	    $msku = &get_next_sku_series($skel, $site, $dbh);
	}
    } 
    
    return ($msku);
}

##############################################################################
=pod

=head2 get_next_sku_series

=over 4

get_next_sku_series - &get_next_sku_series(3-AA, $site, $dbh, "yes");

=item DESCRIPTION

Get the next sku in the series.

For products:  Find all of the other products that use this sku skel, and get the next 
number in the series.

For brands:  Use the sku_skel passed in, and attach 0's for the series, as a brand is
always xxxM#### where xxx is the sku_skel, and M is the store code, and #### is the next
number in the series.  Brands #### is always 0000.

=item INPUT

$sku_series - Object ID of the sku_series (the objid pointing to the brand that
              is selected for the product).  This is not known for a brand generation,
              as the sku_series will eventually become the objid of the brand created.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$is_brand - "yes" if it is requesting a sku for the brand, otherwise undefined.

=item OUTPUT

$msku - The next sku in the series

=item NOTES

=back

=cut
###############################################################################
sub get_next_sku_series {
    my ($skel) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($is_brand) = shift;  # optional, only if this is a brand!


    my ($q, $numRows, $sql_hashref, $msku, $sku_number, $length, $prev_msku);
    my ($sitesku, $thisrow, $where_indx);

    if (!defined $is_brand) {
	$sql_hashref->{'statement'} = "SELECT";
	$sql_hashref->{'TABLE'}->{'NAME'} = "things";
	$sql_hashref->{'TABLE'}->{'FIELDS'}->{'msku'} = "msku";

	$where_indx = 0;
	$sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("msku", "%$skel%", "and");
	$sql_hashref->{'TABLE'}->{'ORDER'} = "msku DESC";
	# override permissions, as if can create a product, must be able to create this
	$q = &build_sql($sql_hashref, $site, $dbh, 1);
	if (!$q) {
	    &debug("get_next_sku_series: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    return $site->{'script_msg'}{2};
	}	
	
	# get the first one, it is the largest
	($numRows, $thisrow) = &query_getNext($q, $site, $dbh);
	if ($numRows > 0) {
	    $sitesku = ($site->{'site_sku_prelen'} + length($site->{'sitecode'}));
	    $prev_msku = substr($thisrow->{'msku'}, $sitesku, $site->{'site_sku_numlen'});
	    $sku_number = $prev_msku + 1;
	    $sku_number = "$sku_number";
	}
    }
    else {
	# This is a brand, first sku in series
	$sku_number = "0";
    }
    

    $length = $site->{'site_sku_numlen'} - length($sku_number);
    
    $msku = $skel . $site->{'sitecode'} ;
    
    while ( 0 < $length ) {
	$msku .= "0";
	$length--;
    }

    $msku .= $sku_number;
    &debug("get_next_sku_series: Just create msku $msku.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $msku;
   
} 

##############################################################################
=pod

=head2 get_hashref_to_permute

get_hashref_to_permute - &get_hashref_to_permute($object_hashref, $site, $info);

=over 4

=item DESCRIPTION

Take an object hashref returned from load_object or load_objects, and copies all
data that will be used to permute down to the first level of a new hashref, with the
appropriate key name so that it will be caught in permutes.

This copipes all language sensitive data down to the main level of the hashref, therefore if the
object is defined in multiple languages, the language sensitive data that applies 
will be pulled down into the main hashref for use.  It also determines if there
is language sensitive data that is NOT defined in the om_render_lang, but is defined
in other languages, it finds the definition of the 'most important language', and
stores in it in the <field>_lang key, so that it will be displayed on the screen
under the input box, showing what was defined in another language.

If this is for a multi-row page, when it moves the information down to the main 
level of the hashref, it confires the key based on the row numbers.  Therefore, it
may define row.1.title_inp if it is a row, and it is on the first row, or it may 
define title_inp if it is the only informaiton on this row.

=item INPUT

$object_hashref - Hashref defining an object, as formated by load_object or load_objects.

$site - A hashref containing information required throughout about the current
        site and session

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

=item OUTPUT

$permute_with  - A hashref with all information required to permute based on the information
                 from the object_hashref.  Everything is at the main level and can be used as
                 a direct parameter to permutebit.

=item NOTES

=back

=cut
###############################################################################
sub get_hashref_to_permute {
    my ($object_hashref) = shift;
    my ($site) = shift;
    my ($object_info) = shift;

    my ($lang_order, $object);
    my ($default_hashref, $field, $i);
    my (@languages, $lang, $permute_with, $previous, $lang_fields, $lang, $already_have);


    # get all of the non language sensitive data
    foreach $field (keys %{$object_hashref}) {
	if ($site->{'languageSensitive'} !~ /$field/) {
	    $previous = &get_row_number($field, $object_info);
	    $previous .= "_pre";
	    $permute_with->{$previous} = $object_hashref->{$field};
	}
    }

    # get all of the language sensitive data that we have defined
    foreach $field (keys %{$object_hashref->{$site->{'om_render_lang'}}}) {
	if ($object_hashref->{$site->{'om_render_lang'}}->{$field} ne "") {
	    $previous = &get_row_number($field, $object_info);
	    $previous .= "_pre";
	    $permute_with->{$previous} = $object_hashref->{$site->{'om_render_lang'}}->{$field};
	    $already_have .= $field;
	}
    }
    

    # get all of the language sensitive data that we do not have defined for this language
    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
    $lang_order =~ s/$site->{'om_render_lang'}\,//gms;
    $object = &set_fields_to_default_lang_val($object_hashref, $site->{'languageSensitive'}, $lang_order, $site);

    # get all of the language sensitive data that we have defined
    foreach $field (keys %{$object}) {
	# if it is language sensitive, and we haven't defined it in primary language, then show it.
	if ($site->{'languageSensitive'} =~ /$field/) {
	    &debug("get_hashref_to_permute_with:  $field is $object->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $lang = &get_row_number($field, $object_info);
	    $lang .= "_lang";
	    if ($already_have !~ /$field/) {
		$permute_with->{$lang} = $object_hashref->{$field};

		# Replace new lines and CR with <P> for display purposes
		if ($permute_with->{$lang} =~ /\r\n/) {
		    &debug("\tget_hashref_to_permute_with:  $field has CR in it", 3, $site->{'debug_level'}, $site->{'debug_log'});
		    $permute_with->{$lang} =~ s/\r\n/<P>/gms;
		}

		if ($permute_with->{$lang} =~ /\n/) {
		    &debug("\tget_hashref_to_permute_with:  $field has NL in it", 3, $site->{'debug_level'}, $site->{'debug_log'});
		    $permute_with->{$lang} =~ s/\n/<P>/gms;
		}
		# Work around for non-language sensitive title in attributes
		if ($field eq "title") {
		    &debug("get_hashref_to_permute: This is $field, and check class: $object->{'objclass'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
		    if ((defined $object->{'objclass'}) && ($object->{'objclass'} eq $site->{'OBJCLASSES'}->{'attribute'})) {
			$previous = &get_row_number($field, $object_info);
			$previous .= "_pre";
			$permute_with->{$previous} = $permute_with->{$lang};
			$permute_with->{$lang} = "";
		    }
		}
	    }
	    else {
		$permute_with->{$lang} = "";  # clear it out so it doesn't take what was in the %FORM before
	    }

	}
    }

    &debug_hashref($permute_with, 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $permute_with;
}

##############################################################################
=pod

=head2 build_html_select

build_html_select - &build_html_select($p_raw, $field, $value, "row", "MULTIPLE", $site);

=over 4

=item DESCRIPTION

=item INPUT

$p_raw - Pointer to an array of raw data, in the value|tag format

$field - Fieldname to replace the select with.  

$value - Default value to set to

$row_or_cell - "row" to retrieve select template from the "rows" directory
               "cell" to retrieve select template from the "cells" directory

$multiple - MULTIPLE if want the select to allow multiple input, nothing otherwise

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$select_template - Select object either cell or row depending on what defined
                   in $row_or_cell, with $value set and $field being tagged
                   for input.
=item NOTES

=back

=cut
###############################################################################
sub build_html_select {
    my ($p_raw) = shift;
    my ($field) = shift;
    my ($value) = shift;
    my ($row_or_cell) = shift;
    my ($multiple) = shift;
    my ($site) = shift;

    my ($select, $select_template, $filename, $select_tag);
    
    $select = &generate_select($value, $p_raw);
    $filename = $row_or_cell . "_select";
    $select_template = &localize_template($filename, "admin", 0, $site);
    $select_template =~ s/__MULTIPLE__/$multiple/gms;
    $select_template =~ s/__select__/$field/gms;
    $select_tag = "__" . $field . "_SELECT__";
    &debug("build_html_select: replacing $select_tag from $select_template with $select.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $select_template =~ s/$select_tag/$select/gms;
    return ($select_template);
}

##############################################################################
=pod

=head2 format_array_into_hashref

format_array_into_hashref - &format_array_into_hashref($key, $p_array);

=over 4

=item DESCRIPTION

Takes an array of hashrefs or text, and copies everything into one large hashref.

=item INPUT

$key - Name of field to key the big hashref on.

$p_array - point to an array of hashrefs, to be extracted and stored in the big_hashref

=item OUTPUT

$big_hashref - Big hashref, keyed by $key, of all entries in $p_array

=item NOTES

=back

=cut
###############################################################################
sub format_array_into_hashref {
    my ($key) = shift;
    my ($p_array) = shift;
    
    my ($big_hashref, $mini_hashref);

    foreach $mini_hashref (@$p_array) {
	# This is just an array of elements, use them as the key & value
	if ($key eq "") {
	    $mini_hashref =~ s/\n//gms;
	    $mini_hashref =~ s/ //gms;
	    next if ($mini_hashref eq "");
	    $big_hashref->{$mini_hashref} = $mini_hashref;
	}
	else {
	    $big_hashref->{$mini_hashref->{$key}} = $mini_hashref;
	}
    }

    return $big_hashref;

}

##############################################################################
=pod

=head2 update_rows

update_rows - &update_rows(\%FORM, $object_info, $site, $dbh);

=over 4

=item DESCRIPTION

Some displays have various rows on them, some of which you can update, and others
of which you can insert new rows (variant products, prices).  This routine sorts
the form into rows, and processes each row.  If it is an existing row, it updates
any changes made to the row.  If it is a new row, then it inserts the new row.  
Validation checking is done prior to any update/inserts as always.

=item INPUT

$FORM - Pointer to the FORM hash

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Error message if error occured, undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub update_rows {
    my ($FORM) = shift;
    my ($object_info) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($form_rows, $modify_form_rows, $object_info, %mini_form, $objid, $error);

    &debug("update_rows: In here.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $form_rows = &sort_form_into_input_rows($FORM, $site);
    &debug_hashref($form_rows, 3, $site->{'debug_level'}, $site->{'debug_log'});
    # update all existing data if it is necessary & create what is required
    foreach $objid (keys %{$form_rows}) {
	%mini_form = %{$form_rows->{$objid}};
	$mini_form{'master_objid'} = $$FORM{'master_objid'};
	$mini_form{'is_a_new_row'} = "yes";
	&debug("update_rows: Objid is $objid.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	if ($objid =~ /row/) { # this is a NEW row
	    $mini_form{'mode'} = "create_object";
	    $error = &create_object(\%mini_form, $site, $dbh);
	    if ($error) {
		&record_errors_in_form($FORM, \%mini_form, $site);
	    }
	    else {
		&debug("update_rows: Created new object.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    }
	}
	else {  # this is an UPDATE row
	    $mini_form{'mode'} = "update_object";
	    $mini_form{'objid'} = $objid;
	    $error = &update_object($objid, \%mini_form, $site, $dbh);
	    if ($error) {
		&record_errors_in_form($FORM, \%mini_form, $site);
	    }
	    else {
		&debug("update_rows: Updating $mini_form{'objid'}.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    }
	}
    }

    &debug("update_rows: finished.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $error;

}

##############################################################################
=pod

=head2 record_errors_in_form

record_errors_in_form - &record_errors_in_form($FORM, \%tmp, $site);

=over 4

=item DESCRIPTION

When we are modifying multiple rows at once, we split the form into 
mini forms, each of which is one object to update or insert.  Therefore,
after each time that we validate the information, we need to copy the errors
back into the main FORM if there are any, because they are in the mini form
and the FORM is what is used to build the next display

=item INPUT

$FORM - Pointer to form hash

$tmp_form - The portion of the FORM which represented one row of data

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$FORM is updated with the errors that were placed into $tmp_form

=item NOTES

=back

=cut
###############################################################################
sub record_errors_in_form {
    my ($FORM) = shift;
    my ($temp_form) = shift;
    my ($site) = shift;

    my ($key);
    
    &debug("In record_errors_in_form", 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $key (keys %{$temp_form}) {
	$$FORM{'previous_error'} = "yes";
	next if ($key !~ /_err/);
	$$FORM{$key} = $$temp_form{$key};
	&debug("record_errors_in_form: Added error for $key, $$FORM{$key}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    }
}

##############################################################################
=pod

=head2 sort_form_into_input_rows

sort_form_into_input_rows - &sort_form_into_input_rows(\%FORM, $site);

=over 4

=item DESCRIPTION

Take the FORM and sort all of the rows.  This is form pages which have multiple
rows showing up in one page.  We need to have all of the data grouped together per
row in order to know what to update/insert.  This copies all of the row information
to a new hashref, and returns it for use in updates and inserts of multiple rows.

=item INPUT

$FORM - Pointer to form hash

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$form_rows - hashref keyed on the row.objid.field_inp, for each row that is displayed on 
             the form.  _inp, _lang, _pre fields are all set as to what is shown of the
             FORM for every row.

=item NOTES

=back

=cut
###############################################################################
sub sort_form_into_input_rows {
    my ($FORM) = shift;
    my ($site) = shift;

    my ($key, $form_rows, $objid, $junk, $field, $previous, $lang, $ini);
    my ($new_ini, $new_prev, $new_lang, $err, $new_err);

    &debug("sort_form_into_rows: In here.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $key (keys %{$FORM}) {
	next if ($key !~ /_inp/);
	next if ($key !~ /row./);

	$previous = $key;
	$previous =~ s/_inp/_pre/gms;
        # no data change, so won't be updating it, if it is anew row, add it anyways
	next if (($$FORM{$key} eq $$FORM{$previous}) && ($key !~ /row.row/));  
	&debug("sort_form_into_rows: key is $key, $$FORM{$key}, previous i $previous $$FORM{$previous}.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$ini = $key;
	$ini =~ s/_inp/_ini/gms;
	$lang = $key;
	$lang =~ s/_inp/_lang/gms;
	$err = $key;
	$err =~ s/_inp/_err/gms;

	($junk,$objid,$field) = split /\./, $key;
	&debug("sort_form_into_rows: key is $key, $$FORM{$key} is junk is $junk, objid is $objid, field is $field.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$form_rows->{$objid}->{'row_info'} = "$junk.$objid";
	$form_rows->{$objid}->{$field} = $$FORM{$key};
	$new_prev = $field;
	$new_prev =~ s/_inp/_pre/gms;
	$new_lang = $field;
	$new_lang =~ s/_inp/_lang/gms;
	$new_ini = $field;
	$new_ini =~ s/_inp/_ini/gms;
	$new_err =~ s/_inp/_err/gms;
	$form_rows->{$objid}->{$new_prev} = $$FORM{$previous};
	$form_rows->{$objid}->{$new_lang} = $$FORM{$lang};
	$form_rows->{$objid}->{$new_ini} = $$FORM{$ini};
	$form_rows->{$objid}->{$new_err} = $$FORM{$err};
    }

    return $form_rows;
}

##############################################################################
=pod

=head2 get_object_info

get_object_info - &get_object_info(\%FORM);

=over 4

=item DESCRIPTION

To retrieve certain information from the FORM and store it in a temporary hash,
which is passed around.  This hash is used to flag certain data.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_info - Hashref of information.

    Flags for dealing with any object
    - $object_info->{'gui_name'} = Name of .gui file used to generate this page
    - $object_info->{'objid'} = Object ID of object, if available
    - $object_info->{'objtype'} = Object type of object, if available
    - $object_info->{'objclass'} = Object class of object, if available
    - $object_info->{'new_object'} = "yes" if this is a new object, undef if it is existing object
    - $object_info->{'objclass_to_attach'} = Object class of data going to attach attribute to
    - $object_info->{'objtype_to_attach'} = Object type of data going to attach attribute to

    Flags for dealing with rows
    - $object_info->{'row_number'} = The row number to use, if this is a new row among multiple rows

    Flags for dealing with variant products
    - $object_info->{'master_objid'} = master product object ID
    - $object_info->{'exists'} = .gui file to use to display existing variant products
    - $object_info->{'new'} = .gui file to use to display new variant products

    Flags for dealing with html_def.pm
    - $object_info->{'rm_field'} is set to the name of fields that are to be removed from
      the html_def.pm file.  They are to be removed, if the attribute is renamed for example.
    - $object_info->{'field'} is set to the name of the new field that is to be added to the file.
    - $object_info->{'html'} is set to the html string for the new field.

=item NOTES

These flags are described here, although all of these are not set here.  THey are set throughout
various routines which use this data structure, although it is easier to define it in one
location, which is what is done here.

=back

=cut
###############################################################################
sub get_object_info {
    my ($FORM) = shift;

    my ($object_info);

    $object_info->{'gui_name'} = $$FORM{'gui_name'};
    $object_info->{'objid'} = $$FORM{'objid'};
    $object_info->{'objtype'} = $$FORM{'objtype_inp'};
    $object_info->{'objclass'} = $$FORM{'objclass_inp'};
    $object_info->{'new_object'} = $$FORM{'new_object'};
    $object_info->{'objclass_to_attach'} = $$FORM{'pre_objclass'};
    $object_info->{'objtype_to_attach'} = $$FORM{'pre_objtype'};
    
    # In case it is master or variant product
    $object_info->{'master_objid'} = $$FORM{'master_objid'};
    $object_info->{'exists'} = "var_exists_gui";
    $object_info->{'new'} = "var_new_gui";

    return ($object_info);
}

##############################################################################
=pod

=head2 initialize_axis_selects

initialize_axis_selects - &initialize_axis_selects($object_body, $object_inf, "", $num_axis, \@v_axis, $site, $dbh);

=over 4

=item DESCRIPTION

This routine fills in all of the __axis__ tags for variant products.  These are
the dynamic pulldowns for each variantional set for variant products.  It sets
the values of these pulldowns based on what the variant product has in it's vcode.

=item INPUT

$object_body - The template built for the variant product, with the __axis__ tags
               in place.

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$vcode - The vcode of the variant product, to use to set the default values of the
         __axis__ pulldowns.

$num_axis - The number of variational sets that the product uses

$variational_axis - pointer to an array of variational sets, each entry the objid of the
                    variational set in the related position (retreived from the master axisnames
                    value).

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$object_body - The template with the __axis__ tags permuted with the dynamically
               generated variational set pulldowns.

=item NOTES

=back

=cut
###############################################################################
sub initialize_axis_selects {
    my ($object_body) = shift;
    my ($object_inf) = shift;
    my ($vcode) = shift;
    my ($num_axis) = shift;
    my ($variational_axis) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@vcodes, $i, $variant_axis, $this_vcode, $axis_title, $axis_tag, $axis_select);
    my ($tag_num, $select_row, $tmp_tag, $col_title);
    
    &debug("initialize_axis_selects: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    @vcodes = split /\:/, $vcode;
    # build the axis selects and set the values, the other selects are taken care of in the main display routine
    for ($i = 0; $i < $num_axis ; $i++) {
	$variant_axis = $$variational_axis[$i];
	$this_vcode = $vcodes[$i];

	# generate the pulldowns for $variant_axis
	($axis_select, $axis_title) = &initialize_one_axis_select($variant_axis, $this_vcode, $site, $dbh);
	&debug("initialize_axis_selects: do we continute?  $axis_select", 3, $site->{'debug_level'}, $site->{'debug_log'});

	if ($axis_select ne "") {
	    $select_row = &localize_template("cell_select", "admin", 0, $site);
	    $axis_tag = "axis" . $i;
	    $axis_tag = &get_row_number($axis_tag, $object_inf);
	    $select_row =~ s/__select__/$axis_tag/gms;
	    &debug("initialize_axis_selects: Just build cell_select: $select_row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $tmp_tag = "__" . $axis_tag . "_SELECT__";
	    $select_row =~ s/$tmp_tag/$axis_select/gms;

	    # for the title, don't worry about getting the row, it is always just the axis#_tag
	    $tmp_tag = "__axis" . $i . "_tag__";
	    $object_body =~ s/$tmp_tag/$axis_title/gms;
	    &debug("initialize_axis_selects: Just build col_title: $col_title, replace with $tmp_tag", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $tmp_tag = "__axis" . $i . "__";
	    $object_body =~ s/$tmp_tag/$select_row/gms;
	    &debug("initialize_axis_selects: replaced $tmp_tag with $select_row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }

    &debug("initialize_axis_selects: returning $object_body", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($object_body);
}

##############################################################################
=pod

=head2 initialize_one_axis_select

initialize_one_axis_select - &initialize_one_axis_select();

=over 4

=item DESCRIPTION

To dynamically generate and set one of the pulldowns for one axis 
of the variant product.

=item INPUT

$load_axis - objid of the axis to load to determine what the variant_set is

$vcode_value - Value of the axis, to set as the default value

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$select - The select file generated from the variational_set, with the $vcode_value
          set

$axis_object->{'title'} - The title to use as the tag for the dynamic pulldown.  Takes
                          into consideration language.

=item NOTES

=back

=cut
###############################################################################
sub initialize_one_axis_select {
    my ($load_axis) = shift;   
    my ($vcode_value) = shift;          
    my ($site) = shift;          
    my ($dbh) = shift;

    my (@results, $q, $num_rows, $sel_file,  @raw, $sel_tag_file, $select_pre, $select_input, $select_tag, $result);
    my ($fields, $next_start, $select, $axis_object, @where, $variant_hashref, $variant);
    my ($row, @variant_set);
    
    &debug("initialize_one_axis_select: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $select = "";

    # Add a Not specified row
    $raw[0] = "|" . $site->{'script_msg'}{'not_specified'};

    # load the axisname information for this axisname
    $axis_object = &load_default_object($load_axis, "things", "title,brief,axisnames,language,variant_set", \@where, $site, $dbh);

    # Get the variant set for this axisname
    @variant_set = split /\:/, $axis_object->{'variant_set'};

    # build the pulldown body
    foreach $variant (@variant_set) {
	$variant_hashref = {};
	$variant_hashref = &load_default_object($variant, "things", "title", \@where, $site, $dbh);
	$row = $variant . "|" . $variant_hashref->{'title'};
	push @raw, $row;
    }

    # if there is something to generate
    if ($#raw > 0) {
	$select = &generate_select($vcode_value, \@raw);
    }
    &debug("initialize_one_axis_selects: Just built $select, $axis_object->{'title'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($select, $axis_object->{'title'});
}


##############################################################################
=pod

=head2 load_main_admin_template

load_main_admin_template - &load_main_admin_template($template_name, $site);

=over 4

=item DESCRIPTION

To load in the main template for admin object objects, and permute all
required rows that make up the template.

=item INPUT

$template_name - Name of the template to load in as the main page

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$template - The main template

=item NOTES

=back

=cut
###############################################################################
sub load_main_admin_template {
    my ($template_name) = shift;
    my ($site) = shift;

    my ($template, @rows);
    $template = &localize_template($template_name, "admin", 0, $site);
    @rows = ('row_spine','row_menu','row_headers','row_form','row_copyright','row_help','row_site_info', 'row_links');
    $template = &permute_rows($template, $site, \@rows);

    return $template;
}

# ****************************************    MOVE TO formtools.cgi  **********************************************************************
##############################################################################
=pod

=head2 build_object_template

build_object_template - &build_object_template($object_inf, $site, $dbh, \@select, \@checks);

=over 4

=item DESCRIPTION

To take an object template configuration, and dynamically create the body 
for the template.  Determine if the attributes should be read only or allow input.

=item INPUT

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$select_array - pointer to an array, to store all names of fields which are HTML select
                boxes, to flag that they need to be dynamically generated with the sel_file
                and the value

$checkbox_array - pointer to an array, to store all names of fields which are HTML checkboxes,
                  to flag that they need to go through initialize_checkboxes.

=item OUTPUT

$body - The contents of the object, dynamically generated to show data that the 
        user is allowed to configure as input, and data they can read only as read only information.

$select_array & $checkbox_array are populated with data needed to go through initialize routines.

=item NOTES

=back


=cut
###############################################################################
sub build_object_template {
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($select_array) = shift;
    my ($checkbox_array) = shift;

    my ($body, $row, @attributes, $ror_titles, $row_template, $link, $attr, $h, $tag);
    my ($col_titles, $row_values, $row_titles, $tag_template, $num_columns, $column_width, $obj_perms);
    my ($class_perms, $type_perms);
    $object_inf->{'class_perms'} = &get_attribute_permissions($object_inf->{'objclass'}, $site);
    if ($object_inf->{'class_perms'} =~ /x/) {
	return "";
    }

    $object_inf->{'type_perms'} = &get_attribute_permissions($object_inf->{'objtype'}, $site);
    if ($object_inf->{'type_perms'} =~ /x/) {
	return "";
    }

    &load_object_template_def($object_inf, $site, $dbh, \@attributes);
    &debug("build_object_template: Have $#attributes + 1 attributes.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $num_columns = 0;
    foreach $row (@attributes) {
	&debug("build_object_template: $row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	next if ($row =~ /<link>/);

	if ($row =~ /<header>/) {
	    $body .= &build_header($row, $site);
	}
	elsif ($row =~ /<row_start>/) {
	    &debug("build_object_template: Started new row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $row_template = &localize_template("row_rowtemplate", "admin", 0, $site);
	}
	# new row is a new row tag, but considered the same row for input purposes, the row_number
	# doesn't change
	elsif (($row =~ /<row_end>/) || ($row =~ /<new_row>/)) {
	    $row_template =~ s/__titles__/$col_titles/gms;
	    $row_template =~ s/__rows__/$row_values/gms;

	    # Add on known columns, for variational sets mostly
	    if ($object_inf->{'additional_cols'}) {
		$num_columns = $num_columns + $object_inf->{'additional_cols'};
	    }

	    &debug("build_object_template: Ending row because of $row, $num_columns", 3, $site->{'debug_level'}, $site->{'debug_log'});

	    if ($num_columns > 0) {
		$column_width = 100/$num_columns;
		$column_width = "WIDTH=$column_width%";
		$row_template =~ s/__column_width__/$column_width/gms;
		$row_template =~ s/__table_width__/$column_width/gms;
	    }

	    $body .= $row_template;
	    $row_template = "";
	    $col_titles = "";
	    $row_values = "";
	    $num_columns = 0;

	    # increase to the next row number, as we have a new row to show
	    if ($row =~ /<row_end>/) {
		if ($object_inf->{'row_number'} ne "") {
		    $object_inf->{'row_number'} += 1;
		    &debug("build_object_template: Just set row_number to $object_inf->{'row_number'}, row was $row", 3, $site->{'debug_level'}, $site->{'debug_log'});
		}
	    }
	    # this was a <new_row>, don't increase the key number, and start the row template again
	    else {
		&debug("build_object_template: Starting a new row because of $row", 3, $site->{'debug_level'}, $site->{'debug_log'});
		$row_template = &localize_template("row_rowtemplate", "admin", 0, $site);
	    }


	}
	elsif ($row =~ /_gui>/) {
	    $row =~ s/<//gms;
	    $row =~ s/>//gms;
	    $h->{'gui_name'} = $row;
	    $body .= &build_object_template($h, $site, $dbh, $select_array, $checkbox_array);
	}
	else {
	    if ($row_template ne "") {
		($attr, $tag) = &build_attribute_template($row, $object_inf, "row", $site, $select_array, $checkbox_array);
		$row_values .= $attr;
		if ($tag !~ /^<_/) {   # place holder
		    $col_titles .= " " . &localize_template("cell_rowtitles", "admin", 0, $site);
		    $tag =~ s/<_/__/gms;
		    $tag =~ s/_>/__/gms;
		    $col_titles =~ s/__tag__/$tag/gms;
		    $num_columns++;
		}
		else {
		    $col_titles .= $tag;  # just a spacer
		}
	    }
	    else {
		($attr, $tag) = &build_attribute_template($row, $object_inf, "cell", $site, $select_array, $checkbox_array);
		if (defined $attr) {
		    $tag_template = &localize_template("cell_tag", "admin", 0, $site);
		    $tag_template =~ s/__tag__/$tag/gms;
		    $column_width = "WIDTH=20%";
		    $tag_template =~ s/__column_width__/$column_width/gms;
		    $attr =~ s/__tag__/$tag_template/gms;
		    $column_width = "WIDTH=80%";
		    $attr =~ s/__column_width__/$column_width/gms;
		    $column_width = "WIDTH=100%";
		    $attr =~ s/__table_width__/$column_width/gms;
		    $row = &localize_template("row_row", "admin", 0, $site);
		    $row =~ s/__row__/$attr/gms;
		    $body .= $row;
		}
	    }
	}

    }

    return $body;
}

##############################################################################
=pod

=head2 build_header

build_header - &build_header($row, $site);

=over 4

=item DESCRIPTION

To build a header to show in the body of a dynamically generated form.

=item INPUT

$row - One row of configuration, in the format info|tag used to build the header

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$header - A permuted header, with the data from $row permuted.

=item NOTES

=back

=cut
###############################################################################
sub build_header {
    my ($row) = shift;
    my ($site) = shift;

    my ($header, @options, $option, $tag, $info, $junk);
    my ($tmp, $replace_with, $val);

    $header = &localize_template("row_header", "admin", 0, $site);
    &debug("build_header: In here with $row, $header", 3, $site->{'debug_level'}, $site->{'debug_log'});
    ($info, $tag) = split /\|/, $row;
    $header =~ s/__header__/$tag/gms;
    ($junk, @options) = split /\:/, $info;
    foreach $option (@options) {
	if ($option =~ /^value=/) {
	    ($tmp, $val) = split /\=/, $option;
	    $header =~ s/$replace_with/$val/gms;
	}
    }

    &debug("build_header: build $header", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($header);
    
}

##############################################################################
=pod

=head2 build_attribute_template

build_attribute_template - &build_attribute_template($row, $object_info, "row", $site, \@select, \@check);

=over 4

=item DESCRIPTION

To take one row for attribute definition, determine whether to display an 
input template or just display the result, and build that individual row 
for that attribute. 

=item INPUT

$row - One row, an attribute definition for the HTML

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$row_or_cell - "row" to retrieve select template from the "rows" directory
               "cell" to retrieve select template from the "cells" directory

$site - A hashref containing information required throughout about the current
        site and session

$select_array - pointer to an array, to store all names of fields which are HTML select
                boxes, to flag that they need to be dynamically generated with the sel_file
                and the value

$checkbox_array - pointer to an array, to store all names of fields which are HTML checkboxes,
                  to flag that they need to go through initialize_checkboxes.

=item OUTPUT

$attr_temp - The template for the individual template

$tag - The tag to display in the FORM, language appropriate

=item NOTES

=back

=cut
###############################################################################
sub build_attribute_template {
    my ($row) = shift;
    my ($object_info) = shift;
    my ($row_or_cell) = shift;
    my ($site) = shift;
    my ($select_array) = shift;
    my ($checkbox_array) = shift;

    my ($field, $tag, $option, @options, $attr_temp, $attr_perm);
    $row =~ s/\n//gms;

    &debug("In build_attribute_template with $row_or_cell, $row", 3, $site->{'debug_level'}, $site->{'debug_log'});
    ($field, $tag) = split /\|/, $row;
    ($field, @options) = split /\:/, $field;
    $attr_perm = &get_attribute_permissions($field, $site);
    if ((!defined $attr_perm) || ($attr_perm =~ /x/)) {
	return;
    }

    if (defined $object_info->{'type_perms'}) {
	&debug("build_attribute_template:  type permissions is enables, so concatenate their permissions", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$attr_perm .= $object_info->{'type_perms'};
    }

    if (defined $object_info->{'class_perms'}) {
	&debug("build_attribute_template:  class permissions is enables, so concatenate their permissions", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$attr_perm .= $object_info->{'class_perms'};
    }

    # place holder
    if ($row =~ /^<_/) {
	$row =~ s/ //gms;
	$attr_perm = "rcmd";
	$tag = &get_place_holder_tag($row);
	$tag = "__" . $tag . "_tag__";
    }

    # special case for generate triggers TARA UNCOMMENT WHEN PERMISSIONS ARE DONE
#    if ($field eq "gen") {
#	if (!&check_generate_triggers($site))
#	    return;
#    }

    foreach $option (@options) {
	if ($option =~ /display/) {
	    $attr_temp = &build_display_template($field, $row_or_cell, $site, \@options, $object_info);
	}
    }

    # we didn't need to 'display' only
    if (!defined $attr_temp) {
	if (($attr_perm =~ /m/) || ($attr_perm =~ /c/)) {
	    $attr_temp = &build_input_template($field, $row_or_cell, $site, $select_array, $checkbox_array, $object_info);
	    $object_info->{'need_enter_button'} = "YES";  # flag to input the enter button
	}
	# Only show the read only information if we are looking at an existing product, don't show new product stuff
	elsif (($attr_perm =~ /r/) && ($object_info->{'new_object'} eq "")) {
	    $attr_temp = &build_display_template($field, $row_or_cell, $site, \@options, $object_info);
	}
	else {
	    &debug("build_attribute_template: No permissions for $field, so not building it", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }

    return ($attr_temp, $tag);
}

##############################################################################
=pod

=head2 get_place_holder_tag

get_place_holder_tag - &get_place_holder_tag($row);

=over 4

=item DESCRIPTION

To parse a row which has <_ and _> surrounding the actual tag (attributes that are
to be dynamically generated), and return the tag in between these delimiters.

=item INPUT

$row - Row with <_ and _>

=item OUTPUT

$tag - $row with <_ and _> stripped.

=item NOTES

=back

=cut
###############################################################################
sub get_place_holder_tag {
    my ($row) = shift;

    my ($tag);

    $tag = $row;	
    # remove spaces
    $tag =~ s/ //gms;
    # remove front tag distinguisher
    $tag =~ s/<_//gms;
    # remove rear tag distinguisher
    $tag =~ s/_>//gms;

    return $tag;
}

##############################################################################
=pod

=head2 build_link

build_link - &build_link($row, $site);

=over 4

=item DESCRIPTION

Take the definition of a link, and load it in. 

=item INPUT

$row - One row from an object HTML gui def

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$attr_temp - A row which consists of a link template.

=item NOTES

=back

=cut
###############################################################################
sub build_link {
    my ($row) = shift;
    my ($site) = shift;


    my ($attr_temp, $link, $tag, $option, @options, $opt, $value);
    my ($replace);

    $attr_temp = &localize_template("row_link", "admin", 0, $site);
    ($link, $tag) = split /\|/, $row;
    ($link, @options) = split /\:/, $link;
    foreach $option (@options) {
	($opt, $value) = split /\=/, $option;
	$replace = "__" . $opt . "__";
	$attr_temp =~ s/$replace/$value/gms;
    }
    $attr_temp =~ s/__tag__/$tag/gms;
    return $attr_temp;
}


##############################################################################
=pod

=head2 build_display_template

build_display_template - &build_display_template();

=over 4

=item DESCRIPTION

To create a row, with the data as read only.  Do not use input fields for this data.

=item INPUT

$field - The name of the field to create this row for.  The attribute name

$row_or_cell - "row" to retrieve select template from the "rows" directory
               "cell" to retrieve select template from the "cells" directory

$site - A hashref containing information required throughout about the current
        site and session

$options - Optional, pointer to an array which defines the options configured, if there are any

$flags -  Hashref with flags of information. Contains row number if this is a
          group of rows.

=item OUTPUT

$template - Template in read only format for the attribute

=item NOTES

=back

=cut
###############################################################################
sub build_display_template {
    my ($field) = shift;
    my ($row_or_cell) = shift;
    my ($site) = shift;
    my ($options) = shift;
    my ($flags) = shift;

    my ($template, $option, @options, $tmp, $val, $replace_with, $filename);

    $filename = $row_or_cell . "_display";
    $template = &localize_template($filename, "admin", 0, $site);
    $field = &get_row_number($field, $flags);
    $template =~ s/__display__/$field/gms;
    foreach $option (@$options) {
	if ($option =~ /^value=/) {
	    ($tmp, $val) = split /\=/, $option;
	    $template =~ s/$replace_with/$val/gms;
	}
    }
    return $template;
}

##############################################################################
=pod

=head2 build_input_template

build_input_template - &build_input_template();

=over 4

=item DESCRIPTION

To create a row, with input locations for user input to set the value of the 
attribute.  Can be of type text, textarea, select, radio, or checkbox.

=item INPUT

$field - The name of the field to create this row for.  The attribute name

$row_or_cell - "row" to retrieve select template from the "rows" directory
               "cell" to retrieve select template from the "cells" directory

$site - A hashref containing information required throughout about the current
        site and session

$select_array - pointer to an array, to store all names of fields which are HTML select
                boxes, to flag that they need to be dynamically generated with the sel_file
                and the value

$checkbox_array - pointer to an array, to store all names of fields which are HTML checkboxes,
                  to flag that they need to go through initialize_checkboxes.

$flags -  Hashref with flags of information. Contains row number if this is a
          group of rows.

=item OUTPUT

$template - A row or cell of the $field requested in input format.  If it is a
            checkbox, then the $field is added to the array $checkbox_array. If it
            it is a select box, then the $field is added to the array $select_array.

=item NOTES

=back

=cut
###############################################################################
sub build_input_template {
    my ($field) = shift;
    my ($row_or_cell) = shift;
    my ($site) = shift;
    my ($select_array) = shift;
    my ($checkbox_array) = shift;
    my ($flags) = shift;

    my ($html_options, $type, @definitions, $filename, $template, $definition, $def_tag, $def_val, $replace);

    # If it is a placeholder, then just put in a place holder
    if ($field =~ /^<_/) {
	$field = &get_place_holder_tag($field);
	return "__" . $field . "__";
    }

    $html_options = $site->{'html_def'}->{$field};
    if (!defined $html_options) {
	return (&build_display_template($field, $site));
    }

    ($type, @definitions) = split /\:/, $html_options;
    $filename = $row_or_cell . "_" . $type;
    $template = &localize_template($filename, "admin", 0, $site);
    $replace = "__" . $type . "__";

    $field = &get_row_number($field, $flags);

    $template =~ s/$replace/$field/gms;  
    foreach $definition (@definitions) {
	($def_tag, $def_val) = split /\=/, $definition;
	$def_val =~ s/\'//gms;
	$def_val =~ s/\"//gms;
#	$def_val = &get_row_number($def_val, $flags);
	$replace = "__" . $def_tag . "__";
	$template =~ s/$replace/$def_val/gms;
    }
    
    # store the sel filename in order to generate the select for it
    if (($type eq "select") && ($definitions[0] !~ /^<_/)) {
	push @$select_array, $definitions[0];
    }

    if (($type eq "checkbox") && ($definitions[0] !~ /^<_/)) {
	push @$checkbox_array, $definitions[0];
    }


    return $template;
}

##############################################################################
=pod

=head2 get_row_number

get_row_number - &get_row_number($field, $flags);

=over 4

=item DESCRIPTION

To surround a field name by unique row properties.  If it is an exisitng row,
use the object ID as part of the row tag.  If it is a new one, use a unique
number.  If it is not part of multiple rows, just use the fieldname.

=item INPUT

$fieldname - The name of the field to build the row key for

$flags - If ->{'row_number'} is specified, then use this to build the $field
         if ->{'objid'} is specified, then this is an existing object
    
=item OUTPUT

$field - The field by itself if this is not part of multiple rows.
         The field with row.<objid>.field if it is part of multiple rows and is
             already has an object (
         The field with row.<row_number>.field if it is a brand new row among 
         multiple rows

=item NOTES

=back

=cut
###############################################################################
sub get_row_number {
    my ($field) = shift;
    my ($flags) = shift;

    if ($flags->{'row_number'} ne "") {
	if ($flags->{'objid'} ne "") {
	    $field = "row\." . $flags->{'objid'} . "." . $field;
	}
	else {
	    $field = "row.row" . $flags->{'row_number'} . "." . $field;
	}
    }

    return $field;
}
##############################################################################
=pod

=head2 load_object_template_def

load_object_template_def - &load_object_template_def();

=over 4

=item DESCRIPTION

Since templates are dynamically generated based on objclass, objtype, 
objid or a specific select file name, we must determine which of these 
methods is appropriate each time that we need to load the object template.  
This subroutine opens the sel definition file, and stores it in an array in 
the order that it is to be rendered.  Any late binding attributes are stored 
in a file sel_$objid_gui for the specific objid, and are loaded in after all 
other gui definitions are loaded in.   From here, the entire definition of 
all of the attributes that are supposed to appear in one form or another is 
loaded into memory and returned to the calling routine.

=item INPUT

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$attributes - Pointer to array where the output will be stored

=item OUTPUT

$attributes		Array of strings, each representing one row of the
			objects definition.  Each row can be composed 
			of any of the following:
				<row_start> - indicate that this is the start of
						a repeating
				<row_end> - indicate that this is the end of 
						a repeating row
				<next_row> - indicate we need a row break, but don't 
                                             change row key
				<field>|<tag> - <field> is the database field
	    					  that the data is for input, and
					             <tag> is what we see on the display

=item NOTES

=back

=cut
###############################################################################
sub load_object_template_def {
    my ($object_inf) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($attributes) = shift;

    my ($filename, @gui_def, $indx, $line, $field, $title, @WHERE, $where_indx);
    my ($num_attr, $next_attr, @attributes, $start, $end, $attribute_hashref, $all_attributes);
    my ($count);

    if ($object_inf->{'gui_name'} ne "") {
	$filename = "<$site->{'selskeletonpath'}/$object_inf->{'gui_name'}.gui";
	$filename =~ s/ //gms;
	$filename =~ tr/A-Z/a-z/;
	&debug("load_object_template_def: gui_name: Opening $filename.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	open FH, $filename;
    }
    elsif ($object_inf->{'objtype'} ne "") {
	$filename = "<$site->{'selskeletonpath'}/$object_inf->{'objtype'}_gui.gui";
	$filename =~ tr/A-Z/a-z/;
	&debug("load_object_template_def: objtype:  Opening $filename.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	open FH, $filename;
    }

    elsif ($object_inf->{'objclass'} ne "") {
	$filename = "<$site->{'selskeletonpath'}/$object_inf->{'objclass'}_gui.gui";
	$filename =~ tr/A-Z/a-z/;
	&debug("load_object_template_def: objclass: Opening $filename.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	open FH, $filename;
    }

    @gui_def = <FH>;
    close(FH);

    $indx = 0;
    foreach $line (@gui_def) {
	$line =~ s/\r\n//gms;
	$line =~ s/\n//gms;
	if ($line) {
	    $$attributes[$indx++] = $line;
	}
    }

    @gui_def = undef @gui_def;
   # Get all attached object gui information
    if (defined $object_inf->{'objid'}) {
	&get_attached_attributes($object_inf, $indx, $attributes, $site, $dbh);
    }
}

##############################################################################
=pod

=head2 get_attached_attributes

get_attached_attributes - &get_attached_attributes($object_inf, 3, \@attributes, $site, $dbh);

=over 4

=item DESCRIPTION

Since any object can have attached attributes, it is necessary to retreive the
attached attributes for each object upon render time.  Therefore, it goes into
the ~conf/attached directory and reads the attached attribute file for the
object if one exists.  Then, it queries the attributes and determines what the 
best tag is to be displayed for the attribute based on the definition of the 
'most important language'.  It builds the line in the value|tag format which is 
expected and implemented in the sel_ files.  

=item INPUT

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$indx - index to start adding the attached attributes to.  All of the main attributes
        are in the attributes array, so start adding after them

$attributes - pointer to an array to populate with attached attributes

$site - A hashref containing information required throughout about the current
        site and session

$dbh - Current active database handler

=item OUTPUT

$attributes is populated with the attached attributes

=item NOTES

=back

=cut
###############################################################################
sub get_attached_attributes {
    my ($object_inf) = shift;
    my ($indx) = shift;
    my ($attributes) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
	

    my ($filename, @gui_def, $where_indx, $count, $line, $start, $end, @WHERE, $where_indx);
    my ($num_attr, $next_attr, @attributes, $all_attributes, $field, $num_gui, @gui, $i);

    $filename = "<$site->{'attach_path'}/$object_inf->{'objid'}_gui.gui";
    $filename =~ tr/A-Z/a-z/;
    &debug("get_attached_attributes: Opening $filename.", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
    if (open ATTACH_FH, $filename) {
	@gui_def = <ATTACH_FH>;
	close(ATTACH_FH);
    }
    else {
	return;
    }
    
    return if ($#gui_def < 0);

    $where_indx = $count = 0;
    $num_gui = $#gui_def + 1;
    foreach $line (@gui_def) {
	$line =~ s/ //gms;
	$line =~ s/\n//gms;
	next if ($line eq "");
	$gui[$i] = $gui_def[$i++];
    }
    
    foreach $line (@gui) {
	$count++;
	&debug("get_attached_attributes: where_indx is $where_indx, count is $count, num_gui is $num_gui", 3, $site->{'debug_level'}, $site->{'debug_log'});
	if ($where_indx == 0) {
	    $WHERE[$where_indx++] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'attribute'}, "and");
	    $start="(";
	    &debug("get_attached_attributes: start is $start", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
	
	if ($count == $num_gui) { # last one, so close it 
	    $end = ")";
	}
	else {
	    $end = "";
	}
	    
	if ($line) {
	    $field = $start . "title";
	    # only use the first time
	    if ($start eq "(") {
		$start = "";
	    }
	    $line = $line . $end;
	    &debug("get_attached_attributes: Adding $field = $line to where clause", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $WHERE[$where_indx++] = &build_where_hash($field, $line, "or");
	}
    }

    if ($#gui_def >= 0) {
	# load in the attributes so that we know their titles
	($num_attr, $next_attr, @attributes) = &load_objects("things", "title,brief,objid", \@WHERE, "", "", $site, $dbh, $site->{'override_permissions'});
	$all_attributes = &format_array_into_hashref("title", \@attributes);
	
	# build the attribute list for this language
	foreach $field (@gui_def) {
	    $field =~ s/\n//gms;
	    $field =~ s/ //gms;
	    next if ($field eq "");
	    $line = $field . "|" . $all_attributes->{$field}->{'brief'};
	    &debug("get_attached_attributes: Added $line", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $$attributes[$indx++] = $line;
	}
    }
}

##############################################################################
=pod

=head2 age_form

age_form - &age_form(\%FORM, $site);

=over 4

=item DESCRIPTION

This routine prepares the FORM for the next submition.  After a valid FORM is
submitted, and the routines have executed, all of the input fields must be copied
to the _pre fields, because right after this when we re-display the form, the 
current input is the previous data.  

=item INPUT

$FORM - pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$FORM with all _inp values copied into the _pre values

=item NOTES

=back

=cut
###############################################################################
sub age_form {
    my ($FORM) = shift;
    my ($site) = shift;

    my ($aged_form, $field, $error, $previous, $input, $field_key);
    &debug("In age form!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($FORM, 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $field_key (keys %{$FORM}) {
	# Clear out the setting of a variant product so we can use it later.
	if (($field_key =~ /axis/) && ($field_key =~ /_pre/)) {
	    $field = $field_key;
	    $field =~ s/_pre//gms;
	    $error = $field . "_err";
	    $input = $field . "_inp";
	    $previous = $field . "_pre";
	    $aged_form->{$input} = "";
	    $aged_form->{$previous} = "";
	    next;
	}

	next if (($field_key !~ /_inp$/) || ($field_key =~ /axis/));
	$field = $field_key;
	$field =~ s/_inp//gms;
	$error = $field . "_err";
	$input = $field . "_inp";
	$previous = $field . "_pre";

	&debug("age_form: Looking at $field, is there error $$FORM{$error}.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&debug("age_form: \tuses: $input, $previous, $error.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	# there was not an error in this field, so set the previous value to be the input so it will display in the next generate
	if (($$FORM{$error} eq "") ||
	    (($input eq "objtype_inp") || ($input eq "objclass_inp"))) {   # NEVER overwrite the class or type
	    &debug("age_form: Setting $previous to $$FORM{$input}.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $aged_form->{$previous} = $$FORM{$input};
	}
	else {
		$aged_form->{$input} = $$FORM{$previous};  # There was an error, so set it back to the previous value
	}
    }

    return $aged_form;
}

##############################################################################
=pod

=head2 set_links

set_links - &set_links($object_inf, \%FORM, $template, $site);

=over 4

=item DESCRIPTION

Various links are dynamically generated, on the spine in the base product.  These
links help ease navigation throughout the administrator.  This routine uses the 
information in $object_inf and FORM to determine what links to display each time
this routine is called.  Only appropriate links are generated.

=item INPUT

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$FORM - pointer to FORM hash

$template - Template that was dynamically built

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$template - with links set in the spine

=item NOTES

=back

=cut
###############################################################################
sub set_links {
    my ($object_inf) = shift;
    my ($FORM) = shift;
    my ($template) = shift;
    my ($site) = shift;

    my ($price_link, $inventory_link, $search_link, $search_info, $new_link);
    my ($back_to_master, $variant_link, $attach_link, $back_to_product, $encrypt_sid);

    $encrypt_sid = &encrypt_sid($site->{'sid'}, $site);
    
    &debug("set_links:  got in here new object is $object_inf->{'new_object'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($object_inf, 3, $site->{'debug_level'}, $site->{'debug_log'});
	
    # Deal with new_object tag, clean it out when it is a variant product or when it is a new product
    # TARA changed to display_variants from display_variant
    if (($object_inf->{'new_object'} eq "yes") || ($object_inf->{'mode'} eq "home") 
	|| ($object_inf->{'mode'} eq "search") || ($object_inf->{'mode'} eq "display_variants") || ($$FORM{'mode'} eq "attach_detach")) {
	$template =~ s/__new_tag__//gms;
	$template =~ s/__new_link__//gms;
    }
    else {
	$new_link = "$site->{'scriptpath'}/$site->{'scriptname'}\?mode=display_object&new_object=yes&objtype_inp=$object_inf->{'objtype'}&objclass_inp=$object_inf->{'objclass'}";
	if ($$FORM{'came_from_search'} eq "yes") {
	    $new_link .= "&came_from_search=yes";
	}

	$template =~ s/__new_link__/$new_link/gms;
	$template =~ s/__new_tag__/$site->{'script_msg'}{'new_tag'}/gms;
    }
	
    # Deal with inventory and pricing links
    if (($$FORM{'objtype_inp'} ne $site->{'OBJTYPES'}->{'product'}) || ($object_inf->{'new_object'})) {
	$template =~ s/__inventory_link__//gms;
	$template =~ s/__inventory_tag__//gms;
	$template =~ s/__price_link__//gms;
	$template =~ s/__price_tag__//gms;
    }
    else {
	&debug("set_links:  build inventory & price  $$FORM{'objtype_inp'}, $$FORM{'objclass_inp'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$price_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editprice", $encrypt_sid, $site);
	$inventory_link = &build_inventory_link_info($FORM, $$FORM{'objid'}, "editsku", $encrypt_sid, $site);
	$template =~ s/__inventory_link__/$inventory_link/gms;
	$template =~ s/__inventory_tag__/$site->{'script_msg'}{'inventory_tag'}/gms;
	$template =~ s/__price_link__/$price_link/gms;
	$template =~ s/__price_tag__/$site->{'script_msg'}{'price_tag'}/gms;
    }


    # Deal with back to search links
    if ($$FORM{'came_from_search'} ne "") {
	$template =~ s/__back_to_search__/$site->{'script_msg'}{'back_to_search'}/gms;

	# Set temporarily to use build_admin_object_link_info
	my $pre_form_val = $$FORM{'came_from_search'};
	$$FORM{'came_from_search'} = "";

	$search_link = &build_admin_object_link_info($FORM,"","search", $encrypt_sid, $site);
	$$FORM{'came_from_search'} = $pre_form_val;

	&debug("set_links:  $$FORM{'search_objtype'}, $$FORM{'search_status'}, $search_link", 3, $site->{'debug_level'}, $site->{'debug_log'});

	
	$template =~ s/__search_link__/$search_link/gms;
    }
    else {
	# clear back to search
	$template =~ s/__back_to_search__//gms;
    }

    # Deal with link to variant products
    if (($object_inf->{'variants'} eq "")  && ($object_inf->{'master_objid'} ne "")) {
	$variant_link = &build_admin_object_link_info($FORM, $$FORM{'objid'}, "display_variants", $encrypt_sid, $site);
	$template =~ s/__variant_link__/$variant_link/gms;
	$template =~ s/__variant_tag__/$site->{'script_msg'}{'goto_variant'}/gms;
    }
    else {
	$template =~ s/__variant_tag__//gms;
    }

    # Deal with back to master product
    if ($object_inf->{'variants'} ne "") {
	$back_to_master = &build_admin_object_link_info($FORM, $$FORM{'master_objid'}, "display_object", $encrypt_sid, $site);
	$template =~ s/__back_to_master_link__/$back_to_master/gms;
	$template =~ s/__back_to_master_tag__/$site->{'script_msg'}{'back_to_master'}/gms;
    }
    else {
	$template =~ s/__back_to_master_tag__//gms;
    }


    # Deal with back to product from attach
    if ($$FORM{'mode'} eq "attach_detach") {
	# put the objtype & objclasses back in order to display the product properly.
	$$FORM{'objtype_inp'} = $$FORM{'pre_objtype'};	
	$$FORM{'objclass_inp'} = $$FORM{'pre_objclass'};
	$back_to_product = &build_admin_object_link_info($FORM, $$FORM{'objid'}, "display_object", $encrypt_sid, $site);
	$template =~ s/__back_to_product_link__/$back_to_product/gms;
	$template =~ s/__back_to_product_tag__/$site->{'script_msg'}{'back_to_product'}/gms;
	$$FORM{'objtype_inp'} = "";
	$$FORM{'objclass_inp'} = "";

    }
    else {
	$template =~ s/__back_to_product_tag__//gms;
    }


    # deal with attach link
    if ($object_inf->{'attach_link'} eq "yes") {
	$attach_link = &build_admin_object_link_info($FORM, $$FORM{'objid'}, "display_attach", $encrypt_sid, $site);
	&debug("set_links:  Setting attach link to $attach_link", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$template =~ s/__attach_link__/$attach_link/gms;
	$template =~ s/__attach_link_tag__/$site->{'script_msg'}{'attach'}/gms;
    }
    else {
	$template =~ s/__attach_link__//gms;
	$template =~ s/__attach_link_tag__//gms;
	
    }

    return $template;
}

##############################################################################
=pod

=head2 set_titles

set_titles - &set_titles($object_inf, \%FORM, $template, $site );

=over 4

=item DESCRIPTION

There are a lot of place holders for templates in the new template styles, as we
are trying to migrate all text out of the templates so that it is less effort
to translate into different languages.

This routine, dynamically sets the titles for displays in admin object.

=item INPUT

$object_inf - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$FORM - pointer to FORM hash

$template - Template that was dynamically built

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$template - with titles set

=item NOTES

=back

=cut
###############################################################################
sub set_titles {
    my ($object_inf) = shift;
    my ($FORM) = shift;
    my ($template) = shift;
    my ($site) = shift;


    my ($title, $desc, $edit, $master, $variants);

    if ($$FORM{'mode'} eq "attach_detach") {
	$title = $edit =  "attach_detach_title";
	$desc = "attach_detach_desc";
    }
    elsif ($object_inf->{'objtype'} ne "") {
	$title = $object_inf->{'objtype'} . "_title";
	$desc = $object_inf->{'objtype'} . "_desc";
	$edit = $object_inf->{'objtype'} . "_edit";
	$master = $object_inf->{'objtype'} . "_master";
	$variants = $object_inf->{'objtype'} . "_variants";
    }
    elsif ($object_inf->{'objclass'} ne "") {
	$title = $object_inf->{'objclass'} . "_title";
	$desc = $object_inf->{'objclass'} . "_desc";
	$edit = $object_inf->{'objclass'} . "_edit";
    }

    $template =~ s/__new_title__/$site->{'script_msg'}{$title}/gms;

    &debug("set_titles: FIELDS: $title, $desc, $master VALUES: $site->{'script_msg'}{$title}, $site->{'script_msg'}{$master}, $site->{'script_msg'}{$desc}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($object_inf->{'master_objid'} ne "") {
	# editing the variant products
	if ($object_inf->{'variants'} ne "") {
	    $edit = "$site->{'script_msg'}{$variants} $object_inf->{'master_objid'}";
	}
	# editing the master product
	else {
	    $edit = "$site->{'script_msg'}{$master} $object_inf->{'master_objid'}";
	}
	$template =~ s/__object_title__/$edit/gms;

    }
    elsif ($object_inf->{'new_object'}) {
	$template =~ s/__object_title__/$site->{'script_msg'}{$title}/gms;
    }
    else {
	# Editing Variant Products for master
	if ($object_inf->{'master_objid'} ne "") {
	    # editing the variant products
	    if ($object_inf->{'variants'} ne "") {
		$edit = "$site->{'script_msg'}{$variants} $object_inf->{'master_objid'}";
	    }
	    # editing the master product
	    else {
		$edit = "$site->{'script_msg'}{$master} $object_inf->{'master_objid'}";
	    }
	}
	else {
	    $edit = "$site->{'script_msg'}{$edit} $$FORM{'objid'}";
	}

	$template =~ s/__object_title__/$edit/gms;
    }

    $template =~ s/__admin__/$site->{'script_msg'}{4}/gms;
    $template =~ s/__admin_title__/$site->{'script_msg'}{5}/gms;
    $template =~ s/__admin_descript__/$site->{'script_msg'}{6}/gms;

    $template =~ s/__object_descript__/$site->{'script_msg'}{$desc}/gms;
    return $template;

}

##############################################################################
=pod

=head2 

initialize_selects - &initialize_selects(\%FORM, $object, $template, $site, \@select_array);

=over 4

=item DESCRIPTION

It is necessary to set the value of all HTML select fields that show up on a 
display.  This routine takes an array of field names which are displayed on the
FORM as selects.  It takes the values from the object defintion and sets the
value of the field to SELECTED if it is the selected value.

=item INPUT

$FORM - Pointer to FORM hash

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$object_hashref - Contents of the object from the database, with all information
                  needed for permuting at the main level.  Use this to set the
                  value of the checkboxes.

$site - A hashref containing information required throughout about the current
        site and session

$p_select_array - name of all fields that are selects and need initialized.

=item OUTPUT

All of the $FORM _inp fields are set to the SELECT file that is generated for the
fields specified in $p_select_array

=item NOTES

=back

=cut
###############################################################################
sub initialize_selects {
    my ($FORM) = shift;
    my ($object_info) = shift;
    my ($object_hashref) = shift;
    my ($site) = shift;
    my ($p_select_array) = shift;


    my ($sel_file, $fieldname, @raw, $sel_tag_file, $select_pre, $select_input, $select_tag);

    foreach $sel_file (@$p_select_array) {
	&debug("initialize_selects: $sel_file to open", 3, $site->{'debug_level'}, $site->{'debug_log'});
        open(FH, "<$site->{'selskeletonpath'}/$sel_file.sel");
        @raw = <FH>;
        chomp @raw;
        close(FH);
        
        # Make the sel_ files work with the automatic generation
        if ($sel_file =~ /sel_/) {
	    $select_tag = $sel_file;
            $select_tag =~ s/sel_//gms;
	    $select_tag = &get_row_number($select_tag, $object_info);
	    &debug("initialize_select: select tag is $select_tag", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $fieldname = $select_tag;
	    $select_tag .= "_SELECT";
	}

	$select_input = $fieldname . "_inp";
	$select_pre = $fieldname  . "_pre";
	&debug("initialize_selects: $select_tag, $select_input, $select_pre", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$$FORM{$select_tag} = &generate_select($object_hashref->{$select_pre}, \@raw);
	&debug("initialize_selects: Just set $$FORM{$select_tag} at $select_tag", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
}

##############################################################################
=pod

=head2 initialize_dynamic_selects

initialize_dynamic_selects - &initialize_dynamic_selects("objid", "title", "3-AA", "things", \@where, $site, $dbh);

=over 4

=item DESCRIPTION

To build a HTML SELECT object, setting the initial value to $default.  The select
data is not retreived from a _sel file however, it is data that is retrieved from
the database.  This can be used to initialize selects which change on a language by
language basis, and as well as for attached attributes.

=item INPUT

$value_field - Name of the field to retreive the data for for the left hand side of the |

$tag_field - Name of the field to retreive the data for for the right hand side of the |

$object_value - value to use to set the default value of the select to

$tables - table to query to get the data from

$where - where clause to use in the query

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$select_html - A HTML SELECT object, created by $p_contents with the value set
               to $default.

=item NOTES

=back

=cut
###############################################################################
sub initialize_dynamic_selects {
    my ($value_field) = shift;  
    my ($tag_field) = shift;  
    my ($object_value) = shift;    
    my ($tables) = shift;         
    my ($where) = shift;          
    my ($site) = shift;          
    my ($dbh) = shift;

    my (@results, $q, $num_rows, $sel_file,  @raw, $sel_tag_file, $select_pre, $select_input, $select_tag, $result);
    my ($fields, $next_start, $select, $line);

    $fields = $value_field. "," . $tag_field;
    ($num_rows, $next_start, @results) = &load_objects($tables, $fields, $where, "", "", $site, $dbh, $site->{'override_permissions'});
    foreach $result (@results) {
	$line = $result->{$value_field} . "|" . $result->{$tag_field} . "\r\n";
	push @raw, $line;
    }
    
    $select = &generate_select($object_value, \@raw);
    return $select;
}

##############################################################################
=pod

=head2 initialize_checkboxes

initialize_checkboxes - &initialize_checkboxes(\%FORM, $object, $template, $site, \@checkbox_array);

=over 4

=item DESCRIPTION

It is necessary to set the value of all check box fields that show up on a 
display.  This routine takes an array of field names which are displayed on the
FORM as checkboxes.  It takes the values from the object defintion and sets the
checkbox to CHECKED if required.

=item INPUT

$FORM - Pointer to FORM hash

$object_hashref - Contents of the object from the database, with all information
                  needed for permuting at the main level.  Use this to set the
                  value of the checkboxes.

$template - The template that was dynamically build to this point.  Will replace the
            tags with CHECKED where the checkbox is supposed to be checked.

$site - A hashref containing information required throughout about the current
        site and session

$p_checkbox_array - name of all fields that are checkboxes and need initialized.

=item OUTPUT

$template - Template with all check box values CHECKED that are supposed to be
            according the the $object_hashref

=item NOTES

=back

=cut
###############################################################################
sub initialize_checkboxes {
    my ($FORM) = shift;
    my ($object_hashref) = shift;
    my ($template) = shift;
    my ($site) = shift;
    my ($p_checkbox_array) = shift;


    my ($field, $input_tag);

    &debug("initialize_checkboxes:  In here", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($object_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $field (@$p_checkbox_array) {
	$field =~ s/name=//gms;
	$input_tag = $field;
	$field =~ s/_inp//gms;
	$field .= "_pre";
	&debug("initialize_checkboxes:  Checking $field, object has $object_hashref->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	if ($object_hashref->{$field} == 1) {
	    $input_tag  = "__" . $input_tag . "__";
	    $template =~ s/$input_tag/ CHECKED/gms;
	    &debug("initialize_checkboxes:  Just set $input_tag in template to CHECKED\n$template", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }

    return ($template);
}

##############################################################################
=pod

=head2 generate_select

generate_select - &generate_select("15000-A", \@status_info);

=over 4

=item DESCRIPTION

To build a HTML SELECT object, setting the initial value to $default.

=item INPUT

$default - The default value to set the select to.  This is the value that is
           in the database, or a default value to set it to.

$p_contents - Pointer to an array, each is a string in the format "key|tag"
              where key is the value to store in the database, and tag is the 
              text shown on the browser for the key              

=item OUTPUT

$select_html - A HTML SELECT object, created by $p_contents with the value set
               to $default.

=item NOTES

$p_contents can be read from a sel_ file, or dynamically generated.

=back

=cut
###############################################################################
sub generate_select {
    my ($default) = shift;
    my ($p_contents) = shift;

    my ($select_html, $default_set_flag, $row, $value, $key);

    $select_html="";
    $default_set_flag=1;
    foreach $row (@$p_contents) {
        ( $key,$value )  = split /\|/, $row;
        $select_html.= "<OPTION VALUE=$key ";  
        
        if (( $default ) && ( $default_set_flag )) {
            if ( "$default" eq "$key" ) {
                $select_html.="SELECTED $key = $default";
                $default_set_flag=0;
            }
        }
        $select_html.="> $value \n";
    }

    return $select_html;
}


################################### VARIANT PRODUCT SUBROUTINES #####################################
##############################################################################
=pod

=head2 saveMasterProducts

saveMasterProducts - &saveMasterProducts($results, \%MasterRec, $site); 

=over 4

=item DESCRIPTION

Save a hash of objids for all master products that have matched at least one search keyword.  Used
later to get the variant products for the master and verify if they too meet the search criteria.

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$MasterRec - pointer to the current hash of master objids.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$master_matched - 0 if no master products matched, 1 if master product matched

%MasterRec - hash of objids, which have been flagged as master records in filterResults

=item NOTES

=back

=cut
##############################################################################
sub saveMasterProducts {
    my ($results) = shift;
    my ($MasterRec) = shift;
    my ($site) = shift;

    my ($master_matched, $obj, $h);

    $master_matched = 0;  # assume no masters matched.

    # Save all master records that match
    foreach $obj (keys %{$results->{"things"}}) {
        $h = $results->{"things"}->{$obj};
        if (defined $h->{'addvariants'}) {
	    &debug("Saving Master product $h->{'objid'} as it matched",3, $site->{'debug_level'}, $site->{'debug_log'});
            $$MasterRec{$h->{'objid'}} = $h;
            $master_matched = 1;
        }
    }
    return ($master_matched, %{$MasterRec});
}

##############################################################################
=pod

=head2 findMatchedExtraprop

findMatchedExtraprop - &findMatchedExtraprop($dbh, %extraprop, \@results, $keyword, $site);

=over 4

=item DESCRIPTION

Find all encodings that match the keywords, and then search for the variant products

=item INPUT

$dbh - current active database handle

$extrarprop_results - The ->{'extraprop'} portion of results.  Only the body matchings for encodings

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search

$keyword - The current keyword to find the variant products for

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$results - updated with results

=item NOTES

THIS WILL NEED TO BE EXPANDED TO INCLUDE ALTTITLE

=back

=cut
##############################################################################
sub findMatchedExtraprop {
    my $dbh = shift;
    my $keyword = shift;
    my $extraprop_results = shift;
    my $results = shift;
    my $site = shift;
    
    my ($h, $objid, $numRows, $numRowsAdded, $indx, $numExtrapropRecs);
    
    &debug("In findMatchedExtraprop with $keyword", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $indx = $numExtrapropRecs = $numRowsAdded =  0;
    foreach $objid (keys %{$extraprop_results}) {
        $h = $extraprop_results->{$objid};
    }
 
}

##############################################################################
=pod

=head2 getVariantProducts

getVariantProducts - &getVariantProducts($dbh, $site, \%MasterRec, \@results);

=over 4

=item DESCRIPTION

To  add all of the variant products for master products
that have matched the search criteria in one form or another.
This is because the variant products inherit all searchable
fields from the master product

=item INPUT

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session


%MasterRec - hash of master records that have matched the
              search criteria in one form or another

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search

=item OUTPUT

$results - Pointer to an array, the results are returned.
           The current set of results, up to this point. An array of  hashrefs of information,
           one for each record that matched the search.  Updated with the variant products

=item NOTES

=back

=cut
##############################################################################
sub getVariantProducts {
    my ($dbh) = shift;
    my ($site) = shift;
    my ($MasterRec) = shift;
    my ($results) = shift;
    
    my ($q4, $sth4, $key, $q4num, $x, $h4, $objtype, $select_hashref);
    
    my ($i, $where_indx);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("ref_from", "?", "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid" , "!?", "and");
    $select_hashref->{'statement'} = "SELECT";
    $q4 = &build_select($select_hashref, $site, $dbh, 1);

    $sth4=$dbh->prepare( $q4 );
    &debug("$q4",2,$site->{'debug_level'}, $site->{'debug_log'});
    &debug("Finding all variant products for master products that matched the search",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    $i = 0;
    foreach $key (keys %{$MasterRec}) {
        $sth4->execute($key, $key);
        while ($$results[$i] = $sth4->fetchrow_hashref) {
            &debug("Variant to add:  $$results[$i]->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
            $i++;
        }
    }
    &debug("Returning $i rows",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $i;
}

################################ DEBUG PRINTING SUBROUTINES #########################################
##############################################################################
=pod

=head2 print_results

print_results - &print_results($site, $results, $numrows, "objid,alttitle");

=over 4

=item DESCRIPTION

Output the results of the search to the debug log if the debug level is set.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$results - The current set of results, up to this point. A hashref of information, initially 
           seperated by tables names, and then by objid, and then the contents of the record,
  	   of records that met the search criteria  

$numrows - The total number of rows found in the search

$fields_to_print - A list of fieldnames which are to be printout out

=item OUTPUT

Outputs the results to the debug log

=item NOTES

=back

=cut
##############################################################################
sub printResults {
    my ($site) = shift;
    my ($results) = shift;
    my ($numrows) = shift;
    my ($fields_to_print) = shift;
    
    my ($table, $obj, $h, $fld);
    
    &debug("\nFOUND $numrows ROWS:",3,$site->{'debug_level'}, $site->{'debug_log'});
    foreach $table (keys %{$results}) {
        &debug("Table is $table",3,$site->{'debug_level'}, $site->{'debug_log'});
        foreach $obj (keys %{$results->{$table}}) {
            foreach $fld (keys %{$results->{$table}->{$obj}}) {
                if ((!$fields_to_print) || ($fields_to_print =~ $fld)){
                    &debug("Found $fld: $results->{$table}->{$obj}->{$fld}",3,$site->{'debug_level'}, $site->{'debug_log'});
                }
            }
        }
    }
}

##############################################################################
=pod

=head2 printQueries

printQueries - &printQueries($site, @search_queries);

=over 4

=item DESCRIPTION

Output the hashref information that will be used to create the select queries to
the debug log if the debug level is set.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

@search_queries - Array of hashrefs consisting of information required to build a 
        select query.

=item OUTPUT

Outputs the query configuration to the debug log

=item NOTES

=back

=cut
##############################################################################
sub printQueries {
    my ($site) = shift;
    my (@search_queries) = @_;
    my ($x, $h, $fld, @where_array, $where_hash);
    
    for ($x=0; $x <= $#search_queries; $x++) {
        $h = $search_queries[$x];
        &debug("Table: $h->{'TABLE'}->{'NAME'}",3,$site->{'debug_level'}, $site->{'debug_log'});
        
	if (defined $h->{'TABLE'}->{'WHERE'}) {
	    @where_array = $h->{'TABLE'}->{'WHERE'};
	    foreach $where_hash (@where_array) { 
		&debug("WHERE: $where_hash->{'field'}, $where_hash->{'value'}, $where_hash->{'and_or'}",3,$site->{'debug_level'}, $site->{'debug_log'});
	    }
	}

        foreach $fld (keys %{$h->{'TABLE'}->{'FIELDS'}}) {
            &debug("WHERE: $fld --> $h->{'TABLE'}->{'FIELDS'}->{$fld}",3,$site->{'debug_level'}, $site->{'debug_log'});
            
        }
    }
}

##############################################################################
=pod

=head2 printSearchArray

printSearchArray - &printSearchArray($site, @search_queries);

=over 4

=item DESCRIPTION

Output the results found from the search after they have been accummulated into
an array for search.cgi

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

@array - Array of hashrefs consisting of information required to build a 
        select query.

=item OUTPUT

Outputs the information from the search query (the information that was converted
into an array datastructure)

=item NOTES

=back

=cut
##############################################################################
sub printSearchArray {
    my ($site) = shift;
    my (@array) = @_;
    my ($i, $fld, $numResults);
    
    $numResults = $#array + 1;
    &debug("Found $numResults entries!",1,$site->{'debug_level'}, $site->{'debug_log'});
    
    for ($i = 0; $i<= $#array; $i++) {
        foreach $fld (keys %{$array[$i]}) {
            if ($array[$i]->{$fld}) {
                &debug("$fld - $array[$i]->{$fld}",1,$site->{'debug_level'}, $site->{'debug_log'});
            }
        }
    }
}

#********************************** ATTACH/DETACH stuff *************************************************8
##############################################################################
=pod

=head2 update_late_binding_attributes

update_late_binding_attributes - &update_late_binding_attributes();

=over 4

=item DESCRIPTION

To take all attributes that were selected in the Attach Attributes SELECT box prior 
to pushing the update button, and add them onto this object.  Take all attributes 
that were selected in the Detach Attributes SELECT box prior to pushing the update 
button, and remove them from the object. It builds the sel_objid_gui.sel for the 
object defined and stores it in the ~/conf/attached directory.

Since the tag to show up on a display is different on a language by language
basis, and since the tag should be chosen for the most appropriate language 
(om_render_lang if defined, otherwise the 'most important language', the only
information that is stored in the attached file is the field name.  This allows
us to attach attributes to objects regardless of language.  Then, the tag name
is dynamically generated based on the best fitting language defined.

=item INPUT

$FORM -	Pointer to the FORM from the browser

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

sel_$objid_gui.sel file is updated in the ~/conf/attached directory.

=item NOTES

=back

=cut
###############################################################################
sub update_late_binding_attributes {
    my ($FORM) = shift;
    my ($object_info) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@attach, @detach, $attribute, $type, $tmp, @each);
    my (@WHERE, $where_indx, $num_attributes, $next_attribute, @attributes, $all_attributes);
    my ($filename, @attached_already);
    my ($default_lang, $attach_hashref, $detach_hashref, $attach_fields);
    my ($row, $field, $values, $junk, $objid);
    my ($write_to_attach);

    &debug("update_late_binding_attributes: in here, showing FORM next!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($FORM, 3,  $site->{'debug_level'}, $site->{'debug_log'});


    # split based on the multiple entry (as defined by cgi-lib.pm
    @attach = split /\0/, $$FORM{'attach_inp'};
    @detach = split /\0/, @$FORM{'detach_inp'};
    $attach_hashref = &format_array_into_hashref("", \@attach);
    $detach_hashref = &format_array_into_hashref("", \@detach);

    foreach $attribute (@attach) {
	&debug("update_late_binding_attributes: attach $attribute!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    foreach $attribute (@detach) {
	&debug("update_late_binding_attributes: detach $attribute!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    $objid = lc($$FORM{'objid'});

    # get the object file if it exists and read in everything
    $filename = "$site->{'attach_path'}/$objid" . "_gui.gui";
    if (!open(TMP, "<$filename")) {
	&debug("update_late_binding_attributes: Can't open filename $filename!", 3, $site->{'debug_level'}, $site->{'debug_log'});	
    }
    else {
	@attached_already = <TMP>;
	close(TMP);
    }

    &debug_hashref($detach_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    $write_to_attach = "";
    if ($#attached_already >= 0){
        # Go through and see if we need to detach the value
	foreach $field (@attached_already) {
	    $field =~ s/\n//gms;
	    $field =~ s/ //gms;
	    &debug("update_late_binding_attributes: Checking $field, $detach_hashref->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    next if (defined $detach_hashref->{$field});  # don't write the detached items
	    &debug("Need to keep: .$field.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $write_to_attach .= "$field\n";
	}
    }

    &debug("After detached: $write_to_attach", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # Get ALL of the attributes
    $where_indx = 0;
    $WHERE[$where_indx++] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'attribute'}, "and");
    ($num_attributes, $next_attribute, @attributes) = &load_objects("things", "title,brief,objid", \@WHERE, "", "", $site, $dbh, $site->{'override_permissions'});
    $all_attributes = &format_array_into_hashref("title", \@attributes);

    if ($#attach >= 0) {
	# print in all of the additional attached items
	foreach $field (@attach) {
	    $field =~ s/\n//gms;
	    $field =~ s/ //gms;
	    $write_to_attach .= "$field\n";
	}
    }
    &debug("After attached: $write_to_attach", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if (!open(TMP, ">$filename")) {
	&debug("update_late_binding_attributes: Can't open filename $filename!", 3, $site->{'debug_level'}, $site->{'debug_log'});	
    }
    &debug("update_late_binding_attributes: WROTE $write_to_attach to $filename!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    
    print TMP "$write_to_attach\n";
    close(TMP);
    
    return;
}

##############################################################################
=pod

=head2 update_html_conf

update_html_conf - &update_html_conf("$site->{'relpath'}/conf/html_def.pm", "new_field", "select|sel_new_field");

=over 4

=item DESCRIPTION

When a new attribute is created, we need to know what the attribute is supposed 
to look like when it is rendered, so we need to update the conf file which has 
this information in it.  This routine writes out a new line line to the end of
the file for the newly attached attribute, in code format.

It also removes old entries, when an attribute name changes, and replaces them
with the new attribute name.  In general, it is not a good idea to rename
attributes, especially after they have been populated in any objects, as then the
inforamtion will not be displayed on the objects that it is attached to.

=item INPUT

$filename - Name of the file, full path and all to the html.conf file

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

    $object_info->{'rm_field'} is set to the name of fields that are to be removed from
     the html_def.pm file.  They are to be removed, if the attribute is renamed for example.

    $object_info->{'field'} is set to the name of the new field that is to be added to the file.

    $object_info->{'html'} is set to the html string for the new field.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

Re-writes the html_def.pm file.

=item NOTES

=back

=cut
###############################################################################
sub update_html_conf {
    my ($filename) = shift;
    my ($object_info) = shift; 
    my ($site) = shift;

    my ($html_def, $rewrite_file, $field, $html);

    $rewrite_file = 0;

    $field = $object_info->{'field'};
    $html = $object_info->{'html'};
    &debug("update_html_conf: In with $filename, $field, $html", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $html_def = &set_html_def();
    &debug_hashref($html_def, 3, $site->{'debug_level'}, $site->{'debug_log'});


    &debug("update_html_conf: Value of field is $html_def->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if (!defined $html_def->{$field}) {
	$html_def->{$field} = $html;
	&debug("update_html_conf: Set rewrite_file because not defined", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$rewrite_file = 1;
    }
    else {
	if ($html_def->{$field} ne $html) {
	    $html_def->{$field} = $html;
	    &debug("update_html_conf: Set rewrite_file because not equal", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    $rewrite_file = 1;
	}
    }

    if ($rewrite_file == 1) {
	&debug("update_html_conf: should have added $html_def->{$field} for $field", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&write_html_conf($filename, $html_def, $object_info, $site);
    }    
}

##############################################################################
=pod 

=head2 write_html_conf

write_html_conf - &write_html_conf($filename, $site->{'html_def'}, $object_info, $site);

=over 4

=item DESCRIPTION

To re-write the file consisting of the subroutine which loads the html_def
into memory.  This file must be re-written so that it is code (formatted
as a subroutine).  It removes fields in the 'rm_field' key in $object_hash
and adds 'field' = 'html' from the $object_hash.

=item INPUT

$filename - Name and path of the html configuration file name.  In the main
            product, it is ~conf/html_def.pm

$html_def - Hashref from site, with the existing html_def

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

Writes out the subroutine set_html_def which re-writes the html_def.pm library routine
that loads in the html configuration data for attributes.

=item NOTES

=back

=cut
###############################################################################
sub write_html_conf {
    my ($filename) = shift;
    my ($html_def) = shift;
    my ($object_info) = shift;
    my ($site) = shift;

    my ($field, $subroutine, $rm_field);
    
    &debug("write_html_conf: Opening $filename to write", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $rm_field = "";
    if (defined $object_info->{'rm_field'}) {
	$rm_field = $object_info->{'rm_field'};
    }

$subroutine = "sub set_html_def {
\tmy (\$h);
\t\$h = {};\n";
	
    foreach $field (keys %{$html_def}) {
	next if ($field eq $rm_field);
	$subroutine .= "\t\$h->{'$field'} = \"$html_def->{$field}\";\n";
    }

    $subroutine .= "\treturn \$h;
}
1;\n";

    &debug("write_html_conf: New items to write: \n $subroutine", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if (!open (FH, ">$filename")) {
	&debug ("Couldn't open file $filename!!!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    else {
	print FH $subroutine;
	close(FH);
    }
}

##############################################################################
=pod  

=head2 update_attribute_html_list 

update_attribute_html_list - &update_attribute_html_list(\%FORM $site);

=over 4

=item DESCRIPTION

When a new attribute is created, we need to know what the attribute is supposed 
to look like when it is rendered, so we need to update the conf file which has 
this information in it.  This routine sees if this was an attribute 
creation/modification, and if it is, then it calls the routine to update it.

=item INPUT

$input_hashref - Pointer the the FORM hashref

$site - A hashref containing information required throughout about the current
        site and session


=item OUTPUT

If we just created a new attribute then add the definition to html_def.pm in conf
directory, if we updated it, then update the information in html_def.pm.

=item NOTES

=back

=cut
###############################################################################
sub update_attribute_html_list {
    my ($input_hashref) = shift;
    my ($site) = shift;

    my ($file, $object_info, $do_something);

    &debug("update_attribute_html_list: In here with title_inp: $$input_hashref{'title_inp'}, title_pre: $$input_hashref{'title_pre'}, html_inp: $$input_hashref{'html_inp'}, html_pre: $$input_hashref{'html_pre'}", 3, $site->{'debug_level'}, $site->{'debug_log'});


    if (($$input_hashref{'title_inp'} eq $$input_hashref{'title_pre'}) &&
	($$input_hashref{'html_inp'} eq $$input_hashref{'html_pre'})) {
	&debug("update_attribute_html_list: Nothing changed, just return.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    # title changed, so remove the old one and insert the new one
    if ($$input_hashref{'title_inp'} ne $$input_hashref{'title_pre'}) {
	$object_info->{'field'} = $$input_hashref{'title_inp'};
	$object_info->{'rm_field'} = $$input_hashref{'title_pre'};
    }
    else {  # just use the old title
	$object_info->{'field'} = $$input_hashref{'title_pre'};
    }

    # html changed, so remove the old one and insert the new one
    if ($$input_hashref{'html_inp'} ne $$input_hashref{'html_pre'}) {
	$object_info->{'html'} = $$input_hashref{'html_inp'};
    }
    else {
	$object_info->{'html'} = $$input_hashref{'html_pre'};
    }


    $file = $site->{'relpath'} . "/conf/html_def.pm";
    &debug("update_attribute_html_list: Going into update_html_conf", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &update_html_conf($file, $object_info, $site);
}

##############################################################################
=pod  

=head2 check_language_sensitive

check_language_sensitive - &check_language_sensitive(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

When an attribute is added to the system, it is possible for that attribute to
be lanaguage sensitive.  If it is, then that attribute needs to be flagged
that it can be defined in multiple languages.  If an attribute is added as
a language Sensitive field, it must be added to the database configuration
describing all language sensitive fields.  If the fieldname is changed,
then the language sensitive field must be updated.  If the field is rendered
non-language sensitive any longer, then it must be removed from the list.

=item INPUT

$input_hashref - Pointer the the FORM hashref

$site - A hashref containing information required throughout about the current
        site and session

$dbh - Current active database handler

=item OUTPUT

Extraprop record with fieldname ac_language_sens is updated appropriately.

=item NOTES

=back

=cut
###############################################################################
sub check_language_sensitive_fields {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($params);

    &debug("check_language_sensitive: In here with title_inp: $$input_hashref{'title_inp'}, title_pre: $$input_hashref{'title_pre'}, lang_sens: $$input_hashref{'lang_sens_inp'}, lang_sens_pre: $$input_hashref{'lang_sens__pre'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    # not language sensitive, and wasn't before this turn either, do nothing
    if (($$input_hashref{'lang_sens_inp'} == 0) && 
	(($$input_hashref{'lang_sens_inp'} eq $$input_hashref{'lang_sens_pre'}) || 
	 ($$input_hashref{'title_pre'} eq ""))) {
	&debug("check_language_sensitive: Nothing changed, just return.", 3, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    # either we are language sensitive now, for the first time OR
    # we were always language sensitive, but the fieldname changed.
    # or something else changed that we don't need to worry about

    $params->{'remove'} = "";
    $params->{'add'} = "";

    # fieldname changed, so remove the old one and insert the new one
    if ($$input_hashref{'title_inp'} ne $$input_hashref{'title_pre'}) {
	if ($$input_hashref{'title_pre'} ne "") {
	    $params->{'remove'} = $$input_hashref{'title_pre'};
	}
	$params->{'add'} = $$input_hashref{'title_inp'};
    }

    # language sensitive for the first time
    if (($$input_hashref{'lang_sens_inp'} == 1) && 
	(($$input_hashref{'lang_sens_pre'} == 0) || $$input_hashref{'lang_sens_pre'} eq "")) {
	if ($$input_hashref{'title_inp'} ne "") {
	    $params->{'add'} = $$input_hashref{'title_inp'};	
	}
	else {
	    $params->{'add'} = $$input_hashref{'title_pre'};	
	}
    }

    # used to be language sensitive, but now not, remove the title
    if (($$input_hashref{'lang_sens_inp'} == 0) && ($$input_hashref{'lang_sens_pre'} == 1)) {
	if ($$input_hashref{'title_inp'} ne "") {
	    $params->{'remove'} = $$input_hashref{'title_inp'};	
	}
	else {
	    $params->{'remove'} = $$input_hashref{'title_pre'};	
	}
    }

    &debug("check_language_sensitive: Going into update_language_sensitive_fields with add: $params->{'add'}, rm: $params->{'remove'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &update_language_sensitive_fields($params->{'add'}, $params->{'remove'}, $site, $dbh);
}

##############################################################################
=pod  

=head2 load_late_binding_attributes

load_late_binding_attributes - &load_late_binding_attributes();

=over 4

=item DESCRIPTION

It is necessary to provide a list of late binding attributes that a user can 
attach to an object.  Late binding attributes can be added to a object type 
or class by adding something to the _gui definition for the class or type.  This 
routine takes into consideration adding an attribute to an instance of a type, or 
in other words, to a specific object. 

=item INPUT

$object_info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - Current active database handler

=item OUTPUT

$already_has -	Hashref of the attributes that are currently displayed on the form

$can_detach -	Hashref of the attributes that are late binding attributes and can be detached.

$can_attach - Hashref of the other attributes that can be attached onto the object 
              that are not currently on the object

=item NOTES

=back

=cut
###############################################################################
sub load_late_binding_attributes {
    my ($object_info) = shift;
    my ($site) = shift;
    my ($dbh) = shift;


    my (@attributes, $tmp_info, $already_has, $can_detach, $can_attach);
    my (@attached, $attribute, $field, $junk);

    &debug("load_late_binding_attributes: In here with $object_info->{'objid'}, $object_info->{'objclass'}, $object_info->{'objtype'}, $object_info->{'sel_name'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $tmp_info->{'objid'} = $object_info->{'objid'};
    &load_object_template_def($tmp_info, $site, $dbh, \@attached);

    $tmp_info = {};
    $tmp_info->{'objclass'} = $object_info->{'objclass_to_attach'};
    $tmp_info->{'objtype'} = $object_info->{'objtype_to_attach'};
    $tmp_info->{'sel_name'} = $object_info->{'sel_name'};

    &load_object_template_def($tmp_info, $site, $dbh, \@attributes);

    &debug("load_late_binding_attributes: There are $#attached + 1 items attached", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # Attached attributes
    foreach $attribute (@attached) {
	&debug("load_late_binding_attributes: Check $attribute", 3, $site->{'debug_level'}, $site->{'debug_log'});
	next if ($attribute =~ /^</);  # skip if special purpose row
	($field, $junk) = split /\|/, $attribute;
	($field, $junk) = split /\:/, $field;
	next if ($field eq "");
	$can_detach->{$field} = $site->{'html_def'}->{$field};
	&debug("load_late_binding_attributes: Can detach $field, set it to $site->{'html_def'}->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    # Object class or type attribute definitions
    foreach $attribute (@attributes) {
	next if ($attribute =~ /^</);  # skip if special purpose row
	($field, $junk) = split /\|/, $attribute;
	($field, $junk) = split /\:/, $field;
	next if ($field eq "");
	$already_has->{$field} = $site->{'html_def'}->{$field};
	&debug("load_late_binding_attributes: Already has $field, set it to $site->{'html_def'}->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }


    foreach $field (keys %{$site->{'html_def'}}) {
	&debug("load_late_binding_attributes: What to do with $field? $already_has->{$field}, $can_detach->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	next if (defined $already_has->{$field});
	next if (defined $can_detach->{$field});
	$can_attach->{$field} = $site->{'html_def'}->{$field};
	&debug("load_late_binding_attributes: Can attach $field", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    return($already_has, $can_detach, $can_attach);
}
1;










