
# ****************************************    MOVE TO search.pm  **********************************************************************
##############################################################################
=pod

=head2 do_admin_object_search

do_admin_object_search - &do_admin_object_search(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Responsible for setting up search filters, performing the search based
on the FORM input, and displaying the results to the browser.

=item INPUT

$FORM - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

The search results are displayed if something was found, or returned to home
page with page error 'Nothing Found'.

=item NOTES

=back

=cut
###############################################################################
sub do_admin_object_search {
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($filters, @results, $link_details, $last_row);

    &debug("In do_admin_object_search", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $filters = &setup_search_filters($FORM);

    &debug("do_admin_object_search: after setup_search_filters, going into do_admin_search", 3, $site->{'debug_level'}, $site->{'debug_log'});
    @results = &do_admin_search($$FORM{'search_keywords'}, "and", "title", $filters, $FORM, $site, $dbh);

    &debug("do_admin_object_search: after do_admin_search", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $link_details = "sid=$$FORM{'sid'}&mode=search&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}";

    &debug("do_admin_object_search: startrow is $$FORM{'startrow'}, found $$FORM{'foundcount'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($$FORM{'foundcount'} > 0) {
	&display_search_results($link_details, $$FORM{'foundcount'}, $$FORM{'startrow'}, $$FORM{'next_startrow'}, $site, $FORM, \@results);
    }
    # Nothing found, go back home
    else {
	$$FORM{'page_error'} = $site->{'script_msg'}{'nothing_found'};
	$$FORM{'mode'} = "home";
    }
}

##############################################################################
=pod

=head2 setSearchNavigation

setSearchNavigation - &setSearchNavigation($pr, $foundcount, $maxrows, $startrow, $lastrow, $scriptinfo);

=over 4

=item DESCRIPTION

To set the Next and Back buttons on a search page

=item INPUT

$pr - location substring:
         sid=$sid&mode=$mode&skuskel=$objid&status_inp=$status_inp

$foundcount - total number of rows returned from the query

$maxrows - maximum number of rows to display on one page

$startrow - row to start displaying

$lastrow - last row to display

$scriptinfo - path the script and the scriptname: ex: /cgi-bin/om/search.cgi

=item OUTPUT

$back - the back link

$next - the next link

$firstrecord - the starting row for this page

=item NOTES

=back

=cut
##############################################################################
sub set_search_navigation {
    my ($pr) = shift;
    my ($foundcount) = shift;
    my ($maxrows) = shift;
    my ($startrow) = shift;
    my ($lastrow) = shift;
    my ($scriptinfo) = shift;
    
    my ($back, $next, $firstrecord, $this_start, $this_page, $startPages, $endPage, $numPages, $totalPages);
    my ($numDisplayPages, $dec, $lastChange, $startPage, $i, $page, $pages);

    
    $numDisplayPages = 3;     # Total number of pages to show on each side of the pages link from the
    # current page
    $totalPages = $numDisplayPages * 2 + 3;    # include the current page, first page and last page
    $this_start = $startrow;
    $this_page = $startrow/$maxrows;
    
    if ($foundcount == 0) {
        $firstrecord = 0;
    }
    else {
        $firstrecord = $startrow+1;
    }
    
    if ($startrow > 0) {
        $startrow = $startrow - $maxrows;
        if ($startrow < 0) {
            $startrow = 0;
        }
        $back="<A HREF=\"$scriptinfo\?$pr";
        $back.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Back</FONT></A>&nbsp;&nbsp;";
    }
    
    if ($lastrow < $foundcount) {
        $startrow = $lastrow;
        $next="\n\n<A HREF=\"$scriptinfo\?$pr";
        $next.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Next</FONT></A>";
    }
    
    $numPages = $foundcount / $maxrows;
    
    ($numPages, $dec) = split /\./, $numPages;
    
    if ($dec) {
        $numPages += 1;
    }
    
    $lastChange = $numPages-$numDisplayPages;
    
    if ($numPages > 1) {
        
        if ($numPages > $totalPages) {
            if ($this_page >= $lastChange) {
                $startPage = $numPages - (($numDisplayPages * 2) + 1);
                $endPage = $numPages;
            }
            elsif ($this_page >=  $numDisplayPages) {
                $startPage = $this_page - $numDisplayPages;
                $endPage = $this_page + $numDisplayPages;
            }
            else {
                $startPage = 0;
                $endPage = $numDisplayPages * 2 + 1;
            }
            
        }
        else {
            $startPage = 0;
            $endPage = $numPages;
        }
        
        for ($i = 0; $i < $numPages; $i++) {
            
            if ((($i >= $startPage) && ($i <= $endPage)) || ($i == ($numPages-1)) || ($i == 0)) {
                $page = $i + 1;
                $startrow = $i * $maxrows;
                $pages.="\n\n<A HREF=\"$scriptinfo\?$pr";
                $pages.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">$page</FONT></A>";
                # put spaces after 1st row and before last row
            }
            if ( ($i == 0) || ($i == ($numPages - 2)) ){
                $pages .= "&nbsp&nbsp&nbsp&nbsp";
            }
            
        }
    }

    return ($back, $next, $pages, $firstrecord);
}

##############################################################################
=pod

=head2 display_search_results

display_search_results - &display_search_results($link_details, $$FORM{'foundcount'}, $$FORM{'startrow'}, $$FORM{'next_startrow'}, $site, $FORM, \@results);

=over 4

=item DESCRIPTION

To display the search results to the browser

=item INPUT

$link_details - Details of the link to build the link to the next page with.

$found - Total number of rows found

$startRow - The starting row

$lastRow - The last row to show on this page

$site - A hashref containing information required throughout about the current
        site and session

$FORM -	Pointer to FORM hash

$p_results - Pointer to array of search results

=item OUTPUT

The search results are displayed

=item NOTES

=back

=cut
###############################################################################
sub display_search_results {
    my ($link_details) = shift;
    my ($found) = shift;
    my ($startRow) = shift;
    my ($lastRow) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($p_results) = shift;


    my ($template, $result, $row, $objid, $prices, $inventory, $linkage, $title, $variants);
    my ($back, $next, $pages, $firstrecord, @rows, $start, $link_info, $link_row, $link_info, $search_info, $object_inf, $saved);

    &debug("In display_search_results", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $template = &localize_template("os_allcommerce-admin_object-search", "admin", 0, $site);
    $template =~ s/__admin_title__/$site->{'script_msg'}{'search_title'}/gms;
    @rows = ('row_spine','row_menu','row_headers','row_copyright','row_help','row_site_info','row_search_results','row_search_navigation');
    $template = &permute_rows($template, $site, \@rows);

    $search_info =  "&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&startrow=$$FORM{'startrow'}&foundcount=$$FORM{'foundcount'}";

    &debug("display_search_results: $search_info", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $result (@$p_results) {
	&debug("display_search_results: Building a row", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$row .= &localize_template("row_search_result", "admin", 0, $site);
	$link_row = &localize_template("cell_link", "admin", 0, $site);
	$link_info = "$site->{'scriptpath'}/__script__\?&__options__";

	if (defined $result->{'title'}) {
	    $row =~ s/__title__/$result->{'title'}/gms;
	}
	# from contacts, so show user name
	else {
	    $row =~ s/__title__/$result->{'uname'}/gms;
	}

	$objid = $link_row;
	$link_info = "$site->{'scriptpath'}/$site->{'scriptname'}\?&sid=$$FORM{'sid'}&mode=display_object&objid=$result->{'objid'}&objtype_inp=$result->{'objtype'}&objclass_inp=$result->{'objclass'}&objclass_pre=$result->{'objclass'}&objtype_pre=$result->{'objtype'}&came_from_search=yes";
	$link_info .= $search_info;

	$objid =~ s/__href__/$link_info/gms;
	$objid =~ s/__tag__/$result->{'objid'}/gms;
	$row =~ s/__objid__/$objid/gms;

	if (($result->{'objclass'} ne "") && ($result->{'objclass'} eq $site->{'OBJCLASSES'}->{'product'}) ||
	    ($result->{'objtype'} ne "") && ($result->{'objclass'} eq $site->{'OBJTYPESS'}->{'product'})) {

	    $prices = $link_row;
	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editprice", $site);
	    &reset_form_values($FORM, $saved, $site);
	    $prices =~ s/__href__/$link_info/gms;
	    $prices =~ s/__tag__/$site->{'script_msg'}{'price_tag'}/gms;

	    $inventory = $link_row;
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_inventory_link_info($FORM, $result->{'objid'}, "editsku", $site);
	    &reset_form_values($FORM, $saved, $site);
	    $inventory =~ s/__href__/$link_info/gms;
	    $inventory =~ s/__tag__/$site->{'script_msg'}{'inventory_tag'}/gms;
	}
	else {
	    $inventory = "";
	    $prices = "";

	}
	$row =~ s/__inventory__/$inventory/gms;
	$row =~ s/__prices__/$prices/gms;

	if ($result->{'objclass'} ne $site->{'OBJCLASSES'}->{'contact'}) {
	    $linkage = $link_row;
	    $link_info = "$site->{'scriptpath'}/linkage.cgi\?&sid=$$FORM{'sid'}&mode=editlink&loadfrom=db&obj1=$result->{'objid'}";
	    $linkage =~ s/__href__/$link_info/gms;
	    $linkage =~ s/__tag__/$site->{'script_msg'}{'linkage_tag'}/gms;
	}
	else {
	    $linkage = "";
	}
	$row =~ s/__linkage__/$linkage/gms;

	# is a master product
	if (($result->{'ref_from'} ne "") && ($result->{'ref_from'} eq $result->{'objid'})) {
	    $variants = $link_row;
	    $result->{'came_from_search'} = "yes";
	    $result->{'objtype_inp'} = $result->{'objtype'};
	    $result->{'objclass_inp'} = $result->{'objclass'};
	    $saved = {};
	    $saved = &set_form_temporary($FORM, "objtype_inp,objclass_inp,came_from_search", $result, $site);
	    $link_info = &build_admin_object_link_info($FORM, $result->{'objid'}, "display_variants", $site);
	    $variants =~ s/__href__/$link_info/gms;
	    $variants =~ s/__tag__/$site->{'script_msg'}{'variant_tag'}/gms;
	    &reset_form_values($FORM, $saved, $site);
	}
	else {
	    $variants = "";
	}

	$row =~ s/__variants__/$variants/gms;
	$result = {};
    }

    $template =~ s/__row_search_result__/$row/gms;
    $template =~ s/__found__/$found/gms;

    # this is the last page
    if (!defined $lastRow) {
	$lastRow = $found;
    }

    &debug("set_search_navigation: start: $startRow, found: $found, last: $lastRow", 3, $site->{'debug_level'}, $site->{'debug_log'});

    ($back, $next, $pages, $firstrecord) = &set_search_navigation($link_details, $found, $site->{'search_results_per_page'}, $startRow, $lastRow, "$site->{'scriptpath'}/$site->{'scriptname'}");

    &debug("set_search_navigation: returned back: $back, next: $next, pages: $pages, first: $firstrecord", 3,  $site->{'debug_level'}, $site->{'debug_log'});


    $object_inf = {};
    $object_inf->{'mode'} = $$FORM{'mode'};
    $template = &set_links($object_inf, $FORM, $template, $site);

    $template =~ s/__end__/$lastRow/gms;
    $template =~ s/__startrow__/$firstrecord/gms;
    $template =~ s/__back__/$back/gms;
    $template =~ s/__next__/$next/gms;
    $template =~ s/__pages__/$pages/gms;

    $$FORM{'startrow'} = $$FORM{'next_startrow'};  # get ready for the next start
    $template = &permutebit($template, $FORM);
    $template = &permutebit($template, $site);
    $template = &permuteclean($template, $site);
    print $template;

}
##############################################################################
=pod

=head2 do_admin_search

do_admin_search -  &do_admin_search($$FORM{'search_keywords'}, "and", "title", $filters, $FORM, $site, $dbh);

=over 4

=item DESCRIPTION

This routine determines if there are any last minute queries that must
be executed in order to prepare for the search, and if there are, then set it
up.  

Sorts the results on the appropriate fields, and then return the results 
to the calling routine.

=item INPUT

$search_words - List of search words, space delimited

$type - Type of search "and", "or", "exact"

$sortfield - Name of field to sort results on.

$filters - hashref of infromatin to filter on, from setup_search_filters

$FORM - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

@search_array - Search results, sorted.

=item NOTES

=back

=cut
###############################################################################
sub do_admin_search {
    my ($search_words) = shift;
    my ($type) = shift;
    my ($sortfield) = shift;
    my ($filters) = shift;
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@search_array, $search_results, $search_indx, $objid, $extras, @results, $q, $numRows, $select_hashref);
    my ($where_indx);

    $extras->{'status'} = $$FORM{'search_status'};
    $extras->{'objid'} = $$FORM{'search_objid'};
    $extras->{'keyword'} = $search_words;
    $extras->{'objtype'} = $$FORM{'search_objtype'};
    $extras->{'contact_type'} = $site->{'OBJCLASSES'}->{'contact'};
    if ($$FORM{'search_sku_series'} ne "") {
	&debug("Adding sku_series to search!", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
	$where_indx = 0;
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", "$$FORM{'search_sku_series'}", "and");
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "sku_skel", "and");
	$q = &build_select($select_hashref, $site, $dbh, 1);
	$numRows = &query_getAll($q, $site, $dbh, \@results);
	$extras->{'sku_skel'} = $results[0]->{'body'};
	&debug("Adding $extras->{'sku_skel'}!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }

    &debug("In do_admin_search", 3, $site->{'debug_level'}, $site->{'debug_log'});
    ($search_results, $$FORM{'next_startrow'}, $$FORM{'foundcount'}) = &search($search_words, $type, $$FORM{'startrow'}, $filters, \&build_admin_search_queries, $extras, $site, $dbh);
    &debug("do_admin_search: after search, next_start: $$FORM{'next_startrow'}, found: $$FORM{'foundcount'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

   $search_indx = 0;
  foreach $objid (keys %{$search_results->{'things'}}) {
	if (&check_all_attribute_permissions($search_results->{'things'}->{$objid}, "r", $site)) {
	    $search_array[$search_indx++] = $search_results->{'things'}->{$objid};
	}
    }
    foreach $objid (keys %{$search_results->{'contacts'}}) {
	if (&check_all_attribute_permissions($search_results->{'contacts'}->{$objid}, "r", $site)) {
	    $search_array[$search_indx++] = $search_results->{'contacts'}->{$objid};
	}
    }

    # sort it, assume text sort
    @search_array = sort { uc($a->{$sortfield}) cmp uc($b->{$sortfield}) } @search_array;    
    return @search_array;
}

##############################################################################
=pod

=head2 setup_search_filters

setup_search_filters - &setup_search_filters(\%FORM);

=over 4

=item DESCRIPTION

Setting up filters that can be used after search results are gathered.  This is
a final check against the results to remove any data that may not fit the search
query.

=item INPUT

$FORM - pointer to the FORM hash

=item OUTPUT

$filters - Hashref of filters, by field and the value to compare to 

=item NOTES

=back

=cut
###############################################################################
sub setup_search_filters {
    my ($FORM) = shift;

    my ($filters);

    $filters->{'things'} = "title,keywords,brief";

    if ($$FORM{'search_status'} ne "") {
	$filters->{'status'} = $$FORM{'search_status'};
    }

    if ($$FORM{'search_objid'} ne "") {
	$filters->{'objid'} = "%$$FORM{'search_objid'}%";
    }

    if ($$FORM{'search_sku_series'} ne "") {
	$filters->{'sku_series'} = $$FORM{'search_sku_series'};
    }
    
    return $filters;
}

##############################################################################
=pod

=head2 build_admin_search_queries

build_admin_search_queries - &build_admin_search_queries("hat", 3, \@queries, $extras);

=over 4

=item DESCRIPTION

It is necessary to build queries based on user input for search criteria.  This routine
builds all of the queries required for the keyword passed in order to do a complete
search of the AllCommerce tables for their requested data.

=item INPUT

$keyword - This keyword to search for

$indx - Next index into the $p_queries that the next queries should be put

$p_queries - Pointer to an array where the $select_hashrefs that are built in
             this routine need to be stored.

$extras - Hashref of extra information, which may be required to build the queries.
    
    $extras->{'contact_type'} = objtype of contact row entry
    $extras->{'objtype_type'} = objtype selected for the search
    $extras->{'status'} = status selected for the search
    $extras->{'sku_skel'} = brand selected for the search    
    $extras->{'objid'} = object ID to search for

=item OUTPUT

$indx - The next location to add more queries.

$p_queries is populated with the queries appropriate for this $keyword

=item NOTES

=back

=cut
###############################################################################
sub build_admin_search_queries {
    my ($keyword) = shift;
    my ($indx) = shift;
    my ($p_queries) = shift;
    my ($extras) = shift;

    my ($select_hashref, $where_hash, @where, $where_indx);

    # THINGS QUERY
    if ($extras->{'objtype'} ne $extras->{'contact_type'}) {
	$where_indx = 0;
	$select_hashref = {};
	$select_hashref->{'statement'} = "SELECT";
	$select_hashref->{'TABLE'}->{'NAME'} = "things";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	
	# don't show initial dataload stuff
	$select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", "!%-A", "and");
	if ($keyword ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("title", "%$keyword%", "or");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("brief", "%$keyword%", "or");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("keywords", "%$keyword%", "and");
	}
	
	if ($extras->{'objtype'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("(objtype", $extras->{'objtype'}, "or");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objclass", "$extras->{'objtype'})", "and");
	}
	
	if ($extras->{'status'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++]  = &build_where_hash("status", "$extras->{'status'}", "and");
	}

	if ($extras->{'objid'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++]  = &build_where_hash("objid", "%$extras->{'objid'}%", "and");
	}

	if ($extras->{'sku_skel'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("msku", "%$extras->{'sku_skel'}%", "and");
	}    

	$select_hashref->{'TABLE'}->{'ORDER'} = "title";
	$$p_queries[$indx++] = $select_hashref;
    }

    # only query CONTACTS if no sku_skel specified
    if ($extras->{'objtype'} eq $extras->{'contact_type'}) {
	$where_indx = 0;
	$select_hashref = {};
	$select_hashref->{'statement'} = "SELECT";
	$select_hashref->{'TABLE'}->{'NAME'} = "contacts";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	
	if ($keyword ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("uname", "%$keyword%", "and");
	}
	
	if ($extras->{'objtype'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("(objtype", $extras->{'objtype'}, "or");
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objclass", "$extras->{'objtype'})", "and");
	}
	
	if ($extras->{'status'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++]  = &build_where_hash("status", "$extras->{'status'}", "and");
	}

	if ($extras->{'objid'} ne "") {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++]  = &build_where_hash("objid", "%$extras->{'objid'}%", "and");
	}
	
	$select_hashref->{'TABLE'}->{'ORDER'} = "uname";
	$$p_queries[$indx++] = $select_hashref;
    }

    return $indx;
}

##############################################################################
=pod

=head2 search

search - &search($dbh, "red blue", "and", $site->{'max_search_results'}, $site, $filters, \&subroutine_name);

=over 4

=item DESCRIPTION

To search the tables for the searchwords.  If $type is and, all search words must match.  If $type
is or, any search word can match, and if $type is exact, the words must match in the order specified.

An 'or' search is always done first, querying for everything that could come back from the tables.
Then, after these are all gathered, the results are filtered.  This method is preferred because 
variant products do not have their title's stored in the things table anywhere, and are dynamically
generated.  Therefore, for an 'and' search, the results portions could be in various locations, 
making the logic more complicated. 

=item INPUT

$dbh - current active database handle

$searchwords - string of words, separated by a space

$type - type of search and, or, exact

$maxrows - the maximum number or rows to strive to return.  DOES NOT WORK AT THE MOMENT,
	   need more thought.

$site - A hashref containing information required throughout about the current
site and session

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

$build_query_routine - pointer to a subroutine which is used to create the queries for each keyword
                       This is unique on a per script basis, depending on what the search is based on.  This way
                       search can be called by any script and used by any script, just by defining this subroutine
                       first!  The subroutine MUST return results in the exact same fashion that build_site_search_queries
                       does and accept the exact same parameters in the same order as build_site_search_queries does.

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$start_row - The next row to start from, will be set to startRow on next interation

$foundRows - Total number of rows that are found

=item NOTES

This should be tested for effeciency on LARGE sites.

AND does not work if all of the words do not appear in one field.  So, if 2 of
the words appear in title, and the other word appears in brief, this does not
work.  This is not really good

This does not limit the number of results returned, this needs to be thought
about, as the data comes from all over the place, and we are supposed to 
order by weight, which means we would have to grab all of the data, and by
that time, we are already done the hard work, so why limit it.  Otherwise, 
we need to do an order by on the select, which may not be very efficient.

=back

=cut
###############################################################################
sub search {
    my ($searchwords) = shift;
    my ($type) = shift;  
    my ($startRow) = shift;
    my ($filter_hashref) = shift;
    my ($build_query_routine) = shift;
    my ($extras) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@rows, $i, $search_key, @words, %MasterRec);
    my (@search_queries, $q, $select_hashref, $obj, $results, $numRows, @res);
    my ($h,@variant_products, $keyword, $numExtAdded, $numRowsAdded);
    my (@encoding_match, $master_matched, $numQueries, $foundRows, $found);
    my ($numSearchResults) = 0;    
    
    $results = {};

    if ($searchwords =~ / /) {
	@words = split / /, $searchwords;
    }
    else {
	$words[0] = $searchwords;
    }

    foreach $search_key (@words) {
	$numQueries = 0;
	$numQueries = &$build_query_routine($search_key,$numQueries,\@search_queries, $extras);
	($results, $startRow, $found) = &do_search($results, \@search_queries, $numQueries, $search_key, $startRow, $type, $filter_hashref, $site, $dbh);
	$foundRows += $found;

    }
    
    if ($#words > 0) {  # more than one word supplied
	if ($type eq "and") {
	    $results->{'things'} = &filterAndResults($results->{'things'}, $site, \@words, $filter_hashref);
	}
	
	if ($type eq "exact") {
	    $results->{'things'} = &filterExactResults($results->{'things'}, $site, $searchwords, $filter_hashref);
	}
    }

    return ($results, $startRow, $foundRows);
}

##############################################################################
=pod

=head2 do_search

do_search - &do_search($dbh, $site, $results, \@search_queries, $numQueries, $search_key, $numRows, $type, $filter_hashref);

=over 4

=item DESCRIPTION

Do the actual search.  Submit the query.   If any master records matched, grab all of the
variant products for that master as well, as they inherit searchable fields from the master.
Then, search the encodings and find all variant products which match the search_key.

=item INPUT

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$results - The current set of results, up to this point. A hashref of information, initially 
           seperated by tables names, and then by objid, and then the contents of the record,
  	   of records that met the search criteria  

$search_queries - Array or hashrefs, each consisting of the elements to create a select statement

$numQueries - The number of queries to submit

$search_key - the keyword to perform the search on.  This is one keyword for and and or, and is
              the exact string for exact matches.

$maxrows - the maximum number or rows to strive to return.  DOES NOT WORK AT THE MOMENT,
	   need more thought.


$type - type of search and, or, exact

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$next_start - The next row to start from, will be set to startRow on next interation

$foundRows - Total number of rows that are found

=item NOTES

=back

=cut
##############################################################################
sub do_search {
    my ($results) = shift;
    my ($search_queries) = shift;
    my ($numQueries) = shift;
    my ($search_key) = shift;
    my ($startRow) = shift;
    my ($type) = shift;
    my ($filters) = shift;
    my ($site) = shift;
    my ($dbh) = shift;    

    my ($i, $select_hashref, $master_matched, $numRows, $numExtAdded, @encoding_match);
    my (%MasterRec, @variant_products, $numSearchResults, $next_start, $found, $foundRows);


    $numRows = 0;

    for ($i = 0; $i < $numQueries; $i++) {
	$select_hashref = {};
	$select_hashref = $$search_queries[$i];
	($numRows, $results, $next_start, $found) = &submit_query($select_hashref, $results, $startRow, $type, $filters, $site, $dbh);
	$foundRows += $found;
	($master_matched) = &saveMasterProducts($results, \%MasterRec, $site);
    }

    # If a master record that matched, we need to find all of the variant products that match
    if ($master_matched == 1) {
	$numRows = &getVariantProducts($dbh, $site, \%MasterRec, \@variant_products);
	$results = &filterResults("things", $numRows, \@variant_products, $dbh,$site,  $results, $filters);
    }
	    
    &findMatchedExtraprop($dbh, $search_key, $results->{"extraprop"}, \@encoding_match, $site);
    $results = &filterResults("things", $numExtAdded, \@encoding_match, $dbh, $site, $results, $filters);
	    
    &debug("do_search: Returning next_start: $next_start, found: $foundRows", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($results, $next_start, $foundRows);

}

##############################################################################
=pod

=head2 submitQuery

submitQuery - &submitQuery($select_hashref, $results, $startRow, $type, $filters, $site, $dbh);

=over 4

=item DESCRIPTION

Generate the select statement and issue it.  Perform first round of filtering

=item INPUT

$select_hashref - hashref with information to build a select query:
                  $select_hashref->{'TABLE'}->{'NAME'} = name of table to query
                  $select_hashref->{'TABLE'}->{'FIELD'}->{$field} = $field
                          where $field is the name of the field to select.  One
                          for each field to select from the table
                  $select_hashref->{'TABLE'}->{'WHERE'} array of where clause 
                          elements.  See NOTES section of db_layer.pm for 
                          details
$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$startRow - the row to start getting data from

$type - type of search (and, or, exact)

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter


$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$numRows - Number of rows found in this execution of the query_getNext

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
                of records that met the search criteria

$nextRow - The next row to start from, will be set to startRow on next interation

$foundRows - Total number of rows that are found

=item NOTES

=back

=cut
##############################################################################
sub submit_query {
    my ($select_hashref) = shift;
    my ($results) = shift;
    my ($startRow) = shift;
    my ($type) = shift;
    my ($filters) = shift;
    my ($site) = shift;
    my ($dbh) = shift;


    my ($q, $numRows, @res, $nextRow, $foundRows);

    &debug("*************************IN submit_query!", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $select_hashref->{'statement'} = "SELECT";
#    $q = &build_select($select_hashref, $site, $dbh, $site->{'override_permissions'});
    $q = &build_select($select_hashref, $site, $dbh, $site->{'override_permissions'});
#    $numRows = &query($q, $dbh, \@res, $maxNumRows);
    &debug("submit_query: max results: $site->{'search_results_per_page'}, start: $startRow", 3, $site->{'debug_level'}, $site->{'debug_log'});
    ($numRows, $nextRow) = &query_getRows($q, $site, $dbh, $startRow, $site->{'search_results_per_page'}, \@res);
    &debug("submit_query: got $numRows, $nextRow rows before filterResults", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($numRows > 0) {
	$results = &filterResults($select_hashref->{'TABLE'}->{'NAME'}, $numRows, \@res, $dbh, $site, $results, $filters);
    }

    $foundRows = &query_findNumRows($q, $site, $dbh);
    &debug("submit_query: got $numRows rows, next is $nextRow, found: $foundRows", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($numRows, $results, $nextRow, $foundRows);
}

################################### FILTER SUBROUTINES #####################################
##############################################################################
=pod

=head2 filterResults

filterResults - &filterResults("things", $numRows, \@sql_results, $dbh, $site, $results, $filters);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the words typed into the search
keywords input by the user through a filter.  Verifies that the object matches the
objtype filter in the $filters hashref, and matches the visibility filter in the
$filters hashref.  If the product is a master product, set a flag to get the variant
products for the matched master.  If it is a variant product, set the title and the
brief.

=item INPUT

$table - Table where the object came from "things" "extraprop"

$numRows - The total number of results returned so far

$sql_results - an array of hashrefs, each being on record returned by the search query.

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$filters - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

=back

=cut
##############################################################################
sub filterResults {
    my ($table) = shift;
    my ($numRows) = shift;
    my ($sql_results) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    my ($results) = shift;
    my ($filters) = shift;  # objtypes that it needs to be
    
    my ($h, $i, $add_it, $res, $master_select);
    
    
    for ($i=0; $i < $numRows; $i++) {
        $add_it = 1;  # assume we'll add it
        
        $h = $$sql_results[$i];
        
        if ($table eq "things") {
	    if (defined $filters->{'objtype'}) {
		if (( $filters->{'objtype'} !~ $h->{'objtype'}) || ($h->{'objtype'} eq "")) {
		    $add_it = 0;
		}
	    }
            
	    if (defined $filters->{'visibility'}) {
		if (($filters->{'visibility'} !~ $h->{'visibility'}) || ($h->{'visibility'} eq "")) {
		    $add_it = 0;
		}
	    }
            
            # This is a master product, so all of the variant products need to be gathered
            if ( $h->{'ref_from'} eq $h->{'objid'}) {
		&debug("Adding $h->{'objid'} to check the variants!  $h->{'addvariants'}",3, $site->{'debug_level'}, $site->{'debug_log'});
                $h->{'addvariants'} = "yes";

            }
            
            
            # set the title for variant products that matched
#            if (( $h->{'ref_from'} ne $h->{'objid'}) && ($h->{'ref_from'} ne "")) {
#		$master_select->{'FIELDS'}->{'brief'} = "brief";
#		$master_select->{'FIELDS'}->{'weight_man'} = "weight_man";
#                $res = &get_fields_and_title(undef ,$h->{'objid'}, $dbh, $master_select, $site);  # can't have spaces in master portion!
#                $h->{'title'} = $res->{'title'};
#                $h->{'brief'} = $res->{'brief'};
#                $h->{'weight_man'} = $res->{'weight_man'};
#            }
        }
        elsif ($table eq "extraprop") {
            if ($h->{'fieldname'} eq "encoding") {
                $h->{'search_encoding'} = "yes";
            }
        }
        
        if ($add_it == 1) {
            $h->{'table'} = $table;
            $results->{$table}->{$h->{'objid'}} = $h;
            &debug("Added $h->{'objid'} from $table",3,$site->{'debug_level'}, $site->{'debug_log'});
            
        }
    }
    
    return $results;
}

##############################################################################
=pod

=head2 filterExactResults

filterExactResults - &filterExactResults($results, $site, $search_string, \%filter_hashref);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the words typed into the search
keywords input by the user through a filter.  If all of the words show up in the
at least one of the $filter_hashref->{'things'} fields, then it is kept.  
If it doesn't then it is set to undef

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$site - A hashref containing information required throughout about the current
        site and session

$word_array - Pointer to an array, each index is one of the keywords inputted by the user

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

THIS WILL NOT WORK IF A PORTION OF THE INFORMATION IS IN THE TITLE AND THE OTHER IS IN
THE KEYWORDS.  THIS IS NOT A GOOD THING.

=back

=cut
##############################################################################
sub filterAndResults {
    my $results = shift;
    my $site = shift;
    my $word_array = shift;
    my $filter_hashref = shift;

    
    my (@filters, $field, $search_word, $h, $is_a_match, $objid);
    my ($this_search, $search_str, $it_matched, @words);


    @words = @$word_array;
    @filters = split /\,/, $filter_hashref->{'things'};

    $search_str = "if (";
    foreach $search_word (@words) {
	if (length ($search_word) > 0) {
	    $search_word = lc ($search_word);
	    $search_str .= "(lc(__field__) =~ /$search_word/) && ";
	}
    }

    $search_str =~ s/&& $//g;
    $search_str .= ") { return 1; } else { return 0; };";
    &debug("Search string is $search_str", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $objid (keys %{$results}) {
	$is_a_match = 0;
	$it_matched = 0;
	foreach $field (@filters) {
	    $this_search = $search_str;
	    $this_search =~ s/__field__/\$results->{'$objid'}->{'$field'}/gms;
	    $is_a_match = eval $this_search;
	    if ($is_a_match == 1) {  # we keep it, so don't remove it!
		$it_matched = 1;
	    }
	}
	if ($it_matched == 0) {
	    &debug ("removing $results->{$objid}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    undef $results->{$objid};
	}
    }
    return $results;
}

##############################################################################
=pod

=head2 filterExactResults

filterExactResults - &filterExactResults($results, $site, $search_string, \%filter_hashref);

=over 4

=item DESCRIPTION

To take the results from the or search, and run the exact phrase typed into the search
keywords input by the user through a filter.  If the phrase shows up in any of the 
$filter_hashref->{'things'} fields, then it is kept.  If it doesn't then it is set
to undef

=item INPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria

$site - A hashref containing information required throughout about the current
        site and session

$search_string - the exact search string that must match.

$filter_hashref - hashref of information to filter on:
		 'things' -> fieldname of things record which match must be true on for AND results
		 'objtype' -> list of valid object types which is compared to for the filter
		 'visibility' -> list of visibility values (only one normally) which is compared to for the filter

=item OUTPUT

$results - a hashref of information, initially seperated by tables names, 
		 and then by objid, and then the contents of the record,
		 of records that met the search criteria.  Items that did NOT match
                 the exact filter, are undef'd in the results hashref.

=item NOTES

=back

=cut
##############################################################################
sub filterExactResults {
    my $results = shift;
    my $site = shift;
    my $search_string = shift;
    my $filter_hashref = shift;

    
    my (@filters, $field, $search_word, $h, $is_a_match, $objid);
    my ($this_search, $search_str, $it_matched, @words);

    @filters = split /\,/, $filter_hashref->{'things'};

    $search_str = "if (";
    $search_word = lc ($search_string);
    $search_str .= "(lc(__field__) =~ /$search_string/)";

    $search_str .= ") { return 1; } else { return 0; };";
    &debug("Search string is $search_str", 3, $site->{'debug_level'}, $site->{'debug_log'});

    foreach $objid (keys %{$results}) {
	$is_a_match = 0;
	$it_matched = 0;
	foreach $field (@filters) {
	    $this_search = $search_str;
	    $this_search =~ s/__field__/\$results->{'$objid'}->{'$field'}/gms;
	    $is_a_match = eval $this_search;
	    if ($is_a_match == 1) {  # we keep it, so don't remove it!
		$it_matched = 1;
	    }
	}
	if ($it_matched == 0) {
	    &debug ("removing $results->{$objid}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	    undef $results->{$objid};
	}
    }
    return $results;
}
