#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# cart-lib.pm
#
# Version 5 NC 20000922
#        pod  
# Version 4 TG 20000915
#        Fixed parameters to localize_template after retrofitting 
#        localization.pm to standards
# Version 3 LCF 20000911
#	integrated further changes for Oracle compliance 
#	and modularity
# Version 2 LCF 20000907
#   moved more of cart.cgi in here
# Version 1 LCF 20000824
#	split out lots of cart.cgi into here.
#
# Library of subroutines for cart.cgi

# Start docs
=pod
=head1 NAME 

cart-lib.pm

=head1 SYNOPSIS

This is a library of functions used in cart.cgi. 

=head1 DESCRIPTION

All of the DB calls formerly found in cart.cgi are here now. Routines that do DB calls will generally return an active (executed) statement handle.

Also, a big chunk of the logic for finding variants and creating the "next page" URLs is now in here.

=head1 NOTES & BUGS

More of cart.cgi should be put in here, and the check_for_variants and make_nextpg_* routines should probably be broken up further.

=cut
# End docs

use strict;

##############################################################################
=pod

=head2 init_script

init_script - &init_script($scriptname, $relpath);

=over 4

=item DESCRIPTION

Load stadard script data, like $site, %FORM, and $dbh.

=item INPUT

$scriptname - Name of the currently running script. 

$relpath    - path to top level of allcommerce 

=item OUTPUT

$site         - Standard $site hash from &setsite

\%FORM        - Reference to standard %FORM hash from &ReadParse

$dbh          - Active database handle, also accessable through $site->{dbh}

\%script_msg  - Reference to hash of error messages for this script. Needs a &message() function to run.

=item NOTES

This version pokes $scriptname and $relpath into $site, for convenient access.

=back

=cut
###############################################################################

sub init_script {
	my ($scriptname, $relpath) = @_;
	
	## Get the site hash
	#
	my $site = &setsite();

	## get the FORM bits
	#
	my $form = &ReadParse();

	## establish connection to database  
	#
	my $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}) 
		|| die "Can't connect to database! ".DBI->errstr();

	## Add dbh to $site, for convenience
	$site->{dbh} = $dbh;

	## Get the language and interface, if possible
	#
	my $h = {};
        # decrypt sid
        $form->{'sid'}=&decrypt_sid($form->{'sid'},$site) if ($form->{'sid'});

	if ($form->{'sid'}) {
	   my $sth = &get_lang_interf($form->{'sid'}, $site->{dbh});
	   $h = $sth->fetchrow_hashref('NAME_lc') || {};
	   $sth->finish();
	}

	$site->{'om_render_lang'} = $h->{'language'} || $site->{'om_locale_lang_def'} || 'eng';
	$site->{'current_interface'} = $h->{'interface'} || "html";
	
	# Add scriptname and relpath to site for convenience
	$site->{scriptname} = $scriptname;
	$site->{relpath} = $relpath;
	
	## Get the script error messages hash
	#
	my %script_msg = &message($site->{'om_render_lang'}, $scriptname, $relpath);          # user interaction messages
	
	return ($site, $form, $dbh, \%script_msg);
}


##############################################################################
=pod

=head2 determine_target_page

determine_target_page - &determine_target_page($site, $form);

=over 4

=item DESCRIPTION

Determine what the target for $form->{'pg_targ'} should be. This will point the "Return to Shopping" button at something sensible.

=item INPUT

$site - Standard $site hashref from &setsite()

$form - Standard %FORM reference from &ReadParse() 

=item OUTPUT

$target - String of the URL for $form->{'pg_targ'}

=item NOTES

=back

=cut
###############################################################################



sub determine_target_page {
	my ($site, $form) = @_;
	
	# if target page element exists
    # user is coming from with script
    # don't need to do anything
	return $form->{'pg_targ'} if ( exists($form->{'pg_targ'}) );
	
	# no target page exists
	# check that refered from inside site
    if (( $ENV{'HTTP_REFERER'} =~ /($site->{'sitedomain'}|$site->{'sitessldomain'})/ ) and
	    ( $ENV{'HTTP_REFERER'} !~ /$site->{scriptname}/ )) {
    	  return $ENV{'HTTP_REFERER'};
	}
	
	# referer is outside site or this 
    # script, default to home page
    return "/html/$site->{om_render_lang}/home.shtml";
}


##############################################################################
=pod

=head2 get_custid 

get_custid - &get_custid($site, \%FORM); 

=over 4

=item DESCRIPTION

get the custid from sessions for this session id.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

=item OUTPUT

$id - The custid, or undef if there wasn't one.

=item NOTES

=back

=cut
###############################################################################
sub get_custid {
	my ($site, $form) = @_;
	
	#my $q   = "SELECT custid from sessions where sid=$form->{'sid'} ";
	my @where;
	$where[0] = &build_where_hash("sid", $form->{sid}, "and");
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'      => 'sessions',
			 'FIELDS'    => {'custid' => 'custid'},
			 'WHERE'     => \@where
			}
	};
	
	my $q = &build_sql(	$select_q, $site, $site->{dbh}, 1);	 
	my $sth = $site->{dbh}->prepare( $q );
	$sth->execute();

	my $id = $sth->fetchrow() || undef;
	$sth->finish();
	return $id;
}

sub get_next_page_url {
	my ($site, $form) = @_;
	my $url;
	
	if ($site->{'custid'} && (length($site->{'custid'}) > 2)) {
		$url = &make_nextpg_user($site, $form);
	} else {
		$url = &make_nextpg_nouser($site, $form);
	}
	
	return $url;
}

##############################################################################
=pod

=head2 make_nextpg_user

make_nextpg_user - &make_nextpg_user($a,$b,$c,$site,\%FORM); 

=over 4

=item DESCRIPTION

do the setups for the next page if the current page already owned by a customer id.

=item INPUT

$site  - A hashref containing information required throughout about the current
         site and session

\%FORM  - a hashref which will be used to describe the current page to display
        - Sets up all CGI relevant information

=item OUTPUT

$url    - The url to stick in $form->{'nextcheckoutpg'}

=item NOTES

$site must include the key 'scriptname', so we know if we're a cart or a wishlist.

=back

=cut
###############################################################################
sub make_nextpg_user {
	my ($site,$form) = @_;
	my $url;
        ## encrypt sid
        my $encrypted_sid=&encrypt_sid($form->{'sid'},$site);
	
    if ($form->{wishlist}) { # wishlist sequencie
		
		# STUB. DON'T KNOW YET WHAT WISHLIST DOES NEXT.
		# SIMPLE DUPE OF CART RETURN
		#$url  = "$site->{'ssl_prefix'}://$site->{'sitesslurl'}/cgi-bin/om/ssl/";
	    $url .= "/cgi-bin/om/cart.cgi";
				
	} else {
		
		$url  = "$site->{'ssl_prefix'}://$site->{'sitesslurl'}/cgi-bin/om/ssl/";
	    $url .= "checkout.cgi?sid=$encrypted_sid";
		
	}    
	
	return $url;

}

##############################################################################
=pod

=head2 make_nextpg_nouser

make_nextpg_nouser - &make_nextpg_nouser($a,$b,$c,$site,\%FORM); 

=over 4

=item DESCRIPTION

do the setups for the next page if the current page still not owned by a customer id.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

\%FORM - a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

=item OUTPUT

$url  - The url to stick in $form->{'nextcheckoutpg'}

=item NOTES

$site must include the key 'scriptname', so we know if we're a cart or a wishlist.

=back

=cut
###############################################################################
sub make_nextpg_nouser {
	my ($site,$form) = @_;
	
	my $url;
        ## encrypt sid
        my $encrypted_sid=&encrypt_sid($form->{'sid'},$site);

    if ($form->{wishlist}) { # wishlist sequence

		## wishlist sequence
		#
		my $q = "/cgi-bin/om/cart.cgi?wishlist=1%26cartclass=$form->{'cartclass'}%26objid=$form->{'objid'}%26add=$form->{'add'}";
		$url  = "http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi?";
		$url .= "sid=$encrypted_sid&pg_targ=$q";

	} else {  # cart sequence

		my $q = "$site->{'ssl_prefix'}://$site->{'sitesslurl'}/cgi-bin/om/ssl/checkout.cgi?";
		$q   .= "sid=$encrypted_sid";
		$url  = "/cgi-bin/om/welcome.cgi?sid=$encrypted_sid&pg_targ=$q";
	
	}
		
	return $url;
}


sub determine_err_msg {
	my $err = shift;
	my $msgs = shift;
	return '' unless $err;
	
	if ( $err eq "1" )  { 
	   $err = $msgs->{4};
	} elsif ( $err eq "2" ) { 
	   $err = $msgs->{5};
	} elsif ( $err eq "3" ) { 
	   $err = $msgs->{6};	  
	} elsif ( $err eq "4" ) {
	   $err = $msgs->{7};  
	} elsif ( $err eq "5" ) {
	   $err = $msgs->{8};
	} elsif ( $err eq "6" ) {
	   $err = $msgs->{9};
	} elsif ( $err eq "7" ) {
	   $err = $msgs->{10};
	} elsif ( $err eq "8" ) {
	   $err = $msgs->{11};
	} 
	return $err;
}

sub create_pulldown {

	my $site = shift;
        my $form = shift;
        my $dbh = shift;

	my ($q,$lineobjid,@lineobjids,$currency,@currencies,$tempcurrences);

        $q = "SELECT lineobjid from tcart where sid=$form->{'sid'}";
        my $sth = $dbh->prepare($q);
        $sth->execute();

        while ( $lineobjid = $sth->fetchrow ) {
     
            push @lineobjids, $lineobjid; 

        }
        $sth->finish();    

        my $first = 1;

        foreach $lineobjid (@lineobjids) {


            $q = "SELECT currency from prices where objid='$lineobjid' and status='15000-A'";

            my $sth = $dbh->prepare($q);
            $sth->execute();
            if ( $first == 1 ) {
                while ( $currency = $sth->fetchrow ) {

                    push @currencies, $currency;

                }    
                $first = 0; 
            } else {

                 while ( $currency = $sth->fetchrow ) { 

                     $tempcurrences .= $currency;

                 }

                 foreach $currency (@currencies) {

                     if ( $currency !~ /$tempcurrences/ ) {

                         shift @currencies; 
                     
                     }  
                 

                 }




            } 


        }
        $sth->finish();


}

sub update_row_values {
	my ($form, $objid, $availquant, $err) = @_;
	
	## set the lineid to a form variable
	#
	$form->{'newrow_objid'} = $objid;

	my $q_err = "row.".$objid.".quant_err";
	my $q_inp = "row.".$objid.".quant_inp";

	$form->{$q_err} = $err if ( $err ); 
	$form->{$q_inp} = $availquant;

	return $form;
}

sub find_objs_to_delete {
	#warn "\tIn find_objs_to_delete()...\n";
	my $form = shift;
	my $obj_del = {};
	foreach my $key (keys %{$form}) {
		# Skip it unless it descvribes something to remove,
		# Either a zero quant or a kill button
		next unless ( 
			($key =~ /kill$/ && $form->{$key} eq "on")  || 
			($key =~ /quant_inp$/ && $form->{$key} eq "0")
		);
		my $objid = ( split /\./, $key )[1];
    	next if ($objid < 1);
		#warn "\tDelete object: $objid\n";
		$obj_del->{$objid} = 1;
	}
	#warn "\tLeaving find_objs_to_delete()...\n";
	return $obj_del;
}

sub delete_objects {
	#warn "\tIn delete_objects()...\n";
	my ($site, $form, $obj_del) = @_;
	
	# The keys are all we care about. Vals are just placeholders. 
	# Hash used so that we know objid's are unique in the list.
	my @objs = keys %{$obj_del};
	# If none, just return
	#warn "\tI have found ".($#objs + 1)." object(s) to delete.\n";
	return unless ($#objs + 1);
	
#	# This join will end up OR'ing all the id's together.
#	my $q_objs = join qq|' OR objid = '|, @objs;
	
	my $db_cart = $site->{db_cart} || 'tcart';
	my $id_field = 'sid';
	my $id_value = $form->{'sid'};
	
	if ($site->{'custid'} && ($db_cart eq 'pcart')) {
		$id_field = 'custid';
		$id_value = $site->{'custid'};
	}
 
        my $rv;
      foreach my $q_objs (@objs) {	
	# Build the delete query
	#my $q = qq|DELETE FROM $db_cart where objid = '$q_objs' AND $id|;
	my @where;
	$where[0] = &build_where_hash("objid", $q_objs, "and");
	$where[1] = &build_where_hash($id_field, $id_value, "and");
	my $delete_q = {
		'statement' => 'delete',
		'TABLE' => 
			{
			 'NAME'  => $db_cart,
			 'WHERE' => \@where
			}
	};
	
	my $q = &build_sql($delete_q, $site, $site->{dbh}, 0);
	
	#warn "\tDelete query is: $q\n";
	# And run it
	my $sth = $site->{dbh}->prepare($q);
	$rv = $sth->execute();
	$sth->finish();
      } # end of foreach
	
	#warn "\tLeaving delete_objects()\n";
	return $rv;
}

	
sub validate_quant_adjust {
	my ($form, $key) = @_;
	
	# Bail if this isn't an update element
	return 0 if (($key !~ /^row/       ) || 
			 ($key !~ /quant_inp/  ) || 
			 ($form->{$key} eq 'no') ||
			 ($key =~ /quant_inp$/ && $form->{$key} eq "0")
	        );
	
	# skip remove items 
	my $kill_key = $key;
	$kill_key =~ s/quant_inp/kill/;	
	return 0 if ( exists($form->{$kill_key}) and ( $form->{$kill_key} eq "on" )); 
	
	# skip newrow
	my $row_id = ( split /\./, $key )[1];
	return 0 if ( $row_id eq $form->{'newrow_objid'} );

	# Check that there's a diff quantity
	my $pre_quant = $key;
	$pre_quant =~ s/quant_inp/quant_pre/;
	
	# Return false unless quant_pre (old) is diff from quant_inp (new)
	return 0 unless ( $form->{$key} != $form->{$pre_quant} );
	
	# If we got there, then the key is good!
	# Return the id, and the old quantity.
	# New (or adjust) quant is in $form->{$key}
	return ($row_id, $form->{$pre_quant});
}


sub adjust_quantity {
	my ($form, $site, $key, $row_id, $old_quant) = @_;
	
	my $objid=$form->{"row.".$row_id.".lineobjid"};
	my $sku;
   
	# promotional sku
	# do not allow quantity changes
	return 1 if ( $form->{$sku} =~ /^xpr/ );
	
	# normal sku
	$form->{$key} = $old_quant + $form->{$key} if ( $form->{$key} < 0 );
	$form->{$key} = 0 if ( $form->{$key} < 0 );

	my ($obj_return_id,$quant,$err_code) = &cartaddcheck($form->{'add'},$form->{$key},$objid,$form->{'sid'},$form->{'cartclass'},'',$site->{'custid'},$form->{'currency'},$site);
	return ($obj_return_id,$quant,$err_code);
}



##############################################################################
=pod

=head2 get_cart_rows

get_cart_rows - &get_cart_rows($site, $dbh, $c_err, %FORM);

=over 4

=item DESCRIPTION

load rows for shopping cart.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$c_err - error message if exsited

%FORM  - A ReadParse version of the FORM to start with

=item OUTPUT

\%rows - a pointer to a hash which holds rows will be displayed in current cart

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

$count - number of rows returned

=item NOTES

=back

=cut
###############################################################################

sub get_cart_rows {
	my ($site, $dbh, %FORM) = @_;
	
	my ($items_sth) = &get_cart_items($site, \%FORM, $dbh);

	my %variant;
	my $count = 0;
	my %rows;
	
	while ( my $items = $items_sth->fetchrow_hashref('NAME_lc') ) {
		$count++;
    	my $row=$items->{'objid'};
		$rows{$row}=$items;

		##warn "Calling check_for_variants\n";
		my $return_rows = &check_for_variants($items, \%rows, $row, $dbh);
		%rows = %{$return_rows};

		## get price data
		#
		my $price_sth = &get_price_data(\%rows, $row, $site);

		for (1..2) {
			my $pp=$price_sth->fetchrow_hashref('NAME_lc');
			foreach my $key ( keys %{$pp} ) {
				next if ($key ne "unitpr");

				if ( $rows{$row}->{$key} > $pp->{$key} || $rows{$row}->{$key} eq "0.00" ) {
					$rows{$row}->{$key} = $pp->{$key} 
				}		
			} #/ done populating price data 
		} #/end of loading the specifics from variants for this one
		$price_sth->finish();

	} #/done loading fields

	# finish sth's
	$items_sth->finish();

	return (\%rows, \%FORM, $count);
}

##############################################################################
=pod

=head2 get_cart_items

get_cart_items - &get_cart_items($site, \%FORM, $dbh);

=over 4

=item DESCRIPTION

load items with the current sid from table 'tcart' in database

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

$dbh - current active database handle

=item OUTPUT

$sth - the query statement handler for retriving data from table 'tcart' 
       it holds all fields of row of data

=item NOTES

=back

=cut
###############################################################################
sub get_cart_items {
	my $site = shift;
	my $FORM = shift;
	my $dbh  = shift;
	my $cart = 'tcart';
	my $id = qq|sid='$FORM->{'sid'}'|;
	
	if ($FORM->{wishlist}) {
		$cart = 'pcart';
		$id = qq|custid='$site->{'custid'}'|;
	}
		
	my $query="SELECT * from $cart where $id AND objclass != '140-A' ";
	my $sth = $dbh->prepare( $query );
	my $rv = $sth->execute;
	return ($sth);
}

##############################################################################
=pod

=head2 check_for_variants

check_for_variants - &check_for_variants($items, \%rows, $row, $dbh);

=over 4

=item DESCRIPTION

Check if the row items is a variant, if so, add variant information to it

=item INPUT

$items - a hash reference for the retriving row of data from database

\%rows - a pointer to a hash which holds rows will be displayed in current cart
 
$row - a string, which holds the item's objid

$dbh - current active database handle

=item OUTPUT

$rows - a pointer to a hash which holds rows will be displayed in current cart

=item NOTES

=back

=cut
###############################################################################
sub check_for_variants {
	###warn "  In check_for_variants...\n";
	my $items  = shift;
	my $rows   = shift;
	my $obj_id = shift;
	my $dbh    = shift;
	
	my $variants_sth = &get_variants($items, $dbh);
		
	while ( my $variants = $variants_sth->fetchrow_hashref('NAME_lc') ) {
      ##warn "  We have a variant...\n";
	  foreach my $key ( keys %{$variants} ) {
         $rows->{$obj_id}->{$key} = $variants->{$key} unless ( $rows->{$obj_id}->{$key} );
      }

      ## do we have a variant
      #
      if ( length($variants->{'ref_from'}) >2 ) {

         my $vardescript="";
         my $vcode="";

         if ( $variants->{'lineobjid'} ne $variants->{'ref_from'} ) {
            $vcode = $variants->{'vcode'};
            
			my $from_sth = &get_ref_from($variants, $dbh);
			
			my $ref_from = $from_sth->fetchrow_hashref('NAME_lc');
			$from_sth->finish();
            my @vcodes = split /:/, $vcode;
            my @axisname = split /:/, $ref_from->{'axisnames'};
            foreach my $axis (@axisname) {
            	my $body = &get_extraprop_body($axis, $dbh);
				my @valuepairs = split /\r\n/, $body;
            	
				foreach my $pair (@valuepairs) {
            		my ($key,$value) = split/:/, $pair;
            		$variants->{$key}=$value;
            	}
            	
				my $code = shift @vcodes;
            	$vardescript .= " ".$variants->{$code};
            }

            $variants->{'title'} = $ref_from->{'title'}.': '.$vardescript;
 
         } 

         foreach ( keys %{$variants} ) {
            $rows->{$obj_id}->{$_} = $variants->{$_} unless ( $rows->{$obj_id}->{$_} );
         }                                           
      } #/done checking for variants
   }
   $variants_sth->finish();
   
   ##warn "  Leaving check_for_variants...\n";
   return $rows;
}

##############################################################################
=pod

=head2 get_variants

get_variants - &get_variants($items, $dbh); 

=over 4

=item DESCRIPTION

get the variant from table 'things'

=item INPUT

$items - a hash reference for the retriving row of data from database

$dbh - current active database handle

=item OUTPUT

$sth - the query statement handler for retriving data from table 'things' 
       it holds all fields of row of data

=item NOTES

=back

=cut
###############################################################################
sub get_variants {
	my $h   = shift;
	my $dbh = shift;
	
	my $q="SELECT * from things where objid='$h->{'lineobjid'}' ";
	my $sth=$dbh->prepare( $q );
	$sth->execute;
	
	return ($sth);
}

##############################################################################
=pod

=head2 get_ref_from 

get_ref_from - &get_ref_from($variants, $dbh); 

=over 4

=item DESCRIPTION

get the ref_from from table 'things'

=item INPUT

$variants - a hash reference for the retriving row of data from database

$dbh - current active database handle

=item OUTPUT

$sth - the query statement handler for retriving data from table 'things' 
       it holds all the fields of row of the data.

=item NOTES

=back

=cut
###############################################################################
sub get_ref_from {
	my $h   = shift;
	my $dbh = shift;
	
	my $q = "SELECT * from things where objid='$h->{'ref_from'}' ";
    my $sth = $dbh->prepare( $q );
    my $rv = $sth->execute;
    return ($sth);
}

##############################################################################
=pod

=head2 get_extraprop_body 

get_extraprop_body - &get_extraprop_body($axis, $dbh);

=over 4

=item DESCRIPTION

get the content of a variant from table 'extraprop'

=item INPUT

$axis - a string, which is the variant objid

$dbh - current active database handle

=item OUTPUT

$body - a string, which is the data of the field 'body' from table 'extraprop' 

=item NOTES

=back

=cut
###############################################################################
sub get_extraprop_body {
	my $axis = shift;
	my $dbh  = shift;
	
	my $s="SELECT body FROM extraprop WHERE objid='$axis' AND fieldname='encoding'";
    my $sth=$dbh->prepare($s);
    $sth->execute;
    my ($body)=$sth->fetchrow;
	$sth->finish();
	return $body;
}


##############################################################################
=pod

=head2 get_price_data

get_price_data - &get_price_data(\%rows, $row, $site); 

=over 4

=item DESCRIPTION

get the query statement handler for data from table 'prices'

=item INPUT

\%rows - a pointer to a hash which holds rows will be displayed in current cart
 
$row - a string, which holds the item's objid

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$sth - the query statement handler for retriving data from table 'prices' 
       it holds all the fields of row of data.

=item NOTES

=back

=cut
###############################################################################
sub get_price_data {
	my $rows      = shift;
	my $curr_row  = shift;
	my $site	  = shift;
	
	my $p="SELECT * FROM prices WHERE objid='$rows->{$curr_row}->{'lineobjid'}' and currency='$rows->{$curr_row}->{'currency'}' "; 
	$p.=" AND status='15000-A' ";
	my $sth=$site->{dbh}->prepare($p);
	$sth->execute;
	
	return $sth;
}

##############################################################################
=pod

=head2 get_lang_interf 

get_lang_interf - &get_lang_interf($FORM{'sid'}, $site->{dbh});

=over 4

=item DESCRIPTION

get the language and interface for the current session

=item INPUT

$FORM{'sid'} - sid for current session

$site->{dbh} - current active database handle

=item OUTPUT

$sth - the query statement handler for retriving data from table 'sessions' 
       it holds the fields 'language' and 'interface'

=item NOTES

=back

=cut
###############################################################################
sub get_lang_interf {
	my $sid = shift;
	my $dbh = shift;
	
	my $q="SELECT language,interface from sessions where sid=$sid";
	my $sth = $dbh->prepare( $q );
	$sth->execute;
	return $sth;
}

##############################################################################
=pod

=head2 cart_perm_template

cart_perm_template - &cart_perm_template($site, $interface);

=over 4

=item DESCRIPTION

load all necessary sub templates shown in cart

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$interface - a string, which is the interface that current session is using

=item OUTPUT

%templates - a hash holds all necessary sub templates shown in cart
             including message, header, list, subtotal, and buttons

=item NOTES

=back

=cut
###############################################################################
sub cart_perm_template {
	my ($site, $interface, $wishlist) = @_;
	
	my $display = 'cart';
	if ($wishlist) {
		$display = 'wishlist';
	}
	
	my $templates = {};
	$templates->{'cell_'.$display.'_message'}=
   		localize_template("cell_".$display."_message",$interface,0, $site);
	$templates->{'cell_'.$display.'_header'}=
   		localize_template("cell_".$display."_header",$interface,0, $site);
	$templates->{'cell_'.$display.'_list'}=
   		localize_template("cell_".$display."_list",$interface,0, $site);
	$templates->{'cell_'.$display.'_subtotal'}=
   		localize_template("cell_".$display."_subtotal",$interface,0, $site);
	$templates->{'cell_'.$display.'_buttons'}=
   		localize_template("cell_".$display."_buttons",$interface,0, $site);
	
	return $templates;
}

##############################################################################
=pod

=head2 new_item_row 

new_item_row - &new_item_row(\%rows, \%FORM, \%templates);

=over 4

=item DESCRIPTION

load in new rows into cart

=item INPUT

\%rows - a pointer to a hash which holds rows will be displayed in current cart

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

\%templates - a pointer for hash which holds all necessary sub templates shown in cart

=item OUTPUT

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

\%templates - a pointer for hash which holds all necessary sub templates shown in cart

=item NOTES

=back

=cut
###############################################################################
sub new_item_row {
	my ($rows, $form, $templates) = @_;
	my %rows = %{$rows};
	my %FORM = %{$form};
	my %templates = %{$templates};
	
	my $display = 'cart';
	if ($FORM{'wishlist'}) {
		$display = 'wishlist';
	}
	
	if ( $FORM{'newrow_objid'} ) {
		my $row=$FORM{'newrow_objid'};
		my $h=$rows{$row};
		my $k="row\.$row\.quant_err";
		$h->{'quant_err'}=$FORM{$k} if ( $FORM{$k} );

		## cleanup decimals on prices
		#
		$h->{'unitpr'} = nicedecimals( $rows{$row}->{'unitpr'} );

		## permute and build cells

		my $u=permutebit( $templates{'cell_'.$display.'_list'}, $h );
		$u=permuteclean( $u );
		$FORM{'newrow_cell_'.$display.'_list'}=$u;
		$FORM{'newrow_cell_'.$display.'_message'}=$templates{'cell_'.$display.'_message'};
		$FORM{'newrow_cell_'.$display.'_header'}=$templates{'cell_'.$display.'_header'};
		$FORM{'newrow_cell_'.$display.'_buttons'}=permutebit( $templates{'cell_'.$display.'_buttons'}, \%FORM );
	}

	return (\%templates, \%FORM);
}

##############################################################################
=pod

=head2 old_item_row 

old_item_row - &old_item_row(\%FORM, \%rows, \%templates); 

=over 4

=item DESCRIPTION

load in the exsited rows into cart

=item INPUT

\%FORM - a pointer to a hashref which will be used to describe the current page to display
       - Sets up all CGI relevant information

\%rows - a pointer to a hash which holds rows will be displayed in current cart

\%templates - a pointer for hash which holds all necessary sub templates shown in cart

=item OUTPUT

%FORM - A hashref which will be used to describe the current page to display
      - Sets up all CGI relevant information

=item NOTES

=back

=cut
###############################################################################
sub old_item_row {
	my ($frm, $rws, $tmpl) = @_;
	my %FORM = %{$frm};
	my %rows = %{$rws};
	my %templates = %{$tmpl};
	
	my $u="";

	my $display = 'cart';
	if ($FORM{'wishlist'}) {
		$display = 'wishlist';
	}
	
	$FORM{'newrow_objid'}=0 if ( ! defined($FORM{'newrow_objid'}) );
		my $skip;
	   foreach my $r ( keys %rows ) {
    	  $rows{$r}->{'sid'}=$FORM{'sid'};
    	  next if ( $r eq $FORM{'newrow_objid'} || $skip );
    	  my $k="row\.$r\.quant_err";
    	  $rows{$r}->{'quant_err'}=$FORM{$k};

    	  $rows{$r}->{'unitpr'} = nicedecimals ( $rows{$r}->{'unitpr'} ) ;

    	  $rows{$r}->{'objid'}=$r;

    	  my $ub=permutebit( $templates{'cell_'.$display.'_list'}, $rows{$r} );
    	  $ub=permuteclean( $ub );
    	  $u.=$ub;
	   } #/end of constructing old rows     

	if ( $u ) {
	   $FORM{'cell_'.$display.'_header'}=$templates{'cell_'.$display.'_header'};
	   $FORM{'cell_'.$display.'_buttons'}=permutebit( $templates{'cell_'.$display.'_buttons'}, \%FORM );
	   $FORM{'cell_'.$display.'_list'}=$u;
	}
	
	return \%FORM;
}

##############################################################################
=pod

=head2 put_checkout_back

put_checkout_back - &put_checkout_back($form, $dbh); 

=over 4

=item DESCRIPTION

this function will check if there is any item in shopping cart has been checked 
out before coming into shopping cart, if so, increment inventory lstock using
hqaunt, and set hqaunt and squant in tcart to 0 for this item. This means that
after running this function, any items have been checked out before coming in 
shopping cart will be treated as new items, they have to be checked out again.
 
=item INPUT

$form - a pointer to a hashref which will be used to describe the current page to display
      - Sets up all CGI relevant information
      - the purpose for inputing it is to pass the sid into the function

$site - Standard $site hashref from &setsite()


=item OUTPUT

There is no output for this function. It updates the table tcart with hqaunt and squant, 
and inventory table with lstock.

=item NOTES

=back

=cut
###############################################################################
sub put_checkout_back {

    my $form = shift;
    my $site = shift;

    my $dbh = $site->{'dbh'};
    my $now = time();

    my $q = " SELECT * from tcart where SID='$form->{'sid'}' ";
       $q .= "AND custid='$site->{'custid'}' ";
    my $stss = $dbh->prepare($q);
       $stss->execute;

    my $first = 1;
    my %del_orderids;
    my %item;
    my $remain_orderid;
    while (my $h=$stss->fetchrow_hashref('NAME_lc')) {

        # just leave one order id for current sid and custid
        # others marked as delete
        if ($first) { 
            $remain_orderid = $h->{'orderid'};
            $first = 0;
        } else {
            # store delete order id
            $del_orderids{$h->{'orderid'}} = 1;
        }

        ## delete shipping and wrapping rows
        #  put lineobjid's data back together from separate orders
        if (($h->{'objclass'} eq "80001-A") || ($h->{'objclass'} eq "140-A") ) {
            my $q = "DELETE FROM tcart WHERE objid='$h->{'objid'}' ";
            my $stq = $dbh->prepare( $q );
               $stq->execute;
        } else { 
            # put each lineobjid back together
            if ($item{$h->{'lineobjid'}}{'quant'}) {
                $item{$h->{'lineobjid'}}{'quant'} += $h->{'quant'};
                $item{$h->{'lineobjid'}}{'hquant'} += $h->{'hquant'};
            } else {
                $item{$h->{'lineobjid'}}{'quant'} = $h->{'quant'};
                $item{$h->{'lineobjid'}}{'hquant'} = $h->{'hquant'};
                $item{$h->{'lineobjid'}}{'objid'} = $h->{'objid'};
                $item{$h->{'lineobjid'}}{'unitpr'} = $h->{'unitpr'};
            } # end of inner if else

            # delete rows not necessary
            if ($h->{'objid'} ne $item{$h->{'lineobjid'}}{'objid'}) {
                my $q = "DELETE FROM tcart WHERE objid='$h->{'objid'}' ";
                my $sth = $dbh->prepare( $q );
                   $sth->execute;
            } # end of if

        } # end of outer if else

    } # end of while

    ## put each lineobjid in one row
    foreach my $lineobjid (keys %item) {
        my $total = &nicedecimals($item{$lineobjid}{'quant'}*$item{$lineobjid}{'unitpr'});
        my $q = "UPDATE tcart SET quant='$item{$lineobjid}{'quant'}', ";
           $q .= "hquant='0', squant='0', mdate='$now', ";
           $q .= "orderid='$remain_orderid', linetotal='$total' ";
           $q .= "WHERE objid='$item{$lineobjid}{'objid'}' "; 
        my $sth = $dbh->prepare( $q );
           $sth->execute;
    } # end of foreach

    ## give stock back to inventory
    foreach my $lineobjid (keys %item) {
        my $q = "UPDATE inventory SET lstock = lstock+$item{$lineobjid}{'hquant'} ";
           $q .= "WHERE objid='$lineobjid' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
           $sth->finish;
    } # end of foreach

    ## put remain orderid into delete, because need to clear other tables
    $del_orderids{$remain_orderid} = 1;

    ## delete rows in orders and extraprop tables
    foreach my $orderid (keys %del_orderids) {
        my $q = "DELETE FROM orders WHERE objid='$orderid' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
           $sth->finish;

        $q = "DELETE FROM extraprop WHERE objid='$orderid' ";
        $sth = $dbh->prepare( $q );
        $sth->execute;
        $sth->finish;
    } # end of foreach

} # end of put_checkout_items_back


##############################################################################
=pod

=head2 message

message - &message($om_render_lang, $scriptname, $relpath); 

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This routine loads in the
messages for the $site->{'om_render_lang'} that is currently active.

=item INPUT

$om_render_lang - a string, which is the directory for the language being used 
                  for the current session

$scriptname - The name of the script 

$relpath - The relative path to files that must be required

=item OUTPUT

%script_msg - a hash, which holds the error message need to used in the script

=item NOTES

=back

=cut
###############################################################################
sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below
	my %script_msg;
	my ($om_render_lang, $scriptname, $relpath) = @_;
	
   if ($om_render_lang eq "eng" ) {
      $script_msg{1}="We're sorry! Limit one per order!";
      $script_msg{2}="You currently have no items in your cart.";
      $script_msg{3}="Sorry you must log in before you can place items into your Wishlist";
      $script_msg{4}="Sorry quantity is limited to this item. ";
      $script_msg{5}="Sorry, we have a quantity limit on this item.";
      $script_msg{6}="Sorry, but this item is now out of stock.";
      $script_msg{7}="This item cannot be added to your cart.";
      $script_msg{8}="You can not add the item you just select into the cart at this time.";
      $script_msg{9}="The item you just selected has been discontinued.";
      $script_msg{10}="The item you just selected has been recalled.";
      $script_msg{11}="You have exceeded the purchase limit.  Delete an item or go to checkout and purchase the items you have now before adding anything else to your cart.";
	  $script_msg{12}="You currently have no items in your wishlist.";


   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$scriptname.$om_render_lang.messages");
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         my @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }
      close(EM);
   }
   return %script_msg;
}


1;
