#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#  ccvs.pm
#  HVC 20000511
#  make OS AllCommerce communicate with CCVS (Credit Card Verification System)
#
# Version 0.9.5 IB 20000601
#    wrong header code 
# Version 0.9.4 HVC 20000523
#    Read CCVS related info from site.pm
#    Handle ERROR conditions
# Version 0.9.3 HVC 20000522
#    Remove ERLOG
# Version 0.9.2 HVC 20000522
#    Clean code
#    Use sockets to comunicate with CCVS (helps if CCVS is on a
#    machine than open merchant)
#    Blow fisher algorithm for encryption/decryption
# Version 0.7.1 HVC 20000511
#    open merchant communicate with CCVS specific API's

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
use strict;
use Cwd;
my $separator = '/'; # What separates dirnames in this architecture?
my ($scriptname, $args, $rundir);
($scriptname, $args) = split /\?/, $0;
if ($scriptname =~ /$separator/) {
	$scriptname =~ /^(.*)$separator(.*?)$/;
	$rundir = $1;
	$scriptname = $2;
} else {
	$rundir = cwd();
}
my @path = split(/$separator/, $rundir);
my ($relpath, $siteshortname);

## short circuit if $s does not contain cgi-bin
# This happens in scripts linked from the top-level
if ( $rundir !~ /cgi-bin/ ) {
   $siteshortname = pop @path;
   $relpath=".";
   @path=();
} 

while (my $d = pop @path) {
	$relpath .= "..$separator";
	if ($d eq 'cgi-bin') {
		$relpath .= "..$separator";
		last;
	}
}
$siteshortname = pop @path;
$relpath .= $siteshortname;

require "$relpath/conf/site.pm";

#get site information
my $site = setsite();          


# Libraries to communicate with CCVS
use Socket;
use IO::Socket;
require "$relpath/lib/blowfisher.pm";


sub run_ccvs {
   my($z, $o);
   $z = $_[0];             # incoming parameters
   $o={};                  # initialize return hashref

   ## make certain everything has defaults that need them...
   #
   $z->{'testing'}=0 if ( ! $z->{'testing'} );
   $z->{'TRKIND'}="to" if ( ! $z->{'TRKIND'} );
   $z->{'orderid'}=0 if ( ! $z->{'orderid'} );
   $z->{'SID'}=0 if ( ! $z->{'SID'} );
   $z->{'SID'}=$z->{'sid'} if ( exists($z->{'sid'}) and $z->{'SID'}==0 );
   $z->{'custnum'}='' if ( ! $z->{'custnum'} );
   $z->{'TOTAL'}=0 if ( ! $z->{'TOTAL'} );
   $z->{'PAYVIA'}='' if ( ! $z->{'PAYVIA'} );
   $z->{'name'}='' if ( ! $z->{'name'} );
   $z->{'CCID'}='' if ( ! $z->{'CCID'} );
   $z->{'CCEXPMO'}=$z->{'CCEXPMO_inp'} if ( ! $z->{'CCEXPMO'} );
   $z->{'CCEXPYR'}=$z->{'CCEXPYR_inp'} if ( ! $z->{'CCEXPYR'} );   
   $z->{'address1'}='' if ( ! $z->{'address1'} );
   $z->{'city'}='' if ( ! $z->{'city'} );
   $z->{'state'}='' if ( ! $z->{'state'} );
   $z->{'country'}='usa' if ( ! $z->{'country'} );
   $z->{'postcode'}='' if ( ! $z->{'postcode'} );

   #talk to CCVS
   $o = mck_ccvs_payment($z);

   #TODO - make entry in required tables

   return $o;

} #/run_ccvs




sub mck_ccvs_payment {
  my $z    = shift;             # incoming parameters
  my $site = shift;
  
  my $remote_host = $site->{'site_ccvs_server'};
  my $remote_port = $site->{'site_ccvs_port'};
  my $flag = "DONE\n"; #signal the end of sending critical information
  my $answer;
  
  # Extract information from FORM
  my $invoice_id = $z->{'orderid'};
  my $creditcardnumber = $z->{'CCID'}  if ( $z->{'CCID'} );
  my $expirationdate = sprintf("%s/%s",$z->{'CCEXPMO'},$z->{'CCEXPYR'});
  my $cost = $z->{'ordtotal'};  

  #list of strings to be encrypted
  my @plain_list = ($invoice_id, $creditcardnumber, $expirationdate, $cost);
  my $count=0;
  
  #Encrypt strings
  my @encrypted_list;
  foreach my $each_plain_string (@plain_list) {
    if(length($each_plain_string)<=8) {
	  $encrypted_list[$count]=&om_encrypt($each_plain_string);
      $encrypted_list[$count].="\n";
      $count ++;
      $encrypted_list[$count]="END";
      $encrypted_list[$count].="\n";
      $count ++; 
	} else {
      my $len = length($each_plain_string);
	  my $x;
      for($x=0; $x < $len; $x=$x+8){
	    my $result = substr($each_plain_string, $x, 8);
	    $encrypted_list[$count] = &om_encrypt($result); 
	    $encrypted_list[$count] .= "\n";
	    $count ++;
	  }
      $encrypted_list[$count] = "END";
      $encrypted_list[$count] .= "\n";
      $count ++;
	}
  }

  #Establish connection with server
  my $socket= IO::Socket::INET->new(
                                PeerAddr=>$remote_host,
				                PeerPort=>$remote_port,
				                Proto=>"tcp",
				                Type=>SOCK_STREAM) 
    or &handle_error("connection to server failed :$@");
  
  #Send critical information to server
  foreach my $each_encrypted_string (@encrypted_list) {
    $socket->send($each_encrypted_string) or &handle_error("can't send message");
  }
  
  $socket->send($flag) or &handle_error("can't send message");
   
  #Wait to hear back from server until flag=DONE
  my @answer;
  my $i = 0;
  do {
    $answer[$i] = <$socket>;
    $i++ ; 
  } while ( ($answer[$i-1] ne "DONEe\n") && ($answer[$i-1] ne "DONE\n") ) ;
  
  my $status;
  if ($answer[$i-1] ne "DONEe\n"){
    #Decrypt the response from CCVS
    chomp($answer[0]);chomp($answer[0]);
    $status = &om_decrypt($answer[0], $site);
    $z->{'STATUSDVALUE'} = $status;
  } else {
    $status = $answer[0];
    $z->{'STATUSDVALUE'} = $status;
  }
  
  #TODO - Handle events accordingly

  #Close up connection
  close($socket); 
  return $z; 
}
  

sub om_encrypt {
	my $input = shift;
	my $site  = shift;
	my $key = $site->{'hash_key'};
	my $encrypt=unpack("H16",os_hash($input,$key,0));
	return $encrypt; 
}

sub om_decrypt {
	my $input = shift;
	my $site  = shift;
	my $key = $site->{'hash_key'};
	my $decrypt = os_unhash( pack("H16",$input),$key,0);
	return $decrypt; 
}


sub handle_error{
	print "Content-type: text/html\n\n"; 
	print "<HTML><HEAD><TITLE>ccvs.pm messages</TITLE></HEAD><BODY><BR><BR>$_[0]</BODY></HTML>";
	print "\n\n";
	exit -1;
}

