#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#####
# cgi-lib.pm
#
#####
use strict;

##############################################################################
=pod

=head1 cgi-lib.pm

cgi-lib.pm - library to handle CGI specific routines

=head1 DESCRIPTION

This library is used for all subroutines that are used for CGI programming.
This allows CGI calls to be hidden in the main CGI scripts.

=head1 NOTES & BUGS

=cut

##############################################################################

#####
# Version 3 TG 200000918
#    Moved ReadParse to ReadParse, retrofit to development standards
# Version 2 LCF 20000828
#    Made ReadParse return a ref to %FORM as a second return value,
#    for scripts that don't use globals.
# Version 1 TG 20000823
#    Added ReadParse2 to use for temporary retrofit for standards
# Version 0.7.4 IB 20000302
#    updated copyright and removed -w
# Version 0.7.3 RF 991027
#    removed the setserver() function as obsolete
# Version 0,7.2 IB 991027
#    eliminated csid code, because it was obsolete
# ditto\ 990830 GF - myhome sambura configuration
# ditto\ 990817 RF - myhomedev localization
# Version 0.7.1 GF 990617
#    cleaned up for release
#
#####


##############################################################################
=pod

=head2 ReadParse

ReadParse - &ReadParse();

=over 4

=item DESCRIPTION

To read the environment information that is set up for CGI programming,
and store the information in a hashref for use throughout the CGI script

=item INPUT

None 

=item OUTPUT

$FORM  - Hashref which will be used to describe the current page to display.
       - Sets up all CGI relevant information

=item NOTES

=back

=cut
###############################################################################
sub ReadParse {
    
    my($in,@in,$k,$v, %in, $kv);
    my (%FORM);
    
    $in="";
    @in=();
    %FORM=();
    %in=();    # for compatability with previous passing
    
    ## NOTE - in case of duplication of a key in multiple data
    #    passing methods, the precedence is COOKIE>POST>GET
    #
    
    # default form method to "GET" (degenerates to a simple URL)
    $ENV{'REQUEST_METHOD'}="GET" if (! $ENV{'REQUEST_METHOD'} );
    
    # get the form data
    read(STDIN,$in,$ENV{'CONTENT_LENGTH'}) if ($ENV{'REQUEST_METHOD'} eq "POST");
    $in .= $ENV{'QUERY_STRING'} if ( $ENV{'QUERY_STRING'} );
    
    # split into key-value entries (entry later split on '=')
    @in = split(/&/,$in);
    
    # cookie data
    @in=(@in,split(/;(\s*)/, $ENV{'HTTP_COOKIE'})) if (defined($ENV{'HTTP_COOKIE'}));
    
    while (@in) {
        $kv = shift @in;
        # backconvert plus to space
        $kv =~ s/\+/ /g;
        # de-hexify hex encoded bits
        $kv =~ s/%(..)/pack("c",hex($1))/ge;
        
        # split key and value
        ($k,$v) = split(/=/,$kv,2);
        
        # create hash entries
        # separate multiple values with \0
        if ( $k ) {
            if (exists($FORM{$k})) {
                if (defined($FORM{$k})) {
                    if ( $k =~ /^sid$/ ) {
                        $FORM{$k}=$v;
                        } else {
                        $FORM{$k}.="\0$v";
                    }
                    } else { # key exists with undef value
                    $FORM{$k}=$v;
                } # done checking for defined key
                } else {
                $FORM{$k}=$v;
            } # done cheking for existing $k
        } # end of "if $k" bit
    }
    
    %in=%FORM;   # for backwards compatability
    
    return \%FORM;
} #/ReadParse

1; #return true
