#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# checkout-lib.pm
#
# Version 1 NC 20001125
#
###########################################################################
=pod

=head1 NAME 

checkout-lib.pm

=head1 SYNOPSIS

This is a function library for checkout.cgi. 

=head1 DESCRIPTION

All the subroutines that are only used by checkout.cgi are here

=head1 NOTES & BUGS

=cut
############################################################################# 

use strict;

##############################################################################
=pod

=head2 init_script

init_script - &init_script();

=over 4

=item DESCRIPTION

Initialize the script, load in %site and %form, setup dbh, language, interface, and so on.

=item INPUT

=item OUTPUT

$site         - Standard $site hash from &setsite

\%form        - Reference to standard %form hash from &ReadParse

=item NOTES

=back

=cut
###############################################################################
sub init_script {

    ## Get the site hash
    #
    my $site = &setsite();

    ## get the FORM bits
    #
    my $form = &ReadParse();

    ## establish connection to database  
    #
    $site->{'dbh'} = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}) || die "Can't connect to database!: $!\n";

    ## Get the language and interface, if possible
    #
    if ($form->{'sid'}) {

       ##
       # decrypt sid
       $form->{'sid'} = &decrypt_sid($form->{'sid'}, $site);

       ##
       # Get data
       my $q = "SELECT language, interface from sessions where sid = '$form->{'sid'}' ";
       my $sth = $site->{'dbh'}->prepare($q);
       $sth->execute;

       ($site->{'om_render_lang'}, $site->{'interface'}) = $sth->fetchrow_array;   
       if (!$site->{'om_render_lang'}) { 
           $site->{'om_render_lang'} = $site->{'om_locale_lang_def'} || "eng";
       }
       if (!$site->{'interface'}) { 
           $site->{'interface'} = "html";
       }

    }

        return ($site, $form);

} # end of sub


##############################################################################
=pod

=head2 get_pagelist

get_pagelist - get_pagelist();

=over 4

=item DESCRIPTION

Setup the hashref for page list which contains the function for
each page

=item INPUT

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
###############################################################################
sub build_pagelist {

    my $pagelist = {
           orderrecap          => \&pg_orderrecap,
           singleshippingadd   => \&pg_singleshippingadd,
           addresscenter       => \&pg_addresscenter,
           multishippingadd    => \&pg_multishippingadd,
           giftwrap            => \&pg_giftwrap,
           shippingmethod      => \&pg_shippingmethod,
           payment             => \&pg_payment,
           orderpreview        => \&pg_orderpreview,
           orderconfirm        => \&pg_orderconfirm
       }; # end of $page_hashref

   return $pagelist;

}


##############################################################################
=pod

=head2 pg_orderrecap

pg_orderrecap - pg_orderrecap($page_hashref, $site);

=over 4

=item DESCRIPTION

this funtion does the logic for the first page of checkout
it will check the stock behaviors and stock level, 
then give out the list of available items, backorder items,
and unavailable items.

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_orderrecap {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};
    my %clist = (); # a hash contains the reference to the three types
                    # of products, available, backorder, and unavailable
    my %quant = (); # a same structure as %clist which just contains 
                    # quantity for permutebit 
    my $flag = 0;   # if there is any available or back order items
                    # it will set to 1, and can go next step of checkout
                    # if there is no available or back order items
                    # cannot go next setp of checkout, prompt customer to
                    # go back shopping 
    ## update custid for items in tcart
    &update_custid_in_tcart($form->{'custid'}, $form->{'sid'}, $dbh);
           
    ##
    # give the page template 
    $page_hashref->{'template'} = 'page_checkout_orderrecap';
    
    ##
    # retrieve data from tcart
    # select * from tcart where sid = $form{'sid'} and objclass != '140-A';

    my $q = "SELECT * FROM tcart WHERE sid = '$form->{'sid'}' AND objclass != '140-A' ";

    my $sth = $dbh->prepare($q);
    $sth->execute;

    while ( my $h = $sth->fetchrow_hashref("NAME_lc") ) {
        ##
        # get the product title
        $h->{'title'} = &get_title($h->{'lineobjid'}, $site->{'dbh'});

        ##
        # get the behavior for the product
        $q=" SELECT behavior, ref_from from things where objid='$h->{'lineobjid'}' ";
        my $stq= $dbh->prepare($q);
        $stq->execute;
        my ($behavior, $ref_from)= $stq->fetchrow_array;

        # if this product is a variant, 
        # it has no behavior by itself, 
        # have to get the behavior from its master product
        if (!$behavior) {
           $q=" SELECT behavior from things where objid='$ref_from' ";
           $stq=$dbh->prepare($q);
           $stq->execute;
           ($behavior)=$stq->fetchrow;
        }

        ##
        # get the local stock for the product
        $q="SELECT lstock from inventory where objid='$h->{'lineobjid'}' ";
        $stq=$dbh->prepare($q);
        $stq->execute;
        my ($instock) = $stq->fetchrow;

        ##
        # run through behavior logic and see if the user can purchase this item

        # now all hqaunt go available items
        # all squant go back order items
        # all quant more than hquant but cannot go squant, go unavailable items

        # 90002-A| JIT Saleable if no stock
        # check to see if quant of items are availavle from lstock
        # if they are, decrement from lstock move to hquant
        # else decrement from lstock move to squant
        # partial move avilable items to hquant, all others move to squant

        # 90005-A| Potentially restockable if no stock (if out of stock, don't sell)
        # check to see if quant of items are available from lstock
        # if they are, decrement from lstock move to hquant
        # else "error message - you cannot buy this item at this time, it is out of stock.

        # 90006-A| Saleable irrespective of stock
        # check to see if quant of items are availavle from lstock
        # if they are, decrement from lstock move to hquant
        # else decrement from lstock move to squant
        # partial move avilable items to hquant, all others move to squant

        # 90007-A| Discontinued
        # check to see if quant of items are availavle from lstock
        # if they are, decrement from lstock move to hquant
        # check to see if quant of items are available from lstock
        # if they are, decrement from lstock move to hquant
        # else "error message - you cannot buy this item at this time, it is out of stock.

        # 90008-A| Recalled
        # do not sell even if inventory exists

        # For all the behavior besides Recalled,
        # if the quantity of local stock is more than the quantity of the order,
        # decrement lstock by quantity ordered in inventory
        # and move quantity ordered to hquant
        if ($instock >= $h->{'quant'} && ($behavior ne "90008-A")) {

            $instock -= $h->{'quant'};
            $q= "UPDATE inventory SET lstock='$instock' ";
            $q.=" where objid='$h->{'lineobjid'}' ";
            $stq = $dbh->prepare($q);
            $stq->execute;

            $q = "UPDATE tcart set hquant='$h->{'quant'}' ";
            $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
            $q.="and custid='$h->{'custid'}' ";
            $stq = $dbh->prepare($q);
            $stq->execute;

            ## set quantity to hquant
            $clist{'avail'}{$h->{'lineobjid'}} = $h;
            $quant{'avail'}{$h->{'lineobjid'}}->{'list_quant'} = $h->{'quant'};
            ## setup flag
            $flag = 1;

        } else {

            if ($behavior eq "90002-A"){

                my $squant = $h->{'quant'} - $instock;
                $q= "UPDATE inventory SET lstock='0' ";
                $q.=" where objid='$h->{'lineobjid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                $q= "UPDATE tcart set hquant='$instock', squant='$squant' ";
                $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
                $q.="and custid='$h->{'custid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                ## if hquant not equal to 0, set quantity to hquant
                #  if hquant equal to 0, means not a available item
                if ($instock != 0) {

                    $clist{'avail'}{$h->{'lineobjid'}} = $h;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'list_quant'} = $instock;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($instock*$h->{'unitpr'});   
                }
              
                $clist{'back'}{$h->{'lineobjid'}} = $h;
                $quant{'back'}{$h->{'lineobjid'}}->{'list_quant'} = $squant;
                $quant{'back'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($squant*$h->{'unitpr'});                    ## setup flag
                $flag = 1;
    
            }
            elsif ($behavior eq "90005-A"){
         
                ## in this $behavior, if there is stock in inventory,
                #  reduce the inventory
                #  set stock quant to available quant
                #  set the difference between quant and stock to unavailable quant
                $q= "UPDATE inventory SET lstock='0' ";
                $q.=" where objid='$h->{'lineobjid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                $q= "UPDATE tcart set hquant='$instock' ";
                $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
                $q.="and custid='$h->{'custid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                ## if hquant not equal to 0, set quantity to hquant
                #  if hquant equal to 0, means not a available item
                if ($instock != 0) {
                    $clist{'avail'}{$h->{'lineobjid'}} = $h;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'list_quant'} = $instock;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($instock*$h->{'unitpr'});
                    ## setup flag
                    $flag = 1;
                }

                $clist{'unavail'}{$h->{'lineobjid'}} = $h;
                $quant{'unavail'}{$h->{'lineobjid'}}->{'list_quant'} = $h->{'quant'} - $instock;

            }
            elsif ($behavior eq "90006-A"){

                my $squant = $h->{'quant'} - $instock;
                $q= "UPDATE inventory SET lstock='0' ";
                $q.=" where objid='$h->{'lineobjid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                $q= "UPDATE tcart set hquant='$instock', squant='$squant' ";
                $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
                $q.="and custid='$h->{'custid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                ## if hquant not equal to 0, set quantity to hquant
                #  if hquant equal to 0, means not a available item
                if ($instock != 0) {

                    $clist{'avail'}{$h->{'lineobjid'}} = $h;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'list_quant'} = $instock;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($instock*$h->{'unitpr'});
                }

                $clist{'back'}{$h->{'lineobjid'}} = $h;
                $quant{'back'}{$h->{'lineobjid'}}->{'list_quant'} = $squant;
                $quant{'back'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($squant*$h->{'unitpr'});    
                ## setup flag
                $flag = 1;
            
            }
            elsif ($behavior eq "90007-A"){

                ## in this $behavior, if there is stock in inventory,
                #  reduce the inventory
                #  set stock quant to available quant
                #  set the difference between quant and stock to unavailable quant
                $q= "UPDATE inventory SET lstock='0' ";
                $q.=" where objid='$h->{'lineobjid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                $q= "UPDATE tcart set hquant='$instock' ";
                $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
                $q.="and custid='$h->{'custid'}' ";
                $stq = $dbh->prepare($q);
                $stq->execute;

                ## if hquant not equal to 0, set quantity to hquant
                #  if hquant equal to 0, means not a available item
                if ($instock != 0) {
                    $clist{'avail'}{$h->{'lineobjid'}} = $h;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'list_quant'} = $instock;
                    $quant{'avail'}{$h->{'lineobjid'}}->{'linetotal'} = &nicedecimals($instock*$h->{'unitpr'});
                    # setup flag
                    $flag =1;
                }

                $clist{'unavail'}{$h->{'lineobjid'}} = $h;
                $quant{'unavail'}{$h->{'lineobjid'}}->{'list_quant'} = $h->{'quant'} - $instock;
            }
       }

       if ($behavior eq "90008-A"){
           $q= "UPDATE tcart set hquant='0', squant='0' ";
           $q.="where objid='$h->{'objid'}' and sid='$h->{'sid'}' ";
           $q.="and custid='$h->{'custid'}' ";
           $stq = $dbh->prepare($q);
           $stq->execute;

           ## in this behavior, set the item to unavailable item
           $clist{'unavail'}{$h->{'lineobjid'}} = $h;
           $quant{'unavail'}{$h->{'lineobjid'}}->{'list_quant'} = $h->{'quant'};
       }

    }

    ## 
    # load in cells

    ## foreach category (available, backorder, and unavailable)
    foreach my $c (keys %clist) {

        ## load header
        my $cellheader = "cell_ck_300_".$c."hdr";
        $form->{$cellheader} = &loadtemplate("$cellheader","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
     
        ## load list
        my $celllist = "cell_ck_300_".$c."list";
        my $celltemplate = &loadtemplate("$celllist","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        # replace the real data in cell list template
        # foreach product under this category 
        foreach my $r (keys %{$clist{$c}} ) {

            ## load in quantity and linetotal first
            my $productlist = &permutebit( $celltemplate, $quant{$c}->{$r} );
            ## load in titel, msku, price, subtotal
            $productlist = &permutebit( $productlist, $clist{$c}->{$r} );
            ## load in template
            $form->{$celllist} .= $productlist; 
        }

    }

    if ( $flag == 1 ) {
        ## To direct the next page, if no address, go address center
        #  if at least one, go singleshipping page
        #  get the shipping address for current custid
        $q = "SELECT * from addresses where custid = '$form->{'custid'}' ";
        $sth = $dbh->prepare($q);
        $sth->execute;

        my $rows=0;
        while ($sth->fetchrow_hashref("NAME_lc")){
            $rows += 1;
        }

        # setup the current page and next page
        if ($rows == 0) {
            $form->{'current_page'} = 'orderrecap';
            $form->{'next_page'} = 'addresscenter';
        } else {
            $form->{'current_page'} = 'orderrecap';
            $form->{'next_page'} = 'singleshippingadd';
        }
    } else {
        $form->{'error'} = $site->{'msg'}->{1};
        $form->{'current_page'} = 'orderrecap';
        $form->{'next_page'} = 'orderrecap';
    } # end of if else

    # put $form back to $page_hashref and pass back
    $page_hashref->{'form'} = $form;

    return $page_hashref;

}


##############################################################################
=pod

=head2 pg_singleshippingadd

pg_singleshippingadd - pg_singleshippingadd($page_hashref, $site);

=over 4

=item DESCRIPTION

this funtion does the logic for the second page of checkout
this page will list out all the existed shipping address for current 
custid, if no available shipping address, then automatically show
the shipping address creation page. On, this page, if customer choose
to use multiple shipping address, then next page is pg_multishipadd

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_singleshippingadd {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};

    ## when first time come to this page
    #  update database for the previous step
    if ( ($form->{'current_page'} eq "orderrecap") ) {
        &update_database_for_orderrecap($form->{'sid'}, $form->{'custid'}, $form, $site);
    }

    ## 
    # load in template, by default the single shipping address
    # page is loaded in
    $page_hashref->{'template'} = 'page_checkout_single_shippingadd';

    ##
    # get the shipping address for current custid
    my $q = "SELECT * from addresses where custid = '$form->{'custid'}' ";
    my $sth = $dbh->prepare($q);
    $sth->execute;
   
    my $rows=0;
    while ($sth->fetchrow_hashref("NAME_lc")){
          $rows += 1;
    }
    ##
    # if there are more than two shipping address, this button will
    # be displayed
    if ($rows >= 2) {
       my $multiaddress_button = &loadtemplate("cell_ck_p2_btn_multiaddress","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
       $form->{'multiaddress_button'} = $multiaddress_button;
    }

    $form->{'address_pulldown'} = &get_address_pulldown($form->{'custid'}, $dbh);

    ## set current page and next page
    $form->{'current_page'} = 'singleshippingadd';
    $form->{'next_page'} = 'shippingmethod';

    # put $form back to $page_hashref
    $page_hashref->{'form'} = $form;

    return $page_hashref;

}


##############################################################################
=pod

=head2 pg_addresscenter

pg_addresscenter - pg_addresscenter($page_hashref, $site);

=over 4

=item DESCRIPTION

This is function allows customer to create new address
it will check the input and give error message if any error existed,
when no error exists, it will direct the page back to 
where it from (single address page or multiple address page) 
it also allow customers to edit their exsiting address.

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_addresscenter {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};
    my $addr;
    my %addressinfo = ();
    my $default_state; # to put into select template

    ## 
    # load in template
    $page_hashref->{'template'} = 'page_checkout_addressenter';

    ## if this page is directly from orderrecap
    #  update database for orderrecap
    if ( ($form->{'current_page'} eq "orderrecap") ) {
        &update_database_for_orderrecap($form->{'sid'}, $form->{'custid'}, $form, $site);
    }

    ##
    # check if this should be blank page to be shown
    # if current page is not equal to next page, this means that
    # this page should be blank and wait for input
    # if not, it means that check the error
    if ($form->{'current_page'} ne $form->{'next_page'}) {

        ## setup back page
        $form->{'return_page'} = $form->{'current_page'};

        ## if this page is directly from orderreap page,
        #  redirect to singleshippingadd page
        if ($form->{'current_page'} eq "orderrecap") {
            $form->{'return_page'} = "singleshippingadd";
        }

        ##
        # set both current and next page tp be addresscenter
        # so that it will come back again to check error 
        $form->{'current_page'} = 'addresscenter';
        $form->{'next_page'} = 'addresscenter';

    } else {

      if ($form->{'editaddress'}) {

          my $q = "SELECT name, address1, address2, city, state, postcode, country, dayphone FROM addresses WHERE objid='$form->{'editaddress'}' ";
          my $sth = $dbh->prepare( $q );
          $sth->execute;

          my $h = $sth->fetchrow_hashref("NAME_lc");

          ($h->{'firstname'}, $h->{'lastname'}) = split /\|/, $h->{'name'};    

          foreach my $key (keys %{$h}) {
              my $key_inp = $key."\_inp";
              $addressinfo{$key_inp} = $h->{$key};
          } # end of foreach

          $default_state = $h->{'state'};
          $form->{'addid'} = $form->{'editaddress'}

      } else {      
        ##
        # check the validation for the address information 
        # if no errors, update the addresses table
        $addr = &validateaddr($form, $site);
        %addressinfo = %{$addr};

        $default_state = $form->{'state_inp'};

        if ( !$addressinfo{'errors'} ) {
           
           my $save = 1; # flag for wether or not save this address
                         # by default, it is "save"
  
           ##
           # get all the addresses for the custid
           # and see if the entered address has been there
           my $q = "SELECT * FROM addresses WHERE custid = '$form->{'custid'}' "; 
           my $sth = $dbh->prepare( $q );
           $sth->execute;

           WHILE: while ( my $h = $sth->fetchrow_hashref("NAME_lc") ) {
                
               ## 
               # setup a flag for wether or not the address is ok
               # by testing every  filed of the address inforation
               my $ok = 0;
   
               ## break name to be last and first names first
               ($h->{'firstname'}, $h->{'lastname'}) = split /\|/, $h->{'name'};

               FOREACH: foreach my $inp (keys %addressinfo) {

                   next if ($inp !~ /_inp$/);

                   my $old = $inp;
                   $old =~ s/_inp$//;                   

                   ##
                   # if there is any difference between the new entered address
                   # and the exsiting address, then mark it as 'ok'
                   if ( $addressinfo{$inp} ne $h->{$old} ) {
                        $ok = 1; 
                        last FOREACH;
                   } # end of if
  
               } # end of foreach

               ##
               # if the new input address is exactly the same as
               # the existing address, then don't need to save it   
               if ( $ok == 0 ) {
                   $save = 0;
                   last WHILE;
               } # end of if

           } # end of while

           if ($save) {

             ## if addid, means edit, just update database 
             #  else it's new address, create new
             if ($form->{'addid'}) {

                 my %update; # a hash for update database
                 my $set;
                 my $q = "SELECT name, address1, address2, city, state, postcode, country, dayphone FROM addresses WHERE objid='$form->{'addid'}' ";
                 my $sta = $dbh->prepare( $q );
                 $sta->execute;
                 ## build name same as database 
                 $addressinfo{'name_inp'} = join '|', $addressinfo{'firstname_inp'}, $addressinfo{'lastname_inp'};
                 my $oldadd = $sta->fetchrow_hashref("NAME_lc");
                 foreach my $fieldname (keys %{$oldadd}) {
                     my $fieldname_inp = $fieldname."_inp";
                     if ($addressinfo{$fieldname_inp} ne $oldadd->{$fieldname}){
                         $set .= "$fieldname="."\'$addressinfo{$fieldname_inp}\',";
                     }
                 }  
                 chop $set;

                 $q = "UPDATE addresses SET $set WHERE objid='$form->{'addid'}' "; 
                 $sta = $dbh->prepare( $q ); 
                 $sta->execute;

             } else {
               ## if the script comes to here,
               # need to update the addresses table
               # update levent first
               my $addressid = &get_new_eventid($site, "addresses", "$form->{'sid'} $form->{'custid'} create an new address id");

               my $now = time();

               # join the last and first name to input into database
               $addressinfo{'name'} = join '\|', $addressinfo{'firstname_inp'}, $addressinfo{'lastname_inp'};

               $q  = "INSERT INTO addresses ";
               $q .= "(objid, custid, name, address1, address2, city, state, ";
               $q .= "postcode, country, dayphone, cdate, mdate) ";
               $q .= "VALUES ('$addressid', '$form->{'custid'}', ";
               $q .= "'$addressinfo{'name'}', ";
               $q .= "'$addressinfo{'address1_inp'}', ";
               $q .= "'$addressinfo{'address2_inp'}', ";
               $q .= "'$addressinfo{'city_inp'}', ";
               $q .= "'$addressinfo{'state_inp'}', ";
               $q .= "'$addressinfo{'postcode_inp'}', ";
               $q .= "'$addressinfo{'country_inp'}', ";
               $q .= "'$addressinfo{'dayphone_inp'}', ";
               $q .= "$now, $now) ";

               $sth = $dbh->prepare( $q );
               $sth->execute;
               $sth->finish;

           } # end of if else

               # everything is fine, setup current and next page
               $form->{'current_page'} = 'addresscenter';
               $form->{'next_page'} = $form->{'return_page'};

          } # end of if

         } else {

           # if error, need to come back again
           $form->{'current_page'} = 'addresscenter';
           $form->{'next_page'} = 'addresscenter';

         } # end of else
   
      } # end of if else    
 
    } # end of else

    ## 
    # get the shipping address for current custid
    my $q = "SELECT * from addresses where custid = '$form->{'custid'}' ";
    my $sth = $dbh->prepare($q);
    $sth->execute;

    my $rows=0;
    while ($sth->fetchrow_hashref("NAME_lc")){
          $rows += 1;
    }

    if ($rows > 0) {
        $form->{'cell_ck_edit_address'} = &loadtemplate("cell_ck_edit_address","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        ## 
        ## load addressbook pulldown
        $form->{'address_pulldown'} = &get_address_pulldown($form->{'custid'}, $dbh);

        $form->{'cell_ck_edit_address'} = &permutebit( $form->{'cell_ck_edit_address'}, $form );
    }

    ## load blank address input template
    my $template = &loadtemplate("cell_ck_addressenter","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

    ## load in state pull down
    my $state = &localize_template("sel_state", $site->{'interface'}, 0, $site);
    my @temp = split /\n/, $state;
    $addressinfo{'STATE_SELECT'} = &gen_select($default_state, 2, @temp);
    
    $form->{'cell_ck_addressenter'} = &permutebit( $template, \%addressinfo );

    $page_hashref->{'form'} = $form;

    return $page_hashref;

} # end of sub


##############################################################################
=pod

=head2 pg_multishippingadd

pg_multishippingadd - pg_multishippingadd($page_hashref, $site);

=over 4

=item DESCRIPTION

this function allows customer to choose multiple shipping addresses
for deliver

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_multishippingadd {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};

    $page_hashref->{'template'} = 'page_checkout_multi_shippingadd';

    ## 
    # load all the shipping addresses for the custid
    # into a pulldown
    my $address_pulldown = &get_address_pulldown($form->{'custid'}, $dbh);

    ##build hash with format $cell_list->{orderid}{lineobjid}=$h 
    my $cell_hashref = &build_available_backorder_item_hash($form, $dbh);
    my %cell_list = %{$cell_hashref};

    ## load data in cell template
    foreach my $item (keys %cell_list) {

        ## load header
        my $cellheader = "cell_ck_multi_".$item."hdr";
        $form->{$cellheader} = &loadtemplate("$cellheader","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        ## load list
        my $celllist = "cell_ck_multi_".$item."list";
        my $celltemplate = &loadtemplate("$celllist","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        ## actually, there is just one orderid for each availability now
        foreach my $orderid (keys %{$cell_list{$item}}) {
            # replace the real data in cell list template
            # foreach product under this category 
            foreach my $lineobjid (keys %{$cell_list{$item}{$orderid}} ) {

                ## put address pulldown in
                $cell_list{$item}{$orderid}{$lineobjid}{'address_pulldown'} = $address_pulldown;

                foreach my $num (1..$cell_list{$item}{$orderid}{$lineobjid}->{'quant'}) {
                    $cell_list{$item}{$orderid}{$lineobjid}->{'num'} = $num;
                    $form->{$celllist} .= &permutebit( $celltemplate, $cell_list{$item}{$orderid}{$lineobjid} );

                } # end of in foreach ($num)

            } # end of midin foreach ($lineobjid)

        } # end of mid foreach ($orderid)

    } # end of out foreach ($item)

    ##
    # setup current page and next page 
    $form->{'current_page'} = 'multishippingadd';
    $form->{'next_page'} = 'shippingmethod';

    $page_hashref->{'form'} = $form;
    return $page_hashref;       
      
} # end of sub


##############################################################################
=pod

=head2 pg_shippingmethod

pg_shippingmethod - pg_shippingmethod($page_hashref, $site);

=over 4

=item DESCRIPTION

this function allwos customer to choose shipping method for 
each shipping address, it divids into available items and 
back order items

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_shippingmethod {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};

    my $page_error = 0;

    &update_database_for_shippingadd($form, $site);

    &update_orders_status("shippingmethod", $form, $site);

    $page_hashref->{'template'} = 'page_checkout_shippingmethod';

    my $cell_hashref = &build_available_backorder_item_hash($form, $dbh);
    my %cell_list = %{$cell_hashref};

    ## available or back order
    foreach my $item (keys %cell_list) {

        ## load header
        my $cellheader = "cell_ck_shippingmethod_".$item."hdr";
        my $header_tem = "cell_ck_".$item."hdr";
        $form->{$cellheader} = &loadtemplate("$header_tem","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        ## load in list 
        my $celllist = "cell_ck_shippingmethod_".$item."list";
    
        ## for each order id
        foreach my $orderid (keys %{$cell_list{$item}}) {    
   
            ## a hash contains shipping weight for each address
            my %weight = ();

            ## get ship to address objid
            my $q = "SELECT shipto FROM orders WHERE objid='$orderid' ";
            my $sth = $dbh->prepare( $q );
            $sth->execute;
            my $addressid = $sth->fetchrow;

            $form->{$celllist} .= &load_addresslist($addressid, $site);
            
            ## load product list header
            my $producttemplate = 'cell_ck_300_product';
            $producttemplate = &loadtemplate($producttemplate, "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

            my $product_list = (); # refernce to hash which will used
                                   # to permute product template            

            ## for each product id
            foreach my $objid (keys %{$cell_list{$item}{$orderid}})  {

                ## load product in
                my $productline = "cell_ck_300_".$item."list";

                my  $productline_tem = &loadtemplate($productline, "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

                ## to load quant, have to be same as template __list_quant__ name
                $cell_list{$item}{$orderid}{$objid}->{'list_quant'} = $cell_list{$item}{$orderid}{$objid}->{'quant'};
                $product_list->{$productline} .= &permutebit( $productline_tem, $cell_list{$item}{$orderid}{$objid} );

                ## get total shipping weight fro each shipping
                $weight{$orderid} += &get_shipping_weight($dbh, $objid, $cell_list{$item}{$orderid}{$objid}->{'unitpr'});                

            } # end of foreach objid

            ## permute in cell_ck_300_prduct
            #  load __cell_ck_shippingmethod_avail(or back)_prod__
            $form->{$celllist} .= &permutebit( $producttemplate, $product_list );

            ## done with product list, begin build shipping method
            $q = "SELECT country, postcode FROM addresses WHERE objid='$addressid' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            my ($country, $postcode) = $sth->fetchrow_array;

            my @shipping_method = &list_ship_methods($site, $country, $postcode, $weight{$orderid});

            my @methods = ();
            my $shippingmethod = ();
            my $this_error = 0;
            FOREACH: foreach my $method (@shipping_method) {
                # if there is a error, put error message there
                if ($method =~ /^ERROR/) {
                    $shippingmethod->{'shippingmethod_error'} = $method;
                    $this_error = 1;
                    $page_error = 1;
                    next FOREACH;
                }
                my @a = split /\t/, $method;
                $b = "\"$a[0]\:$a[1]\""."|".$a[2].":\t\$".$a[1];
                push @methods, $b;
            }

            if ($this_error) {
                ## if error, not load in pulldown
            } else {
                ## if no error, load in pulldown
                #  build pulldown using obtained shipping method
                $shippingmethod->{'shippingmethod_pulldown'} = &gen_select(' ', 35, @methods);
            }

            my $shippingmethod_template = 'cell_ck_shippingmethod_pulldown';
            $shippingmethod_template = &loadtemplate($shippingmethod_template,"$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

            ## load orderid into template
            $shippingmethod->{'orderid'} = $orderid;
            ## load __cell_ck_shippingmethod_avail(or back)_meth__
            $form->{$celllist} .= &permutebit( $shippingmethod_template, $shippingmethod );  

            ## load in gift checkbox
            my $gift_checkbox = 'cell_ck_gift_checkbox';
            $gift_checkbox = &loadtemplate($gift_checkbox,"$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
            my $gift; # for load in availbility and orderid into template
            $gift->{'ava'} = $item;
            $gift->{'orderid'} = $orderid;
            $form->{$celllist} .= &permutebit( $gift_checkbox, $gift );
 
        } # end of foreach orderid
        
    } # end of foreach item

    if ($page_error) {
        $form->{'error'} = $site->{'msg'}->{2};
        $form->{'current_page'} = 'shippingmethod';
        $form->{'next_page'} = 'shippingmethod';
    } else {
        $form->{'current_page'} = 'shippingmethod';
        $form->{'next_page'} = 'payment';
    }

    $page_hashref->{'form'} = $form;
    return $page_hashref;

} # end of sub


##############################################################################
=pod

=head2 pg_payment

pg_payment - pg_payment($page_hashref, $site);

=over 4

=item DESCRIPTION

this function allows customer to choose payment method
it can be single or multiple payment methods

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_payment {

    my $page_hashref = shift;
    my $site = shift;
    my $dbh = $site->{'dbh'};
    my $form = $page_hashref->{'form'};

    ## if not from giftwrap page, need to decide which is next page
    if ($form->{'current_page'} ne "giftwrap") {
        ## determine if go gift wrapping page or not
        my $gift = 0;
        foreach my $key (keys %{$form}) {
            next if ($key !~ /_isgift$/);
            $gift = 1;
        } # end of foreach
        ## if there is gift wrapping, go gift wrapping page
        if ($gift == 1) {
            $page_hashref = &pg_giftwrap($page_hashref, $site);
            return $page_hashref;
        } # end of if
    } # end of if   
    
    ## if not gift wrapping, do payment page
    my %total = (); # a hash contains order total and pay total 
                    # for each orderid
    my %payment = (); # a hash of all payments with format
    my $payment_cell; # a refernce to a hash for permute the payment 
                      # template

    my $ok_flag = 1; # check if can go next page, by default is ok

    ## update database for previous page
    if ($form->{'current_page'} eq "giftwrap") {
        &update_database_for_giftwrap($form, $site);
    } else {
        &update_database_for_shippingmethod($form, $site);
    }

    # update order status
    &update_orders_status("payment", $form, $site);

    # load page template
    $page_hashref->{'template'} = 'page_checkout_payment';
    
    ##
    # get all the orderids from table orders 
    # which are still table intcart owned by current sid and custid
    my @order_ids = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});

    my @namenumber;
    ## delete the payment that be checked to be removed
    foreach my $key (keys %{$form}) {
        next if ($key !~ /\_remove$/);
        push @namenumber, (split /_/, $key)[0];
    }

    foreach my $orderid (@order_ids) {
        foreach my $name_number (@namenumber) {
            my ($name, $number) = split /\-/, $name_number;
            my $q = "DELETE FROM extraprop WHERE objid='$orderid' ";
               $q .= "AND body LIKE '%|$number|$name|%' ";
            my $sth = $dbh->prepare( $q );
            $sth->execute;
         }
    }        

    ## get order total information $total{orderid}{order}=subtotal
    &get_order_total(\%total, $dbh, @order_ids);

    ## add to order total for this sid and custid
    foreach my $orderid (keys %total) {
        $payment_cell->{'order_total'} += $total{$orderid}{'order'};
    }
    
    ## get paid total inforamtion $total{orderid}{pay}=subtotal
    #  also get every payment infomation $payment{namenumber}->{'name'}
    #                                    $payment{namenumber}->{'number'}
    #                                    $payment{namenumber}->{'expire'}
    #                                    $payment{namenumber}->{'amount'}
    &get_pay_total(\%total, \%payment, $dbh, @order_ids);

    ## add to paid total for this sid and custid
    foreach my $namenumber (keys %payment) {
        $payment_cell->{'pay_total'} += $payment{$namenumber}->{'amount'};
    }

    ## if case no payment yet
    $payment_cell->{'pay_total'} = &nicedecimals(0.00) if (!$payment_cell->{'pay_total'});

    ## before permute summary, load in balance
    $payment_cell->{'balance'} = &nicedecimals($payment_cell->{'order_total'} - $payment_cell->{'pay_total'});


    ## now check the new input validation
    #  if any error, show errors
    my ($cc_error, $co_error);

    ## check if no any input, error message
    if ((!$form->{'COID_inp'}) && (!$form->{'CCID_inp'}) && (!$form->{'CCEXPMO_inp'}) && (!$form->{'CCEXPYR_inp'}) && (!$form->{'CCNAME_inp'}) && (!$form->{'CCAMT_inp'})) {

        $form->{'info_err'} = &validate_message($site, 85);

    }
    
    ## if coupon number is entered,
    #  check coupon validation
    if ( $form->{'COID_inp'} ) {
        my $co_info = &validate_coupon($form->{'COID_inp'}, $site);

        ## for coupon, just one field need to be check
        #  so error message store in form->{'CO_err'}
        if ( $co_info->{'CO_err'} ) {

            $form->{'CO_err'} = $co_info->{'CO_err'};
            $form->{'info_err'} = &validate_message($site, 41);
            $ok_flag = 0;

        } else {

            ## test if coupon amount is bigger than balance
            #  if so, give error message, if not update database
            if ( $co_info->{'amount'} > $payment_cell->{'balance'} ){

                # Error - The coupon amount you put is more than the 
                #         amount you need to pay, please remove your
                #         credit card payment and try to input coupon 
                #         before credit card payment.
                $form->{'CO_err'} = &validate_message($site, 82);
                $ok_flag = 0;

            } else {

                # give nice decimals to CCAMT_inp
                $co_info->{'amount'} = &nicedecimals($form->{'amount'});

                # get order ids that should be updated by current payment
                my $pay_orderid = &get_payment_orderid(\%total, $co_info->{'amount'});

                # if no error, update database
                &store_coupon($co_info->{'number'}, $pay_orderid, $site);

            } # end of if else

        } # end of if else

    } # end of if 

    ## if any credit card field is input,
    #  check credit card validation
    if ( $form->{'CCID_inp'} || $form->{'CCEXPMO_inp'} || $form->{'CCEXPYR_inp'} || $form->{'CCNAME_inp'} || $form->{'CCAMT_inp'}) {

        $cc_error = &validate_creditcard($form, $site);

        ## amt has to be positive number
        if ( $form->{'CCAMT_inp'} !~ /^(\d+.?\d*|\.\d+)$/ || $form->{'CCAMT_inp'} <= 0 ) {
            # Error - You must input a positive payment amount.
            $form->{'CCAMT_err'} = &validate_message($site, 84);
            $form->{'info_err'} = &validate_message($site, 41);            
            $cc_error->{'CC_ok'} = 0;
            $ok_flag = 0;
        }

        ## if error, put into $form
        if ($cc_error->{'CC_ok'} == 0) {

            # put into $form
            foreach my $err (keys %{$cc_error}) {

                # don't need CC_ok, and CC_err now
                next if ($err eq 'CC_ok');
                next if ($err eq 'CC_err');
                $form->{$err} = $cc_error->{$err};

            } # end of foreach

            # put page error
            # There were problems with your information
            $form->{'info_err'} = &validate_message($site, 41);
            $ok_flag = 0;
        } else {

            if ( $form->{'CCAMT_inp'} > $payment_cell->{'balance'} ) {
             
                # Error - The payment amount you put is more than 
                #         the amount you need to pay, please enter again.
                $form->{'CCAMT_err'} = &validate_message($site, 83);
                $form->{'info_err'} = &validate_message($site, 41);
                $ok_flag = 0;

            } else {
                
                # if reach this point, need to update database
                # get order ids that should be updated by current payment
                my $pay_orderid = &get_payment_orderid(\%total, $form->{'CCAMT_inp'});
                
                # if no error, store in database
                &store_creditcard($form, $pay_orderid, $site);
  
            } # end of if elsif else

        } # end of if else

    } # end of if 

    ## After update database, need to get data from database again to 
    #  list on page
   
    #  need to re-initialize %payment and %total
    %payment = ();   
    %total = ();
    $payment_cell = ();
     
    # get all the orderids from table orders 
    # which are still table intcart owned by current sid and custid
    @order_ids = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});

    ## get order total information $total{orderid}{order}=subtotal
    &get_order_total(\%total, $dbh, @order_ids);

    ## add to order total for this sid and custid
    foreach my $orderid (keys %total) {
        $payment_cell->{'order_total'} += $total{$orderid}{'order'};
    }

    ## get paid total inforamtion $total{orderid}{pay}=subtotal
    #  also get every payment infomation $payment{namenumber}->{'name'}
    #                                    $payment{namenumber}->{'number'}
    #                                    $payment{namenumber}->{'expire'}
    #                                    $payment{namenumber}->{'amount'}
    &get_pay_total(\%total, \%payment, $dbh, @order_ids);

    ## add to paid total for this sid and custid
    foreach my $namenumber (keys %payment) {
        $payment_cell->{'pay_total'} += $payment{$namenumber}->{'amount'};
    }

    ## permute payment summary details
    my $summary_detail = 'cell_ck_payment_detail';
    $summary_detail = &loadtemplate($summary_detail,"$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

    foreach my $type (keys %payment) {
        ## load each payment method
        $payment_cell->{'cell_ck_payment_detail'} .= &permutebit( $summary_detail, $payment{$type} );
    }
    # if there is at least one payment, load payment summary,
    # if not, don't load the header
    if ($payment_cell->{'cell_ck_payment_detail'}) {
        my $header = "cell_ck_payment_listsummary";
        $payment_cell->{$header} = &loadtemplate("$header","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
    }

    ## permute summary template
    my $summary_template = 'cell_ck_payment_summary';
    $summary_template = &loadtemplate($summary_template,"$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

    ## if case no payment yet
    $payment_cell->{'pay_total'} = 0 if (!$payment_cell->{'pay_total'});

    ## give nice decimals before showing
    $payment_cell->{'pay_total'} = &nicedecimals($payment_cell->{'pay_total'});
    $payment_cell->{'order_total'} = &nicedecimals($payment_cell->{'order_total'});

    ## before permute summary, load in balance
    $payment_cell->{'balance'} = &nicedecimals($payment_cell->{'order_total'} - $payment_cell->{'pay_total'});
    $form->{'balance'}= $payment_cell->{'balance'};

    $form->{'cell_ck_payment_summary'} = &permutebit( $summary_template, $payment_cell );
    ## done with payment loading   
    #  now build credit card pulldowns for this template
    my @cc_pulldown = ("ccname", "ccexpmo", "ccexpyr");
    foreach my $pulldown (@cc_pulldown) {
       my $key_length;
       if ($pulldown eq "ccname"){
           $key_length = 4;
       } else {
           $key_length = 2;
       }
       my $file_info = $pulldown."|".$key_length;
       my $in_template = uc($pulldown."_select");
       $form->{$in_template} = &load_select_pulldown($file_info, $form, $site);
    }

    ## load address pulldown
    $form->{'address_select'} = &get_address_pulldown($form->{'custid'}, $dbh);

    # done of loading and error checking, 
    # set up current page and next page
    # if no error and balance equal to 0,
    # can go next step, 
    # if not, this step again
    if ( $ok_flag && $payment_cell->{'balance'} == 0) {       
        $form->{'next_page'} = 'orderpreview';
    } else {        
        $form->{'next_page'} = 'payment';
    }

    $form->{'current_page'} = 'payment';
    $page_hashref->{'form'} = $form;

    return $page_hashref;

} # end of sub


##############################################################################
=pod

=head2 pg_giftwrap

pg_giftwrap - pg_giftwrap($page_hashref, $site);

=over 4

=item DESCRIPTION

This fuction allow customer to choose which item to be packed, 
but gift card is not available to back order items

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

gift card is not available to back order items

wrapping style should be created by content manager at admin site
by now, content manager doesn't do it, so for wrap style and price, it is hard 
code in template, later it should be replaced with new stuff when 
content manager can create gift wrapping style

=back

=cut
##############################################################################
sub pg_giftwrap {

    my $page_hashref = shift;
    my $site = shift;
    my $form = $page_hashref->{'form'};
    my $dbh = $site->{'dbh'};
   
    ## load template for this page
    $page_hashref->{'template'} = 'page_checkout_giftwrap';

    ## update shippingmethod
    &update_database_for_shippingmethod($form, $site);

    ## udpate order status
    &update_orders_status("giftwrap", $form, $site);

    my $gava; # contains the availbility for gift orderid
    my $gorderid; #contains the orderids for gift warpping
    ## load in order ids for giftwrapping
    foreach my $key (keys %{$form}) {
        next if ($key !~ /_isgift$/);
        my @values = split /_/, $key;

        $gava .= "$values[0] ";
        $gorderid .= "$values[1] ";
    }

    ## build hash for avail->orderid->objid
    my $cell_hashref = &build_available_backorder_item_hash($form, $dbh);
    my %cell_list = %{$cell_hashref};

    foreach my $item (keys %cell_list) {

        ## if don't need this availbility, next one         
        next if ($gava !~ /$item/);

        my $subcell; # a reference to hash of sub cell

        ## load header for sub cell
        my $cellheader = "cell_ck_".$item."hdr";
        $form->{$cellheader} = &loadtemplate("$cellheader","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        ## for each order id
        foreach my $orderid (keys %{$cell_list{$item}}) {

            ## if don't need this orderid, next one
            next if ($gorderid !~ /$orderid/);

            ## get ship to address objid
            my $q = "SELECT shipto FROM orders WHERE objid='$orderid' ";
            my $sth = $dbh->prepare( $q );
            $sth->execute;
            my $addressid = $sth->fetchrow;

            ## load in shipping list for sub cell
            $subcell->{'shippinglist'} = &load_addresslist($addressid, $site);
           
            my $pro_tem; 
            ## per product
            foreach my $objid (keys %{$cell_list{$item}{$orderid}})  {
                          
                ## load gift list for sub cell
                my $giftlist_tem = "cell_ck_gift_productlist";
                my $giftlist = &loadtemplate("$giftlist_tem","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
       
                ## one line one piece of one item
                foreach (1..$cell_list{$item}{$orderid}{$objid}->{'quant'}) {
                    $cell_list{$item}{$orderid}{$objid}->{'num'} = $_;
                    $pro_tem .= &permutebit($giftlist, $cell_list{$item}{$orderid}{$objid});                
                } # end of foreach quantity

            } # end of foreach product

            $subcell->{'cell_ck_gift_productlist'} = $pro_tem;

            ## load list for sub cell
            my $giftlist_tem = "cell_ck_gift_".$item."list";
            my $giftlist = &loadtemplate("$giftlist_tem","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
            $subcell->{'orderid'} = $orderid;
            $form->{$giftlist_tem} .= &permutebit($giftlist, $subcell);

        } # end of foreach orderid

    } # end of foreach item

    ## done with loading
    #  now setup current page and next page
    $form->{'current_page'} = "giftwrap";
    $form->{'next_page'} = "payment";

    $page_hashref->{'form'} = $form;

    return $page_hashref;

} # end of giftwrap


##############################################################################
=pod

=head2 pg_orderpreview

pg_orderpreview - pg_orderpreview($page_hashref, $site);

=over 4

=item DESCRIPTION

this page show thw customers all the information about their orders
by now, it doesn't allow customers edit their order, but later on, 
it will allow customers top edit their order by each individual 
shipment.

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_orderpreview {

    my $page_hashref = shift;
    my $site = shift;

    my $form = $page_hashref->{'form'};

    ## load template for this page
    $page_hashref->{'template'} = 'page_checkout_orderpreview';

    ## update order status
    &update_orders_status("orderpreview", $form, $site);

    $form = &build_ordersummary_page($form, $site);

    $form->{'current_page'} = "orderpreview";
    $form->{'next_page'} = "orderconfirm";

    $page_hashref->{'form'} = $form;

    return $page_hashref;

} # end of sub


##############################################################################
=pod

=head2 pg_orderconfirm

pg_orderconfirm - pg_orderconfirm($page_hashref, $site);

=over 4

=item DESCRIPTION

This page actually move table tcart stuff into table ocart,
update orders status in table orders,
send email the orderpreview page,
then show the message to customers

=item INPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

$site - Standard $site hash from &setsite

=item OUTPUT

$page_hashref - the refernce of the hash which contains the variables
                for the page

=item NOTES

=back

=cut
##############################################################################
sub pg_orderconfirm {

    my $page_hashref = shift;
    my $site = shift;

    my $dbh = $site->{'dbh'};
    my $form = $page_hashref->{'form'};

    ## load template for this page
    $page_hashref->{'template'} = 'page_checkout_orderconfirm';

    my $now = time();
    
    ## move tcart to ocart
    my $q = "SELECT * FROM tcart WHERE sid='$form->{'sid'}' ";
       $q .= "AND custid='$form->{'custid'}' ";
    my $sth = $dbh->prepare ( $q );
    $sth->execute;

    ## record order id for update table orders
    my %order_id;

    while (my $tcart = $sth->fetchrow_hashref("NAME_lc") ) {

        my $objid = &get_new_eventid($site, "ocart", "sid:$form->{'sid'} custid:$form->{'custid'} orderid:$tcart->{'orderid'} lineobjid:$tcart->{'lineobjid'} get objid for table ocart row");
   
        my ($field, $value);
        foreach my $fieldname (keys %{$tcart}) {

            if ($fieldname eq "cdate" || $fieldname eq "mdate") {
                 $value .= "\'$now\',";
                 $field .= "$fieldname,";                 
            } elsif ($fieldname eq "objid") {
                 $value .= "\'$objid\',";
                 $field .= "$fieldname,";                 
            } else {
                 $value .= "\'$tcart->{$fieldname}\',";
                 $field .= "$fieldname,";  
            }
        } # end of foreach

        chop $field;
        chop $value;

        ## insert into ocart table
        my $q = "INSERT INTO ocart ($field) VALUES ($value) "; 
        my $stq = $dbh->prepare( $q );
        $stq->execute;
        ## remove the original row in table tcart
        $q = "DELETE FROM tcart WHERE objid='$tcart->{'objid'}' ";
        $stq = $dbh->prepare( $q );
        $stq->execute;
        ## record order id for update table orders
        $order_id{$tcart->{'orderid'}} = 1;        
    }
         
    my @orderids;
    foreach my $orderid (keys %order_id) {
        push @orderids, $orderid;

        ## this part will need to put into update_orders_status subroutine
        #  update order status
      
        #  should call this subroutine to get authorization
        #  $status = &payment_authorization($payment, $site) if ($payment);
        my $status = "15018-A";
 
        my $q = "UPDATE orders SET status='$status' ";
           $q .= "WHERE objid='$orderid' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
           $sth->finish;
    }    
    
    # send confirmation email
    &sendmail($form, $site, @orderids);

    # setup current page and next page
    $form->{'current_page'} = "orderconfirm";
    $form->{'next_page'} = "orderconfirm";

    $page_hashref->{'form'} = $form;

    return $page_hashref;

} # end of sub
##############################################################################
=pod

=head2 build_ordersummary_page

build_ordersummary_page - build_ordersummary_page($form, $site);

=over 4

=item DESCRIPTION

this fuction builds up the order summary page
including shipping address, shipping method, ordertotal, product list
information. It is listed by availability and shipping 
addresses(orderid).

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

$form - Reference to standard %form hash from &ReadParse
        it already holds all the information need to 
        load in form page for ordersummary

=item NOTES

=back

=cut
##############################################################################
sub build_ordersummary_page {

    my $form = shift;
    my $site = shift;

    my $dbh = $site->{'dbh'};
    
    ## build payment summary first
    my (%total, %payment);
    # get all the orderids from table orders 
    # which are still table intcart owned by current sid and custid
    my @order_ids = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});

    &get_pay_total(\%total, \%payment, $dbh, @order_ids);

    ## permute payment summary details
    my $summary_detail = 'cell_ck_payment_detail_noremove';
    $summary_detail = &loadtemplate($summary_detail,"$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

    foreach my $type (keys %payment) {
        ## load each payment method
        $form->{'cell_ck_payment_detail'} .= &permutebit( $summary_detail, $payment{$type} );
    }

    ## now load in order infomation

    ## build the hash with format $cell_list{avail}{orderid}{lineobjid}=$h
    #                             $cell_list{back}{orderid}{lineobjid}=$h
    my $cell = &build_available_backorder_item_hash($form, $dbh);
    my %cell_list = %{$cell};

    foreach my $ava (keys %cell_list) {

        ## load header
        my $header = "cell_ck_".$ava."hdr";
        $form->{$header} = &loadtemplate("$header","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

        foreach my $orderid (keys %{$cell_list{$ava}} ) {
           
            my $ordersum; # for permutebit
            my $ordsum_cell; # for permutebit
 
            ## load shipping address in
            my $q = "SELECT shipto, st_taxable, st_tax, taxrate, ordtotal ";
               $q .= "FROM orders WHERE objid='$orderid' ";
            my $sth = $dbh->prepare( $q );
            $sth->execute;
            # get address id for this orderid
            my ($addressid, $itemtotal, $tax, $taxrate, $ordtotal) = $sth->fetchrow_array;
            # load in template
            $ordsum_cell->{'shippingaddress'} = &load_addresslist($addressid, $site);

            ## load in shipping method
            $q = "SELECT lineobjid, linetotal FROM tcart WHERE orderid='$orderid' ";
            $q .= "AND objclass='140-A' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            # get shipping method objid
            my ($methodid, $shipcost) = $sth->fetchrow_array;
            # get title for the shipping method
            $q = "SELECT title FROM things WHERE objid='$methodid' ";           
            $sth = $dbh->prepare( $q );
            $sth->execute;
            # get shipping method title
            $ordersum->{'shippingmethod'} = $sth->fetchrow;
            # get wrapping cost
            $q = "SELECT linetotal FROM tcart WHERE orderid='$orderid' ";
            $q .= "AND objclass='45002-A' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            # get wrapping cost
            my $wrappingcost = $sth->fetchrow;            
            $wrappingcost = &nicedecimals(0) if (!$wrappingcost);

            my $shipmethod_template = &loadtemplate("cell_ck_shippingmethod","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
            $ordsum_cell->{'shippingmethod'} = &permutebit( $shipmethod_template, $ordersum );

            my $subtotal = &nicedecimals($itemtotal+$wrappingcost);

            $ordtotal = &nicedecimals($subtotal+$tax+$shipcost);
            ## load order total
            $ordersum->{'itemcost'} = $itemtotal;
            $ordersum->{'st_tax'}   = $tax;
            $ordersum->{'shipcost'} = $shipcost;
            $ordersum->{'ordtotal'} = $ordtotal;
            $ordersum->{'wrapcost'} = $wrappingcost;            
            $ordersum->{'subtotal'} = $subtotal;
            $ordersum->{'taxrate'}  = &nicedecimals($taxrate);
    
            my $ordtotal_template = &loadtemplate("cell_ck_smallorder","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
            $ordsum_cell->{'ordertotalsum'} = &permutebit( $ordtotal_template, $ordersum );

            ## for load in items
            foreach my $objid (keys %{$cell_list{$ava}{$orderid}})  {

                ## load product in
                my $productline = "cell_ck_300_".$ava."list";

                my  $productline_tem = &loadtemplate($productline, "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

                ## to load quant, have to be same as template __list_quant__ name
                $cell_list{$ava}{$orderid}{$objid}->{'list_quant'} = $cell_list{$ava}{$orderid}{$objid}->{'quant'};
                $ordsum_cell->{'productlist'} .= &permutebit( $productline_tem, $cell_list{$ava}{$orderid}{$objid} );

            } # end of foreach objid
 
            # give the orderid to template
            $ordsum_cell->{'orderid'} = $orderid;

            ## load in availbility cell
            my $detail_tem = &loadtemplate("cell_ck_allorder","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");
            my $ava_detail = "cell_ck_".$ava."detail";
            $form->{$ava_detail} .= &permutebit( $detail_tem, $ordsum_cell );

        } # end of foreach (orderid)

    } # end of foreach ($ava)      
   
    return $form;

} # end of sub


##############################################################################
=pod

=head2 sendmail

sendmail - sendmail($form, $site, @orderids)
=over 4

=item DESCRIPTION

this fuction get all information about the orders
and send an email to customer in their e-mail account

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

@orderids - an array holds all orderids for current sid and custid
            for this time shopping

=item OUTPUT

NONE

=item NOTES

=back

=cut
##############################################################################
sub sendmail {

    my $form = shift;
    my $site = shift;
    my @orderids = @_;

    my $dbh = $site->{'dbh'};

    ## get user e-mail address
    my $q = "SELECT uname FROM contacts WHERE objid='$form->{'custid'}' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;
    my $email = $sth->fetchrow;

    ## setup email program
    my $mailprog = $site->{'mailserver'};
    if ($mailprog) {
        ## header
        my $title = "$site->{'msg'}->{3} $site->{'sitename'} $site->{'msg'}->{4}";
        open (MAIL, "|$mailprog -t");
        print MAIL "To: $email\n";
        print MAIL "FROM: $site->{'order_email_address'}\n";
        print MAIL "Bcc: $site->{'order_email_address'}\n";
        print MAIL "Subject: $title\n\n";
 
        ## body part
        print MAIL "$site->{'msg'}->{5} $site->{'sitename'}, $site->{'msg'}->{6}\n\n";

        ## payment summary
        print MAIL "$site->{'msg'}->{7}\n";

        my (%payment, %total);
        &get_pay_total(\%total, \%payment, $dbh, @orderids);

        foreach my $type (keys %payment) {
            $payment{$type}->{'amount'} = &nicedecimals($payment{$type}->{'amount'});
            print MAIL "$payment{$type}->{'name'}\t$payment{$type}->{'number'}\t$payment{$type}->{'expire'}\t$payment{$type}->{'amount'}\n";
        }  

        print MAIL "\n\n";

        ## individual order infomation
        foreach my $orderid (@orderids) {

            ## get data from database
            my $q = "SELECT shipto FROM orders WHERE objid='$orderid' ";
            my $sth = $dbh->prepare( $q );
            $sth->execute;
            my $addressid = $sth->fetchrow;
            # shipping address
            $q = "SELECT * FROM addresses WHERE objid='$addressid' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            my $add = $sth->fetchrow_hashref("NAME_lc");
            ($add->{'firstname'}, $add->{'lastname'}) = split /\|/, $add->{'name'};
            $add->{'country'} = "USA" if ( !$add->{'country'});

            my $address = "$add->{'firstname'} $add->{'lastname'}: $add->{'address1'}, ";
            $address .= "$add->{'address2'}, " if ($add->{'address2'});
            $address .= "$add->{'city'}, $add->{'state'}$add->{'postcode'}, ";
            $address .= "$add->{'country'} ";
      
            # shipping method
            $q = "SELECT lineobjid, linetotal FROM ocart WHERE orderid='$orderid' AND objclass='140-A' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            my ($method, $shippingcost) = $sth->fetchrow_array;
            # get shipping method title
            $q = "SELECT title FROM things WHERE objid='$method' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            my $shippingmethod = $sth->fetchrow;

            # get wrap cost
            $q = "SELECT linetotal FROM ocart WHERE orderid='$orderid' AND objclass='45002-A' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            my $wrapcost = $sth->fetchrow;
            $wrapcost = &nicedecimals(0) if (!$wrapcost);
 
            print MAIL "$site->{'msg'}->{8} $orderid\n";
            print MAIL "$site->{'msg'}->{9} $address\n";
            print MAIL "$site->{'msg'}->{10} $shippingmethod\n\n";
            print MAIL "$site->{'msg'}->{11}\n";
            
            $q = "SELECT * FROM ocart WHERE orderid='$orderid' AND objclass != '140-A' ";
            my $stq = $dbh->prepare( $q );
            $stq->execute;

            while ( my $product = $stq->fetchrow_hashref("NAME_lc") ) {
                my $title = &get_title($product->{'lineobjid'}, $dbh);
                print MAIL "$title\t$product->{'quant'}\t$site->{'this_currency_symbol'}$product->{'unitpr'}\t$site->{'this_currency_symbol'}$product->{'linetotal'}\n";
            } # end of while

            ## load total infomation
            $q = "SELECT st_taxable, st_tax, taxrate, ordtotal ";
            $q .= "FROM orders WHERE objid='$orderid' ";
            $stq = $dbh->prepare( $q );
            $stq->execute;

            my ($total, $tax, $taxrate, $ordertotal) = $stq->fetchrow_array;

            $total = &nicedecimals($total);
            $tax = &nicedecimals($tax);
            $ordertotal = &nicedecimals($ordertotal);

            print MAIL "\n";
            print MAIL "$site->{'msg'}->{12} $site->{'this_currency_symbol'}$total\n";
            print MAIL "$site->{'msg'}->{13} $site->{'this_currency_symbol'}$wrapcost\n";
            print MAIL "$site->{'msg'}->{14} $site->{'this_currency_symbol'}$tax\n";
            print MAIL "$site->{'msg'}->{15} $site->{'this_currency_symbol'}$shippingcost\n";
            print MAIL "$site->{'msg'}->{16} $site->{'this_currency_symbol'}$ordertotal\n";
            print MAIL "\n\n";

        } # end of foreach orderid

    } # end of if

} # end of sub
##############################################################################
=pod

=head2 get_order_total

get_order_total - get_order_total(\%total, $dbh, @order_ids)

=over 4

=item DESCRIPTION

this function get all the orderids for current sid and custid
and build a hash with format $total{orderid}{'order'}=subtotal
                             $total{orderid}{'paid'}=amountpaid
=item INPUT

\%total - the refernce to a hash whilh holds order subtotal and
          paid total information
       
$dbh - current database handler

@order_ids - an array holds all the order ids for current sid 
             and custid

=item OUTPUT

NONE

=item NOTES

=back

=cut
##############################################################################
sub get_order_total {

    my $total = shift;
    my $dbh =shift;
    my @order_ids = @_;

    foreach my $order_id (@order_ids) {

        ## get total for each orderid
        my $q = "SELECT ordtotal FROM orders WHERE objid='$order_id' ";
        my $sth = $dbh->prepare( $q );
        $sth->execute;
        my $subtotal = $sth->fetchrow;

        ## setup order total for each order id
        $total->{$order_id}{'order'} = $subtotal;

    } # end of foreach

} # end of sub


##############################################################################
=pod

=head2 get_pay_total

get_pay_total - get_pay_total(\%total, \%payment, $dbh, @order_ids);

=over 4

=item DESCRIPTION

this fuction stores customer's payment information into two hash
one is $total{orderid}{'paid'}=amountpaid
another is $payment{namenumber}{'name'}
           $payment{namenumber}{'number'}
           $payment{namenumber}{'expire'}
           $payment{namenumber}{'amount'}
                   
=item INPUT

\%total - the refernce to a hash whilh holds order subtotal and
          paid total information

\%payment - a refernce to a hash which holds all the payment infomation
            for current sid and custid

$dbh - current database handler

@order_ids - an array holds all the order ids for current sid 
             and custid

=item OUTPUT

NONE

=item NOTES

=back

=cut
##############################################################################
sub get_pay_total {

    my $total = shift;
    my $payment =shift;
    my $dbh = shift;
    my @order_ids = @_;

    foreach my $order_id (@order_ids) {
 
        ## get payments have been made from table extraprop
        my $q = "SELECT body FROM extraprop WHERE objid='$order_id' ";
        $q .= "AND (fieldname LIKE 'CC|%' OR fieldname LIKE 'CO|%') ";
        my $sth = $dbh->prepare( $q );
        $sth->execute;

        while (my $statement = $sth->fetchrow) {

           ## split payment information
           #  with format: [0]type [1]number [2]name [3]amount
           #               [4]expire month [5]expire year
           #               [6]start month  [7]start year (for coupon)
           my @info = split /\|/, $statement;

           # if this is a coupon, need to get inforation
           # from table extraprop
           if ($info[0] eq "CO") {

              my $q = "SELECT body FROM extraprop ";
              $q .= "WHERE fieldname LIKE 'CO|id|amount|exmonth|exyear|%' ";
              $q .= "AND body LIKE 'CO|$info[1]|%' ";
              my $sth = $dbh->prepare( $q );
              $sth->execute;

              my $body = $sth->fetchrow;
              my @coupon_info = split /\|/, $body;
              # build same format as credit card
              $info[1] = $coupon_info[1]; # number
              # move $info[2] to $info[3] before new value assign to it
              $info[3] = $info[2];        # amount
              $info[2] = "coupon";        # name
              $info[4] = $coupon_info[3]; # expire month
              $info[5] = $coupon_info[4]; # expire year
              $info[6] = $coupon_info[5]; # start month
              $info[7] = $coupon_info[6]; # start year

          } # done with get information for coupon

           ## build a hash using the foramt
           #  $payment{namenumber}->{name}
           #  $payment{namenumber}->{number} 
           #  $payment{namenumber}->{amount}
           #  $payment{namenumber}->{expire}
           #  $payment{namenumber}->{start} (for coupon)
           #  if name and number both are same, then is same payment
           if ($payment->{$info[2].$info[1]}->{'name'}) {

               ## add amount to the same card 
               $payment->{$info[2].$info[1]}->{'amount'} += $info[3];
               $payment->{$info[2].$info[1]}->{'amount'} = &nicedecimals($payment->{$info[2].$info[1]}->{'amount'});
               ## add amount to paid for each orderid
               $total->{$order_id}{'paid'} += $info[3];
           } else {

               ## if there are 7th and 8th element
               #  it means that it's a coupon's starting date               
               if ($info[6] && $info[7]) {
                   $payment->{$info[2].$info[1]}->{'expire'} = $info[6].'/'.$info[7].'-';
               }

               ## if the card not exsiting, create it 
               #  with expire date and amount
               $payment->{$info[2].$info[1]}->{'name'} = $info[2];
               $payment->{$info[2].$info[1]}->{'number'} = $info[1];
               $payment->{$info[2].$info[1]}->{'expire'} .= $info[4].'/'.$info[5];
               $payment->{$info[2].$info[1]}->{'amount'} = $info[3];
               $payment->{$info[2].$info[1]}->{'amount'} = &nicedecimals($payment->{$info[2].$info[1]}->{'amount'});
               ## add amount to paid for each orderid
               $total->{$order_id}{'paid'} += $info[3];

           } # end of if else

        } # end of while

    } # end of foreach 

} # end of sub


##############################################################################
=pod

=head2 get_custid

get_custid - get_custid($site);

=over 4

=item DESCRIPTION

Look up the database to get the custid for the current sid, if no custid 
for current sid, return undefined value

=item INPUT

$site - Standard $site hash from &setsite

$form - Reference to standard %form hash from &ReadParse

=item OUTPUT

$custid - custid for the current sid, if unavailable, return undef

=item NOTES

=back

=cut
###############################################################################
sub get_custid {

    my $site = shift;
    my $form = shift;

    my $q = " SELECT custid FROM sessions where sid = $form->{'sid'} ";
    my $sth = $site->{'dbh'}->prepare($q);
    $sth->execute;

    my ($custid) = $sth->fetchrow;
    my $row = $sth->rows;
    if ($row == 1) {
       return $custid;
    } else {
       return undef;
    }

} # end of sub


##############################################################################
=pod

=head2 store_credircard

store_creditcard - store_creditcard($form, $pay_orderid, $site);

=over 4

=item DESCRIPTION

this function store credit card information
into table extraprop after the information validation

=item INPUT

$form - the reference of a hash which contains credit card 
        information, ID, NAME, EXPIRE, MONTH, EXPYEAR, email ,length
 
$pay_orderid - a refernce to hash with order id as key and amount as value

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

this is a function for no payment processing
it will always return a success

=back

=cut
##############################################################################
sub store_creditcard {

   my ($h,$q,$now,$pay_order,$CCnum,$CCname,$CCmo,$CCyr);
   my ($ccemail,$cclength,$total,@numbers,$stop,$i,$j);
   my ($CCnum_email,$amount,$site, $billto);
   $now=time();
   $h = shift;
   $pay_order= shift;
   $site = shift;
   $CCnum=$h->{'CCID_inp'};
   $CCname=$h->{'CCNAME_inp'};
   $CCmo=$h->{'CCEXPMO_inp'};
   $CCyr=$h->{'CCEXPYR_inp'};
   $billto = $h->{'BILLTO_inp'};
   $ccemail=$site->{'ccemail'};
   $cclength=$site->{'cclength'};

   foreach my $orderid ( keys %{$pay_order} ) {

    my $amount = $pay_order->{$orderid};  

    if ( $cclength == 4 | $cclength == 8 | $cclength == 12 ) {

      @numbers=split //, $CCnum;
      $total=$#numbers+1;
      $stop=$total-$cclength;
      $CCnum="";
      $i=$stop;
      while ( $i<$total ) {
         $CCnum.=$numbers[$i];
         $i++;
      }
      $j=0;
      $CCnum_email="";
      if ( $ccemail eq "yes" ) {
         while ( $j<$stop ) {
            $CCnum_email.=$numbers[$j];                                                                   $j++;
         }
         open(MAIL,"|$site->{'mailserver'} -t");
         print MAIL "$site->{'msg'}->{17} $site->{'order_email_address'}\n";
         print MAIL "$site->{'msg'}->{18} $site->{'webmaster_email_address'}\n";
         print MAIL "$site->{'msg'}->{19} $orderid\n\n";
         print MAIL "$CCnum_email\n";
         close(MAIL);
      }
   }

   if ( $cclength eq "all" ) {

      my @cc=split //, $CCnum, 2;

      my $cc_crypt1=os_hash($cc[0],$site->{'hash_key'},0);
      my $cc_crypt2=os_hash($cc[1],$site->{'hash_key'},0);

      $CCnum="$cc_crypt1|$cc_crypt2";

   }

   if ( $cclength == 4 || $cclength == 8 || $cclength == 12 || $cclength eq "all" ) {

      $q="INSERT INTO extraprop (objid,fieldname,body,cdate,mdate) VALUES ('$orderid','CC|$ccemail|$cclength','CC|$CCnum|$CCname|$amount|$CCmo|$CCyr|$billto','$now','$now')";
      my $sth=$site->{'dbh'}->prepare($q);
      $sth->execute;

   } # end of if

  } # end of foreach

} # end of sub


##############################################################################
=pod

=head2 store_coupon

store_coupon - store_coupon($number, $pay_order, $site);

=over 4

=item DESCRIPTION

this function save the payment using coupon to table extraprop

=item INPUT

$number - coupon id number 

$pay_order - a refernece to a hash with order id as key and amount as value

$site - Standard $site hash from &setsite

=item OUTPUT

NONE

=item NOTES

=back

=cut
##############################################################################
sub store_coupon {

    my $number = shift;
    my $pay_order = shift;
    my $site = shift;
    my $dbh = $site->{'dbh'};
    my $now = time();

    foreach my $orderid (keys %{$pay_order}) {
        my $amount = $pay_order->{$orderid};

        my $q = "INSERT INTO extraprop (objid,fieldname,body,cdate,mdate) ";
           $q .= "VALUES ('$orderid','CO|id|amount','CO|$number|$amount','$now','$now')";
        my $sth=$dbh->prepare($q);
        $sth->execute;
        $sth->finish;
    }

    return;

} # end of sub


##############################################################################
=pod

=head2 get_payment_orderid

get_payment_orderid - get_payment_orderid($total_hashref, $thispay, $site);

=over 4

=item DESCRIPTION

this function calculate the difference between the order total 
and paid total for each orderid, if there is no difference, it means
that this order has been paid off, don't need to worry about, go next 
orderid. If there is a difference, then use current payment amount
substract the difference, if the payment amount is more than the 
difference, this means that this order can be paid off this time,
add the orderid to the orderid array, take amount off from current 
payment amount, and test next orderid until current payment amount is
not enough to pay a orderid, then use the left pauyment amount as the
payment to current orderid, add the current orderid to orderid array
also, it has to wait next payment to get full pay  

=item INPUT

$total_hashref - a reference to the hash which contains the order total
                 and paid total for each orderid

$thispay - a scale which is the current payment amount

=item OUTPUT

\%pay_orders - a refernce to a hash with orderid as key and amount as value 
             it contains all the order is and amount to be updated by 
             current payment

=item NOTES

=back

=cut
##############################################################################
sub get_payment_orderid {

    my $total_hashref = shift;
    my $thispay = shift;
    my %pay_orders = ();

    foreach my $orderid ( %{$total_hashref} ) {
    
        my $needpay = $total_hashref->{$orderid}{'order'} - $total_hashref->{$orderid}{'paid'}; 

        # if $needpay equal to 0, it means that it has been paid off
        next if ($needpay == 0);

        # need give nice decimals before store into database
        $needpay = &nicedecimals($needpay);
        $thispay = &nicedecimals($thispay);

        if ($thispay >= $needpay){            
            # this order has been fully paid off
            $pay_orders{$orderid} = $needpay;
            # take off paid amount from the current payment amount
            $thispay -= $needpay;
            # add to paid amount for this orderid
            $total_hashref->{$orderid}{'paid'} += $needpay;
        } else {
            # this order cannot be fully paid off
            # can just get paid by whatever left from this payment
            # must wait until next payment to get full paid
            $pay_orders{$orderid} = $thispay;
            # take off paid amount from the current payment amount, 
            # should be 0 now
            $thispay -= $thispay;
            # add to paid amount for this orderid
            $total_hashref->{$orderid}{'paid'} += $thispay;

        } # end of if else

        # if $thispay equal to 0, it means that no amount left for
        # current payment, need go get next payment
        last if ($thispay == 0);

    } # end of foreach

    return \%pay_orders;

} # end of sub
 
##############################################################################
=pod

=head2 get_orderid

get_orderid - get_orderid($dbh, $sid, $custid); 

=over 4

=item DESCRIPTION

this function get all the order ids which are 
still in table tcart owned by current sid and custid

=item INPUT

$dbh - current database handler

$sid - current sid

$custid - current custid

=item OUTPUT

@order_ids - an array contains aall the order ids which are 
             still in table tcart owned by current sid and custid

=item NOTES

=back

=cut
##############################################################################
sub get_orderid {

    my $dbh = shift;
    my $sid = shift;
    my $custid = shift;
    
    my @order_ids;    

    my $q = "SELECT orderid FROM tcart WHERE sid='$sid' AND custid='$custid' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;

    my %tem;
    while ( my $orderid = $sth->fetchrow ) {
       if (! $tem{$orderid}) {
           push @order_ids, $orderid;
           $tem{$orderid}=1;
       }
    }

    return @order_ids;

}


##############################################################################
=pod

=head2 get_shipping_weight

get_shipping_weight - get_shipping_weight();

=over 4

=item DESCRIPTION

this function takes the objid for the product
and return its shipping weight

=item INPUT

$dbh - current database handler

$objid - the objid of the product

$unitpr  - the price for this product, this is needed 
           to determine which is the weight when 
           multiple weight existed

=item OUTPUT

$weight - a scale which is the shipping weight of
          the product

=item NOTES

=back

=cut
##############################################################################
sub get_shipping_weight {

    my $dbh = shift;
    my $objid = shift;
    my $unitpr = shift;
    my $weight = ();

    my $q = "SELECT body FROM extraprop WHERE objid='$objid' AND ";
       $q .= "fieldname='ship_weight' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;

    WHILE: while ( my $body = $sth->fetchrow ) {
        #break body, get price and weight
        my @contents = split /\|/, $body;

        if ($contents[2]){
           next WHILE if ($contents[2] != $unitpr);
        }

        $weight = $contents[0];

    } # end of while

    return $weight;

}
##############################################################################
=pod

=head2 build_available_backorder_item_hash

build_available_backorder_item_hash - build_available_backorder_item_hash($from,$dbh);

=over 4

=item DESCRIPTION

this function get all the items for current sid and custid from tcart, 
and divid them into available item (positive with hquant) and 
back order item (with possitive squant)

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$dbh - current database handler

=item OUTPUT

\%cell_list - reference to a hash which contains available and 
              back order items with the inforation
              the format is $cell_list{'avail'}{lineobjid} = $h
                            $cell_list{'back'}{lineobjid} = $h
              $h is the hashref of the row in tcart

=item NOTES

=back

=cut
##############################################################################
sub build_available_backorder_item_hash {

    my $form = shift;
    my $dbh = shift;
    my %cell_list = ();

    ##
    # get all the items with the same sid and custid
    # including available items and back order items
    my $q = "SELECT * FROM tcart WHERE sid='$form->{'sid'}' AND custid='$form->{'custid'}' AND objclass != '140-A' ";
    my $sth = $dbh->prepare($q);
    $sth->execute;

    while ( my $h = $sth->fetchrow_hashref("NAME_lc") ) {

        ## get title for the product
        $h->{'title'} = &get_title($h->{'lineobjid'}, $dbh);

        ##
        # divid into available items and back order items
        # order with hquant means available
        # order with squant means back order
        if ($h->{'hquant'}) {
            $cell_list{'avail'}{$h->{'orderid'}}{$h->{'lineobjid'}} = $h;
        } elsif ($h->{'squant'}) {
            $cell_list{'back'}{$h->{'orderid'}}{$h->{'lineobjid'}} = $h;
        }
    } # end of while 

    return \%cell_list;
                                         
} # end of sub                                     


##############################################################################
=pod

=head2 update_custid_in_tcart

update_custid_in_tcart - update_custid_in_tcart($form->{'custid'}, $form->{'sid'}, $dbh);

=over 4

=item DESCRIPTION

this function setup the custid for the items in tcart using the current sid

=item INPUT

$custid - the custid for current sid

$sid - the current sid

$dbh - current database handler

=item OUTPUT

update tcart

=item NOTES

=back

=cut
##############################################################################
sub update_custid_in_tcart {

    my $custid = shift;
    my $sid = shift;
    my $dbh = shift;

    ## get objid and custid of rows with the current sid
    my $q = "SELECT objid, custid FROM tcart WHERE sid = '$sid' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;

    while ( my ($objid, $exist_custid) = $sth->fetchrow_array ) {

        ## if no exsiting custid for the row with current sid
        #  set it up
        if (!$exist_custid) {

            my $qq = "UPDATE tcart SET custid='$custid' WHERE objid='$objid' ";
            my $stq = $dbh->prepare( $qq );
            $stq->execute;
            $stq->finish;

        } # end of if

    } # end of while
    
    return;

} # end of sub

    
##############################################################################
=pod

=head2 get_new_eventid

get_new_eventid - get_new_eventid($site, $table, $value);

=over 4

=item DESCRIPTION

this function insert new event to levent table,
and return the event id for the inserted envent

=item INPUT

$site - Standard $site hash from &setsite

$table - name of table to be updated

$value - what will be updated in the $table

=item OUTPUT

$eventid - event id for the inserted envent

=item NOTES

=back

=cut
##############################################################################
sub get_new_eventid {

    my $site = shift;
    my $table = shift;
    my $value = shift;

    my $cdate = time();

    
#    my $q = "INSERT INTO levents (dtable, dvalue, cdate, series) VALUES ('$table', '$value', '$cdate', '$site->{'series'}') ";
#    my $sth = $site->{'dbh'}->prepare($q);
#    $sth->execute;
#    $sth->finish;

#    $q = "SELECT event, series FROM levents WHERE dtable = '$table' AND dvalue = '$value' AND cdate = '$cdate' ";
#    $sth = $site->{'dbh'}->prepare($q);
#    $sth->execute;
#    my ($event, $series) = $sth->fetchrow_array;
#    $sth->finish;

#    my $eventid = $event."-".$site->{'series'};

    my $eventid = &get_new_objid($site, $site->{'dbh'}, $table, $value);

    return $eventid;

}


##############################################################################
=pod

=head2 get_taxrate

get_taxrate - &get_taxrate($orderid, $site);

=over 4

=item DESCRIPTION

load taxrate for the orderid

=item INPUT

$orderid - the orderid

$site - Standard $site hash from &setsite

=item OUTPUT

$taxrate - the tax rate for the orderid

=item NOTES

=back

=cut
###############################################################################
sub get_taxrate {

    my $orderid = shift;
    my $site = shift;

    my $dbh = $site->{'dbh'};

    ## get addressid for orderid
    my $q = "SELECT shipto FROM orders WHERE objid='$orderid' ";
    my $sth = $dbh->prepare( $q );
    $sth->execute;
    my $addid = $sth->fetchrow;

    ## get shipping to state to determine the tax rate
    #  if the shipping state is the same as storestate, 
    #  then charge tax, if not, no tax
    $q = "SELECT state FROM addresses WHERE objid='$addid' ";
    $sth = $dbh->prepare( $q );
    $sth->execute;
    my $state = $sth->fetchrow;

    my $taxrate = 0;
    
    my $site_storestate = $site->{'storestate'};
    my $site_taxrate = $site->{'taxrate'};

    $site_storestate =~ s/(\s+)//g;
    $site_storestate =~ tr/a-z/A-Z/;
    $site_taxrate =~ s/(\s+)//g;

    my @storestate = split /,/, $site_storestate;
    my @storetaxrate = split /,/, $site_taxrate;

    my %rate;
    my $num = @storestate;
    foreach (0..$num-1){
       if ($storetaxrate[$_] =~ /^(\d+.?\d*|\.\d+)$/) {
           $rate{$storestate[$_]} = $storetaxrate[$_];
       } else {
           $rate{$storestate[$_]} = 0;
       } # end of if else
    } # end of foreach

    foreach my $store_state (keys %rate) {
       $taxrate = $rate{$store_state} if ($state eq $store_state);
    } # end of foreach

    return $taxrate;

} # end of sub


##############################################################################
=pod

=head2 get_currency_and_symbol

get_currency_and_symbol - &get_currencyand__symbol($form, $site);

=over 4

=item DESCRIPTION

load currency symbol 

=item INPUT

$form - - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

$currency - the currency

$this_symbol{$currency} - the currency symbol

=item NOTES

=back

=cut
###############################################################################
sub get_currency_and_symbol {

    my $form = shift;
    my $site = shift;
    my $dbh = $site->{'dbh'};

    ## search table tcart for currency
    my $q = "SELECT currency FROM tcart WHERE sid='$form->{'sid'}' ";
       $q .= "AND custid='$form->{'custid'}' ";
    my $sth = $dbh->prepare( $q );
       $sth->execute;

    my %this_symbol;
    my $currency;
    while ($currency = $sth->fetchrow) {
        last;
    }

    ## get currency symbol from site.pm
    my $cur = "$site->{'om_locale_curr_def'},$site->{'om_locale_curr_act'}";
    my $sym = $site->{'currency_symbol'};

    $cur =~ s/\s+//g;
    $sym =~ s/\s+//g;
    $cur =~ tr/a-z/A-Z/;

    my @store_cur = split /,/, $cur;
    my @store_sym = split /,/, $sym;

    my $num = @store_cur;
    foreach (0..$num-1) {
        $this_symbol{$store_cur[$_]} = $store_sym[$_] if $store_sym[$_];
    }
    
    ## return the currency symbol
    return ($currency, $this_symbol{$currency});

}


##############################################################################
=pod

=head2 get_title

get_titel - get_title($lineobjid, $dbh); 

=over 4

=item DESCRIPTION

load in title for the product

=item INPUT

$lineobjid - objid for the product  

$dbh - the standard database handler

=item OUTPUT

$title - a scale which is the title of the product

=item NOTES

=back

=cut
###############################################################################
sub get_title {

    my $lineobjid = shift;
    my $dbh = shift;

    my $vcode="";
    my $vardescript="";
    my %variant = ();
    my $title;

    # Get title for product
    my $q=" SELECT title from things where objid='$lineobjid' ";
    my $sth=$dbh->prepare($q);
    $sth->execute;

    $title = $sth->fetchrow;

    return $title;
}


##############################################################################
=pod

=head2 get_address_pulldown

get_address_pulldown - get_address_pulldown($form->{'cuitid'}, $dbh) 

=over 4

=item DESCRIPTION

create a pulldown of all addresses of the custid

=item INPUT

$custid - the custid 

$dbh - the current database handler

=item OUTPUT

$address_pulldown - a scale contains the html of pulldown 

=item NOTES

=back

=cut
###############################################################################
sub get_address_pulldown {

    my $custid = shift;
    my $dbh = shift;

    ## 
    # load all the shipping addresses for the custid
    # into a pulldown
    my $q = "SELECT * from addresses where custid = '$custid' ";
    my $sth = $dbh->prepare($q);
    $sth->execute;

    my @addresses = ();

    while ( my $h = $sth->fetchrow_hashref("NAME_lc") ) {

        ## break last name and first name
        my @names = split /\|/, $h->{'name'};

        my $name = $names[0].' '.$names[1];

        ## build the address in one line
        my $address = "\"$h->{'objid'}\"\|$name: $h->{'address1'}, ";
        
        if ($h->{'address2'}) {
            $address .= "$h->{'address2'}, ";
        }

        $address .= "$h->{'city'}, $h->{'state'}$h->{'postcode'}, ";
        ## by default, it's usa
        $h->{'country'} = "USA" if (! $h->{'country'});
        $address .= $h->{'country'};

        ## add this address to the address array for the custid
        push @addresses, $address;

    } # end of while

    ## build pulldown using obtained addresses
    my $address_pulldown = &gen_select(' ', 8, @addresses);

    return $address_pulldown;

} # end of sub


##############################################################################
=pod

=head2 load_addresslist

load_addresslist - &load_addresslist($addressid, $site);

=over 4

=item DESCRIPTION

load address into template for display

=item INPUT

$addressid - the objid id of the addresss

$site - Standard $site hash from &setsite

=item OUTPUT

$addresstemplate - a scale holds address list template

=item NOTES

=back

=cut
###############################################################################
sub load_addresslist {

    my $addressid = shift;
    my $site = shift;
           
    ## load address for the orderid
    my $q = "SELECT * FROM addresses WHERE objid='$addressid' ";
    my $sth = $site->{'dbh'}->prepare( $q );
    $sth->execute;
    my $h = $sth->fetchrow_hashref("NAME_lc");

    ## break names
    ($h->{'firstname'}, $h->{'lastname'}) = split /\|/, $h->{'name'};

    ## load __cell_ck_shippingmethod_avail(or back)_addr__             
    my $addresstemplate = 'cell_ck_addresslist_shipto';
    $addresstemplate = &loadtemplate("$addresstemplate","$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}");

    $addresstemplate = &permutebit( $addresstemplate, $h );

    return $addresstemplate;

}


##############################################################################
=pod

=head2 load_select_pulldown

load_select_pulldown - &load_select_pulldown($file, $form, $site);

=over 4

=item DESCRIPTION

To read in the specified .sel file and create pulldown 

=item INPUT

$file - a scale holds value of a the select file:  
        'PULLDOWN|NUM' where PULLDOWN is the name of 
        the file before the .sel and NUM is maximum 
        length of the key

$form  - Hashref to the current page that is displayed.

$site - Standard $site hash from &setsite

=item OUTPUT

$template - a scale holds the pulldown template

=item NOTES

=back

=cut
###############################################################################
sub load_select_pulldown {

    my $file = shift;
    my $form =shift;
    my $site =shift;

    my $select_path = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/$site->{'interface'}/sel";

    my ($filename,$key_length) = split /\|/, $file;
    $filename = "sel_".$filename;
    open FH, "<$select_path/$filename.sel";

    my @raw = <FH>;
    chomp @raw;
    close FH;

    # Make the sel_ files work with the current procedure.
    if ($filename =~ /sel_/) {
        $filename =~ s/sel_//gms;
    }

    my $select_input = lc($filename."_inp");

    my $template = &gen_select( $form->{$select_input},$key_length,@raw );

    return $template;
}


##############################################################################
=pod

=head2 update_orders_status

update_orders_status - update_orders_status($step, $form, $site);

=over 4

=item DESCRIPTION

this function updates the orders' status when in each step in checkout

=item INPUT

$step - the name of checkout steps

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

$payment - a reference to a hash contains the paymetn info

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_orders_status {
 
    my $step = shift;
    my $form = shift;
    my $site = shift;
    my $payment = shift;

    my $dbh = $site->{'dbh'};

    my $status;
    if ($step eq "shippingaddress") {
        $status = "15012-A";
    } elsif ($step eq "shippingmethod") {
        $status = "15012-A";
    } elsif ($step eq "giftwrap") {
        $status = "15012-A";
    } elsif ($step eq "payment") {
        $status = "15012-A";
    } elsif ($step eq "orderpreview") {
        $status = "15012-A";
    } elsif ($step eq "orderconfirm") {
        ## should call this subroutine to get authorization
        #$status = &payment_authorization($payment, $site) if ($payment);
        $status = "15018-A";
    }

    my @orderids = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});

    foreach my $orderid (@orderids) {
        my $q = "UPDATE orders SET status='$status' ";
           $q .= "WHERE objid='$orderid' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
           $sth->finish;
    } # end of foreach

} # end of sub
##############################################################################
=pod

=head2 update_database_for_orderrecap

update_database_for_orderrecap($form->{'sid'}, $form->{'custid'}, $form, $site);

=over 4

=item DESCRIPTION

update the database that need to be done when finish step orderrecap
split order if more than one availablility existed, one oder is avail items
the other order is back order items with new orderid

=item INPUT

$sid - current sid 

$custid - current custid

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_database_for_orderrecap {

    my $sid = shift;
    my $custid = shift;
    my $form = shift;
    my $site =shift;
    my $dbh = $site->{'dbh'};
    my $first_squant = 1; # a flag 
    my $now = time();

    my $q = "SELECT * FROM tcart WHERE sid='$form->{'sid'}' AND custid='$form->{'custid'}'";
    my $stq = $dbh->prepare( $q );
    $stq->execute;
    
    my $orderid;
    while ( my $h = $stq->fetchrow_hashref("NAME_lc") ) {

        my $sth;
        my %insert;

        if ($first_squant == 1) {
     
            ## if this is the first time detect squant
            #  get a new orderid       
            if ($h->{'squant'}) {
               $orderid = &get_new_eventid($site, "tcart", "sid:$form->{'sid'} custid:$form->{'custid'} create a new order id");
               $first_squant = 0;
            } # end of if           
           
        }

        if ($h->{'squant'}>0) {

           ## if thre is squant, break the order to two
           #  create a new order with new order id and
           #  quant equal to squant
           #  and reduce the quant of the origin order 
           #  to be hquant

           # get objid for create a new row
           my $objid = &get_new_eventid($site, "tcart", "sid:$form->{'sid'} custid:$form->{'custid'} lineobjid:$h->{'lineobjid'} create a new objid for tcart");
           
           my ($field, $value);    
           # build the insert fields and values
           foreach my $key ( %{$h} ) {

              next if (! exists $h->{$key});

              if ($key eq "objid") {
                  $insert{$key} = $objid;
              } elsif ($key eq "orderid") {
                  $insert{$key} = $orderid;                  
              } elsif ($key eq "quant") {
                  $insert{$key} = $h->{'squant'};
              } elsif ($key eq "hquant") {
                  $insert{$key} = 0;
              } elsif ($key eq "cdate") {
                  $insert{$key} = $now;
              } elsif ($key eq "mdate") {
                  $insert{$key} = $now;
              } elsif ($key eq "linetotal") {
                  $insert{$key} = $h->{'squant'} * $h->{'unitpr'};
              } else {
                  $insert{$key} = $h->{$key};
              }

           } # end of foeeach

              foreach (keys %insert) {
                  $field .= "$_,";
                  $value .= "\'$insert{$_}\',";
              }

              chop $field;
              chop $value;

           $q = "INSERT INTO tcart ($field) VALUES ($value) "; 
           $sth = $dbh->prepare( $q );
           $sth->execute;

           # update original row
           # if hquant equal to 0, means that this items has to 
           # moved to the back order order id, so should remove it
           # from available order
           my $hquant_linetotal = $h->{'hquant'} * $h->{'unitpr'};
           
           if ($h->{'hquant'}==0) {
               $q = "DELETE FROM tcart WHERE objid='$h->{'objid'}' ";
               $sth=$dbh->prepare( $q );
               $sth->execute;
           } else {
               $q = "UPDATE tcart SET quant='$h->{'hquant'}', squant='0', mdate='$now', linetotal='$hquant_linetotal' ";
               $q .= "WHERE objid='$h->{'objid'}' ";
               $sth = $dbh->prepare( $q );
               $sth->execute;
           }

        } elsif ($h->{'quant'} > $h->{'hquant'}) {

           my $hquant_linetotal = $h->{'hquant'} * $h->{'unitpr'};

           ## if hquant equal to 0, means this is an unavailable item
           #  remove from the table
           if ($h->{'hquant'}==0) {

               $q = "DELETE FROM tcart WHERE objid='$h->{'objid'}' ";
               $sth=$dbh->prepare( $q );
               $sth->execute;
           } else {

               ## if quant > hqaunt, this means there are unavailable items
               #  remove the unavailable quant, and set qaunt to hquant
               $q = "UPDATE tcart SET quant='$h->{'hquant'}', mdate='$now', linetotal='$hquant_linetotal' ";
               $q .= "WHERE objid='$h->{'objid'}' ";
               $sth = $dbh->prepare( $q );
               $sth->execute;
               $sth->finish;
           } # end of else

        } # end of if elsif

    } # end of while
                      
} # end of sub


##############################################################################
=pod

=head2 update_database_for_shippingadd

update_database_for_shippingadd($form, $site);

=over 4

=item DESCRIPTION

update the database that need to be done when finish step shipping address
create a new row with new orderid for each shipping address in table tcart,
insert every orderid into table orders

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_database_for_shippingadd {

    my $form = shift;
    my $site = shift;
 
    my $dbh = $site->{'dbh'};
    my $now = time();

    # if from single shpping address page
    # update the tcart table
    # if multiple, need to split orders based on shipping address
    # each shipping address has a sperate order id
    if ($form->{'current_page'} eq "singleshippingadd") {

        ## get orderids for current sid and custid
        my @orderid = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});

        my $orderadd; # a hashref with format $orderadd->{orderid}=addid
        foreach (@orderid) {
            $orderadd->{$_} = $form->{'addid'};
        }
        &create_orders_row($orderadd, $form, $site);
    } elsif ($form->{'current_page'} eq "multishippingadd") {

        ## build a hash contains the hashref of orderid in tcart 
        #  for current sid and custid
        my %tcart;

        # get the original infomation from tcart
        my $q = "SELECT * FROM tcart WHERE sid='$form->{'sid'}' AND custid='$form->{'custid'}' ";
        my $sth = $dbh->prepare ( $q );
        $sth->execute;

        while ( my $h = $sth->fetchrow_hashref("NAME_lc") ) {
            $tcart{$h->{'orderid'}}{$h->{'lineobjid'}} = $h;

        } # end of while


        # break the ship info from $form 
        # into orderid->addressid->lineobjid        

        my %shipto; # this the hash contain the shipto infomation

        foreach my $key ( keys %{$form} ) {

            # just want ship to infomation
            next if ($key !~ /_shipto$/);
            my $addid = $form->{$key};
            my ($orderid, $lineobjid, @dummy) = split /\_/, $key;

            if ($shipto{$orderid}{$addid}{$lineobjid}) {
                $shipto{$orderid}{$addid}{$lineobjid} += 1;
            } else {
                    $shipto{$orderid}{$addid}{$lineobjid} = 1;
            } # end of if else

        } # end of foreach

        my $orderadd; # for update table orders $orderadd->{orderid}=addid
        ## break the order and create new row in table orders
        foreach my $orderid (keys %shipto) {

            foreach my $addid (keys %{$shipto{$orderid}}) {

                # get a new order id for tcart
                my $new_orderid = &get_new_eventid($site, "tcart", "sid:$form->{'sid'} custid:$form->{'custid'} orderid:$orderid addid:$addid get a new order id for multiple shipping addresses");

                foreach my $lineobjid (keys %{$shipto{$orderid}{$addid}}) {

                # get a new objid id for tcart row
                my $new_objid = &get_new_eventid($site, "tcart", "sid:$form->{'sid'} custid:$form->{'custid'} orderid:$orderid addid:$addid lineobjid:$lineobjid get a new objid for multiple shipping addresses row");

                    my %insert;
                    my ($field, $value);
                    my $h = $tcart{$orderid}{$lineobjid}; 
                    # build the insert fields and values
                    foreach my $key ( %{$h} ) {

                       next if (! exists $h->{$key});

                       if ($key eq "objid") {
                          $insert{$key} = $new_objid;
                       } elsif ($key eq "orderid") {
                          $insert{$key} = $new_orderid;
                       } elsif ($key eq "quant") {
                          $insert{$key} = $shipto{$orderid}{$addid}{$lineobjid};
                       } elsif ($key eq "hquant") {
                          $insert{$key} = $shipto{$orderid}{$addid}{$lineobjid} if ($h->{'hquant'});
                       } elsif ($key eq "squant") {
                          $insert{$key} = $shipto{$orderid}{$addid}{$lineobjid} if ($h->{'squant'});
                       } elsif ($key eq "cdate") {
                          $insert{$key} = $now;
                       } elsif ($key eq "mdate") {
                          $insert{$key} = $now;
                       } elsif ($key eq "linetotal") {
                          $insert{$key} = $shipto{$orderid}{$addid}{$lineobjid} * $h->{'unitpr'};
                       } else {
                          $insert{$key} = $h->{$key};
                       }

                   } # end of foeeach

                   foreach (keys %insert) {
                      $field .= "$_,";
                      $value .= "\'$insert{$_}\',";
                   }

                   chop $field;
                   chop $value;

                   $q = "INSERT INTO tcart ($field) VALUES ($value) ";
                   $sth = $dbh->prepare( $q );
                   $sth->execute;

                   # build order and addid hash to update table orders
                   $orderadd->{$new_orderid} = $addid;

                } # end of inner foreach ($lineobjid)

            } # end of middle foreach ($addid)

            # delete the original row at tcart
           $q = "DELETE FROM tcart WHERE sid='$form->{'sid'}' AND custid='$form->{'custid'}' AND orderid='$orderid' ";
           $sth = $dbh->prepare( $q );
           $sth->execute;

       } # end of outer foreach ($orderid)

       # update table orders
       &create_orders_row($orderadd, $form, $site);
   } # end of if else


   ## update taxrate for tcart table
 
    ##
    # get all the orderids from table orders 
    # which are still table intcart owned by current sid and custid
    my @order_ids = &get_orderid($dbh, $form->{'sid'}, $form->{'custid'});
    
    foreach my $orderid (@order_ids) {
        my $taxrate = &get_taxrate($orderid, $site);

        my $q = "UPDATE tcart SET taxrate='$taxrate' WHERE orderid='$orderid' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
           
    } # end of foreach
            
} # end of sub

            
##############################################################################
=pod

=head2 create_orders_row 

create_orders_row - create_orders_row($orderadd, $form, $site) 

=over 4

=item DESCRIPTION

this function takes the ship to address id and
create an order row in table orders

=item INPUT

$orderadd - a refernce to a hash with format of $orderadd->{orderid}=addid

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub create_orders_row {

    my $orderadd = shift;
    my $form = shift;
    my $site = shift;
    my $now = time();

    my $dbh = $site->{'dbh'};
    
    foreach my $orderid (keys %{$orderadd}) {

        my $qq = "INSERT INTO orders (objid, sid, custid, ";
           $qq .= "shipto, shipwt, st_nontax, ";
           $qq .= "cdate, mdate, status) ";
           $qq .= "VALUES ('$orderid', '$form->{'sid'}', '$form->{'custid'}', ";
           $qq .= "'$orderadd->{$orderid}', '0', '0', ";
           $qq .= "'$now', '$now', '15012-A') ";
        my $stq = $dbh->prepare( $qq );
           $stq->execute;

        ## get tax rate
        my $taxrate = &get_taxrate($orderid, $site);

        ## Get info for orders in tcart for current sid and custid
        my $q = "SELECT linetotal FROM tcart WHERE orderid='$orderid' AND sid='$form->{'sid'}' AND custid='$form->{'custid'}' ";
        my $sth = $dbh->prepare( $q );
        $sth->execute;
        
        my $linetotal = 0;
        while (my $subtotal = $sth->fetchrow) {            
           $linetotal += $subtotal;
        }

        my $tax = $linetotal * $taxrate / 100;
           $tax = &nicedecimals($tax);
        my $order_total = $linetotal + $tax;

        $order_total = &nicedecimals($order_total);

        $qq = "UPDATE orders SET st_taxable='$linetotal', ";
        $qq .= "st_tax='$tax', taxrate='$taxrate', ordtotal='$order_total' ";
        $qq .= "WHERE objid='$orderid' ";  
        $stq = $dbh->prepare( $qq );
        $stq->execute;

    } # end of foreach

} # end of sub    


##############################################################################
=pod

=head2 update_database_for_shippingmethod

update_database_for_shippingmethod($form, $site);

=over 4

=item DESCRIPTION

update the database that need to be done when finish step shipping method
record shipping method in table tcart, add shipping cost to ordertotal in 
table orders

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_database_for_shippingmethod {

    my $form = shift;
    my $site = shift;
    
    my $dbh = $site->{'dbh'};
    my $now = time();

    # update by order id
    foreach my $key (keys %{$form}) {
        # just want shipping method for each order
        next if ($key !~ /method$/);
       
        my ($methodid, $price) = split /:/, $form->{$key};
        my $orderid = (split /\_/, $key)[0];

        # search table tcart to see if there is shipping method for this order id
        # if so, update it; if not, create it.
        my $q = "SELECT objclass FROM tcart WHERE sid='$form->{'sid'}' ";
           $q .= "AND custid='$form->{'custid'}' AND orderid='$orderid' ";
           $q .= "AND objclass='140-A' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
        my $objclass = $sth->fetchrow;
                
        if ($objclass) {
           $q = "UPDATE tcart SET mdate='$now', lineobjid='$methodid', ";
           $q .= "unitpr='$price', linetotal='$price' WHERE sid='$form->{'sid'}' ";
           $q .= "AND custid='$form->{'custid'}' AND orderid='$orderid' ";
           $q .= "AND objclass='$objclass' ";
           $sth = $dbh->prepare( $q );
           $sth->execute;
        } else {

           ## insert shipping method in table tcart
           my $objid = &get_new_eventid($site, "tcart", "sid=$form->{'sid'} custid=$form->{'custid'} orderid=$orderid create new row objid for shipping method");

           $q = "INSERT INTO tcart (objid, objclass, sid, custid, ";
           $q .= "orderid, cdate, mdate, lineobjid, ";
           $q .= "currency, unitpr, quant, linetotal) ";
           $q .= "VALUES ('$objid', '140-A', '$form->{'sid'}', '$form->{'custid'}', ";
           $q .= "'$orderid', '$now', '$now', '$methodid', ";
           $q .= "'$site->{'om_locale_curr_def'}', '$price', '1', '$price') ";
           $sth = $dbh->prepare( $q );
           $sth->execute;
        } # end of if else

        ## get order total
        #$q = "SELECT linetotal, taxrate FROM tcart WHERE sid='$form->{'sid'}' ";
        #$q .= "AND custid='$form->{'custid'}' AND orderid='$orderid' ";
        #$q .= "AND objclass != '140-A' ";
        #$sth = $dbh->prepare( $q );
        #$sth->execute;

        #my $ordtotal = 0;
        #while ( my ($linetotal, $taxrate) = $sth->fetchrow ) {
        #    $ordtotal += $linetotal*(1+$taxrate/100);
        #} # end of while

        ## add shipping cost to order total
        #$ordtotal += $price;

        ## update order total in table orders
        $q = "UPDATE orders SET ordtotal=ordtotal+$price, mdate='$now' WHERE objid='$orderid' ";

        $sth = $dbh->prepare( $q );
        $sth->execute;

   } # end of foreach

} # end of sub


##############################################################################
=pod

=head2 update_database_for_giftwrap

update_database_for_giftwrap($form, $site);

=over 4

=item DESCRIPTION

update the database that need to be done when finish step gift wrap
record wrapping infomation into table extraprop

=item INPUT

$form - Reference to standard %form hash from &ReadParse

$site - Standard $site hash from &setsite

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_database_for_giftwrap {

    my $form = shift;
    my $site = shift;

    my $dbh = $site->{'dbh'};
    my $now = time();

    my (%gc, %wrap);
    foreach my $key (keys %{$form}) {
        next if ($key !~ /_inp$/);
        my @values = split /\_/, $key;
        # wrapping and greeting card store seperately
        if ($key =~ /gcline/) {
            my $orderid = $values[0];
            my $text = $form->{$key};
            $gc{$orderid} .= "$text|";
        } else {
            my $orderid = $values[0];
            my $lineobjid = $values[1];
            my $num = $values[2];
            my $type = $values[3];
            $wrap{$orderid}{$lineobjid}{$num}{'style'}=$form->{$key} if ($type eq "wrapstyle");
            $wrap{$orderid}{$lineobjid}{$num}{'to'}=$form->{$key} if ($type eq "towho");
            $wrap{$orderid}{$lineobjid}{$num}{'from'}=$form->{$key} if ($type eq "fromwho");
        } # end of if else        
    } # end of foreach        
      
    ## update database
    my %total; # a hash holds the total cost for this order wrapping

    # greeting card
    foreach my $orderid (keys %gc) {
        # if there is no content, next
        next if ($gc{$orderid} eq "|||");

        # this price line should be changed in the future
        # should get price from database
        $total{$orderid} = 7.00;

        # search if greeting card already been there,
        # if yes, update; if no, insert
        my $q = "SELECT objid FROM extraprop WHERE objid='$orderid' ";
           $q .= "AND fieldname='gc|line1|line2|line3' ";
        my $sth = $dbh->prepare( $q );
        $sth->execute;
        my $gcobjid = $sth->fetchrow;

        if ($gcobjid) {
            $q = "UPDATE extraprop SET body='$gc{$orderid}', mdate='$now' ";
            $q .= "WHERE objid='$orderid' AND fieldname='gc|line1|line2|line3' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            $sth->finish;
        } else {
            $q = "INSERT INTO extraprop (objid, fieldname, body, mdate, cdate) ";
            $q .= "VALUES ('$orderid', 'gc|line1|line2|line3', '$gc{$orderid}', ";
            $q .= "'$now', '$now') ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
            $sth->finish;
        } # end of if elase

    } # end of foreach
             
    # wrapping
    foreach my $orderid (keys %wrap) {
       foreach my $lineobjid (keys %{$wrap{$orderid}}) {
          foreach my $num (keys %{$wrap{$orderid}{$lineobjid}}) {
             # if there is no wrap and from or to, next       
             next if ($wrap{$orderid}{$lineobjid}{$num}{'style'} eq "nowrap");

             # add to order wrapping total
             my $price = (split /:/, $wrap{$orderid}{$lineobjid}{$num}{'style'})[1];

             $total{$orderid} += $price;

             # search if this data already been record, if so, update,
             # if not, insert
             my $q = "SELECT objid FROM extraprop WHERE objid='$orderid' ";
                $q .= "AND fieldname='wrap|lineobjid|num' ";
                $q .= "AND body LIKE 'wrap|$lineobjid|$num|%' ";                
             my $sth = $dbh->prepare( $q );
                $sth->execute;
             my $wrapobjid = $sth->fetchrow;

             if ($wrapobjid) {
                 $q = "UPDATE extraprop SET body='wrap|lineobjid=$lineobjid|num=$num|style=$wrap{$orderid}{$lineobjid}{$num}{'style'}|to=$wrap{$orderid}{$lineobjid}{$num}{'to'}|from=$wrap{$orderid}{$lineobjid}{$num}{'from'}', mdate='$now' ";
                 $q .= "WHERE objid='$orderid' ";
                 $q .= "AND fieldname='wrap|lineobjid|num' ";
                 $q .= "AND body LIKE 'wrap|$lineobjid|$num|%' ";
                 $sth = $dbh->prepare( $q );
                 $sth->execute;
                 $sth->finish;
             } else {
                 $q = "INSERT INTO extraprop (objid, fieldname, body, mdate, cdate) ";
                 $q .= "VALUES ('$orderid', 'wrap|lineobjid|num', ";
                 $q .= "'wrap|lineobjid=$lineobjid|num=$num|style=$wrap{$orderid}{$lineobjid}{$num}{'style'}|to=$wrap{$orderid}{$lineobjid}{$num}{'to'}|from=$wrap{$orderid}{$lineobjid}{$num}{'from'}', ";
                 $q .= "'$now', '$now') ";
                 $sth = $dbh->prepare( $q );
                 $sth->execute;
                 $sth->finish;
             } # end of if else

          } # end of foreach(num)
       } # end of foreach(lineobjid)
    } # end of orderid
            
    ## update table tcart and orders
    foreach my $orderid (keys %total) {

        my $taxrate = &get_taxrate($orderid, $site);
        
        # search if already in tcart,
        # if so, update; if not, insert
        my $q = "SELECT objid FROM tcart WHERE orderid='$orderid' ";
           $q .= "AND objclass='45002-A' ";
        my $sth = $dbh->prepare( $q );
           $sth->execute;
        my $thisobjid = $sth->fetchrow;

        if ($thisobjid) {
            $q = "UPDATE tcart SET linetotal='$total{$orderid}' ";
            $q .= "WHERE orderid='$orderid' AND objclass='45002-A' ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
        } else {
            my $objid = &get_new_eventid($site, "tcart", "sid:$form->{'sid'} custid:$form->{'custid'} orderid:$orderid get objid for table tcart row");
            $q = "INSERT INTO tcart (objid, objclass, sid, custid, ";
            $q .= "orderid, cdate, mdate, currency, ";
            $q .= "taxable, taxrate, unitpr, quant, linetotal) ";
            $q .= "VALUES ('$objid', '45002-A', '$form->{'sid'}', '$form->{'custid'}', ";
            $q .= "'$orderid', '$now', '$now', '$site->{'om_locale_curr_def'}', ";
            $q .= "'1', '$taxrate', '$total{$orderid}', '1', '$total{$orderid}') ";
            $sth = $dbh->prepare( $q );
            $sth->execute;
        }

        # update ordtotal of table orders
        my $wrap_tax = &nicedecimals($total{$orderid}*($taxrate/100));
        my $wraptotal = &nicedecimals($total{$orderid}+$wrap_tax);

        $q = "UPDATE orders SET ordtotal=ordtotal+$wraptotal, st_tax=st_tax+$wrap_tax ";
        $q .= "WHERE objid='$orderid' ";
        $sth = $dbh->prepare( $q );
        $sth->execute;

    } # end of foreach

} # end of sub


##############################################################################
=pod

=head2 checkout_message

checkout_message - &checkout_message($site); 

=over 4

=item DESCRIPTION

load in error messages for this script

=item INPUT

$site - Standard $site hash from &setsite

=item OUTPUT

%script_msg - a hash, which holds the error message need to used in the script

=item NOTES

=back

=cut
###############################################################################
sub checkout_message {

    my $site = shift;
    my %script_msg = ();

    open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'scriptname'}.messages");  

    while(<EM>) {
        chomp;
        next if /^\#/;
        next if (! $_);
        my @temp=split /\t/;
        $script_msg{$temp[0]}=$temp[1];
    }
    close(EM);

    return \%script_msg;

}

1;
