#!/usr/local/perl
############################################################################
#    Copyright (C) 2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict; 

##############################################################################
=pod

=head1 configure.pm

configure.pm - configuration script library

=head1 DESCRIPTION

This library is used for configure.pl.

=head1 NOTES & BUGS

=cut

############################################################################## 

##############################################################################
=pod

=head2 yesno 

yesno - &yesno($yesno);

=over 4

=item DESCRIPTION

Checks input to see if person has answered positively.
Returns 1 for positive or 0 for negitive.

=item INPUT

$yesno - answer to be evaluated.

=item OUTPUT

$yesno - 1 for positive or 0 for negitive.

=item NOTES

=back

=cut
###############################################################################    

sub yesno {

    my ($yesno) = shift;

    $yesno = (split //, $yesno)[0];
    $yesno =~ tr/a-z/A-Z/;

    if ( $yesno eq "Y" ) { 
       return 1;
    } else {
       return 0;
    }
}


##############################################################################
=pod

=head2 get_db_info 

get_db_info - &get_db_info($site,$username,$passwd,$servername,$port,$new,$current);

=over 4

=item DESCRIPTION

Prompts for DBI parameters to be used.

=item INPUT

$site - site hash
$username - DB user name
$passwd - DB passwd
$servername - DB server name
$port - DB port
$new - new information for the conf files
$current - current information from the conf files.

=item OUTPUT

$username - DB user name
$passwd - DB passwd
$servername - DB server name
$port - DB port
$new - new information for the conf files
$current - current information from the conf files.

=item NOTES

=back

=cut
###############################################################################  

sub get_db_info {

    my ($site) = shift;
    my ($username) = shift;
    my ($passwd) = shift;
    my ($servername) = shift;
    my ($port) = shift;
    my ($new) = shift;
    my ($current) = shift;

    print "$site->{'script_msg'}{'info_username1'}$current->{'site_dbiuser'}$site->{'script_msg'}{'info_username2'}";
    $username = <STDIN>;
    chomp $username;
    $username="$current->{'site_dbiuser'}" if ( ! $username );
    $new->{'site_dbiuser'}=$username;

    print "$site->{'script_msg'}{'info_password1'}$current->{'site_dbipwd'}$site->{'script_msg'}{'info_password2'}";
    $passwd = <STDIN>;
    chomp $passwd;
    $passwd="$current->{'site_dbipwd'}"  if ( ! $passwd );
    $new->{'site_dbipwd'}=$passwd;

    print "$site->{'script_msg'}{'info_servername1'}$current->{'site_database_serv'}$site->{'script_msg'}{'info_servername2'}";
    $servername = <STDIN>;
    chomp $servername;
    $servername="$current->{'site_database_serv'}" if ( ! $servername );
    $new->{'site_database_serv'}=$servername;

    print "$site->{'script_msg'}{'info_port1'}$current->{'site_database_port'}$site->{'script_msg'}{'info_port2'}";
    $port = <STDIN>;
    chomp $port;
    $port="$current->{'site_database_port'}" if ( ! $port );
    $new->{'site_database_port'}=$port; 

    return ($username,$passwd,$servername,$port,$new);
}

##############################################################################
=pod

=head2 header

header - &header($site);

=over 4

=item DESCRIPTION

Displays header of software product.

=item INPUT

$site - site hash 

=item OUTPUT

None.

=item NOTES

=back

=cut
###############################################################################    

sub header {

    my ($site) = shift;

    my ($answer,$yesno);

    print "$site->{'script_msg'}{'header_header'}";
    print "$site->{'script_msg'}{'header_welcome'}";
    print "$site->{'script_msg'}{'header_continue'}";

    $answer = <STDIN>;
    $yesno = &yesno($answer);

    if ( $yesno == 1 ) {
        # continue 
    } else {
        die "$site->{'script_msg'}{'header_exiting'}";
    }
} 

##############################################################################
=pod

=head2 version 

version - &version($site);

=over 4

=item DESCRIPTION

Prints Perl version and checks related perl information.

=item INPUT

$site - site hash

=item OUTPUT

None.

=item NOTES

=back

=cut
###############################################################################    

sub version {

    my ($site) = shift;  

    print "$site->{'script_msg'}{'version_header'}";
    print "$site->{'script_msg'}{'version_version'}";
    if ( $] > 5 ) {
        print "$site->{'script_msg'}{'version_ok'}";

        eval 'use LWP::UserAgent; '; # check for libwww-perl

        if ( $@ ) {
            print "$site->{'script_msg'}{'version_notinstalled'}";
        } else {
            print "$site->{'script_msg'}{'version_installed'}";
        }

    } else {
        die "$site->{'script_msg'}{'version_update'}";
    }

}

##############################################################################
=pod

=head2 dbi 

dbi - &dbi($site);

=over 4

=item DESCRIPTION

Checks for Perl DBI information.

=item INPUT

$site - site hash

=item OUTPUT

None.

=item NOTES

=back

=cut
############################################################################### 

sub dbi {

    use DBI;

    my ($site) = shift;
    my ($i,$driver,$l,$dataSource,$answer,$yesno);
    my (@drivers,@dataSources,@database);

    ## checking DBI interfaces installed
    #

    print "$site->{'script_msg'}{'dbi_header'}";

    print "$site->{'script_msg'}{'dbi_remote'}";

    $answer = <STDIN>;
    $yesno = &yesno($answer);

    if ( $yesno == 0 ) { 

        print "$site->{'script_msg'}{'dbi_checking'}";

        @drivers = DBI->available_drivers();

        die "$site->{'script_msg'}{'dbi_nodrivers'}" unless @drivers;

        print "$site->{'script_msg'}{'dbi_done'}";

        print "$site->{'script_msg'}{'dbi_listing'}";
        $i=1;
        foreach $driver ( @drivers ) {
            next if ( $driver eq "Proxy" );
            next if ( $driver eq "ADO" );
            next if ( $driver eq "ExampleP" );
            print "$site->{'script_msg'}{'dbi_driver'} $driver\n";
            @dataSources = DBI->data_sources( $driver );

            if ( $DBI::err ) {
                $l=$DBI::errstr;
                print "$site->{'script_msg'}{'dbi_error'} $l";
            } # done testing for DBI error     
            foreach $dataSource ( @dataSources ) {
                next if ( ! $dataSource );
                print " ($i)$site->{'script_msg'}{'dbi_datasource'} $dataSource \n";
                $database[$i]=$dataSource;
                $i++;
            }
            print "\n";
        }

    }

    return ($yesno);

}

##############################################################################
=pod

=head2 db

db - &db($site);

=over 4

=item DESCRIPTION

Databse setup

=item INPUT

$site - site hash

=item OUTPUT

$new - conf file info
$dbh - database handler
$site - site hash

=item NOTES

=back

=cut
############################################################################### 

sub db {

    use DBI;

    my ($site) = shift;
    my ($remote) = shift;

    my ($i,$key,$value,$dbnamehold,$dr,$q,$sth);
    my ($username,$passwd,$servername,$port,$datadriver,$driver,$yesno,$dropok,$answer);
    my ($dbs,$dbnames,$choice,$dataSource,$dbname,$ORACLE_HOME,$ORACLE_SID,$tablespace,$dbh);
    my (@db,@dataSources,@dataSource,@database);
    my (%current,%new,$new,@drivers,$driversold);

    ## read database specific information in case they are reinstalling

    open( FH, "<./conf/database.conf" ); 
    @db=(<FH>);
    close(FH);

    while ( @db ) {             
        $_ = shift @db;
        next if ( $_ =~ /^#/ || $_ =~ /^$/ );
        chomp;

        ( $key,$value ) = split /\s/, $_, 2;
        next if ( ! $key );
        $value =~ s/^\s+//g;
        $value =~ s/\s+$//g;

        $current{$key}=$value;
    }

    $username=$current{'site_dbiuser'};
    $passwd=$current{'site_dbipwd'};
    $servername=$current{'site_database_serv'};
    $port=$current{'site_database_port'};
    $datadriver=$current{'site_datadriver'};                       

    ## Database selection
    #
    print "$site->{'script_msg'}{'db_header'}";

    print "$site->{'script_msg'}{'db_which'}$datadriver$site->{'script_msg'}{'db_datadriver'}";

    $driver=<STDIN>;
    chomp $driver;
    $driver = $datadriver if ( ! $driver );

    print "$site->{'script_msg'}{'db_header2'}";

    if ( $driver !~ /Oracle/ ) { 

        print "$site->{'script_msg'}{'db_new'}";

        $answer = <STDIN>;
        $yesno = &yesno($answer);

    } elsif ( $driver =~ /Oracle/ ) {

        $yesno = 1;
    
    }

    if ( $yesno == 1 ) {

        $new{'site_datadriver'}=$driver;

        if ( $driver !~ /Oracle/ ) { 

            print "$site->{'script_msg'}{'db_name'}";
            $dbname = <STDIN>;
            chomp $dbname;
            $dbname="osallcommerce" if ( ! $dbname );
            $new{'site_database'}=$dbname;

        }


        @dataSources = DBI->data_sources( $driver );

        foreach $dataSource ( @dataSources ) {
            next if ( ! $dataSource );
            $dbnames = (split /:/, $dataSource)[2];
            $dbs = (split /:/, $dataSource)[1];
            die "$site->{'script_msg'}{'db_already'}" if ($dbnames eq $dbname and $dbs eq $driver);
        }

        ($username,$passwd,$servername,$port,$new) = &get_db_info($site,$username,$passwd,$servername,$port,\%new,\%current);

        $new->{'site_database'}=$dbname;

        if ( $driver =~ /Oracle/ ) {
            print "$site->{'script_msg'}{'db_tablespace'}";
            $tablespace=<STDIN>;
            chomp $tablespace;
            $site->{'tablespace'} = $tablespace; 

            print "$site->{'script_msg'}{'db_home'}";
            $ORACLE_HOME=<STDIN>;
            chomp $ORACLE_HOME;

            print "$site->{'script_msg'}{'db_sid'}";
            $ORACLE_SID=<STDIN>;
            chomp $ORACLE_SID;

            $site->{'ora_home'}=$ORACLE_HOME;
            $site->{'ora_sid'}=$ORACLE_SID;
        }

        if ( $driver =~ /Pg/ ) {
            if ( $passwd ) {
                `createdb -p $port -h $servername create $dbname`;
            } else {
                `createdb -p $port -h $servername create $dbname`;
            }
        }

        if ( $driver =~ /mysql/ && $^O !~ /MSWin32/ ) {
            if ( $passwd ) {
                `mysqladmin -h $servername -u $username -p$passwd create $dbname`;
            } else {
                `mysqladmin -h $servername -u $username create $dbname`;
            }
        } elsif ( $driver =~ /mysql/ && $^O =~ /MSWin32/ ) {

            $dbh = DBI->connect("DBI:mysql:test:$servername:$port", "$username","$passwd");
            $q = "CREATE DATABASE $dbname";
            $sth = $dbh->prepare($q);
            $sth->execute;
            $dbh->disconnect;
        }

        $dbh = DBI->connect("DBI:mysql:$dbname:$servername:$port", "$username", "$passwd", { PrintError => 0, RaiseError => 0 }) if ( $driver =~ /mysql/ ); 

        $dbh = DBI->connect("DBI:$driver:dbname=$dbname;host=$servername;port=$port", "$username", "$passwd", { PrintError => 1, RaiseError => 1 }) if ( $driver =~ /Pg/ );     

        $dbh = DBI->connect("dbi:Oracle:host=$servername;sid=$ORACLE_SID", "$username", "$passwd", { PrintError => 1, RaiseError => 1 }) if ( $driver =~ /Oracle/ );


        print "\n $dbname $site->{'script_msg'}{'db_created'}" if ( $driver !~ /Oracle/ );

    } elsif ( $yesno == 0 && ! @dataSource ) {
        
        print "$site->{'script_msg'}{'db_recreate'}";

        $answer = <STDIN>;
        $yesno = &yesno($answer);

        if ( $yesno == 1 ) {

            @dataSources = DBI->data_sources( $driver );

            print "$site->{'script_msg'}{'db_whichdb'}" if ( ! $remote );

            print "$site->{'script_msg'}{'db_whichdb_alt'}" if ( $remote );

            $choice = <STDIN>;

        @drivers = DBI->available_drivers();
   
        $i=1;
 
        foreach $driver ( @drivers ) {
            next if ( $driver eq "Proxy" );
            next if ( $driver eq "ADO" );
            next if ( $driver eq "ExampleP" );
            @dataSources = DBI->data_sources( $driver );
   
            foreach $dataSource ( @dataSources ) {
                next if ( ! $dataSource );
                $database[$i]=$dataSource;
                $i++;
            }
 
        }   

            if ( $choice =~ /\d/  && $database[$choice] || $remote ) {

                ($username,$passwd,$servername,$port,$new) = &get_db_info($site,$username,$passwd,$servername,$port,\%new,\%current);
                if ( ! $remote ) {
        
                    $dbname = (split /:/, $database[$choice])[2];
                    $dbname = (split /=/, $dbname)[1] if ( $dbname =~ /=/ );
                    $driver = (split /:/, $database[$choice])[1];

                } else {

                    $dbname = $choice;
                    chomp $dbname; 
                    $database[$choice]="DBI:$driver:$dbname";

                }

                print "$site->{'script_msg'}{'db_sure1'} $dbname $site->{'script_msg'}{'db_sure2'}";

                $answer = <STDIN>;
                $dropok = &yesno($answer);

                if ( $dropok == 1 ) {

                    $dbh = DBI->connect("$database[$choice]:$servername:$port", "$username", "$passwd", { PrintError => 0, RaiseError => 0 }) if ( $driver =~ /mysql/ );
                    $dbnamehold=$dbname if ( $driver =~ /Pg/ );
                    $dbname="template1" if ( $driver =~ /Pg/ );
                    $dbh = DBI->connect("DBI:$driver:dbname=$dbname;host=$servername;port=$port", "$username", "$passwd", { PrintError => 1, RaiseError => 1 }) if ( $driver =~ /Pg/ );
#                    DBI->trace( 4, 'dbi_debug.log' ) if ( $debug );

                    print "\n Dropping $dbname ... " if ( $driver !~ /Pg/ );
                    print "\n Dropping $dbnamehold ... " if ( $driver =~ /Pg/ );
                    $new{'site_database'}=$dbname if ( $driver !~ /Pg/ );
                    $new{'site_database'}=$dbnamehold if ( $driver =~ /Pg/ );
                    $dr = (split /:/, $database[$choice])[1];
                    $new{'site_datadriver'}=$dr;
                    $q = "DROP DATABASE $dbname" if ( $driver !~ /Pg/ );
                    $q = "DROP DATABASE $dbnamehold" if ( $driver =~ /Pg/ );

                    $sth = $dbh->prepare( $q );
                    $sth->execute;
                    $dbh->disconnect if ($driver =~ /Pg/ || $^O =~ /MSWin32/ );
                    $dbname=$dbnamehold if ( $driver =~ /Pg/ );
                    print "$site->{'script_msg'}{'db_done1'} $dbname ... "; 

##                   $q = "CREATE DATABASE $dbname";
##                   $sth = $dbh->prepare( $q );
##                   $sth->execute;

#                    $dbh->disconnect;

                    if ( $driver =~ /Pg/ ) {
                        if ( $passwd ) {
                            `createdb -p $port -h $servername create $dbname`;
                        } else {
                            `createdb -p $port -h $servername create $dbname`;
                        }
                    }

                    $dbh = DBI->connect("DBI:$driver:dbname=$dbname;host=$servername;port=$port", "$username", "$passwd", { PrintError => 1, RaiseError => 1 }) if ( $driver =~ /Pg/ );
                    if ( $driver =~ /mysql/ && $^O !~ /MSWin32/ ) {
                        if ( $passwd ) {
                            `mysqladmin -h $servername -u $username -p$passwd create $dbname`;
                        } else {
                            `mysqladmin -h $servername -u $username create $dbname`;
                        }
                    } elsif ( $driver =~ /mysql/ && $^O !=~ /MSWin32/ ) {

                        $dbh = DBI->connect("DBI:mysql:test:$servername:$port","$username","$passwd");
                        $q = "CREATE DATABASE $dbname";
                        $sth = $dbh->prepare($q);
                        $sth->execute;
                        $dbh->disconnect;
                        
                        $dbh = DBI->connect("DBI:mysql:$dbname:$servername:$port","$username","$passwd");
                    }

                    print "$site->{'script_msg'}{'db_done2'}";
                     
                } else {
                    die "$site->{'script_msg'}{'db_abort'}";
                }
            } else {
                die "$site->{'script_msg'}{'db_invalid'}";
            }
        } else { 
            die "$site->{'script_msg'}{'db_abort'}";
        }
    } else {
       die "$site->{'script_msg'}{'db_exiting'}";
    }

    $site->{'site_datadriver'}=$driver;

    return ($new,$dbh,$site);
}

##############################################################################
=pod

=head2 db

httpd - &httpd($site);

=over 4

=item DESCRIPTION

Apache configuration

=item INPUT

$site - site hash

=item OUTPUT

$param - parameters to be used with conf files

=item NOTES

=back

=cut
############################################################################### 

sub httpd {

    my ($site) = shift;

    my (@infile,@outfile,@allfiles);
    my ($yesno,$directory,$file,$httpok,$httpsok,$updatefile,$orignal,$newfile);
    my ($usessl,$sname,$sport,$sadmin,$docroot,$eld,$tld,$cgibin,$vh,$nomess,$sslcacf,$vhssl);
    my ($nogo,$line,$writetoconf,$sslsname,$sslsport,$sslsadmin,$ssldocroot,$ssleld,$ssltrd);
    my ($sslcgibin,$sslckf,$sslcf,$sslvd,$sslvc,$writetosslconf,$param); 

    print "$site->{'script_msg'}{'httpd_header'}";

    print "$site->{'script_msg'}{'httpd_add'}";

    $yesno = <STDIN>;
    $yesno = &yesno($yesno);

    if ( $yesno == 1 ) {

        print "$site->{'script_msg'}{'httpd_directory'}";
        $directory = <STDIN>;
        chomp $directory;

        opendir DIR, "$directory" || die "$site->{'script_msg'}{'httpd_openerr'}";
        @allfiles = readdir DIR;
        closedir DIR;

        shift @allfiles; # getting rid of .
        shift @allfiles; # getting rid of ..

        foreach $file ( @allfiles ) {
            chomp $file;
            $httpok=1 if ( $file eq "httpd.conf" );
            $httpsok=1 if ( $file eq "httpsd.conf" );
        }

        if ( $httpok != 1 && $httpsok != 1 ) {
            die "$site->{'script_msg'}{'httpd_founderr'}";
        } else { 

            print "$site->{'script_msg'}{'httpd_found'}";
            print "httpd.conf " if ( $httpok == 1 );
            print "$site->{'script_msg'}{'httpd_and'} " if ( $httpsok == 1 && $httpok == 1 );
            print "httpsd.conf " if ( $httpsok == 1 );
            print "\n\n";

            if ( $httpok == 1 && $httpsok == 1 ) {
                print "$site->{'script_msg'}{'httpd_update'}";
                $yesno = <STDIN>;
                $yesno = &yesno($yesno);
                $updatefile="httpsd.conf" if ( $yesno == 1 );
            } elsif ( $httpok == 1 ) {
                $updatefile="httpd.conf";
            } else {
                $updatefile="httpsd.conf";
            }

            $orignal="$directory\/$updatefile";
            $newfile="$directory\/$updatefile.orig";
            if ( -e $newfile ) {
                die " $updatefile.orig $site->{'script_msg'}{'httpd_rename'} $updatefile.orig $site->{'script_msg'}{'httpd_before'} $updatefile $site->{'script_msg'}{'httpd_saved'}";
            }

            print "$site->{'script_msg'}{'httpd_back'} $updatefile.orig ... ";
            $orignal="$directory\/$updatefile";
            $newfile="$directory\/$updatefile.orig";

            @infile=();
            @outfile=();
            open(FH, "<$orignal" ) || die "$site->{'script_msg'}{'httpd_opennot1'} $orignal\n";
            @infile=(<FH>);
            close(FH);

            @outfile=@infile;  

            open(FH, ">$newfile" ) || die "$site->{'script_msg'}{'httpd_opennot2'} $newfile\n";
            print FH @outfile;
            close(FH);

            @infile=();
            @outfile=();

            print "$site->{'script_msg'}{'httpd_done'}";

            $usessl=0;

            print "$site->{'script_msg'}{'httpd_ssl'}";
            $yesno = <STDIN>;
            $yesno = &yesno($yesno);
            $usessl=1 if ( $yesno == 1 );

            print "\n ServerName?\n\n";
            $sname = <STDIN>;
            chomp $sname;

            print "\n Port?\n\n";
            $sport = <STDIN>;
            chomp $sport;

            print "\n Server Admin?\n\n";
            $sadmin = <STDIN>;
            chomp $sadmin;

            print "\n Document Root?\n\n";
            $docroot = <STDIN>;
            chomp $docroot;

            print "\n Error Log?\n\n";
            $eld = <STDIN>;
            chomp $eld;

            print "\n Transfer Log?\n\n";
            $tld = <STDIN>;    
            chomp $tld;

            print "\n cgi-bin Directory?\n\n";
            $cgibin = <STDIN>;
            chomp $cgibin;

            print "$site->{'script_msg'}{'httpd_host'}";

            $vh = "<VirtualHost $sname:$sport>\n\n";

            if ($usessl==1) {
                $vh .= " SSLDisable\n";
            }

            $vh .= " Port $sport
  DirectoryIndex index.html index.cgi index.shtml
  ServerAdmin $sadmin
  DocumentRoot $docroot
  ServerName $sname
  ErrorLog $eld
  TransferLog $tld

<Directory \"$docroot\">
  AllowOverride all
  Options Includes FollowSymLinks ExecCGI
  Order allow,deny
  Allow from all
<\/Directory>

<DirectoryMatch \"$docroot (DOCS|archive|bits|conf|crons|data|lib|tools)\">
  Options None
  Order deny,allow
  Deny from all
<\/DirectoryMatch>

AddHandler cgi-script .cgi
AddType text/html       .shtml
AddHandler server-parsed        .shtml
ScriptAlias /cgi-bin/ $cgibin
<\/VirtualHost>  
";

            print "$vh";

            print "$site->{'script_msg'}{'httpd_out'}";
            $yesno = <STDIN>;
            $yesno = &yesno($yesno);

            $nomess=0;

                if ( $yesno == 1 ) {

                    open(FH, "<$directory/$updatefile" ) || die "$site->{'script_msg'}{'httpd_opennot1'} $directory/$updatefile\n";
                    @infile=(<FH>);
                    close(FH);

                    @outfile=@infile;

                    $nogo=0;

                    foreach $line (@infile) {
                    $nogo=1 if ( $line =~ /$sname/ );
                }

                if ( $nogo == 1 ) {
                    print "$site->{'script_msg'}{'httpd_appears'}";
                } else {

                    print "$site->{'script_msg'}{'httpd_append'}";
                    push @outfile, $vh;
                    open(FH, ">$directory/$updatefile" ) || die "$site->{'script_msg'}{'httpd_opennot2'} $directory/$updatefile\n";
                    print FH @outfile;
                    close(FH);
                    print "$site->{'script_msg'}{'httpd_done'}";

                    $writetoconf="ok";

                }

            } else {

                $nomess=1;

            }


        ##  SSL specific parameters
        #

        if ( $usessl == 1  && $nogo == 0 ) {

            print "\n SSL ServerName?\n\n";
            $sslsname = <STDIN>;
            chomp $sslsname;

            print "\n SSL Port?\n\n";
            $sslsport = <STDIN>;
            chomp $sslsport;

            print "\n SSL Server Admin?\n\n";
            $sslsadmin = <STDIN>;
            chomp $sslsadmin;

            print "\n SSL Document Root?\n\n";
            $ssldocroot = <STDIN>;
            chomp $ssldocroot;

            print "\n SSL Error Log?\n\n";
            $ssleld = <STDIN>;
            chomp $ssleld;

            print "\n SSL Transfer Log?\n\n";
            $ssltrd = <STDIN>;
            chomp $ssltrd;

            print "\n SSL cgi-bin Directory?\n\n";
            $sslcgibin = <STDIN>;
            chomp $sslcgibin;

            print "\n SSLCertificateKeyFile?\n\n";
            $sslckf = <STDIN>;
            chomp $sslckf;

            print "\n SSLCertificateFile?\n\n";
            $sslcf = <STDIN>;
            chomp $sslcf;

            print "\n SSLVerifyDepth?\n\n";
            $sslvd = <STDIN>;
            chomp $sslvd;

            print "\n SSLVerifyClient?\n\n";
            $sslvc = <STDIN>;
            chomp $sslvc;

            print "\n SSLCACertificateFile?\n\n";
            $sslcacf = <STDIN>;
            chomp $sslcacf;

        }

        if ( $usessl == 1 && $nogo == 0 ) {

        print "$site->{'script_msg'}{'httpd_sslhost'}";

     $vhssl = "\n<VirtualHost $sslsname:$sslsport>  
  SSLEnable
  ServerName $sslsname
  ServerAdmin $sslsadmin
  DocumentRoot $ssldocroot
  ScriptAlias \/cgi-bin\/ $sslcgibin
  SSLCertificateKeyFile $sslckf
  SSLCertificateFile $sslcf
  SSLVerifyDepth $sslvd
  SSLVerifyClient $sslvc

  <Directory \"$ssldocroot\">
    AllowOverride all
    Options Includes FollowSymLinks ExecCGI
    Order allow,deny
    Allow from all
  <\/Directory>

  <DirectoryMatch \"$ssldocroot (DOCS|archive|bits|conf|crons|data|lib|tools)\">
    Options None
    Order deny,allow
    Deny from all
  <\/DirectoryMatch>

  SSLCACertificateFile $sslcacf
  TransferLog $ssltrd
  ErrorLog $ssleld
<\/VirtualHost>
";

        print "$vhssl";

        print "$site->{'script_msg'}{'httpd_sslout'}";
        $yesno = <STDIN>;
        $yesno = &yesno($yesno);

        $writetosslconf="yes";

        if ( $yesno == 1 ) {    

            @infile=();
            @outfile=();

            open(FH, "<$directory/$updatefile" ) || die "$site->{'script_msg'}{'httpd_opennot1'} $directory/$updatefile\n";
            @infile=(<FH>);
            close(FH);

            @outfile=@infile;

            $nogo = 0;

            foreach $line (@infile) {
                $nogo=1 if ( $line =~ /<VirtualHost $sslsname:$sslsport>/ );
            }

            if ( $nogo == 1 ) {
                print "$site->{'script_msg'}{'httpd_appears'}";
                $writetosslconf="no";

            } else {

                print "$site->{'script_msg'}{'httpd_append'}";
                push @outfile, $vhssl;
                open(FH, ">$directory/$updatefile" ) || die "$site->{'script_msg'}{'httpd_opennot2'} $directory/$updatefile\n";
                print FH @outfile;
                close(FH);
                print "$site->{'script_msg'}{'httpd_done'}";

           }

        } else {

            $writetosslconf="no";

        }

        }  

        }

        print "$site->{'script_msg'}{'httpd_effect'}" if ( $nogo == 0 && ( $writetosslconf ne "no" && $nomess == 0 ) );

    }

    $param->{'usessl'}=$usessl;
    $param->{'writetoconf'}=$writetoconf;
    $param->{'writetosslconf'}=$writetosslconf;
    $param->{'sname'}=$sname;
    $param->{'sslsname'}=$sslsname;
    $param->{'docroot'}=$docroot;

    return ($param);

}

##############################################################################
=pod

=head2 conf 

conf - &conf($site);

=over 4

=item DESCRIPTION

Configuration file setup

=item INPUT

$site - site hash
$new - new conf values from other subroutines
$param - parameter values that need to be merged into conf files

=item OUTPUT

None.

=item NOTES

=back

=cut
############################################################################### 

sub conf {

    my ($site) = shift;
    my ($new) = shift;
    my ($param) = shift; 

    my $usessl=$param->{'usessl'};
    my $writetoconf=$param->{'writetoconf'};
    my $writetosslconf=$param->{'writetosslconf'}; 
    my $sname=$param->{'sname'};
    my $sslsname=$param->{'sslsname'};
    my $docroot=$param->{'docroot'};

    my (%current,%new,$hsite);
    my (@skip,@already,@snames,@sslsnames,@allfiles,@info,@lines,@keys,@modified,@docroots);
    my ($f,$place,$header,$k,$v,$ok,$siteshortname,$parameter,$key,$ORACLE_HOME,$ORACLE_SID,$PWD);

    @skip=("clusterid","om_version","series");
    @already=("site_datadriver","site_database_serv","site_database_port","site_dbiuser","site_dbipwd","site_database");

    ## check to see if user has gone through httpd subroutine and use
    #  those entries instead of having him input them again
    #

    if ( $usessl == 1 && $writetoconf eq "ok" ) {

        $new->{'ssl_prefix'}="https";
        push @already, "ssl_prefix";

    } elsif ( $usessl == 0 && $writetoconf eq "ok" ) {

        $new->{'ssl_prefix'}="http";
        push @already, "ssl_prefix";

    }

    if ( $sname && $writetoconf eq "ok" ) {

        @snames = split /\./, $sname, 2;
        @sslsnames = split /\./, $sslsname, 2 if ( $writetosslconf ne "no" );

        $new->{'siteserver'}=$snames[0];
        $new->{'sitedomain'}=".".$snames[1];
        $new->{'sitesslserver'}=$sslsnames[0] if ( $writetosslconf ne "no" );
        $new->{'sitessldomain'}=".".$sslsnames[1] if ( $writetosslconf ne "no" );

        push @already, "sitedomain";
        push @already, "siteserver";
        push @already, "sitessldomain" if ( $writetosslconf ne "no" );
        push @already, "sitesslserver" if ( $writetosslconf ne "no" );
    }       

    if ( $docroot && $writetoconf eq "ok" ) {

        @docroots = split /\/$siteshortname/, $docroot, 2;

        $new->{'sr'}=$docroots[0];

        push @already, "sr";
    }


    print "$site->{'script_msg'}{'conf_header'}";

    print "$site->{'script_msg'}{'conf_file'}";

    opendir (CONF, "./conf");
    @allfiles = readdir CONF;
    closedir (CONF);

    while (@allfiles) {
    $f = pop @allfiles;              # get a file from the list
    next if ( length( $f ) <= 5 );   # .conf has five characters
    next if ( $f !~ /\.conf$/ );     # it has to end in .conf
    next if ( $` =~ /[^a-zA-Z]/ );   # compare name for non-letters
    next if ( $f =~ /INDEX\.conf/ );
    next if ( $f =~ /MAINTAINERS\.conf/ );

    open( FH, "<./conf/$f" );       # read the file
    @info=(<FH>);
    @lines=@info;
    close(FH);

    $place="beginning";
    while ( @lines ) {               # go through the lines
        $_ = shift @lines;
        if ( $_ =~ /^###*/ ) {
            if ( $place eq "beginning" ) {
                $place="end";
            } elsif ( $place eq "end" ) {
                $place="beginning";
            }
            next;
        };

        if ( $_ =~ /^#/ ) {
            next if ( $place eq "end" );
            chomp;
            $header =~ s/#//g;
            $header .= $_ ."\n" if ( $_ =~ /^#/ );
            $header .= $_ if ( $_ =~ /^$/ );
            next;
        }
        if ( $_ =~ /^$/  ) {
            $place="beginning" if ( $place eq "end" );
            next;
        }

        chomp;

        ( $k,$v ) = split /\s/, $_, 2;

        next if ( ! $k );

        $k =~ s/\$sr/sr/g;  # for $sr

        push @keys, $k;
        push @skip, @already;

        $ok=0;

        foreach $parameter (@skip) {
            $ok=1 if ( $k eq $parameter);
        }  

        $v =~ s/^\s+//g;
        $v =~ s/\s+$//g;

        $current{$k}=$v;
        $header =~ s/#//g;

        if ( $ok == 0 ) {
            print "\n\n$header\n";
            print " $k [$v] ";
            $new->{$k} = <STDIN>;
            chomp $new->{$k};
        }
        $new->{$k}=$current{$k} if ( ! $new->{$k} );
        $header="";
        $place = "beginning";
    }                                # done reading conf data

    ## now put the information into the conf files
    @modified=();

    while (@info) {
        $_ = shift @info;
        foreach $key (@keys) {
            $_ =~ s/$current{$key}/$new->{$key}/ if ( $key eq "sr" && $_ =~ /^\$sr/ );
            if ( $_ =~ /^$key/ && $current{$key} ne $new->{$key} ) {
                $_ =~ s/$key\s+$current{$key}/$key\t$new->{$key}/;
            }
        }
        push @modified, $_;
    }

    open( FH, ">./conf/$f" );       # write the new file
    print FH @modified;
    close(FH);

    }

    $PWD=`pwd`;
    chomp $PWD; 
#    `$PWD/crons/update_conf.pl $site->{'ora_sid'}` if ( $site->{'ora_sid'} ); 
#    `$PWD/crons/update_conf.pl` if ( ! $site->{'ora_sid'} );
&update_conf($site->{'ora_sid'},$site->{'ora_home'});

    require "./conf/site.pm";

    $hsite = &setsite();

    foreach $key (keys %{$hsite}) {
        $site->{$key} = $hsite->{$key};
    }   

    return ($site);


} 

##############################################################################
=pod

=head2 tables 

tables - &tables($site,$dbh);

=over 4

=item DESCRIPTION

Database Table input

=item INPUT

$site - site hash
$dbh - database handler info

=item OUTPUT

None.

=item NOTES

=back

=cut
############################################################################### 

sub tables {

    use DBI;

    my ($site) = shift;
    my ($dbh) = shift;

    my (@lines,@drops,@tables);
    my ($skel,$l,$table,$driver,$yesno,$drop,$q,$sth,$entry,$tablespace);

    $tablespace = $site->{'tablespace'};

    print "$site->{'script_msg'}{'tables_header'}";

    $driver = $site->{'site_datadriver'};
    $skel = $site->{'site_datadriver'};
    $skel =~ tr/A-Z/a-z/;

    open(TABLE, "<./conf/tables.$skel.skel") || die "$site->{'script_msg'}{'tables_die1'}$skel$site->{'script_msg'}{'tables_die2'}";

    @lines=(<TABLE>);
    chomp @lines;

    while ( @lines ) {
        $l = shift @lines;
        next if ( $l =~ /#/ );
        next if ( $l !~ /\w/ and $l !~ /\)/ );

        $table.=$l;
    }

    close(TABLE);

    @tables=split(/\;/, $table);
    @tables=split(/\)\;/, $table) if ( $driver =~ /mysql/ );

    print "\n Creating tables ... ";

    @drops=("addresses","clicks","contacts","conversions","extraprop","gevents","inventory","levents","linkage","ocart","orders","pcart","prices","sessions","tcart","thesaurus","things","vparts","vprices","zonemap","zonerate");

    if ( $driver =~ /Oracle/ ) {

        print "$site->{'script_msg'}{'tables_drop'}";
        $yesno = <STDIN>;
        $yesno = &yesno($yesno);
        if ( $yesno == 1 ) {  

            foreach $drop (@drops) {
                $q = "drop table ".$drop;
                $sth = $dbh->prepare( $q );
                $sth->execute;
            }

            $q="DROP SEQUENCE levents_event";
            $sth = $dbh->prepare( $q );
            $sth->execute;

            $q="DROP SEQUENCE sessions_sid";
            $sth = $dbh->prepare( $q );
            $sth->execute;

        }
    }
    foreach $entry (@tables) {
        $entry =~ s/\$tablespace/$tablespace/;
        $q = $entry if ( $driver =~ /Pg/ );
        $q = $entry."\)" if ( $driver =~ /mysql/ );
        $q = $entry if ( $driver =~ /Oracle/ );
        $sth = $dbh->prepare( $q );
        $sth->execute;
    }

    if ( $driver =~ /Oracle/ ) {

        $q="CREATE SEQUENCE levents_event INCREMENT BY 1 START WITH 1 MINVALUE 0";
        $sth = $dbh->prepare( $q );
        $sth->execute;

        $q="CREATE SEQUENCE sessions_sid INCREMENT BY 1 START WITH 1 MINVALUE 0";
        $sth = $dbh->prepare( $q );
        $sth->execute;

    }

    print "$site->{'script_msg'}{'tables_done'}";
}               

sub idl {

    use DBI;

    my ($site) = shift;
    my ($dbh) = shift;

    my ($now,$series,%current,%new,$h,$m,$objid,$a,$b,$k,$statement,$dvalue,$driver);
    my (@infile,@dataload,$file,$line,@fieldparts,$fieldparts,@parts,$fieldhash,$fields,$values,$field);                                                      
    my ($q,$sth,$event,$k,$relpath,%h,%objid);

    $now=time();
    $series=$current{'series'};
    $series=$new{'series'} if ( $new{'series'} );

    $series=$site->{'series'};

    print "\n #####\n";
    print " #\n";
    print " # AllCommerce Initial Data Load \n";
    print " #\n";
    print " #####\n";

    print " \n\n Adding AllCommerce initial data load ... ";

$driver = $site->{'site_datadriver'};

@infile=();
@dataload=("configure_dataload.txt","idl_dataload.txt");

foreach $file (@dataload) {

#open(FH, "<./conf/$file");
#@infile=(<FH>);
#chomp @infile;
#close(FH);

foreach $line (@infile) {

next if ( $line !~ /\"/ );
if ( $line =~ /\#\:/ ) {
$fieldparts=();
$line =~ s/\#\://;
$line =~ s/\"//g;
#print "-->$line\n";
@fieldparts=split /\s+/,$line;
#print "->$fieldparts[0]<- \t ->$fieldparts[1]<- \t ->$fieldparts[2]<- \t ->$fieldparts[3]<- \t ->$fieldparts[4]<- ->$fieldparts[5]<- \t ->$fieldparts[6]<- \t ->$fieldparts[7]<-\n";

} else {
@parts=();
next if ( $line !~ /^\"/ );
#print "\n\n$line\n\n";
$line =~ s/\'/\'\'/g;
$line =~ s/\"/\'/g;
@parts=split /\t/,$line;
$fieldhash = {};

$fieldhash->{$fieldparts[0]}=$parts[0];
$fieldhash->{$fieldparts[1]}=$parts[1] if ( $fieldparts[1] );
$fieldhash->{$fieldparts[2]}=$parts[2] if ( $fieldparts[2] );
$fieldhash->{$fieldparts[3]}=$parts[3] if ( $fieldparts[3] );
$fieldhash->{$fieldparts[4]}=$parts[4] if ( $fieldparts[4] );
$fieldhash->{$fieldparts[5]}=$parts[5] if ( $fieldparts[5] );
$fieldhash->{$fieldparts[6]}=$parts[6] if ( $fieldparts[6] );
$fieldhash->{$fieldparts[7]}=$parts[7] if ( $fieldparts[7] );

$fields="";
$values="";
$q="";

foreach $field ( keys %{$fieldhash} ) {
next if ( ! $field );
next if ( ! $fieldhash->{$field} );
next if (($field ne "status") && ($field ne "title") && ($field ne "objid") && ($field ne "objtype") && ($field ne "objclass") && ($field ne "brief"));
$fields.="," if ( $fields );
$fields.=$field;

$values.="," if ( $values );
$fieldhash->{$field}="\'\'" if ( ! $fieldhash->{$field} );
$values.=$fieldhash->{$field};

}

#print "$fields\n";
#$q="INSERT INTO things ($fields) VALUES ($values)";
#print "->$q \n";
#$sth=$dbh->prepare($q);
#$sth->execute;

} #end of else
} #end of foreach
}    
    ## sql insert statement hash
    #    
    $h = {
34 => "INSERT into things (objid,objclass,title,status) VALUES ('50000-A','140-A','FedEx 2 Day','15000-A')",
35 => "INSERT into things (objid,objclass,title,status) VALUES ('50001-A','140-A','FedEx Express Saver','15000-A')",
36 => "INSERT into things (objid,objclass,title,status) VALUES ('50002-A','140-A','FedEx First Overnight','15000-A')",
37 => "INSERT into things (objid,objclass,title,status) VALUES ('50003-A','140-A','FedEx Freight','15000-A')",
38 => "INSERT into things (objid,objclass,title,status) VALUES ('50004-A','140-A','FedEx Priority Overnight','15000-A')",
39 => "INSERT into things (objid,objclass,title,status) VALUES ('50005-A','140-A','FedEx Standard Overnight','15000-A')",
40 => "INSERT into things (objid,objclass,title,status) VALUES ('50100-A','140-A','UPS 2nd Day Air','15000-A')",
41 => "INSERT into things (objid,objclass,title,status) VALUES ('50101-A','140-A','UPS 2nd Day Air AM','15000-A')",
42 => "INSERT into things (objid,objclass,title,status) VALUES ('50102-A','140-A','UPS 3 Day Select Commercial','15000-A')",
43 => "INSERT into things (objid,objclass,title,status) VALUES ('50103-A','140-A','UPS 3 Day Select Residental','15000-A')",
44 => "INSERT into things (objid,objclass,title,status) VALUES ('50104-A','140-A','UPS Ground to Commercial','15000-A')",
45 => "INSERT into things (objid,objclass,title,status) VALUES ('50105-A','140-A','UPS Ground to Residential','15000-A')",
46 => "INSERT into things (objid,objclass,title,status) VALUES ('50106-A','140-A','UPS Ground to Residential (area surcharge)','15000-A')",
47 => "INSERT into things (objid,objclass,title,status) VALUES ('50107-A','140-A','UPS Next Day Air','15000-A')",
48 => "INSERT into things (objid,objclass,title,status) VALUES ('50108-A','140-A','UPS Next Day Air Early AM','15000-A')",
49 => "INSERT into things (objid,objclass,title,status) VALUES ('50109-A','140-A','UPS Next Day Air Saver','15000-A')",
50 => "INSERT into things (objid,objclass,title,status) VALUES ('50150-A','140-A','USPS Bound Printed Matter','15000-A')",
51 => "INSERT into things (objid,objclass,title,status) VALUES ('50151-A','140-A','USPS Express Mail','15000-A')",
52 => "INSERT into things (objid,objclass,title,status) VALUES ('50152-A','140-A','USPS First Class Mail','15000-A')",
53 => "INSERT into things (objid,objclass,title,status) VALUES ('50153-A','140-A','USPS Parcel Post','15000-A')",
54 => "INSERT into things (objid,objclass,title,status) VALUES ('50154-A','140-A','USPS Priority Mail','15000-A')",
55 => "INSERT into things (objid,objclass,title,status) VALUES ('50155-A','140-A','USPS Special starndard Mail (Book Rate)','15000-A')",
56 => "INSERT into contacts VALUES ('1123-AA','128-A','39004-A','','eng','admin','allcommerce','','','','','',0,959735890,'15000-A','')",
}; 

## Get objid and now(time/date) information
#  and insert into database 
#

sub numerically { $a <=> $b; }
foreach $k (sort numerically keys %{$h} ) {

$statement=$h->{$k};
$statement =~ s/\'//g;

$dvalue="1\t Create ObjID\t $statement";

if ( $driver =~ /mysql/ ) {

   if ( $statement =~ /things/ && $statement !~ /140-A/ ) {
   $q="INSERT INTO levents (dtable,dvalue,cdate,series) VALUES ('things','$dvalue','$now','$series')";
   $sth=$dbh->prepare($q);
   $sth->execute;

   $q="SELECT event,series from levents where dtable='things' and dvalue='$dvalue' and cdate='$now'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   ($event,$series)=$sth->fetchrow;
   }
   $objid{$k}=$event."-"."AA";

} elsif ( $driver =~ /Oracle/ ) {

   if ( $statement =~ /things/ && $statement !~ /140-A/ ) {
   $q="INSERT INTO levents (event,dtable,dvalue,cdate,series) VALUES (levents_event.nextval,'things','$dvalue','$now','AA')";
   $sth=$dbh->prepare($q);
   $sth->execute;

   $q="SELECT event,series from levents where dtable='things' and dvalue='$dvalue' and cdate='$now'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   ($event,$series)=$sth->fetchrow;
   }
   $objid{$k}=$event."-".$series;
} elsif ( $driver =~ /Pg/ ) {

    if ( $statement =~ /things/ && $statement !~ /140-A/ ) { 
       $q="INSERT INTO levents (event,dtable,dvalue,cdate,series) VALUES (NEXTVAL('event'),'things','$dvalue','$now','AA')";
       $sth=$dbh->prepare($q);
       $sth->execute;

       $q="SELECT CURRVAL('event'),series from levents where dtable='things' and dvalue='$dvalue' and cdate='$now'";
       $sth=$dbh->prepare($q);
       $sth->execute;
       ($event,$series)=$sth->fetchrow;
    }
       $objid{$k}=$event."-".$series;

}
} # end of foreach

## sql insert statement hash
#
$m = {
34 => "INSERT into things (objid,objclass,title,status) VALUES ('50000-A','140-A','FedEx 2 Day','15000-A')",
35 => "INSERT into things (objid,objclass,title,status) VALUES ('50001-A','140-A','FedEx Express Saver','15000-A')",
36 => "INSERT into things (objid,objclass,title,status) VALUES ('50002-A','140-A','FedEx First Overnight','15000-A')",
37 => "INSERT into things (objid,objclass,title,status) VALUES ('50003-A','140-A','FedEx Freight','15000-A')",
38 => "INSERT into things (objid,objclass,title,status) VALUES ('50004-A','140-A','FedEx Priority Overnight','15000-A')",
39 => "INSERT into things (objid,objclass,title,status) VALUES ('50005-A','140-A','FedEx Standard Overnight','15000-A')",
40 => "INSERT into things (objid,objclass,title,status) VALUES ('50100-A','140-A','UPS 2nd Day Air','15000-A')",
41 => "INSERT into things (objid,objclass,title,status) VALUES ('50101-A','140-A','UPS 2nd Day Air AM','15000-A')",
42 => "INSERT into things (objid,objclass,title,status) VALUES ('50102-A','140-A','UPS 3 Day Select Commercial','15000-A')",
43 => "INSERT into things (objid,objclass,title,status) VALUES ('50103-A','140-A','UPS 3 Day Select Residental','15000-A')",
44 => "INSERT into things (objid,objclass,title,status) VALUES ('50104-A','140-A','UPS Ground to Commercial','15000-A')",
45 => "INSERT into things (objid,objclass,title,status) VALUES ('50105-A','140-A','UPS Ground to Residential','15000-A')",
46 => "INSERT into things (objid,objclass,title,status) VALUES ('50106-A','140-A','UPS Ground to Residential (area surcharge)','15000-A')",
47 => "INSERT into things (objid,objclass,title,status) VALUES ('50107-A','140-A','UPS Next Day Air','15000-A')",
48 => "INSERT into things (objid,objclass,title,status) VALUES ('50108-A','140-A','UPS Next Day Air Early AM','15000-A')",
49 => "INSERT into things (objid,objclass,title,status) VALUES ('50109-A','140-A','UPS Next Day Air Saver','15000-A')",
50 => "INSERT into things (objid,objclass,title,status) VALUES ('50150-A','140-A','USPS Bound Printed Matter','15000-A')",
51 => "INSERT into things (objid,objclass,title,status) VALUES ('50151-A','140-A','USPS Express Mail','15000-A')",
52 => "INSERT into things (objid,objclass,title,status) VALUES ('50152-A','140-A','USPS First Class Mail','15000-A')",
53 => "INSERT into things (objid,objclass,title,status) VALUES ('50153-A','140-A','USPS Parcel Post','15000-A')",
54 => "INSERT into things (objid,objclass,title,status) VALUES ('50154-A','140-A','USPS Priority Mail','15000-A')",
55 => "INSERT into things (objid,objclass,title,status) VALUES ('50155-A','140-A','USPS Special starndard Mail (Book Rate)','15000-A')",
56 => "INSERT into contacts VALUES ('1123-AA','128-A','39004-A','','eng','admin','allcommerce','','','','','',0,959735890,'15000-A','')",
};  

foreach $k (sort numerically keys %{$m} ) {

$q=$m->{$k};
$q =~ s/objids/$objid{$k}/g;
$sth=$dbh->prepare($q);
$sth->execute;

} 

require "./conf/site.pm";
require "./lib/objects.pm";
$site = &setsite();
($site->{'OBJCLASSES'}, $site->{'OBJTYPES'}) = &getClassesAndTypes();
$site->{'om_render_lang'} = "eng";
print "\n Inserting English GUI mappings ... \n";
&insert_eng_gui_mappings($site, $dbh); 
&insert_eng_pulldowns($site, $dbh);
`chown nobody:nobody $site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/eng/html/sel/*.*` if ( $^O ne 'MSWin32');
print " done.\n\n"; 

}

##############################################################################
=pod

=head2 ship 

ship - &ship($site,$dbh);

=over 4

=item DESCRIPTION

Shipping table information input.

=item INPUT

$site - site hash
$dbh - database handler

=item OUTPUT

None.

=item NOTES

=back

=cut
############################################################################### 

sub ship {

use DBI;

    my ($site) = shift;
    my ($dbh) = shift;

    my (@entries, $count, $q, $sth,); 
print "\n #####\n";
print " #\n";
print " # Shipping Tables Data Load \n";
print " #\n";
print " #####\n";

print "\n Running import_shipping_tables.pl ... ";

`./tools/import_shipping_tables.pl` if ( $^O !~ /MSWin32/);

print "done. \n\n Adding shipping entries (This may take a few minutes) ... \n";

@entries=();
$count=0;

open SH, "<./tools/data/shipping.skel";
@entries=(<SH>);
close(SH);

chomp @entries;

$dbh->disconnect if ($dbh and ! @entries);

die "\n No shipping entries to add. \n" if ( ! @entries );

while ( @entries ) {
   $q = shift @entries;
   $q =~ s/\\g//;
   $sth = $dbh->prepare( $q );
   $sth->execute;
   $count++;
}
print " $count shipping entries entered.\n\n";
$dbh->disconnect;
}    


1;
