#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# conversions.pm
#
# Version 3 TG 20000912
#    retrofit for development standards
# Version 2 IB 20000302
#    updated copyright and removed -w
# Version 1 RF 981001

use strict;

##############################################################################
=pod

=head1 conversion.pm

conversion.pm - routines to handle converting to different formats or units

=head1 DESCRIPTION

This library is used for all subroutines that convert strings to different 
formats, or convert values to different units.

=head1 NOTES & BUGS

=cut

##############################################################################

##############################################################################
=pod

=head2 nicedatetime

nicedatetime - &nicedatetime($gmt);

=over 4

=item DESCRIPTION

Take a time in GMT format, and converts it to a readable format.  If a time is
not passed in, it uses the current time.

=item INPUT

$GMTtime - seconds since epoch (if no args uses current time) 

=item OUTPUT

$readableTime - Time in DD Month, YYYY at hh:mm:ss

=item NOTES

=back

=cut
###############################################################################
sub nicedatetime {
    my ($GMTtime) = shift;
    
    my (@date, @months, $readableTime);
    
    @months=("January","February","March","April","May","June");
    @months=(@months,"July","August","September","October","November","December");
    
    $GMTtime = time() if (! $GMTtime);
    @date=localtime($GMTtime);
    
    #
    # correct century
    #
    $date[5]+=1900;
    $readableTime="$date[3] $months[$date[4]], $date[5] at $date[2]\:$date[1]\:$date[0]";
    return $readableTime;
}

1; 






