#/usr/bin/perl

use strict;

sub deleteAttributes {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $numRows, @results, $delete_hashref);

    $delete_hashref->{'TABLE'}->{'NAME'} = "things";
    $delete_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objclass", "101-A", "and");
    $delete_hashref->{'statement'} = "DELETE";
    $q = &build_delete($delete_hashref, $site, $dbh, 1);
    $numRows = &query_execute($q, $site, $dbh);
    &debug ("$q\n\t-->$numRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
}

sub insertAttributes {
    my ($file) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@lines, $line);

    open FH, "<$file";
    @lines = <FH>;
    close FH;
    foreach $line (@lines) {
	chop $line;  # get rid of newline
	next if ($line =~ /#/);   # comment line
	next if (length($line) == 0);
	if (!&insertAttribute($line, $site, $dbh)) {
	    &debug("Could not insert:\n\t$line", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }
}


sub insertAttribute {
    my ($line) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($q, $numRows, $insert_hashref, @insert_results, @fields);

    $line =~ s/\"//gms;
    @fields = split "\t", $line;

    $insert_hashref->{'TABLE'}->{'NAME'} = "things";
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = string_todbi($fields[0]);
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'objclass'} = string_todbi($fields[1]);
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = string_todbi($fields[2]);
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'title'} = string_todbi($fields[3]);
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'brief'} = string_todbi($fields[4]); 
    $insert_hashref->{'statement'} = "insert";
    $q = &build_insert($insert_hashref, $site, $dbh, 1);
    $numRows = &query_execute($q, $site, $dbh);
    &debug("$q\n\t$numRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
    return ($numRows);
}

sub loadAttributes {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($select_hashref, $q, $numRows, $attributes, @select_results);
    my ($attribute);


    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objclass'} = "objclass";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'title'} = "title";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'brief'} = "brief";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "status";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objclass", "101-A", "and");
    $select_hashref->{'statement'} = "select";
    $q = &build_select($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@select_results);
    
    if ($numRows > 0) {
	foreach $attribute (@select_results) {
	    $attributes->{$attribute->{'title'}} = $attribute;
	}
    }

    return ($attributes);
}

sub printAttributes {
    my ($attributes) = shift;
    my ($site) = shift;

    my ($attribute, $field);

    foreach $attribute (keys %{$attributes}) {
	foreach $field (keys %{$attributes->{$attribute}}) {
	    &debug("$field: $attributes->{$attribute}->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
	&debug("\n", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
}


# These need to be translated to other appropriate languages
sub insert_eng_gui_mappings {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, @defs, $i, $h, $hash, $object_hashref);

    # Insert the language sensitive fields
    &insert_language_sensitive_fields("title,brief,keywords,descript,encoding,uperm,gui", $site, $dbh);

    $site->{'languageSensitive'} = &get_language_sensitive_fields($site, $dbh);

    # Create the conf file of html data information
    $site->{'relpath'} = ".";
    &initialize_html_conf($site);

    $h = {};

    $h->{'title_inp'} = "Brand";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
descript|Description
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
sku_skel|Sku Code
gen|Generate Page
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'brand'} . "_gui";
    $defs[$i++] = $h;


if (defined $site->{'autosku'}) {
    $h = {};
    $h->{'title_inp'} = "Product";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
descript|Description
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
behavior|Stock Behavior
axisnames|Variant Set
ship_weight|Shipping Weight
sku_series|Brand
gen|Generate Page
";
}
# include a space to manually enter a sku
    else {
    $h = {};
    $h->{'title_inp'} = "Product";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
descript|Description
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
behavior|Stock Behavior
axisnames|Variant Set
ship_weight|Shipping Weight
sku_series|Brand
msku|SKU
gen|Generate Page
";
    }
    
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'product'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Variant Products";
    $h->{'gui_inp'} = 
"<row_start>
<var_new_gui>
<row_end>
<row_start>
<var_exists_gui>
<row_end>";
    $h->{'gui_name_inp'} = "variant_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Existing Variant Products";
    $h->{'gui_inp'} = 
"<row_start>
objid:display|Object ID
title:display|Name
<row_end>
<row_start>
status|Status
visibility|Display on Site
weight_man|Ranking
ship_weight|Shipping Weight
gen|Generate Page?
<row_end>
";

    $h->{'gui_name_inp'} = "var_exists_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Add Variant Product";
    $h->{'gui_inp'} = 
"<row_start>
<_axis0_>
<_axis1_>
<_axis2_>
<_axis3_>
<new_row>
status|Status
visibility|Display on Site
weight_man|Ranking
ship_weight|Shipping Weight
gen|Generate Page?
<row_end>
";
    $h->{'gui_name_inp'} = "var_new_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Variant";
    $h->{'gui_inp'} = 
	"title|Name";
    $h->{'gui_name_inp'} = $site->{'OBJCLASSES'}->{'variant'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Pulldown";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
encoding|Pulldown
sel_name|Select Filename";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'encoding'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Product Line";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
descript|Description
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
gen|Generate Page
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'product_line'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Article";
    $h->{'gui_inp'} = 
"title|Title
brief|Brief
descript|Article
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
author|Author
gen|Generate Page
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'article'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Internal Document";
    $h->{'gui_inp'} = 
"title|Title
brief|Brief
descript|Article
keywords|Keywords
status|Status
visibility|Display on Site
pagestyle|Page Style
weight_man|Ranking
author|Author
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'internal_doc'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Permission Set";
    $h->{'gui_inp'} = 
"title|Title
brief|Brief
uperm|Permissions";
    $h->{'gui_name_inp'} = $site->{'OBJCLASSES'}->{'permission_set'} . "_gui";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "User";
    $h->{'gui_inp'} = 
"uname|User Name
passwd|Password
hint|Hint
gen_trigs|Generate Site Pages
uperms|Permission Sets
uperm|Unique Permissions";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'contact'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Attribute";
    $h->{'gui_inp'} = 
"title|Field Name
brief|Description
html|HTML String
lang_sens|Language Sensitive
";
    $h->{'gui_name_inp'} = $site->{'OBJCLASSES'}->{'attribute'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Prices";
    $h->{'gui_inp'} = 
"<row_start>
<row_price_new_gui>
<row_end>
<row_start>
<row_price_exist_gui>
<row_end>
gen|Generate Page
";

    $h->{'gui_name_inp'} = "price_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Existing Prices";
    $h->{'gui_inp'} = 
"<row_start>
unitct:display|Unit Count
unitpr|Unit Price
currency|Currency
status|Status
autogen:display|Rule Generated?
<row_end>";
    $h->{'gui_name_inp'} = "price_exist_gui";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Add Price";
    $h->{'gui_inp'} = 
"<row_start>
unitct:display|Unit Count
unitpr|Unit Price
currency|Currency
status|Status
autogen:display|Rule Generated?
<row_end>";
    $h->{'gui_name_inp'} = "price_exist_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Inventory";
    $h->{'gui_inp'} = 
"<row_start>
wh:display|Warehouse
lstock:display|In Stock
status|Status
adjust|Adjust
<row_end>
";
    $h->{'gui_name_inp'} = "inventory_gui";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Shipping Zones";
    $h->{'gui_inp'} = 
"shipzone|Shipping Zone
shipmeth|Shipping Method
fromzip|From zipcode
fromcntry:display:value=USA|From Country
destcntry:display:value=USA|To Country
destzipmin|Minimum Zipcode in range
destzipmax|Maximum Zipcode in range
status|Status
";
    $h->{'gui_name_inp'} = "zonemap_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Shipping Rates";
    $h->{'gui_inp'} = 
"shipmeth|Shipping Method
shipzone|Shipping Zone
shipwt|Max Shipping Weight
shipcost|Cost per item per zone, weight and method
currency|Currency
status|Status
";
    $h->{'gui_name_inp'} = "zonerate_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Object Definition";
    $h->{'gui_inp'} = 
"
title|Name
brief|Brief
gui|Definition
gui_name|Filename
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'object'} . "_gui";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Variant Set";
    $h->{'gui_inp'} = 
"
title|Name
brief|Brief
variant_set|Variant Object IDs
";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'variant_set'} . "_gui";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Warehouse";
    $h->{'gui_inp'} = 
"title|Name
brief|Brief
keywords|Keywords
status|Status";
    $h->{'gui_name_inp'} = $site->{'OBJTYPES'}->{'warehouse'} . "_gui";
    $defs[$i++] = $h;

    foreach $hash (@defs) {
	$object_hashref = {};
	$object_hashref->{'objclass_inp'} = $site->{'OBJCLASSES'}->{'object'};
	$object_hashref->{'objtype_inp'} = $site->{'OBJTYPES'}->{'object'};
	$object_hashref->{'status_inp'} = "15000-A";
	$object_hashref->{'brief_inp'} = "This is an internal field for AllCommerce.";
	$object_hashref->{'language_inp'} = "eng";
	$object_hashref->{'title_inp'} = $hash->{'title_inp'};
	$object_hashref->{'gui_inp'} = $hash->{'gui_inp'};
	$object_hashref->{'gui_name_inp'} = lc($hash->{'gui_name_inp'});
	$error = &create_object($object_hashref, $site, $dbh);
    }


    &load_permission_sets($site, $dbh);
    &create_users($site, $dbh);

}

##############################################################################
=pod

=head2 insert_language_sensitive_fields

insert_language_sensitive_fields - &insert_language_sensitive_fields($fields, $site, $dbh);

=over 4

=item DESCRIPTION

To add a record in the database, with fieldname = 'ac_language_sens', 
objid = "112000-A", and the body defining the list of language sensitive 
fields.  This is called on initial data load.

=item INPUT

$fields - Comma seperated list of fields that are language sensistive

$site - The site hashref, use for relative path and message sub

$dbh - Current active database handler

=item OUTPUT

The ac_language_sens field is inserted.

=item NOTES

This record is defined in extraprop with the fieldname ac_language_sens

=back

=cut
###############################################################################
sub insert_language_sensitive_fields {
    my ($fields) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $insert_hashref, $numRes, @results);
    my ($languageSensitive, @add_fields, @rm_fields);
    my ($field);
    
    &debug("insert_language_sensitive_fields: In with $field.", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $insert_hashref->{'statement'} = "INSERT";
    $insert_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "112000-A";
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15000-A";
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "ac_language_sens";
    $insert_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $fields;
    $q = &build_sql($insert_hashref, $site, $dbh, 1);
    if (!&query_execute($q, $site, $dbh)) {
	&debug("insert_language_sensitive_fields: Could not execute $q.", 1, $site->{'debug_level'}, $site->{'debug_log'});
    }
    return;
}

# These need to be translated to other appropriate languages
sub insert_eng_pulldowns {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, @defs, $i, $h, $hash, $object_hashref);

    
    $site->{'languageSensitive'} = &get_language_sensitive_fields($site, $dbh);

    # Create the conf file of html data information
    $site->{'relpath'} = ".";
    $h = {};

    $h = {};
    $h->{'title_inp'} = "Object Types";
    $h->{'sel_name_inp'} = "sel_objtype";
    $h->{'encoding_inp'} = "|All Types\r\n47001-A|Articles\r\n47004-A|Brands\r\n47000-A|Internal Documents\r\n99002-A|Object Definitions\r\n147-A|Permission Sets\r\n45000-A|Products\r\n47005-A|Product Lines\r\n99003-A|Pulldowns\r\n181-A|Variant\r\n99004-A|Variant Set\r\n128-A|User\r\n74003-A|Warehouses";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Order Status";
    $h->{'sel_name_inp'} = "sel_orderstatus";
    $h->{'encoding_inp'} = "| Not Specified\r\n15018-A | Tentative order -  Authorization Pending\r\n15019-A | Tentative order -  Pending Clearance\r\n15020-A | Awaiting Fulfillment\r\n15021-A | Picking from Inventory\r\n15022-A | Packing for Shipment\r\n15023-A | Packed for Shipping\r\n15024-A | Partial Packed for Shipping\r\n15025-A | Left Facility and in Route to Customer\r\n15026-A | Pending Funds Capture\r\n15027-A | Finished with Funds Capture\r\n15028-A | Voided Prior to Handling\r\n15029-A | Voided After Pick but Before Departure";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Credit Card Expiry Months";
    $h->{'sel_name_inp'} = "sel_ccexpmo";
    $h->{'encoding_inp'} = "|- Not Specified\r\n01|January\r\n02|February\r\n03|March\r\n04|April\r\n05|May\r\n06|June\r\n07|July\r\n08|August\r\n09|September\r\n10|October\r\n11|November\r\n12|December";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Credit Card Expiry Years";
    $h->{'sel_name_inp'} = "sel_ccexpyr";
    $h->{'encoding_inp'} = "|- Not Specified\r\n00|2000\r\n01|2001\r\n02|2002\r\n03|2003\r\n04|2004\r\n05|2005";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Credit Card Names";
    $h->{'sel_name_inp'} = "sel_ccname";
    $h->{'encoding_inp'} = "|- Not Specified\r\nAMEX|American Express\r\nVISA|Visa\r\nMC|Master Card\r\nDISC|Discover";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Cell Styles";
    $h->{'sel_name_inp'} = "sel_cellstyle";
    $h->{'encoding_inp'} = "|Not Specified\r\ncell_220_product|Display Cell with Photo (220)\r\ncell_140_product_related|Related Product Cell (140)\r\ncell_310_intermediate_box|Intermediate Cell (310)\r\ncell_220_article|Article Cell (220)\r\ncell_220_intermediate_box|Intermediate Cell (220)";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Page Styles";
    $h->{'sel_name_inp'} = "sel_pagestyle";
    $h->{'encoding_inp'} = "page_product_display|Product Page\r\npage_product_related|Product Page w/related items (Cell 140)\r\npage_product_master|Product Page for Variants\r\npage_intermediate_display|Intermediate Page (Cell 220)\4\npage_home_display|Home Page (Cell 310,220)\r\npage_article_display|Article Page\r\npage_intermediate_article|Intermediate Article Page (Cell 220)";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "States";
    $h->{'sel_name_inp'} = "sel_state";
    $h->{'encoding_inp'} = "|- Not Specified\r\nAL|Alabama\r\nAK|Alaska\r\nAS|American Samoa\r\nAZ|Arizona\r\nAR|Arkansas\r\nCA|California\r\nCO|Colorado\r\nCT|Connecticut\r\nDE|Delaware\r\nDC|District Of Columbia\r\nFM|Federated States Of Micronesia\r\nFL|Florida\r\nGA|Georgia\r\nGU|Guam\r\nHI|Hawaii\r\nID|Idaho\r\nII|Illinois\r\nIN|Indiana\r\nIA|Iowa\r\nKS|Kansas\r\nKY|Kentucky\r\nLA|Louisiana\r\nME|Maine\r\nMH|Marshall Islands\r\nMD|Maryland\r\nMA|Massachusetts\r\nMI|Michigan\r\nMN|Minnesota\r\nMS|Mississippi\r\nMO|Missouri\r\nMT|Montana\r\nNE|Nebraska\r\nNV|Nevada\r\nNH|New Hampshire\r\nNJ|New Jersey\r\nNM|New Mexico\r\nNY|New York\r\nNC|North Carolina\r\nND|North Dakota\r\nMP|Northern Mariana Islands\r\nOH|Ohio\r\nOK|Oklahoma\r\nOR|Oregon\r\nPW|Palau\r\nPA|Pennsylvania\r\nPR|Puerto Rico\r\nRI|Rhode Island\r\nSC|South Carolina\r\nSD|South Dakota\r\nTN|Tennessee\r\nTX|Texas\r\nUT|Utah\r\nVT|Vermont\r\nVI|Virgin Islands\r\nVA|Virginia\r\nWA|Washington\r\nWV|West Virginia\r\nWI|Wisconsin\r\nWY|Wyoming\r\nAE|Armed Forces Africa\r\nAA|Armed Forces America (Except Canada)\r\nAE|Armed Forces Canada\r\nAE|Armed Forces Europe\r\nAE|Armed Forces Middle East\r\nAP|Armed Forces Pacific\r\nFT|Canada";
    $defs[$i++] = $h;


    $h = {};
    $h->{'title_inp'} = "Behavior";
    $h->{'sel_name_inp'} = "sel_behavior";
    $h->{'encoding_inp'} = "90002-A| JIT Saleable if no stock\r\n90005-A| Potentially restockable if no stock\r\n90006-A| Saleable irrespective of stock\r\n90007-A| Discontinued\r\n90008-A| Recalled";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Status";
    $h->{'sel_name_inp'} = "sel_status";
    $h->{'encoding_inp'} = "15000-A| Active\r\n15001-A| In-Active";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Visibility";
    $h->{'sel_name_inp'} = "sel_visibility";
    $h->{'encoding_inp'} = "110000-A|Display\r\n110001-A|Non-Display";
    $defs[$i++] = $h;

    foreach $hash (@defs) {
	$object_hashref = {};
	$object_hashref->{'objclass_inp'} = $site->{'OBJCLASSES'}->{'encoding'};
	$object_hashref->{'objtype_inp'} = $site->{'OBJTYPES'}->{'encoding'};
	$object_hashref->{'status_inp'} = "15000-A";
	$object_hashref->{'visibility_inp'} = "11000-A";
	$object_hashref->{'brief_inp'} = "This is an internal field for AllCommerce.";
	$object_hashref->{'language_inp'} = "eng";
	$object_hashref->{'title_inp'} = $hash->{'title_inp'};
	$object_hashref->{'encoding_inp'} = $hash->{'encoding_inp'};
	$object_hashref->{'sel_name_inp'} = lc($hash->{'sel_name_inp'});
	$object_hashref->{'override_selname_check'} = "yes";
	$error = &create_object($object_hashref, $site, $dbh);
    }
}

##############################################################################
=pod   

=head2 initialize_html_conf

initialize_html_conf - &initialize_html_conf($site);

=over 4

=item DESCRIPTION

Builds the initial html_def.pm file for html object configurations.  This is
called from configure.pl.  NOTE, you do not want to call this if the user has
already added to html_def.pm, as it will overwrite all of their information.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

Writes out the subroutine set_html_def which re-writes the html_def.pm library routine
that loads in the html configuration data for attributes.

=item NOTES

=back

=cut
###############################################################################
sub initialize_html_conf {
    my ($site) = shift;

    my ($h);

    $h = {};
    $h->{'adjust'} = "text:size=9:maxlength=9";
    $h->{'author'} = "text:size=40:maxlength=80";
    $h->{'axisnames'} = "text:size=40:maxlength=80";
    $h->{'behavior'} = "select:sel_behavior";
    $h->{'brief'} = "textarea:rows=3:cols=38:wrap=visual";
    $h->{'currency'} = "select:sel_currency";
    $h->{'descript'} = "textarea:rows=9:cols=38:wrap=visual";
    $h->{'encoding'} = "textarea:rows=9:cols=38:wrap=visual";
    $h->{'gen'} = "checkbox:name=generate_inp:value=1";
    $h->{'gen_trigs'} = "select:sel_gen_trigs";
    $h->{'gui'} = "textarea:rows=9:cols=38:wrap=visual";
    $h->{'gui_name'} = "text:size=40:maxlength=80";
    $h->{'hint'} = "text:size=40:maxlength=80";
    $h->{'html'} = "text:size=40:maxlength=80";
    $h->{'keywords'} = "text:size=40:maxlength=80";
    $h->{'lang_sens'} = "select:sel_lang_sens";
    $h->{'manual_sku'} = "text:size=40:maxlength=80";
    $h->{'msku'} = "text:size=40:maxlength=80";
    $h->{'pagestyle'} = "select:sel_pagestyle";
    $h->{'passwd'} = "text:size=40:maxlength=80";
    $h->{'sel_name'} = "text:size=40:maxlength=80";
    $h->{'ship_weight'} = "text:size=9:maxlength=9";
    $h->{'sku_series'} = "select:sel_sku_series";
    $h->{'sku_skel'} = "text:size=3:maxlength=3";
    $h->{'status'} = "select:sel_status";
    $h->{'title'} = "text:size=40:maxlength=80";
    $h->{'uname'} = "text:size=40:maxlength=80";
    $h->{'unitpr'} = "text:size=11:maxlength=11";
    $h->{'uperm'} = "textarea:rows=9:cols=38:wrap=visual";
    $h->{'uperms'} = "text:size=40:maxlength=80";
    $h->{'variant_set'} = "text:size=40:maxlength=80";
    $h->{'visibility'} = "select:sel_visibility";
    $h->{'weight_man'} = "text:size=3:maxlength=3";

    &write_html_conf("$site->{'relpath'}/conf/html_def.pm", $h, {}, $site);
    if ( $^O !~ /MSWin32/ ) {
        my $gid = getgrnam("nobody");
        my $uid = getpwnam("nobody") || die "no \"nobody\" user. \n";
        chown $gid, $uid, "$site->{'relpath'}/conf/html_def.pm";
    }
}

sub load_permission_sets {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($h, $hash, $i, $object_hashref, @defs, $error);
    $i = 0;
    $h = {};
    $h->{'title_inp'} = "Full Access";
    $h->{'brief_inp'} = "This is the permission set for complete access to the system";
    $h->{'uperm_inp'} = "rcmd\t*\t*";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Read Only Access";
    $h->{'brief_inp'} = "This is the permission set for complete read only access to the system";
    $h->{'uperm_inp'} = "r\t*\t*";
    $defs[$i++] = $h;

    $h = {};
    $h->{'title_inp'} = "Data Entry";
    $h->{'brief_inp'} = "This is the permission set for complete access to the information which is required for configuring the store side of the system";
    $h->{'uperm_inp'} = "rcmd\t*\t*
x\t$site->{'OBJCLASSES'}->{'contact'}\t*
x\t$site->{'OBJCLASSES'}->{'permission_set'}\t*";

    $defs[$i++] = $h;

    foreach $hash (@defs) {
	$object_hashref = {};
	$object_hashref->{'objclass_inp'} = $site->{'OBJCLASSES'}->{'permission_set'};
	$object_hashref->{'objtype_inp'} = $site->{'OBJTYPES'}->{'permission_set'};
	$object_hashref->{'status_inp'} = "15000-A";
	$object_hashref->{'language_inp'} = "eng";
	$object_hashref->{'title_inp'} = $hash->{'title_inp'};
	$object_hashref->{'brief_inp'} = $hash->{'brief_inp'};
	$object_hashref->{'uperm_inp'} = $hash->{'uperm_inp'};
	$error = &create_object($object_hashref, $site, $dbh);
    }
}

sub create_users {
    my ($site) = shift;
    my ($dbh) = shift;

    my ($h, $hash, @defs, $i, $object_hashref, $error, $admin_objid);
    my ($select_hashref, $q, $numRows, $objid, @res);

    $i = 0;

    $select_hashref = {};
    $select_hashref->{'statement'} = "select";
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'permission_set'}, "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("title", "Read Only Access", "and");
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@res);
    $objid = $res[0]->{'objid'};

    $h = {};
    $h->{'uname_inp'} = "read_only";
    $h->{'passwd_inp'} = "read_only";
    $h->{'uperms_inp'} = $objid;
    $defs[$i++] = $h;

    $select_hashref = {};
    $select_hashref->{'statement'} = "select";
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'permission_set'}, "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("title", "Data Entry", "and");
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@res);
    $objid = $res[0]->{'objid'};

    $h = {};
    $h->{'uname_inp'} = "data_entry";
    $h->{'passwd_inp'} = "data_entry";
    $h->{'gen_trigs_inp'} = "1";
    $h->{'uperms_inp'} = $objid;
    $defs[$i++] = $h;

    # create new users
    foreach $hash (@defs) {
	$object_hashref = {};
	$object_hashref->{'objclass_inp'} = $site->{'OBJCLASSES'}->{'contact'};
	$object_hashref->{'objtype_inp'} = $site->{'OBJTYPES'}->{'contact'};
	$object_hashref->{'status_inp'} = "15000-A";
	$object_hashref->{'language_inp'} = "eng";
	$object_hashref->{'uname_inp'} = $hash->{'uname_inp'};
	$object_hashref->{'uperms_inp'} = $hash->{'uperms_inp'};
	$object_hashref->{'passwd_inp'} = $hash->{'passwd_inp'};
	$error = &create_object($object_hashref, $site, $dbh);
    }

    # Update the admin user with it's permission set

    $select_hashref = {};
    $select_hashref->{'statement'} = "select";
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'permission_set'}, "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("title", "Full Access", "and");
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@res);
    $objid = $res[0]->{'objid'};

    $h = {};
    $h->{'uname_inp'} = "admin";
    $h->{'passwd_inp'} = "allcommerce";
    $h->{'gen_trigs_inp'} = "1";
    $h->{'uperms_inp'} = $objid;
    $defs[$i++] = $h;

    $object_hashref = {};
    $object_hashref->{'objclass_inp'} = $site->{'OBJCLASSES'}->{'contact'};
    $object_hashref->{'objtype_inp'} = $site->{'OBJTYPES'}->{'contact'};
    $object_hashref->{'language_pre'} = "";
    $object_hashref->{'language_inp'} = "eng";
    $object_hashref->{'uperms_pre'} = "";
    $object_hashref->{'uperms_inp'} = $h->{'uperms_inp'};
    $object_hashref->{'gen_trigs_pre'} = "";
    $object_hashref->{'gen_trigs_inp'} = $h->{'gen_trigs_inp'};
    $error = &update_object("1123-AA", $object_hashref, $site, $dbh);
}

1;






