#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# filetools.pm
# Version 3 TG 20000913
#    Added filetools_msg
# Version 2 TG 20000912
#    Retrofit for development standards
# Version 1 TG 20000823
#    Started retrofit to development standards
# Version 35 TG 20000816
#     Added debug subroutine
# Version 34 TG 20000706
#     removed second version of loadskutohash
# Version 33 JC 20000705
#    fixed bug with variants in permutebit
# Version 32 TG 20000607
#    fixed loadskutohash
# Version 31 IB 20000605
#    openmerchant->os_allcommerce
# Version 30 GF 20000524
#    added back loadtemplate
# Version 29 GF 20000421
#    renamed into proper script versioning scheme
# Version 28 IB 20000420
#    eliminated loadtemplate
# Version 27 JC 20000415
#    changes for multilingual support
# Version 26 JC 20000330
#    changed connect statement
# Version 25 JC 20000330
#    changed DBI prepare/connect
# Version 24 IB 20000302
#    removed -w
# Version 23 IB 20000301
#    updated copyright
# Version 22 IB 20000229
#    changed table to database
# Version 21 IB 20000226
#    changed store to site
# Version 20 IB 991115
#    mysql to dbi
# Version 19 IB 991114
#    corrected database cleanup
# Version 18 IB 991110
#    corrected database call
# Version 17 SH 990708
#    done system 6 retrofitting
#    changed queries and database connections
# Version 16 SH 990615
#    modified loadtemplate to default to "openmerchant" template files
#    eliminated loadtemplate's check in the all directory
# Version 15 RF 990428
#    DISABLED comment removal from loadtemplate pending javascript detect
# Version 14 RF 990208
#    fixed use of fsrelativeroot and storecode -> store_sku_code
# Version 13 RF 981219
#    fixed errorlog streaming errors
# Version 12 RF 981211
#    fixed loadskutohashref
# Version 11 RF 981209
#    added loadskutohashref(sku):\% - returns all fiels of sku (+variant)
# Version 10 RF 981205
#    modified loadtemplate to look in store bits, then all/bits
# Version 9 RF 981127
#    bugfix on loadskutohash
# Version 8 RF 981125
#    expanded loadtemplate to handle .sel files
# Version 7 RF 981122
#    fixed loadtemplate - it looks for an "all" template
#       if it cannot find the $storeshortname file...
#    added deterministic variant in loadskutohash
# Version 6 RF 981120
#    fixed undesried kill of include statements
# Version 5 RF 981112
#    modified permuteclean to kill self-referencing links if second arg is filename
#    modified permutebit to handle replacements with leading / and
#       trailing _ characters
#    modified loadtemplate to handle include files as well, prefix i_
# Version 4 RF 981107
#    added function to read template vars and data
#    added readtemplate - reads file into string
#    ADDED these functions:
#       loadskutohash($sku) - \%h
#       loadtemplate($filename w/o extension, $bitspath) - $s
#       permutebit( $templatestring, $hashref ) - $s
#       permuteclean( $templatestring ) - $s
#
# Version 3 RF 981022
#    added
# Version 2 RF 981021
#    added hashthefile(targetpathfromroot,templatetouse);
#

use strict;

##############################################################################
=pod

=head1 filetools.pm

filetools.pm - library to handle flat files

=head1 DESCRIPTION

This library is used for all subroutines that modify or use flat files.

=head1 NOTES & BUGS

The script which requires this library, must also require "eapi.pm" prior
to the require filetools.pm.

=cut

##############################################################################

##############################################################################
=pod

=head2 permutebit

permutebit - permutebit($string, $h);

=over 4

=item DESCRIPTION

Takes a template string, and a hash ref, replaces all tags in __tag__ form,
where $h->{'tag'} is defined.  Returns the string, can possibly contain tags
that have not be replaced with values.

=item INPUT

$string - Template in string format, after calling permutebit.

$h - Hashref of tag and values to replace tags in $string with.

=item OUTPUT

$string - String with __tags__ replaced for all tags in $h that match and
are defined.  There could be some __tags__ that have not been replaced
yet.

=item NOTES

=back

=cut
###############################################################################
sub permutebit {
    my ($string) = shift;
    my ($h) = shift;
    
    my ($tag_match,$tag_value,$key);
    if ( ! $h ) {
        return $string;
    } #
    
    foreach ( keys %{$h} ) {
        $tag_value="";
        $tag_value =  $h->{$_} if ( defined($h->{$_}) );
        if ( $tag_value =~ /HASH/ ) {
            foreach $key ( keys %{$h->{$_}} ) {
                $tag_value = $h->{$_}->{$key};
                $tag_match = "__".$key."__";
                $string =~ s/$tag_match/$tag_value/gms ;
            }
        }
        $tag_match = "__".$_."__";
        $string =~ s/$tag_match/$tag_value/gms ;
    }
    
    return $string;
} #/permutebit


##############################################################################
=pod

=head2 permuteclean

permuteclean - permuteclean($template);

=over 4

=item DESCRIPTION

Takes a template string that has already undergone var expansion and cleans
out remaining unexpanded variables.

=item INPUT

$string - Template in string format, after calling permutebit.

=item OUTPUT

$string - String with any additional __tags__ removed from the template.

=item NOTES

=back

=cut
###############################################################################
sub permuteclean {
    my ($string) = shift;
    
    $string =~ s/(\s|[;:,<>()"^$?+.])__(\w|-|\.)+__(\s|[;:,<>()"^$?+.])/$1$3/g;
    
    return $string;
} #/permuteclean

##############################################################################
=pod

=head2 loadtemplate

loadtemplate - &loadtemplate($filename, $file_path);

=over 4

=item DESCRIPTION

Reads a template file into a string format, and returns the string to the calling
routine.

=item INPUT

$filename - Name of the file to load into a string

$path - Path to the file

=item OUTPUT

$file_string - File read into string format

=item NOTES

* SELECT pregeneration files are passed with their .sel extensions

* If there's no template w/ the site's shortname, defaults to os_allcommerce

=back

=cut
###############################################################################
sub loadtemplate {
    my ($filename) = shift;
    my ($path) = shift;
    
    my($file_string, $ok, $path_and_file, $i);
    my (@file_words, $default_filepath);
    my ($built_filename);
    
    local(*FH);
    
    @file_words= split /-/,$filename;
    $built_filename="os_allcommerce";
    for ($i=1; $i <= $#file_words; $i++) {
        $built_filename=$built_filename."-".$file_words[$i];
    }
    # subdirs are "pages", "cells", "rows", "subcells","sel"
    $ok=0;
    if ( $filename =~ /\.sel$/ ) {
        # it is a .sel file
        $path_and_file="$path/sel/$filename";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/sel/$built_filename";
            $ok=1 if ( -e "$path_and_file" );
        }
    } elsif ( $filename =~ /^cell/ ) {
        # it is a cell
        $path_and_file="$path/cells/$filename.txt";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/cells/$built_filename.txt";
            $ok=1 if ( -e "$path_and_file" );
        }
    } elsif ( $filename =~ /^(subc|sc)/ ) {
        # subcell
        $path_and_file="$path/subcells/$filename.txt";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/subcells/$built_filename.txt";
            $ok=1 if ( -e "$path_and_file" );
        }
    } elsif ( $filename =~ /^row/ ) {
        # row
        $path_and_file="$path/rows/$filename.txt";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            
            $path_and_file="$path/rows/$built_filename.txt";
            $ok=1 if ( -e "$path_and_file" );
        }
    } elsif ( $filename =~ /^inc_/ ) {
        # include
        $path_and_file="$path/include/$filename.html";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/include/$built_filename.html";
            $ok=1 if ( -e "$path_and_file" );
        }
    } else {
        # default guess - page template
        $path_and_file="$path/pages/$filename.txt";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/pages/$built_filename.txt";
            $ok=1 if ( -e "$path_and_file" );
        }
    }
    $ok=1 if ( -e "$path_and_file" );
    ## if that failed, try includes
    #
    if ( ! $ok ) {
        $path_and_file="$path/include/$filename.html";
        if ( -e "$path_and_file" ) {
            $ok=1;
	} else { # look at os_allcommerce
            $path_and_file="$path/include/$built_filename.html";
            $ok=1 if ( -e "$path_and_file" );
        }
    }
    ## if that failed, try the $path directory itself
    #
    if ( ! $ok ) {
        $path_and_file="$path/$filename.txt";
        $ok=1 if ( -e "$path_and_file" );
        if ( ! $ok ) {
            $path_and_file="$path/$built_filename.txt";
            $ok=1 if ( -e "$path_and_file" );
        }
    }
    
    ## if OK, get the file, otherwise return an error
    if ( $ok ) {
        # success
        open(FH, "<$path_and_file");
        $file_string = join "", (<FH>);
        close(FH);
    } else {
	
        $file_string="filename=$filename <br> built filename=$built_filename <br> path=$path \n";
    }
    
    ## now remove all html comments from the template, EXCEPT includes
    #
    # $file_string =~ s/<!--(?!(#include|#exec|# include|# exec))(.+?)-->//gsm ;
    
    return $file_string;
} #/loadtemplate

##############################################################################
=pod
    
=head2 loadObjidToHashref
    
loadObjidToHashref - &loadObjidToHashref($objid, $site, $dbh);

=over 4

=item DESCRIPTION

Takes an objid, and returns the attributes of the object in a hashref

=item INPUT

$objid - objid to search for prices

$site - site hashref

$dbh - current active database handle

=item OUTPUT

$h - hashref of attributes of the object from things and extraprop tables.

=item NOTES

Will need to be modified for multi-language support

=back

=cut
###############################################################################
sub loadObjidToHashref {
    my $objid = shift;
    my $site = shift;
    my $dbh = shift;
    
    my ($extraprop_objid, $sth, $q, $h, $i, $fieldname, @temp, $key, $body);
    my ($select_hashref, @things_results, @extraprop_results, $extra_hashref);
    my ($numRows);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $objid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@things_results);
    
    if ($numRows == 1) {
        $h = $things_results[0];
    }
    else {
        $h = {};
    }
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "fieldname";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $objid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    
    $numRows = &query_getAll($q, $site, $dbh, \@things_results);
    if ($numRows == 1) {
        $h = $things_results[0];
    }
    else {
        $h = {};
    }
    
    for ($i=0; $i < $numRows; $i++ ) {
        $extra_hashref = $extraprop_results[$i];
        $h->{$extra_hashref->{'fieldname'}} =  $extra_hashref->{'$body'};
    }
    
    @temp=keys %{$h};
    
    # remove dbi-safe encodings
    while ( @temp ) {
        $key = shift @temp;
        $h->{$key}=string_exdbi( $h->{$key} );
    } # end of de-dbi-safe-encoding strings
    
    return $h;
    
} #/loadObjidToHashref

##############################################################################
=pod

=head2 debug

debug - &debug("Debug message", 3, $site->{'debug_level'}, $site->{'debug_log'});

=over 4

=item DESCRIPTION

To format and print out a debug string if the  message level
is less than or equal to the script debug level.

=item INPUT

$msg - Debug message to print to the debug log

$message_level - The level assigned to the current debug request.

$debug_level - The assigned debug level of the calling script.

$file - The output debug file to write to.  Must be the full path and file name,
and start with > for write, or >> for append.

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub debug {
    my ($msg) = shift;
    my ($message_level) = shift;
    my ($debug_level) = shift;
    my ($file) = shift;
    
    my ($now);
    
    $now = nicedatetime();
    
    if ($message_level <= $debug_level) {
        $msg = "$now - $msg\n";
        open FH, "$file";
        print FH $msg;
        close FH;
    }
}


##############################################################################
=pod

=head2 debug_hashref

debug_hashref - &debug_hashref($h, 3, $site->{'debug_level'}, $site->{'debug_log'});

=over 4

=item DESCRIPTION

The contents of the hash are printed out to the debug file if the message level
is less than or equal to the script debug level.

=item INPUT

$h - hashref of information to be printed to the debug log

$message_level - The level assigned to the current debug request.

$debug_level - The assigned debug level of the calling script.

$file - The output debug file to write to.  Must be the full path and file name,
and start with > for write, or >> for append.

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub debug_hashref {
    my ($h) = shift;
    my ($message_level) = shift;
    my ($debug_level) = shift;
    my ($file) = shift;
    
    my ($now, $key, $msg);
    
    $now = nicedatetime();
    
    if ($message_level <= $debug_level) {
        $msg = "$now\n";
        foreach $key (keys %{$h}) {
            if ($h->{$key}) {
                $msg .= "$key - $h->{$key}\n";
            }
        }
        open FH, "$file";
        print FH $msg;
        close FH;
    }
}

##############################################################################
=pod

=head2 filetools_message

filetools_msg - $ret = &filetools_msg($site, 1);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This returns the error 
message based the number passed in.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$msg_number - The number of the message to be returned

=item OUTPUT

The text of the message in the appropriate language.

=item NOTES

=back

=cut
###############################################################################
sub filetools_message {
    my ($site) = shift;
    my ($msg_number) = shift;
    
    my (@temp);
    my (%script_msg);
    
    # check the language that is being used and open appropriate
    # message language file or get messages below
    if ($site->{'om_render_lang'} eq "eng" ) {
        $script_msg{1}="";  # ADD MESSAGES HERE
        } else {
        open( EM, "<$site->{'relpath'}/bits/$site->{'om_render_lang'}/system/$site->{'script'}.messages");
        while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
        }
        close(EM);
    }
    return $script_msg{$msg_number};
}
1;










