#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# formtools.pm
#
# Set of form and page generation tools
#
# Version 4 TG 20000929
#    Added CR_to_HTML - not used at the moment but will be by admin_object
# Version 3 TG 20000912
#    Retrofit for development standards
# Version 2 TG 20000830
#    Moved search stuff to search.pm.  Need to remove setSearchNavigation after
#    all scripts use it from search.pm
# Version 1 TG 20000823
#    Starting retrofit for coding standards
# Version 0.7.19 JC 20000807
#    bug fixes
# Version 0.7.18 TG 20000713
#    Added search routines, searchVariantEncodings, addVariantProducts,
#    getVariantCode, getVariantLocation, retrofit for language -
#    formSearchQuery
# Version 0.7.17 TG 20000712
#    Limited number of page links in setSearchNavigation
# Version 0.7.16 TG 20000707
#    Put OR statements in () for formSearchQuery
# Version 0.7.15 TG 20000706
#    Added links to pages for search results
# Version 0.7.14 TG 20000705
#    Added objtype to formSearchQuery
# Version 0.7.13 TG 20000629
#    Added setSearchNavigation & formSearchQuery
# Version 0.7.12 TG 20000623
#    Added loadseltomem
# Version 0.7.11 TG 20000621
#    added loadselects
# Version 0.7.10 IB 20000608
#    corrected the first line
# Version 0.7.9 TG 20000606
#    fixed sortlinks to use the proper fields
# Version 0.7.8 IB 20000301
#    updated copyright
# Version 0.7.7 IB 991112
#    adding subroutines string_todbi and string_exdbi to the string_tomysql
#    and string_exmysql
# Version 0.7.6 IB 991110
#    minor cleanup
# Version 0.7.5 RF 990830
#    made string_exmysql sensitive to \\_ stuff
# Version 0.7.4 RF 990830
#    fixed bug in string_tomysql for "_"->"\_"
# Version 0.7.3 RF 990830
#    fixed bug in string_exmysql for "\_"->"_"
# Version 0.7.2 RF 990829
#    added variant sensitivity to sortlinks()
# Version 0.7.1 GF 990829
#    removing quotes from selects in gen_select
# Version 9 SH 990819
#    hack fix of note on sortlinks
# SH Note:  sortlinks needs to be rewritten
# Version 8 RF 990208
#    corrected use of fsrelativeroot and storecode -> store_sku_code
# Version 7 RF 981211
#    added nicedecimals( $s ):$s - evens up trailing zeros to currency
# Version 6 RF 981208
#    added getformrows( \% ):\% - returns hashref of hashrefs of rows
# Version 5 RF 981127
#    added sortlinks
#    fixed gen_select matching bug
# Version 4 RF 981125
#    modified keylength operator from equality to >= in gen_select
#    added make_hash_htmlsafe - html encodes certain fields
# Version 3 RF 981123
#    added gen_safeselect - generate select w/only one choice
# Version 2 RF 981013
#    added string to/from mysql conversion operations
#    added string to/from html conversion operations
# version 1 RF 981009
#
#

use strict;

##############################################################################
=pod

=head1 formtools.pm

formtools.pm - subroutines to help with form manipulation

=head1 DESCRIPTION

This library is used for all subroutines that help with form manipulation, 
or form displays.

=head1 NOTES & BUGS

=cut

##############################################################################

##############################################################################
=pod

=head2 permute_rows

permute_rows - &permute_rows($template, $site, \@rows);

=over 4

=item DESCRIPTION

Takes reads in all of the templates from the rows directory in the admin tree
that are listed in $rows, and replaces their tag in the main template passed in.

Therefore, it loads in all of the row templates to build the one main template.

=item INPUT

$template - The current template, as is, with the __row__ tags for each row defined
            in $rows

$site - A hashref containing information required throughout about the current
        site and session

$rows - Pointer to an array, which is a list of row templates to load in and replace
        their respective __tag__ values with.

=item OUTPUT

$template - The template with the rows permuted.

=item NOTES

=back

=cut
###############################################################################
sub permute_rows {
    my ($template) = shift;
    my ($site) = shift;
    my ($rows) = shift;

    my ($row, $row_template, $target);
    foreach $row (@$rows) {
	$row_template = &localize_template($row, "admin", 0, $site);
	if ($row eq "row_admin_object_home") {
	    $target = "__row_body__";
	}	    
	else {
	    $target = "__" . $row . "__";
	}
	$template =~ s/$target/$row_template/gms;
    }

    return $template;
}

##############################################################################
=pod

=head2 nicedecimals

nicedecimals - &nicedecimals();

=over 4

=item DESCRIPTION

Take a floating point value, and round it to 2 decimal places

=item INPUT

$number - A floating point number.

=item OUTPUT

A number rounded to two decimal places.

=item NOTES

=back

=cut
###############################################################################
sub nicedecimals {
   my $i = shift;
   my ( $left,$right ) = split /\./, $i;
   my $round_digit = substr($right, 2, 1);
   $right =~ s/^(..).*/$1/;
   $right = sprintf("%-2s", $right);
   $right =~ s/\s/0/g;

   if ($round_digit >= 5 && $right < 99) {
      $right += 1;
   } elsif ($round_digit >= 5 && $right == 99) {
   	  $right = '00';
	  $left += 1;
   }
   
   return sprintf("%d\.%02d", $left, $right);
} #/nicedecimals


##############################################################################
=pod

=head2 sortlinks

sortlinks - &sortlinks (@links);

=over 4

=item DESCRIPTION

Take an array of links and sort them.

=item INPUT

@links - an array of linkage records

=item OUTPUT

@sorted_links - array of linkage records, sorted by linkwt_use.

=item NOTES

=back

=cut
###############################################################################
sub sortlinks {
    my (@links) = @_;

    my(@sorted_links, @temp, $i, $link_weight, $padding, $link_key );
    @sorted_links = ();
    @temp=();
    
    for ( $i=0; $i<=$#links; $i++ ) {
        $link_weight='0';
        $link_weight=$links[$i]->{'linkwt_use'};
        $link_weight=$links[$i]->{'linkwt_man'} if ( ( ! $link_weight ) and ( $links[$i]->{'linkwt_man'} ) );
        $link_weight=$links[$i]->{'linkwt_sys'} if ( ( ! $link_weight ) and ( $links[$i]->{'linkwt_sys'} ) );
        $link_weight=$links[$i]->{'linkwt_man_inp'} if ( ( ! $link_weight ) and ( $links[$i]->{'linkwt_man_inp'} ) );
        $link_weight='10' if ( ! $link_weight );
        
        # now construct sort key
        $padding='0' if ( ! $padding );
        $link_key = "$padding"."$link_weight X$i";  # bootstrap to make work with 100
        push @temp, $link_key;
    } # end of running through key creation for sorting
    
    sub bywidthweight { $b <=> $a; } # sort numerically descending
    @temp=sort bywidthweight @temp;
    
    while ( @temp ) {
        ( $link_weight, $i ) = split /X/, shift @temp;
        push @sorted_links, $links[$i] ;
    } # end of assembling new array

    return @sorted_links;
    
} #/sortlinks

##############################################################################
=pod

=head2 gen_select

gen_select - &gen_select("15000-A", 7, @status_sel_data);

=over 4

=item DESCRIPTION

Generates an OPTION value [SELECTED] pulldown.

=item INPUT

$default - Default value to be selected.

$key_length - Maximum length of the key information (the data before the |)

@sel_contents - key-value pairs, vertical slash delimited array

=item OUTPUT

$select_html - HTML for the OPTION with default value set.

=item NOTES

=back

=cut
###############################################################################
sub gen_select {
    my ($default) = shift;
    my ($key_length) = shift;
    my (@sel_contents) = @_;

    my($key, $value, $select_html, $default_set_flag);

    local(*FH);
    $select_html="";
    $default_set_flag=1;
    while (@sel_contents) {
        ( $key,$value )  = split /\|/, shift @sel_contents;
        $select_html.= "<OPTION VALUE=$key ";  # GF removed quotes
        
        if (( $default ) and ( $default_set_flag ) and ( length($default)>=$key_length )) {
            if ( "$default" eq "$key" ) {
                $select_html.="SELECTED $key = $default";
                $default_set_flag=0;
            }
        }
        $select_html.="> $value \n";
    }

    return $select_html;
} # /gen_select


##############################################################################
=pod

=head2 loadselects

loadselects - &loadselects(\%FORM, \%NVP, $site->{'selskeletonpath'}, @pulldowns);

=over 4

=item DESCRIPTION

To read in all .sel files specified in @pulldowns and load them into $NVP for display

=item INPUT

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$selskeletonpath - Path to the sel files

@pulldowns - Array of 'PULLDOWN|NUM' where PULLDOWN is the name of the file before 
             the .sel and NUM is maximum length of the key

=item OUTPUT

Updates the NVP hashref and the FORM hashref with the generated OPTION pulldowns.

=item NOTES

=back

=cut
###############################################################################
sub loadselects {
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($selskeletonpath) = shift;
    my (@pulldowns) = @_;
    
    my ($filename, $key_length, $select_input, @raw, $select_tag);
    my (@temp, $select_pre);
 

    &debug("In loadselects with $selskeletonpath, @pulldowns", 3, 3, ">>/tmp/formtools");
    foreach ( @pulldowns) {
        
        ( $filename,$key_length) = split /\|/, $_;
        open(FH, "<$selskeletonpath/$filename.sel");
        @raw = <FH>;
        chomp @raw;
        close(FH);
        
        # Make the sel_ files work with the current procedure.
        if ($filename =~ /sel_/) {
            $filename =~ s/sel_//gms;
            $filename = uc($filename);
        }
        
        $select_input = $filename."_inp";
        $select_pre = lc($filename."_pre");
        $select_tag = $filename."_SELECT";
        $select_input = lc($select_input);    # make it lower case for pulldowns!
	&debug("$select_tag, $select_input, $select_pre", 3, 3, ">>/tmp/formtools");
        $$NVP{$select_tag} = gen_select( $$FORM{$select_input},$key_length,@raw );
	&debug("Generated $$NVP{$select_tag}", 3, 3, ">>/tmp/formtools");
        $$NVP{$select_pre} = $$FORM{$select_input};
        $$FORM{$select_pre} = $$FORM{$select_input};
    }
}

##############################################################################
=pod

=head2 loadseltomem

loadseltomem - &loadseltomem("status", $site->{'selskeletonpath'});

=over 4

=item DESCRIPTION

To load in the contents of a sel file, with the key being the value of the 
select HTML object, and the value being the text of the select HTML object.  
This can be used to display  Active vs 15000-A on forms for example.

=item INPUT

$pulldown - Name of the select file to parse and load

$selskeletonpath - Path to the sel files

=item OUTPUT

$h - Hashref with the key being the value before the | in the .sel file, and the
     value being the value after the | in the .sel file

=item NOTES

=back

=cut
###############################################################################
sub loadseltomem {
    my ($pulldown) = shift;
    my ($selskeletonpath) = shift;
    
    my ($key, $value, @raw, $h, $i);
    
    open(FH, "<$selskeletonpath/$pulldown.sel");
    @raw=<FH>;
    chomp @raw;
    close(FH);
    
    for ($i = 0; $i <= $#raw; $i++) {
        ($key, $value) = split /\|/, $raw[$i];
        $h->{$key} = $value;
    }
    
    return $h;
}

##############################################################################
=pod

=head2 string_tohtml

string_tohtml - &string_tohtml("Hello\"There\"");

=over 4

=item DESCRIPTION

Replace " characters in strings to &quot for display in HTML

=item INPUT

$string - Input string to convert

=item OUTPUT

$string - String, with all " replaced with &quot

=item NOTES

=back

=cut
###############################################################################
sub string_tohtml {
    my ($string) = shift;

    $string =~ s/"/&quot;/g;
    return $string;

} # /string_tohtml

##############################################################################
=pod

=head2 string_exmysql

string_exmysl - &string_exmysql($string);

=over 4

=item DESCRIPTION

Replaces \'d characters that have be stored in the database to just the character
for display in HTML.

=item INPUT

$string - Input string to convert

=item OUTPUT

$string - String, with all \ removed

=item NOTES

SHOULD BE PHASED OUT, use string_exdbi
=back

=cut
###############################################################################
sub string_exmysql {
    my ($string) = shift;

    $string =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
    $string =~ s/\\('|"|%|_|t|n|r)/$1/g ;
    
    return $string;
} # /string_exmysql

##############################################################################
=pod

=head2 string_todbi

string_todbi - &string_todbi($string);

=over 4

=item DESCRIPTION

Replace special characters with the quoted version of them to insert them into
the database.  \'d.

=item INPUT

$string - Input string to convert

=item OUTPUT

$string - String, with all key characters quoted

=item NOTES

=back

=cut
###############################################################################
sub string_todbi {
    my ($string) = shift;

    $string =~ s/('|")/\\$1/g ;
    
    return $string;
} # /string_todbi

##############################################################################
=pod

=head2 string_exmysql

string_exmysl - &string_exmysql($string);

=over 4

=item DESCRIPTION

Replaces \'d characters that have be stored in the database to just the character
for display in HTML.

=item INPUT

$string - Input string to convert

=item OUTPUT

$string - String, with all \ removed

=item NOTES

=back

=cut
###############################################################################
sub string_exdbi {
    my ($string) = shift;
    $string =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
    $string =~ s/\\('|"|%|_|t|n|r)/$1/g ;
    
    return $string;
} # /string_exdbi


##############################################################################
=pod

=head2 CR_to_HTML

CR_to_HTML- &CR_to_HTML(@text);

=over 4

=item DESCRIPTION

To replace \r\n with <P> for fields that use HTML.  Stands for Carriage return
to HTML.

=item INPUT

@text - Array of strings, in which all occurances of \r\n are replaced with <P>

=item OUTPUT

$htmlrows - A string of all rows of @rows concatenated and
           joined by <P>\r\n.  Only one <P> will be inserted
           for each \r\n, and will contain the \r\n at the
           end of the <P> for readability

=item NOTES

=back

=cut
###############################################################################
sub CR_to_HTML {
   my @rows = @_;

   my ($row, $htmlrow, $i);
   
   $htmlrow = "";
   
   for ($i = 0; $i <= $#rows; $i++) {
      $row = $rows[$i];
      # Don't put it on the last line
      if ( ($row !~ /<P>/) && ($i != $#rows) ) {
         $row .= " <P>";
      }
      $row .= "\r\n";
      $htmlrow .= $row;
   }
   $htmlrow=string_tohtml($htmlrow);
   return $htmlrow;
}
##############################################################################
=pod

=head2 set_headers

set_headers - &set_headers("content", $template, $site);

=over 4

=item DESCRIPTION

The main headers on the administator highlights the active script with
orange.  This is done by using the gif with the _over.gif extention.  This
routine determines which script to set to the _over and sets the others to
"", defaulting at the normal gif.

=item INPUT

$script - The name of the script to highlight

$template - The template with the tags in it for replacement

$site - A hashref containing information required throughout about the current
        site and session


=item OUTPUT

$template - with the headers tag replaced

=item NOTES

=back

=cut
###############################################################################
sub set_headers {
    my ($script) = shift;
    my ($template) = shift;
    my ($site) = shift;

    my ($content, $inventory, $stats, $linkage, $order, $admin);

    $content = $inventory = $stats = $linkage = $order = $admin = "";
    if ($script eq "content") {
	$content = "_over";
    }
    elsif ($script eq "inventory") {
	$inventory = "_over";
    }
    elsif ($script eq "linkage") {
	$linkage = "_over";
    }
    elsif ($script eq "stats") {
	$stats = "_over";
    }
    elsif ($script eq "order") {
	$order = "_over";
    }

    $template =~ s/__content_over__/$content/gms;
    $template =~ s/__inventory_over__/$inventory/gms;
    $template =~ s/__linkage_over__/$linkage/gms;
    $template =~ s/__order_over__/$order/gms;
    $template =~ s/__stats_over__/$stats/gms;

    return ($template);
}

##############################################################################
=pod

=head2 build_inventory_link_info

build_inventory_link_info - &build_inventory_link_info(\%FORM, "2-AA", "editprice", $site);

=over 4

=item DESCRIPTION

To build the body of the link information to jump into inventory.cgi into the edit
price mode or edit inventory mode for a particular object.

=item INPUT

$FORM - Pointer to the %FORM hash

$objid - Object ID to go into the mode for

$mode - Mode to go into inventory with (editprice is modifying prices, and editsku
        is modifying inventory)

$encrypt_sid - The sid to pass along in encrypted form

$site - A hashref containing information required throughout about the current
        site and session


$master - OPTIONAL, if set this is the object ID of the master product.

=item OUTPUT

$link_info - The link information required to make a link to inventory.cgi

=item NOTES

=back

=cut
###############################################################################
sub build_inventory_link_info {
    my ($FORM) = shift;
    my ($objid) = shift;
    my ($mode) = shift;
    my ($encrypt_sid) = shift;
    my ($site) = shift;
    my ($master) = shift;

    my ($link_info);

    if (!defined $master) {
	$link_info = "$site->{'scriptpath'}/inventory.cgi\?&sid=$encrypt_sid&mode=$mode&loadfrom=db&objid=$objid&warehouse_inp=&objtype_inp=$$FORM{'objtype_inp'}&objclass_inp=$$FORM{'objclass_inp'}&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'this_startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
    }
    else {  # THIS is a master product link
	if ($objid eq "") {
	    $link_info = "$site->{'scriptpath'}/inventory.cgi\?&sid=$encrypt_sid&mode=$mode&loadfrom=db&master_objid=$master&warehouse_inp=&objtype_inp=$$FORM{'objtype_inp'}&objclass_inp=$$FORM{'objclass_inp'}&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'this_startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
	}
	else {
	    $link_info = "$site->{'scriptpath'}/inventory.cgi\?&sid=$encrypt_sid&mode=$mode&loadfrom=db&objid=$objid&master_objid=$master&warehouse_inp=&objtype_inp=$$FORM{'objtype_inp'}&objclass_inp=$$FORM{'objclass_inp'}&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'this_startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
	}
    }
    return ($link_info);
}

##############################################################################
=pod

=head2 build_admin_object_link_info

build_admin_object_link_info - &build_admin_object_link_info(\%FORM, "2-AA", "display_object", $site);

=over 4

=item DESCRIPTION

To build the body of the link information to jump into admin_object.cgi to the particular mode.

=item INPUT

$FORM - Pointer to the %FORM hash

$objid - Object ID to go into the mode for

$mode - Mode to go into admin_object with, "home" is to display home, "display_object" is to
        display the object in edit mode.

$encrypt_sid - The sid to pass along in encrypted form

$site - A hashref containing information required throughout about the current
        site and session


=item OUTPUT

$link_info - The link information required to make a link to admin_object.cgi

=item NOTES

=back

=cut
###############################################################################
sub build_admin_object_link_info {
    my ($FORM) = shift;
    my ($objid) = shift;
    my ($mode) = shift;
    my ($encrypt_sid) = shift;
    my ($site) = shift;

    my ($link_info);

    &debug("Building admin object link: in with $objid, $mode", 3, $site->{'debug_level'}, $site->{'debug_log'});

    if ($mode eq "display_attach") {
	# put the objtype_inp & objclass_inp in a pre_objtype & pre_objclass field
	$link_info = "$site->{'scriptpath'}/admin_object.cgi\?&sid=$encrypt_sid&mode=$mode&objid=$objid&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&pre_objtype=$$FORM{'objtype_inp'}&pre_objclass=$$FORM{'objclass_inp'}&startrow=$$FORM{'startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
    }
    elsif ($mode eq "display_variants") {
	$link_info = "$site->{'scriptpath'}/admin_object.cgi\?&sid=$encrypt_sid&mode=display_object&master_objid=$objid&objclass_inp=$$FORM{'objclass_inp'}&objtype_inp=$$FORM{'objtype_inp'}&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";

    }
    elsif ($mode eq "search") {
	$link_info = "$site->{'scriptpath'}/admin_object.cgi\?&sid=$encrypt_sid&mode=$mode&came_from_search=$$FORM{'came_from_search'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
    }
    else {
	$link_info = "$site->{'scriptpath'}/admin_object.cgi\?&sid=$encrypt_sid&mode=$mode&objid=$objid&objclass_inp=$$FORM{'objclass_inp'}&objclass_pre=$$FORM{'objclass_inp'}&objtype_pre=$$FORM{'objtype_inp'}&objtype_inp=$$FORM{'objtype_inp'}&came_from_search=$$FORM{'came_from_search'}&gui_name=$$FORM{'gui_name'}&search_keywords=$$FORM{'search_keywords'}&search_objid=$$FORM{'search_objid'}&search_objtype=$$FORM{'search_objtype'}&search_sku_series=$$FORM{'search_sku_series'}&search_status=$$FORM{'search_status'}&foundcount=$$FORM{'foundcount'}&startrow=$$FORM{'this_startrow'}&next_startrow=$$FORM{'next_startrow'}&previous_startrow=$$FORM{'previous_startrow'}&lang_startrow=$$FORM{'lang_startrow'}&next_lang_startrow=$$FORM{'next_lang_startrow'}&pre_lang_startrow=$$FORM{'pre_lang_startrow'}&this_startrow=$$FORM{'this_startrow'}";
    }
    return ($link_info);
}


##############################################################################
=pod

=head2 reset_form_values

reset_form_values - &reset_form_values(\%FORM, $saved, $site);

=over 4

=item DESCRIPTION

Sometimes it is necessary to temporarily change the values of the FORM hash
in order to use pre-defined library routines, and to ease complexity.  When
this is necessary, the routine set_form_temporary is called to do so.  

After the work is complete, it is necessary to restore the FORM how it was
prior to the call set_form_temporary, and this is what this routine does.  It
takes all of the data that was saved, and restores the FORM how it used to be.

=item INPUT

$FORM - Pointer to the FORM hash

$saved - Hashref of information that was in the FORM hashref before set_form_temporary
         was called.  These are used to reset the FORM values to what they used to be

site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$FORM - Back the way it was before the call to set_form_temporary

=item NOTES

=back

=cut
###############################################################################
sub reset_form_values {
    my ($FORM) = shift;
    my ($saved) = shift;
    my ($site) = shift;

    my ($field);
    foreach $field (keys %{$saved}) {
	$$FORM{$field} = $saved->{$field};
    }
}

##############################################################################
=pod

=head2 reset_form_values

reset_form_values - &reset_form_values(\%FORM, $saved, $site);

=over 4

=item DESCRIPTION

Sometimes it is necessary to temporarily change the values of the FORM hash
in order to use pre-defined library routines, and to ease complexity.  When
this is necessary, this routine is used.

This routine takes a list of fields, and a hashref of new values.  It copies
the current FORM values for each of the fields into a hash which is saved, and
sets the FORM values to the new values.  It then returns the saved values, which
should call reset_form_values with when it is time to restore the FORM to it's
original state.

=item INPUT

$FORM - Pointer to the FORM hash

$fields - Comma delimited list of fields (keys) that are in the FORM hash and
          need to be replaced.

$set_to - Hashref with key being each field in $fields, and the value being the
          new value to set the FORM to.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$FORM - All the $fields are set to the $set_to values

$saved - Hashref with keys being each field in $fields, and the values being the
         initial FORM values before setting them to $set_to.

=item NOTES

=back

=cut
###############################################################################
sub set_form_temporary {
    my ($FORM) = shift;
    my ($fields) = shift;
    my ($set_to) = shift;
    my ($site) = shift;
    
    my (@all_fields, $saved, $field);
    @all_fields = split /\,/, $fields;
    foreach $field (@all_fields) {
	$saved->{$field} = $$FORM{$field};
	$$FORM{$field} = $set_to->{$field};
    }
    return $saved;
}


1;



