#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

use strict;

##############################################################################
=pod

=head1 hint.pm

hint.pm - subroutines for hint.cgi

=head1 DESCRIPTION

This library is used for all subroutines that are used exclusively by hint.cgi.
This module allows removal of data processing from the main cgi script, allowing
it to only be responsible for displaying the information gathered in this module.

=head1 NOTES & BUGS

At a miniumum, the The calling script must use or require all of the following
libraries to use this library:
    hint.pm
    db_layer.pm
    permission_layer.pm
    validate.pm

=cut

##############################################################################

######################### HINT SUBROUTINES ############################

##############################################################################
=pod

=head2 validateHintInput

validateHintInput - &validateHintInput(\%FORM, $site);

=over 4

=item DESCRIPTION

To validate appropriate email address is entered, if not, then return an error
message

=item INPUT

$FORM - A  pointer to the FORM hash

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$$FORM{'email_err'} = blank if no error, and an error message indicating
    what is wrong with the email address if there is an error

=item NOTES

=back

=cut
###############################################################################
sub validateHintInput {
    my ($FORM) = shift;
    my ($site) = shift;
    
    my ($u);
    
    $u=validateemail ( $$FORM{'email_inp'} );
    
    ## if this is not a correct email address
    #  what is the correct error to display
    
    if ( $u eq "a" ) {
        # Error - No email address entered!
        $$FORM{'email_err'}=$site->{'script_msg'}{3};
        
        } elsif ( $u eq "b" ) {
        # Error - Wrong email address format, need \@
        $$FORM{'email_err'}=$site->{'script_msg'}{4};
        
        } elsif ( $u eq "c" ) {
        # Error - Invalid email address!
        $$FORM{'email_err'}=$site->{'script_msg'}{2};
        
        ## the email address is in the correct format, see if we have a
        #  record on file for the user
        
    }
    
    return $$FORM{'email_err'};
}


##############################################################################
=pod

=head2 retreiveHint

retreiveHint - &retreiveHint(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

A valid email address was entered, so retreive the hint as it is if it
exists.

=item INPUT

$FORM - A  pointer to the FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$$FORM{'hint_err'} = error message if there was an error, and the requested
    hint if it is found

=item NOTES

=back

=cut
###############################################################################
sub retreiveHint {
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $numRows, @results, $select_hashref, $h);
    
    $select_hashref->{'TABLE'}->{'NAME'} = "contacts";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'uname'} = "uname";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'hint'} = "hint";
    $select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("uname", $$FORM{'email_inp'}, "and");
    # used to be hardcoded to 39003-A
    #$select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("objtype", $site->{'OBJTYPES'}->{'contact'}, "and");
	 # back to hardcoding
    $select_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("objtype", "39003-A", "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@results);
    
    if ( $numRows==1 ) {
        ## is there a hint on file for this account
        #
        $h = $results[0];
        if ( length($h->{'hint'})>1 )  {
            $$FORM{'hint_err'}=$site->{'script_msg'}{1}.$h->{'hint'};
            
            } else {
            $$FORM{'hint_err'}="No hint on file for this account!";
        } #/done looking for hint on file
        
        } else {
        $$FORM{'email_err'}=$site->{'script_msg'}{2};
    } #/done checking if we have a record or not
}


##############################################################################
=pod

=head2 checkIfLoggedIn

checkIfLoggedIn - &checkIfLoggedIn($site, \%FORM);

=over 4

=item DESCRIPTION

Verifies that the user is logged in.  If they are not, then it redirects them
to the welcome page.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$FORM - A  pointer to the FORM hash

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub checkIfLoggedIn {
    my ($site) = shift;
    my ($FORM) = shift;
    my ($u, $q);
	 $$FORM{'sid'} = &encrypt_sid($$FORM{'sid'},$site);
    if ($site->{'sid'} ne "") {
        if (length($site->{'custid'}) > 2) { # event - series
            # user is logged in
            $$FORM{'nextcheckoutpg'}=$$FORM{'pg_targ'};
            
        }
        else {
            # user not logged in
            $u="http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi";
            $u.="?sid=$$FORM{'sid'}&pg_targ=$$FORM{'pg_targ'}";
            $$FORM{'nextcheckoutpg'}=$u;
        } # done checking on login when session registered
        
        } else {
        # this is an error
        $q="$site->{'ssl_prefix'}://$site->{'sslpath'}/checkout.cgi?sid=$$FORM{'sid'}";
        $u="http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi";
        $u.="?sid=$$FORM{'sid'}&pg_targ=$q";
        $$FORM{'nextcheckoutpg'}=$u;
    } #/end of checking on sid for login
}
1;
