#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

##############################################################################
=pod

=head1 inventory.pm

inventory.pm - subroutines for inventory.cgi

=head1 DESCRIPTION

This library is used for all subroutines that are used exclusively by inventory.cgi.
This module allows removal of data processing from the main cgi script, allowing
it to only be responsible for displaying the information gathered in this module.

=head1 NOTES & BUGS

=cut

##############################################################################

######################### INVENTORY SUBROUTINES ############################

##############################################################################
=pod

=head2 getExistingInv

getExistingInv - &getExistingInv($objid, $current_inv, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

To get all existing prices for $objid that match $where
clause, and store in $prices array.

=item INPUT

$objid - objid to search for prices

$current_inv - array of inventory that are returned from query,
              each index is a hash with one row from the price table

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$numWarehouses - number of active warehouses

$prices - array of prices that are returned from query, each
          index is a hash with  one row from the price table

=item NOTES

=back

=cut
###############################################################################
sub getExistingInv {
    my ($objid) = shift;
    my ($current_inv) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $numWarehouses, $titles, $wh_inv, $invrow, @warehouse_inv, $i, $wh_inp);
    my ($title, @pulldowns, $k, $v, @raw, $kt, $inp, $key, $kp, $kn, $ki, $query_hashref);
    
    $query_hashref = {};
    $query_hashref->{'TABLE'}->{'NAME'} = "things";
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'title'} = "title";
    $query_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objtype", $site->{'OBJTYPES'}->{'warehouse'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("objclass", $site->{'OBJCLASSES'}->{'warehouse'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("status", "15000-A", "and");
    $query_hashref->{'statement'} = "SELECT";
    # Insert the price new price record
    $q = &build_sql($query_hashref, $site, $dbh);
    $numWarehouses = &query_getAll($q, $site, $dbh, \@warehouse_inv);
    &debug("$q\nFound $numWarehouses results",2,$site->{'debug_level'}, $site->{'debug_log'});
    
    $titles = &get_fields_and_title(undef,$objid, $dbh, undef, $site);
    $$NVP{'pagetitle'}="$site->{'script_msg'}{6} \($objid\) $titles->{'title'} \n";
    
    for ($i = 0; $i < $numWarehouses; $i++) {
        $wh_inv = $warehouse_inv[$i];
        $wh_inv->{'wh_inp'} = $wh_inv->{'objid'};
        $wh_inv->{'wh'} = $wh_inv->{'objid'};
        $wh_inp = "row.$wh_inv->{'objid'}.wh_inp";
        $wh_inv->{$wh_inp} = $wh_inv->{'objid'};
        $wh_inv->{'wh_title'} = $wh_inv->{'title'};
        $invrow = &getInventoryData($objid, $wh_inv->{'wh_inp'}, '', $dbh, $site);
        $wh_inv->{'lstock'} = $invrow->{'lstock'};
        $wh_inv->{'status'} = $invrow->{'status'};
        @pulldowns = ('sel_status|2');
        foreach ( @pulldowns) {
            ( $k,$v) = split /\|/, $_;
            open(FH, "<$site->{'selskeletonpath'}/$k.sel");
            @raw=<FH>;
            chomp @raw;
            close(FH);
            # Make the sel_ files work with the current procedure.
            if ($k =~ /sel_/) {
                $k =~ s/sel_//gms;
                $k = uc($k);
                
                &debug("New pulldown name is $k",3,$site->{'debug_level'}, $site->{'debug_log'});
            }
            $kt=$k."_SELECT";
            $k = lc($k);
            $$NVP{$kt}= gen_select( $wh_inv->{$k},$v,@raw );
            $wh_inv->{$kt} = $$NVP{$kt};
            $inp = $k . "_inp";
            &debug("JUST MADE \$\$NVP{$kt} --> $$NVP{$kt} with \$wh_inv->{$k} --> $wh_inv->{$k}  INP was $wh_inv->{$inp}",3,$site->{'debug_level'}, $site->{'debug_log'});
        }
        
        foreach $key (keys %{$wh_inv}) {
            next if ($key =~ /SELECT$/);
            $v = string_exdbi($wh_inv->{$key});
            $kp=$key."_pre";
            $kn=$key."_ini";
            $ki=$key."_inp";
            $wh_inv->{$key}=$v;
            $wh_inv->{$kp}=$v;
            $wh_inv->{$kn}=$v;
            $wh_inv->{$ki}=$v;
        }
        $$current_inv[$i] = $wh_inv;
    }
    
    return ($numWarehouses);
}

##############################################################################
=pod

=head2 getInventoryData

getInventoryData - &getInventoryData($objid, $wh, $status, $dbh, $site);

=over 4

=item DESCRIPTION

Get the inventory data for the objid, warehouse and status.
Used in search and loading the edit page for editing inventory

=item INPUT

$objid - objid to get the inventory for

$wh - warehouse to get inventory for

$status - status of data to get inventory for

$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$invdata - hash consisting of the row selected from inventory

=item NOTES

=back

=cut
###############################################################################
sub getInventoryData {
    my $objid = shift;
    my $wh = shift;
    my $status = shift;
    my $dbh = shift;
    my $site = shift;
    
    my ($sth, $num, $q, $query_hashref, @sql_results);
    
    my ($invdata, $where_indx);
    
    if ($status =~ /Not Specified/ig) {  # Ignore Parameter if "Not Specified"
    $status = ""; }
    
    if ($status eq '') {
	$query_hashref = {};
	$where_indx = 0;
	$query_hashref->{'TABLE'}->{'NAME'} = "inventory";
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
	if ($wh ne "") {
	    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("wh", $wh, "and");
	}
	$query_hashref->{'statement'} = "SELECT";
	$q = &build_sql($query_hashref, $site, $dbh);
    }
    else {
	$query_hashref = {};
	$where_indx = 0;
	$query_hashref->{'TABLE'}->{'NAME'} = "inventory";
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("status", "%status%", "and");
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
	if ($wh ne "") {
	    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("wh", $wh, "and");
	}
	$query_hashref->{'statement'} = "SELECT";
	$q = &build_sql($query_hashref, $site, $dbh);
    }
    
    $num = &query_getAll($q, $site, $dbh, \@sql_results);
    if ($num > 0) {
	$invdata = $sql_results[0];
    }
    else {
	return;
    }

    $invdata->{'rows'} = $num;
    return($invdata);
}

##############################################################################
=pod

=head2 submitInventory

submitInventory - &submitInventory($objid, \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Submit all inventory that are entered and validated in the
%FORM for $objid.  Determines whether an update or insert
is required, and sets error messages if necessary

=item INPUT

$objid - objid to submit inventory for

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

%FORM error message set, %FORM mode set to editsku

=item NOTES

=back

=cut
###############################################################################
sub submitInventory {
    my $objid = shift;
    my $FORM = shift;
    my $NVP = shift;
    my $site = shift;
    my $dbh = shift;
    
    my ($inv_indx, @form_inv, $invkey, $arrIndx, $h, $check, $x);
    my $rv = "";
    
    # load all of the prices that may be modified
    $inv_indx = &loadFormInventory(\@form_inv, $FORM, $site);
    &debug("There are $inv_indx->{'numInv'} rows to deal with!",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    foreach $invkey (keys %{$inv_indx}) {
        
        # Get the array index for the objid
        $arrIndx = $inv_indx->{$invkey};
        
        # Get the objid hash
        $h = $form_inv[$arrIndx];
        next if ($invkey =~ /numInv/);
        next if (length($invkey) == 0);
        
        $h->{'key'} = $invkey;
        if (($h->{'lstock_pre'} ne '') || ($h->{'adjust_inp'} ne '')) {
            $check = &validate_row($h,$FORM, $NVP, $site, $dbh);
            foreach $x (keys %{$h}) {
                &debug("\$h->{$x} = $h->{$x}",3,$site->{'debug_level'}, $site->{'debug_log'});
            }
            $form_inv[$arrIndx] = $h;
            
            if (($check->{'changestowrite'} eq "yes") && ($check->{'validationcheck'} eq "ok")) {
                &debug("OBJID is $h->{'objid'}!",3,$site->{'debug_level'}, $site->{'debug_log'});
                &updateInventory($h, $objid, $dbh, $site);
            }
        }
    }
    
    $$FORM{'adjust_inp'} = '';
    $$FORM{'mode'} = "editsku";
}

##############################################################################
=pod

=head2 loadFormInventory

loadFormInventory - &loadFormInventory(\@inv, \%FORM, $site);

=over 4

=item DESCRIPTION

Sorts the %FORM hash into inventory rows, which have objid and
all values to insert into the tables from the FORM

=item INPUT

$inv - pointer to an array which will be populated with
       the sorted %FORM price data

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$objid_indx - hash of keys which are objids with return the
              index into the price array where the row exists

$inv - array of prices, each index being one unique row
              of data.

=item NOTES

=back

=cut
###############################################################################
sub loadFormInventory {
    my ($inv) = shift;
    my ($FORM) = shift;
    my ($site) = shift;
    
    my ($key, $row, $objid, $field, $numInv,$objid_indx, $varIndx, $arrIndx);
    
    my ($varheader_template, $manual_header);
    
    &debug("In loadFormInventory!",3,$site->{'debug_level'}, $site->{'debug_log'});
    $numInv = 0;
    $arrIndx = 0;
    foreach $key (keys %{$FORM}) {
        if ($key =~ /row/) {
            ($row, $objid, $field) = split(/\./,$key);
            if (length($objid) == 0) {
                
                &debug("loadFormInv: Got a blank objid for $key!",1,$site->{'debug_level'}, $site->{'debug_log'});
                
                
            }
            else {
                #            if (!($objids =~ /$objid/)) {
                $numInv++;
                
                # Save the index into the array to know what index to put all of the data in
                if (!defined $objid_indx->{$objid}) {
                    $objid_indx->{$objid} = $arrIndx;
                    &debug("Saving the index $arrIndx in \$objid_indx->{$objid}!",3,$site->{'debug_level'}, $site->{'debug_log'});
                    
                    $arrIndx++;
                    #               }
                }
                
                # get the index for the objid
                $varIndx = $objid_indx->{$objid};
                
                # set the field
                &debug("Trying to set \$\$inv[$varIndx]->{$field} = \$\$FORM{$key}, $$inv[$varIndx]->{$field} = $$FORM{$key}",3,$site->{'debug_level'}, $site->{'debug_log'});
                
                $$inv[$varIndx]->{$field} = $$FORM{$key};
            }
        }
    }
    
    $objid_indx->{'numInv'} = $arrIndx-1;
    return $objid_indx;
}

##############################################################################
=pod

=head2 updateInventory

updateInventory - &updateInventory($h, $objid, $dbh, $site);

=over 4

=item DESCRIPTION

To update the inventory record with the values from the hash

=item INPUT


$dbh - current active database handle

$site - A hashref containing information required throughout about the current
        site and session

$h - hash with the appropriate _inp values to update the
     row with

$objid - objid of the inventory row to update

=item OUTPUT

None

=item NOTES

=back

=cut
###############################################################################
sub updateInventory {
    my ($h) = shift;
    my ($objid) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    
    my ($q, $sth, $stu, $num, $flag, $stock, $ki, $now, $numRows, @res, @cur_inv);
    my (@res_array, $v, $l, $query_hashref, $where_indx);
    
    &debug("in updateInventory for $objid",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    foreach $ki (keys %{$h}) {
        &debug("\$h->{$ki} --> $h->{$ki}",3,$site->{'debug_level'}, $site->{'debug_log'});
    }
    
    # Insert
    if ($h->{'lstock_pre'} eq '') {
	$query_hashref = {};
	$where_indx = 0;
	$query_hashref->{'TABLE'}->{'NAME'} = "inventory";
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = $objid;
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'lstock'} = $h->{'adjust_inp'};
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'wh'} = $h->{'wh_ini'};
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = $h->{'status_inp'};
	# Insert the price new price record
	$query_hashref->{'statement'} = "INSERT";
	$q = &build_sql($query_hashref, $site, $dbh);
        $numRows = &query_execute($q, $site, $dbh);
        &debug("$q \n\t--> $numRows rows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
    }
    
    # Update
    else {

	$query_hashref = {};
	$where_indx = 0;
	$query_hashref->{'TABLE'}->{'NAME'} = "inventory";
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'lstock'} = "lstock";
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("wh", $h->{'wh_ini'}, "and");
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
	$query_hashref->{'statement'} = "SELECT";
	$q = &build_sql($query_hashref, $site, $dbh);

        $numRows = &query_getAll($q, $site, $dbh, \@cur_inv);
        
        $stock = $cur_inv[0]->{'lstock'};
        $stock= $stock + $h->{'adjust_inp'};
        &debug("New stock is $stock!",3,$site->{'debug_level'}, $site->{'debug_log'});

	$query_hashref = {};
	$where_indx = 0;
	$query_hashref->{'TABLE'}->{'NAME'} = "inventory";
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = $h->{'status_inp'};
	$query_hashref->{'TABLE'}->{'FIELDS'}->{'lstock'} = $stock;
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
	$query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("wh", $h->{'wh_ini'}, "and");
	$query_hashref->{'statement'} = "UPDATE";
	$q = &build_sql($query_hashref, $site, $dbh);
        $numRows = &query_execute($q, $site, $dbh);
    }
}

######################### PRICING SUBROUTINES ############################
##############################################################################
=pod

=head2 getExistingPrices

getExistingPrices - &getExistingPrices($objid, $fields, $where, $prices, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

To get all existing prices for $objid that match $where
clause, and store in $prices array.

=item INPUT

$objid - objid to search for prices

$fields - fields to return from search

$where - Array of hashrefs of additional where clauses, see NOTES of db_layer.pm for details

$prices - array of prices that are returned from query, each
          index is a hash with one row from the price table

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$prices - array of prices that are returned from query, each
          index is a hash with  one row from the price table

=item NOTES

=back

=cut
###############################################################################
sub getExistingPrices {
    my ($objid) = shift;
    my ($fields) = shift;
    my ($where) = shift;
    my ($prices) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $sth, $res, @res_arr, $numRows);
    my ($numRes, @ext_arr, $junk, $k, $v, @raw, $kt, $inp, $wh_inv, $key);
    my ($kp, $kn, $ki, $arrIndx, $i, @pulldowns, $select_hash, @flds, $fld, $where_indx);
    my ($hash);
    $select_hash->{'TABLE'}->{'NAME'} = "prices";
    @flds = split (/\,/,$fields);
    foreach $fld (@flds) {
	$select_hash->{'TABLE'}->{'FIELDS'}->{$fld} = $fld;
    }

    $where_indx = 0;
    $select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

    foreach $hash (@$where) {
	&debug("getExistingPrices: Adding $hash->{'field'}, $hash->{'value'}, $hash->{'and_or'} to WHERE", 3, $site->{'debug_level'}, $site->{'debug_log'});
	$select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = $hash;
    }
    $select_hash->{'TABLE'}->{'ORDER'} = "unitpr";
    $select_hash->{'statement'} = "SELECT";
    $q = &build_sql($select_hash, $site, $dbh);
    $numRows = &query_getAll($q, $site, $dbh, \@res_arr);
    
    for ($i = 0; $i < $numRows; $i++) {
        $res = $res_arr[$i];
        # set the pulldowns for the row
        @pulldowns = ('sel_status|2', 'sel_currency|2');
        foreach ( @pulldowns) {
            ( $k,$v) = split /\|/, $_;
            open(FH, "<$site->{'selskeletonpath'}/$k.sel");
            @raw=<FH>;
            chomp @raw;
            close(FH);
            # Make the sel_ files work with the current procedure.
            if ($k =~ /sel_/) {
                $k =~ s/sel_//gms;
                $k = uc($k);
            }
            $kt=$k."_SELECT";
            $k = lc($k);
            $$NVP{$kt}= gen_select( $res->{$k},$v,@raw );
            $res->{$kt} = $$NVP{$kt};
            $inp = $k . "_inp";
        }
        
        
        foreach $key (keys %{$res}) {
            next if ($key =~ /SELECT$/);
            $v = string_exdbi($res->{$key});
            $kp=$key."_pre";
            $kn=$key."_ini";
            $ki=$key."_inp";
            $res->{$key}=$v;
            $res->{$kp}=$v;
            $res->{$kn}=$v;
            $res->{$ki}=$v;
        }
        
        # save the hash of the row
        $$prices[$i] = $res;
    }
    
    return $numRows;
}

##############################################################################
=pod

=head2 submitPrice

submitPrice - &submitPrice($objid, \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Submit all prices that are entered and validated in the
%FORM for $objid.  Determines whether an update or insert
is required, and sets error messages if necessary

=item INPUT

$objid - objid to submit prices for

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$$FORM error message set, %FORM mode set to editprice

=item NOTES

<document information about known bugs, limitations, assumptions, or any notes 
that may help people understand what the subroutine can be used for>

=back

=cut
###############################################################################
sub submitPrice {
    my $objid = shift;
    my $FORM = shift;
    my $NVP = shift;
    my $site = shift;
    my $dbh = shift;
    
    my ($price_indx, $arrIndx, $h, $check, $k, $row, $new, $now, $q, $res, @res_insert);
    my ($int_val, $dec, @form_prices, $pricekey, $query_hashref);
    my (@insert_fields);
    my $rv = "";
    
    # load all of the prices that may be modified
    $price_indx = &loadFormPrices(\@form_prices, $FORM, $site);
    &debug("There are $price_indx->{'numPrices'} rows to deal with!",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    foreach $pricekey (keys %{$price_indx}) {
        next if ($pricekey =~ /numPrices/);
        next if (length($pricekey) == 0);
        
        # Get the array index for the objid
        $arrIndx = $price_indx->{$pricekey};
        
        # Get the objid hash
        $h = $form_prices[$arrIndx];
        $h->{'objid'} = $pricekey;
        $h->{'key'} = $pricekey;

        $check = &validate_row($h, $FORM, $NVP, $site, $dbh);
        
        foreach $k (keys %{$h}) {
            &debug("\$h->{$k} = $h->{$k}",3,$site->{'debug_level'}, $site->{'debug_log'});
        }
        $form_prices[$arrIndx] = $h;
        
        if (($check->{'changestowrite'} eq "yes") && ($check->{'validationcheck'} eq "ok")) {
            
            ($objid, $row) = split /_/,$h->{'objid'};
            $$FORM{'objid'} = $objid;
            
            &debug("OBJID is $h->{'objid'}\t FORM objid is $$FORM{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
            &updatePriceRecord($h, $FORM, $dbh, $site);
        }
    }
    
    
    
    # To remove multiple unitct support for this release UCT
    $$FORM{'unitct_inp'} = '1';
    &debug("Unitct is $$FORM{'unitct_inp'}!",3,$site->{'debug_level'}, $site->{'debug_log'});
    if ($$FORM{'unitpr_inp'} ne '') {
        $new->{'unitct_pre'} = $$FORM{'unitct_pre'};
        $new->{'unitct_inp'} = $$FORM{'unitct_inp'};
        $new->{'unitct_ini'} = $new->{'unitct_inp'};   # put it like this for validation
        $new->{'unitpr_pre'} = $$FORM{'unitpr_pre'};
        $new->{'unitpr_inp'} = $$FORM{'unitpr_inp'};
        $new->{'status_inp'} = $$FORM{'status_inp'};
	$new->{'currency_inp'} = $$FORM{'currency_inp'};
        $new->{'objid'} = $objid;
            
	($int_val, $dec) = split /\./, $new->{'unitpr_inp'};
	
	if (length($dec) == 0) {
	    $new->{'unitpr_inp'} = $new->{'unitpr_inp'} . ".00";
	    
	}
	if (length($dec) == 1) {
	    $new->{'unitpr_inp'} = $new->{'unitpr_inp'} . "0";
	}
	&debug("New price is: $new->{'unitpr_inp'}",3,$site->{'debug_level'}, $site->{'debug_log'});

        $check = &validate_row($new, $FORM, $NVP, $site, $dbh);
        if (($check->{'changestowrite'} eq "yes") && ($check->{'validationcheck'} eq "ok")) {
	    &debug("Need to add a new row!",3,$site->{'debug_level'}, $site->{'debug_log'});
            $now = time();
	    $query_hashref->{'TABLE'}->{'NAME'} = "prices";
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = $$FORM{'status_inp'};
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'unitpr'} = $new->{'unitpr_inp'};
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'unitct'} = $$FORM{'unitct_inp'};
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'currency'} = $$FORM{'currency_inp'};
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = $$FORM{'objid'};
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'autogen'} = "N";
	    $query_hashref->{'TABLE'}->{'FIELDS'}->{'pkflag'} = "N";
	    # Insert the price new price record
	    $query_hashref->{'statement'} = "INSERT";
	    $q = &build_sql($query_hashref, $site, $dbh);
            $res = &query_execute($q, $site, $dbh);
        }

    }
    
    if ($$FORM{'regenerate_inp'} == 1) {
        &debug("Going into generate triggers with $$FORM{'objid'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	&generate_specific($$FORM{'objid'}, $site, $dbh);
        &debug("After generate triggers", 3, $site->{'debug_level'}, $site->{'debug_log'});
    }
    
    $$FORM{'unitpr_inp'} = '';
    $$FORM{'unitct_inp'} = '1';
    $$FORM{'mode'} = "editprice";
}

##############################################################################
=pod

=head2 loadFormPrices

loadFormPrices - &loadFormPrices(\@prices, \%FORM, $site);

=over 4

=item DESCRIPTION

Sorts the %FORM hash into price rows, which have objid and
all values to insert into the tables from the FORM

=item INPUT

$price - pointer to an array which will be populated with
         the sorted %FORM price data

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

$objid_indx - hash of keys which are objids with return the
              index into the price array where the row exists

$prices - array of prices, each index being one unique row
          of data.

=item NOTES

=back

=cut
###############################################################################
sub loadFormPrices {
    my ($prices) = shift;
    my ($FORM) = shift;
    my ($site) = shift;
    
    my ($key, $row, $objid, $field, $numPrices, $arrIndx, $objid_indx, $varIndx, $now);
    
    $numPrices = 0;
    $arrIndx = 0; 
    foreach $key (keys %{$FORM}) {
        if ($key =~ /row/) {
            ($row, $objid, $field) = split(/\./,$key);
            &debug("loadFormPrices: $row. $objid. $field!",3,$site->{'debug_level'}, $site->{'debug_log'});
            if (length($objid) == 0) {
                &debug("loadFormPrices: Got a blank objid for $key",1,$site->{'debug_level'}, $site->{'debug_log'});
            }
            else {
                #            if (!($objids =~ /$objid/)) {
                $numPrices++;
                
                # Save the index into the array to know what index to put all of the data in
                if (!defined $objid_indx->{$objid}) {
                    $objid_indx->{$objid} = $arrIndx;
                    $arrIndx++;
                    #               }
                }
                
                # get the index for the objid
                $varIndx = $objid_indx->{$objid};
                
                # set the field
                $$prices[$varIndx]->{$field} = $$FORM{$key};
            }
        }
    }
    
    $objid_indx->{'numPrices'} = $arrIndx-1;
    return $objid_indx;
}

##############################################################################
=pod

=head2 updatePriceRecord

updatePriceRecord - &updatePriceRecord($h, \%FORM, $dbh, $site);

=over 4

=item DESCRIPTION

To update the price record with the values from the hashref

=item INPUT

$h - hash with the appropriate _inp values to update the
     row with

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

None

=item NOTES

=back

=cut
###############################################################################
sub updatePriceRecord {
    my ($h) = shift;
    my ($FORM) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    
    my ($q, $sth, $stu, $num, $flag, $stock, @res_array, $int_val, $dec);
    my ($now, $numRows, $query_hashref, $where_indx);
    
    &debug("in updatePriceRecord!",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    ($int_val, $dec) = split /\./, $h->{'unitpr_inp'};
    
    if (length($dec) == 0) {
	$h->{'unitpr_inp'} = $h->{'unitpr_inp'} . ".00";
	
    }
    if (length($dec) == 1) {
	$h->{'unitpr_inp'} = $h->{'unitpr_inp'} . "0";
    }
    
    $query_hashref = {};
    $where_indx = 0;
    $query_hashref->{'TABLE'}->{'NAME'} = "prices";
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = $h->{'status_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'currency'} = $h->{'currency_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'unitpr'} = $h->{'unitpr_inp'};
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $$FORM{'objid'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("unitct", $h->{'unitct_ini'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("unitpr", $h->{'unitpr_pre'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("currency", $h->{'currency_pre'}, "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("autogen", "N", "and");
    $query_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("pkflag", "N", "and");
    $query_hashref->{'statement'} = "UPDATE";
    $q = &build_sql($query_hashref, $site, $dbh);
    $numRows = &query_execute($q, $site, $dbh);
}
############################# ROW SUBROUTINES ##############################

##############################################################################
=pod

=head2 loadRows

loadRows - &loadRows($numRows, "prices | inventory", \@vals, @rownames, $l, \%FORM, $site);

=over 4

=item DESCRIPTION

To take in an array of hashes (one entry per row), which has
all of the values that are needed for in each @rownames.  The
__ values in the template are then permuted with the values
from the $vals array

=item INPUT

$numRows - number of rows to permute

$type - type of rows that we are loading - inventory or price

$vals - array of hashes, each row representing 1 row, and
        containing the values of the __ values in the row template

@rownames - list of row names that are to be permuted.  These
            names MUST be the exact same as the names in the
            main template where the rows are to be placed,
             as well as the same name as the actual row
             templates

$l - the loaded template to replace the rows into

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
        site and session

=item OUTPUT

Replaces the __row__ where __row__ is the name of each of the
row templates passed in by @rownames with the permuted values.

=item NOTES

The main template with the __row__ where __row__ is the name
of the row template, must be opened prior to calling this
routine

$type should be removed with a key field in the hash, which
will be used to populate the __objid__ portion of the row,
so that this can be used for any row!

Should be eventually moved to a row.pm library, and should
split up the number of rows to show at one time, with a
Next and Back link

=back

=cut
###############################################################################
sub loadRows {
    my ($numRows) = shift;
    my ($type) = shift;
    my ($vals) =  shift;
    my (@rownames) = shift;
    my ($l) = shift;
    my ($FORM) = shift;
    my ($site) = shift;
    
    
    my ($thisrow, $target, $rowval, $rowtemplate, $errortrap, $i, $ROW, $row_objid);
    
    &debug("In load rows with $numRows, $vals, @rownames",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    ## expand row entries
    #
    while (@rownames) {
        
        $thisrow=shift @rownames;
        $target=$thisrow;   # compute from $rownames[$i]
        $rowval="";
        
        $rowtemplate=loadtemplate( $target, "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin" );
        ## run through each linkage...
        #
        $errortrap="";
        
        for ( $i=0; $i<$numRows; $i++ ) {
            $rowtemplate=loadtemplate( $target, "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_render_lang'}/admin" );
            $ROW = $$vals[$i];
            if ($type eq "price") {
                $row_objid = $$FORM{'objid'} . "_" . $ROW->{'unitct_ini'} . "_" . $i;
                $ROW->{'objid'} = $row_objid;
            }
            else {
                $row_objid = $ROW->{'wh_inp'};
                $ROW->{'objid'} = $row_objid;
            }
            $ROW = &agerow($ROW);
            $ROW = &rowsafehtml( $ROW);
            
            $rowval.=permutebit( $rowtemplate, $ROW);
        }
        ## now substitute back into page template
        #
        $thisrow="__".$thisrow."__";
        $l =~ s/$thisrow/$rowval/gs ;
    }
    return $l;
}

##############################################################################
=pod

=head2 agerow

agerow - $h = &agerow($h);

=over 4

=item DESCRIPTION

Moves _inp to _pre values

=item INPUT

$h - hashref for a row

=item OUTPUT

$h - hashref with the _pre values set

=item NOTES

Could be moved to a library routine, as this is also in  other adminstrator CGI scripts

=back

=cut
###############################################################################
sub agerow {
    my $h = shift;
    my(@temp,$k,$v, $k2);
    
    @temp = keys %{$h};
    while (@temp) {
        $k = shift @temp;
        next if ( $k !~ /_inp$/ );   # only want input values
        $v =  $h->{$k} ;
        $k2 =~ s/_inp$/_pre/ ;
        $h->{$k2}=$v if ( $v );
    }
    return $h;
} #/agerow

##############################################################################
=pod

=head2 rowsafehtml

rowsafehtml - $h = &rowsafehtml($h);

=over 4

=item DESCRIPTION

Makes title and alttitle keys html-safe

=item INPUT

$h - hashref of input values

=item OUTPUT

 $h - hashref with title and alttile converted to safe html

=item NOTES

Could be moved to a library routine, as this is also in  other adminstrator CGI scripts

=back

=cut
###############################################################################
sub rowsafehtml {
    my ($h) = shift;
    my(@temp,$k,$v);
    #   $h = @_[0];
    
    @temp = keys %{$h};
    while (@temp) {
        $k = shift @temp;
        next if ( $k !~ /(title|alttitle)/ );   # only want titles
        $h->{$k} = string_tohtml( $h->{$k} ) ;
    }
    return $h;
} #rowsafehtml

##############################################################################
=pod

=head2 checkformbits

checkformbits - &checkformbits(\%FORM);

=over 4

=item DESCRIPTION

checks population of _inp

=item INPUT

$FORM  - Hashref to the current page that is displayed.

=item OUTPUT

updates $$FORM 

=item NOTES

=back

=cut
###############################################################################
sub checkformbits {
    my ($FORM) = shift;
    my($k,$kt,@keys);
    @keys = keys %{$FORM};
    while (@keys) {
        $k=shift @keys;
        next if ( $k =~ /_ini$/ );   # nothing to do with ini
        if ( $k =~ /_pre$/ ) {
            # check to see if the inp key exists
            # if it doesn't exist, then create and set to _pre
            $kt=$k;
            $kt =~ s/_pre$/_inp/ ;
            $$FORM{$kt}=$$FORM{$k} if ( ! exists($$FORM{$kt}) );
        } # only really concerned with _pre/_inp pairs
    } # done with all keys
    
    ## special case - objclass
    #
    
    if ((! $$FORM{'objclass'} ) and ( $$FORM{'objclass_inp'} )) {
        $$FORM{'objclass'}=$$FORM{'objclass_inp'};
    }
    if (( $$FORM{'objclass'} ) and (! $$FORM{'objclass_inp'} )) {
        $$FORM{'objclass_inp'}=$$FORM{'objclass'};
    } # end of objclass fillin
    return 1;
} #/checkformbits

##############################################################################
=pod

=head2 writetonvp

writetonvp - &writetonvp(\%FORM, \%NVP);

=over 4

=item DESCRIPTION

Write NVP data for FORM

=item INPUT

$FORM  - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

=item OUTPUT

updates $$NVP

=item NOTES

=back

=cut
###############################################################################
sub writetonvp {
    my ($FORM) = shift;
    my ($NVP) = shift;
    
    my(@keys,$k,$kt);
    @keys = keys %{$FORM};
    while (@keys) {
        $k=shift @keys;
        $kt = $k;
        $kt =~ s/(_ini|_inp|_pre|_err)$// ;
        if ( $k =~ /_inp$/ ) {           # _inp -> _pre
            $kt.="_pre";
            $$NVP{$kt}=$$FORM{$k};
            } elsif ( $k =~ /_ini$/ ) {      # _ini -> _ini
            $$NVP{$k}=$$FORM{$k};
            } elsif ( $k =~ /_err$/ ) {      # _err -> _err
            $$NVP{$k}=$$FORM{$k};
            } elsif ( $k eq $kt ) {          # X -> X
            if (( $kt !~ /[a-z]/ ) and ( ! exists($$NVP{$kt}) )) {
                # this looks like a fieldname and is not yet set
                $$NVP{$kt}=$$FORM{$kt} if ( $$FORM{$kt} );
            }
        } # any other inheritance modes?
    } # done running through form keys
    ## special cases
    #
    $$NVP{'objclass'}=$$FORM{'objclass_inp'} if (( ! $$NVP{'objclass'} ) and ( $$FORM{'objclass_inp'} ));
    return 1;
} #/writetonvp


1;
