#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
use strict;

##############################################################################
=pod

=head1 linkage.pm

linkage.pm - subroutines for linkage.cgi

=head1 DESCRIPTION

This library is used for all subroutines that are used exclusively by linkage.cgi.
This module allows removal of data processing from the main cgi script, allowing
it to only be responsible for displaying the information gathered in this module.

=head1 NOTES & BUGS

=cut

##############################################################################

###
#
# linkage.pm
#
#  Routines for processing linkage.cgi
#
#  Version 5 TG 20000918
#     Got rid of ->rows
#  Version 4 TG 20000913
#     Fixed subroutine parameters after retrofitting them to standards
#  Version 3 TG 20000912
#     Replaced loadskutohash with loadObjidToHashref
#  Version 2 TG 20000824
#     Fixed problem with alttitle
#  Version 1 TG 20000823
#     Created

############################# SUBROUTINES ##################################

##############################################################################
=pod

=head2 loadlinkages

loadlinkages - &loadlinkages("form|db",$objid,$site,\%FORM,$dbh);

=over 4

=item DESCRIPTION

To load object by object,  linkage data from form or database.

=item INPUT

$source - where to get the data from, db or form

$objid - objid of the object to get the data for (obj1 in linkage table)

$site - A hashref containing information required throughout about the current
site and session

$FORM  - Hashref to the current page that is displayed.

$dbh - current active database handle

=item OUTPUT

@links - Sorted array of hashrefs.  Each hash is for one objid, and are sorted on column width and weight

=item NOTES

Sorting is commented out for this release, will be added in the future

=back

=cut
###############################################################################
sub loadlinkages {
    my($source) = shift;
    my ($objid ) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($dbh) = shift;
    
    my(%tfld,$sti,$n,$m,$q,$sku,@linkah,$h,$k,$k2,$k3, $where_indx);
    my ($i, %rowids, @formkeys, $s, $rid, $rn, @temp, $key, $select_hash, @sql_results);
    # initialize hasref
    @linkah = ();      # array of hashrefs
    $h = {};           # temp for each linkage
    
    &debug("in loadlinkages with $source, $objid",3,$site->{'debug_level'}, $site->{'debug_log'});
    if ( $source eq "db" ) {
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "linkage";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'obj2'} = "obj2";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'status'} = "status";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'linkwt_man'} = "linkwt_man";
	$where_indx = 0;
        $select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("obj1", $objid, "and");
        $select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("status", "15000-A", "and");
	$select_hash->{'statement'} = "SELECT";
        $q = &build_sql($select_hash, $site, $dbh);
        
	$n = &query_getAll($q, $site, $dbh, \@sql_results);
	
        &debug("$q\n\t-->$n affected",2,$site->{'debug_level'}, $site->{'debug_log'});
        
        ## go through the returned rows
        #
        for ( $i=0; $i<$n; $i++ ) {
	    $h = &loadlinktohash( $sql_results[$i]->{'objid'}, $site, $dbh);
            &debug_hashref($h,3,$site->{'debug_level'}, $site->{'debug_log'});
            push @linkah, $h;
            
        }
        
        ## create the rows for new entries
        #
        for ( $i=1; $i<= $site->{'blankrows'}; $i++ ) {
            push @linkah, &createnewrow($i);
        }
        
        ## fill in lost _inp elements
        #
        for ( $i=0; $i<=$#linkah; $i++ ) {
            $linkah[$i] = &filllostinp( $linkah[$i] );
        } # done building linkah form-ish fields
        
        } else {  # SOURCE if FORM
        
        ## assume load from form
        #
        %rowids = ();
        @formkeys = sort keys %{$FORM};
        ## now run through formkeys and construct $linkhh
        #
        while ( @formkeys ) {
            $s = shift @formkeys;
            next if ( $s !~ /^row/ );      # skip over the non-repetitive elements
            ## extract the rowid
            #
            $rid = (split /\./, $s )[1];
            ## check to see if the rowid has already been created
            #     if not, push a new hashref onto @linkah and record the
            #     mapping of rowid onto array element number
            
            if ( defined($rowids{$rid}) ) {
                ## get the rowid for hash referencing
                #
                $rn=$rowids{$rid};
                
                } else {
                ##
                #
                push @linkah, {};
                $rowids{$rid} = $#linkah;
                $rn=$#linkah;
                $linkah[$rn]->{'objid'}=$rid;
                
            }
            ## now have the row number in $rn
            #     compute the field-key name from the form key
            
            @temp = split /\./, $s;
            $k = join ".", @temp[2..$#temp] ;
            
            ## write the hash entry
            
            $linkah[$rn]->{$k} = $$FORM{$s};
            
        } #/formkeys
        
        #
        ## linkah now constructed, perform data validation elsewhere
        #     each linkage is now a hashref of it's own
        #
        
        for ( $i=0; $i<=$#linkah; $i++ ) {
            
            ## check for lost inputs
            #
            $linkah[$i] = &filllostinp( $linkah[$i] );
            $linkah[$i]->{'cellwidth_ini'} = ( split /_/,
            $linkah[$i]->{'cellstyle_inp'} )[1];
            
        } # done checking for lost inputs
        
    } #/source variations
    
    # return the hashref array...
    return @linkah;
} #/loadlinkages


##############################################################################
=pod

=head2 populate_nvp

populate_nvp - &populate_nvp($site, \%NVP, \%FORM);

=over 4

=item DESCRIPTION

Populate the NVP hash from %FORM elements

=item INPUT

$site - A hashref containing information required throughout about the current
site and session

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$FORM  - Hashref to the current page that is displayed.

=item OUTPUT

The FORM hash updated with anything that was set in the NVP hash

=item NOTES

=back

=cut
###############################################################################
sub populate_nvp {
    my ($site) = shift;
    my ($NVP) = shift;
    my ($FORM) = shift;
    
    my(@keys,$k);
    
    @keys = sort keys %{$FORM};
    ## separate repetitive elements from non-repetitive
    #
    while (@keys) {
        $k = shift @keys;
        next if ( $k =~ /^row/ );       # skip repetitive bits
        $$NVP{$k}=$$FORM{$k} if ( ! defined($$NVP{$k}) );
    } #/while running through keys
    
    ## stuff from localization specific data
    #
    #$$NVP{'script'}="$site->{'scriptpath'}/login.cgi?mode=main";
    $$NVP{'script'}="$site->{'scriptpath'}/$site->{'scriptname'}";
    $$NVP{'scriptname'}=$site->{'scriptname'};
    $$NVP{'scriptpath'}=$site->{'scriptpath'};
    $$NVP{'sitename'}=$site->{'sitename'};
    
    return 1;
} # /populate_nvp


##############################################################################
=pod

=head2 createnewrow

createnewrow - &createnewrow($n);

=over 4

=item DESCRIPTION

Create and initialize a new row

=item INPUT

$n - the number of rows to initialize and create

=item OUTPUT

$h - returns a hashref to a blank newrow of number n

=item NOTES

=back

=cut
###############################################################################
sub createnewrow {
    my ($n) = shift;
    my($h);
    my ($k, $k2, @temp);
    $h = {};
    $h->{'obj2'}="";
    $h->{'title'}="";
    $h->{'linkwt_use'}="";
    $h->{'linkwt_man'}="";
    $h->{'cellstyle'}="";
    $h->{'celldepth'}="";
    $h->{'status'}="";
    @temp=keys %{$h};
    while (@temp) {
        $k = shift @temp;
        $k2 = $k."_ini";
        $h->{$k2}="";
        $k2 = $k."_inp";
        $h->{$k2}="";
    }
    
    $h->{'objid'}="newrow$n";
    return $h;
} #/createnewrow


##############################################################################
=pod

=head2 filllostinp

filllostinp - &filllostinp($h);

=over 4

=item DESCRIPTION

Fill in lost _inp fields

=item INPUT

$h - hashref for a row of links

=item OUTPUT

$h - hashref for the row of links, setting the _inp to the _pre values if they
were not inputted by the user via the FORM

=item NOTES

=back

=cut
###############################################################################
sub filllostinp {
    my ($h) = shift;
    my(@temp,$k,$k2);
    
    @temp = keys %{$h};
    foreach $k ( @temp ) {
        next if ( $k !~ /_pre$/ );
        $k2 = $k;
        $k2 =~ s/_pre$/_inp/ ;
        if (( ! exists( $h->{$k2} ) ) and ( $h->{$k} )) {
            $h->{$k2}=$h->{$k};
        }
    }
    return $h;
} #/filllostinp

##############################################################################
=pod

=head2 writelink

writelink - &writelink($h, $site, \%FORM, $dbh);

=over 4

=item DESCRIPTION

To update exisiting modified rows, insert new rows, and  remove any rows requested
by the user via the FORM

=item INPUT

$h - hashref of one row to write to the table

$site - A hashref containing information required throughout about the current
site and session

$FORM  - Hashref to the current page that is displayed.

$dbh - current active database handle

=item OUTPUT

$h - new hashref of the updated record

=item NOTES

=back

=cut
###############################################################################
sub writelink {
    my ($h) = shift;
    my ($site) = shift;
    my ($FORM) = shift;
    my ($dbh) = shift;
    
    my ($q, $stw, $numw, @temp, $k, @res, $fields_linkage, $t, $fn, $k2, $numRows);
    my ($mdate, $cdate, $select, $select_hash, $query_hashref, $update_hashref);
    my ($where_indx, $worked);
    &debug("In write link",3,$site->{'debug_level'}, $site->{'debug_log'});
    $h->{'obj1_inp'}=$$FORM{'obj1'};
    $h->{'obj1_ini'}="";
    $h = &inheritValues($h, $site, $dbh);
    
    $t = time();
    
    $fields_linkage="obj1,obj2,objclass,title,linkwt_man,";
    $fields_linkage.="manweight,cellstyle,celldepth,";
    $fields_linkage.="objclass,status,cdate,mdate";
    
    $q="";
    @temp = sort keys %{$h};
    while (@temp) {
        
        $k = shift @temp;
        
        next if ( $k !~ /_inp$/ );  # only care about entered fields
        next if ( $k =~ /^KILL/ );  # only care about entered fields
        next if ( $k =~ /^objid/ ); # NEVER update the objid
        
        $fn = $k;
        $fn =~ s/_inp$// ;         # fieldname root
        $k2 = $fn."_ini";          # initial value
        
        next if ( $fields_linkage !~ /$fn/ );  # a real field?
        
        # now compare and construct the insert clause
        if ( exists($h->{$k2}) and ( $h->{$k2} ne $h->{$k} )) {
            # meaningful comparison
            #            $q.="," if ( $q );
            $update_hashref->{'TABLE'}->{'FIELDS'}->{$fn} = string_todbi($h->{$k});
            
            #            $q.="$fn='".string_todbi($h->{$k})."'";
        } #/done evaluating if this is a change or not
    } # end of running through fields
    
    ## finish the query
    $update_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15000-A";
    $update_hashref->{'TABLE'}->{'FIELDS'}->{'linkwt_man'} = $h->{'linkwt_man_inp'};
    
    if ($h->{'objid'} =~ /newrow/) {
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "linkage";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
	$where_indx = 0;
        $select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("obj1", $h->{'obj1_inp'}, "and");
        $select_hash->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("obj2", $h->{'obj2_inp'}, "and");

	$select_hash->{'statement'} = "SELECT";
        $q = &build_sql($select_hash, $site, $dbh);
#        $numRows = &query_getAll($select, $site, $dbh, \@res);
        $numRows = &query_getAll($q, $site, $dbh, \@res);
        if ($numRows == 1) {
            $h->{'objid'} = $res[0]->{'objid'};
        }
    }

    $numRows = 0;  # set it to 0 to check if update worked

    # If the older version existed, not activate it, and it is a row that
    # is currently active, then update it
    if ($h->{'objid'} !~ /newrow/) {
        $update_hashref->{'TABLE'}->{'NAME'} = "linkage";
	$where_indx = 0;
        $update_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $h->{'objid'}, "and");

	$update_hashref->{'statement'} = "UPDATE";
        $q = &build_sql($update_hashref, $site, $dbh);
        $worked  = &query_execute($q, $site, $dbh);
        &debug("$q\n\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
    }
    
    # There was a new row entered that has never been inserted before, so
    # insert it
    if (!$worked) {
        $h = &insertLink($h, $site, $dbh);
    }
    
    if ($h->{'alttitle_changed'} eq "yes") {
        # assume update and if fails, then insert it
        $h->{'alttitle_inp'} = string_todbi($h->{'alttitle_inp'});
        &debug("Altittle changed to $h->{'alttitle_inp'}",3,$site->{'debug_level'}, $site->{'debug_log'});

        $mdate = time();
        $query_hashref = {};
        $query_hashref->{'TABLE'}->{'NAME'} = "extraprop";
        $query_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $h->{'alttitle_inp'};
        $query_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $h->{'objid'}, "and");
        $query_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("fieldname", "alttitle", "and");
        $query_hashref->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("language", $site->{'om_render_lang'}, "and");
        $query_hashref->{'TABLE'}->{'WHERE'}[3] = &build_where_hash("status", "15000-A", "and");
# upsed to be update_hashref
	$query_hashref->{'statement'} = "UPDATE";
#        $q = &build_sql($update_hashref, $site, $dbh);
        $q = &build_sql($query_hashref, $site, $dbh);
        $worked = &query_execute($q, $site, $dbh);
        &debug("$q\n\t$numRows-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
        
        if (!$worked) {
            $cdate = $mdate = time();
            
            $query_hashref = {};
            $query_hashref->{'TABLE'}->{'NAME'} = "extraprop";
            $query_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = $h->{'objid'};
            $query_hashref->{'TABLE'}->{'FIELDS'}->{'fieldname'} = "alttitle";
            $query_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = $h->{'alttitle_inp'};
            $query_hashref->{'TABLE'}->{'FIELDS'}->{'language'} = $site->{'om_render_lang'};
            $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15000-A";
	    $query_hashref->{'statement'} = "INSERT";
	    $q = &build_sql($query_hashref, $site, $dbh);
            $numRows = &query_execute($q, $site, $dbh);
            
            &debug("$q\n\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
            
        }
    }
    
    $h=&loadlinktohash( $h->{'objid'},$site,$dbh);
    &debug("Returning from writelink",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $h;
} #/writelink

##############################################################################
=pod

=head2 insertLink

insertLink - &insertLink($h, $site, $dbh);

=over 4

=item DESCRIPTION

To insert a new row into the linkage table.  It is known at time of calling
this routine that an insert is necessary

=item INPUT

$h - hashref of the new row to insert into the table

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

=item OUTPUT

$h - hashref of the newly updated record after insertion

=item NOTES

=back

=cut
###############################################################################
sub insertLink {
    my ($h) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($dvalue, $p, $t, $stp, $e, $ste, $num, $event, $series, $objid, $q, $sti, $x);
    my ($cdate, $mdate, $select_hash, $query_hashref, $numRows, @res, @sql_results);
    
    # added status to the query
    $query_hashref = {};
    $query_hashref->{'TABLE'}->{'NAME'} = "linkage";
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'obj1'} = $h->{'obj1_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'obj2'} = $h->{'obj2_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'linkwt_man'} = $h->{'linkwt_man_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'cellstyle'} = $h->{'cellstyle_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'celldepth'} = $h->{'celldepth_inp'};
    $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15000-A";
    $query_hashref->{'append_objid'} = "true";
    $query_hashref->{'statement'} = "INSERT";
    $q = &build_sql($query_hashref, $site, $dbh);
    ($q, $objid) = split /\:\:/, $q;
    $num  = &query_execute($q, $site, $dbh);
    $h->{'objid'} = $objid;
    &debug("insertLink:  Added objid $h->{'objid'}",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $h;
}

##############################################################################
=pod

=head2 inheritValues

inheritValues - &inheritValues($h, $site, $dbh);

=over 4

=item DESCRIPTION

Check for passable values of linkwt_use from the things tabel

=item INPUT

$h - hashref of a linkage row

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

=item OUTPUT

$h - hashref of the updated linkage row, with the weight_man from things set in
the linkwt_man field if the linkwt_man was not set.

=item NOTES

This may need to inherit the cellstyle as well,  although it does not do this!

=back

=cut
###############################################################################
sub inheritValues {
    my ($h) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    
    my ($q, $sti, $num, $select_hash, @sql_results);
    
    if ( ! $h->{'linkwt_man_inp'} ) {
        
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "things";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'weight_man'} = "weight_man";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $h->{'obj2_inp'}, "and");
	$select_hash->{'statement'} = "SELECT";
	$q = &build_sql($select_hash, $site, $dbh);
        $num = &query ($q, $dbh, \@sql_results);
        &debug("$q\n\t-->$num affected",2,$site->{'debug_level'}, $site->{'debug_log'});
        
	if ($num == 1) {
            $h->{'linkwt_man_inp'} = $sql_results[0]->{'weight_man'};
            $h->{'linkwt_man_ini'}="" if ( ! $h->{'linkwt_man_ini'} );
        } # else wrong number of rows...
        
    } # end of evaluation of value of LNKWEIGHT
    
    return $h;
}

##############################################################################
=pod

=head2 loadlinktohash

loadlinktohash - &loadlinktohash($objid, $site, $dbh);

=over 4

=item DESCRIPTION

Load a link to the hash for a specified objid from the tables

=item INPUT

$objid - objid of the linkage record to load to a hash

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

=item OUTPUT

$h - hashref of the rows data from the tables

=item NOTES

=back

=cut
###############################################################################
sub loadlinktohash {
    my ($objid)=shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my($h,$q, $kp, $alt, $res, @temp, $k, $v, $kn, $ki, $select_hash);
    my ($numRows, @sql_results, $product_select);

    &debug("IN loadlinktohash with $objid",3,$site->{'debug_level'}, $site->{'debug_log'});
    $h={};
    
    if ( $objid ) {
        #  Changed status check
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "linkage";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
        $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $objid, "and");
        $select_hash->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("status", "15000-A", "and");
	$select_hash->{'statement'} = "SELECT";
	$q = &build_sql($select_hash, $site, $dbh);
	$numRows = &query_getAll($q, $site, $dbh, \@sql_results);
        &debug("$q\n\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});	

        if ( $numRows == 1 ) {
            $h = $sql_results[0];
            $h->{'objid'}=$objid;
            $select_hash = {};
            $select_hash->{'TABLE'}->{'NAME'} = "extraprop";
            $select_hash->{'TABLE'}->{'FIELDS'}->{'body'} = "body";
	    $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $h->{'objid'}, "and");
	    $select_hash->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("fieldname", "alttitle", "and");
	    $select_hash->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("language", $site->{'om_render_lang'}, "and");
	    $select_hash->{'TABLE'}->{'WHERE'}[3] = &build_where_hash("status", "15000-A", "and");
	    $select_hash->{'statement'} = "SELECT";
	    $q = &build_sql($select_hash, $site, $dbh);
	    $numRows = &query_getAll($q, $site, $dbh, \@sql_results);
	    &debug("$q\n\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});	
            if ($numRows == 1) {  
                $h->{'alttitle_pre'} = $sql_results[0]->{'body'};
            }

            ## now load the default title,weight...
            # Start variant support
            # Use get_fields_and_title from the variant.pm library, as variants require a lot of work to get the title
	    $product_select->{'FIELDS'}->{'weight_man'} = "weight_man";
            $res = &get_fields_and_title($product_select,$h->{'obj2'},$dbh, undef, $site);
            if ($res) {
                $h->{'title'} = $res->{'title'};
                $h->{'weight_man'} = $res->{'weight_man'};
            } # end of variant support
            
            &debug("Set title: $res->{'title'} AND weight: $res->{'weight_man'} for linkages!",3,$site->{'debug_level'}, $site->{'debug_log'});
            @temp = keys %{$h};
            while ( @temp ) {
                $k = shift @temp;
                $v = string_exdbi($h->{$k});
                $kp=$k."_pre";
                $kn=$k."_ini";
                $ki=$k."_inp";
                $h->{$k}=$v;
                $h->{$kp}=$v;
                $h->{$kn}=$v;
                $h->{$ki}=$v;
            } # done permuting fieldnames
        } # done loading if one record
    } # done acting on good rowid
    
    &debug("Returing from loadlinktohash",3,$site->{'debug_level'}, $site->{'debug_log'});
    return $h;
} #/loadlinktohash

##############################################################################
=pod

=head2 agelink

agelink - &agelink($h);

=over 4

=item DESCRIPTION

Moves _inp to _pre values

=item INPUT

$h - hashref for a row

=item OUTPUT

$h - hashref with the _pre values set

=item NOTES

Could be moved to a library routine, as this is also in other adminstrator CGI scripts

=back

=cut
###############################################################################
sub agelink {
    my ($h) = shift;
    my(@temp,$k,$v, $k2);
    
    #    @temp = sort keys %{$h};
    #    @temp = keys %{$h};
    foreach $k (keys %{$h}) {
        next if ( $k !~ /_inp/ );   # only want input values
        $v =  $h->{$k} ;
        $k2 = $k;
        $k2 =~ s/_inp/_pre/ ;
        if ( $v ) {
            $h->{$k2}=$v
        }
    }
    return $h;
} #/agelink

##############################################################################
=pod

=head2 linksafehtml

linksafehtml - &linksafehtml();

=over 4

=item DESCRIPTION

Makes title and alttitle keys html-safe, catching key perl variables, such as ', $ ...

=item INPUT

$h - hashref to a linkage row

=item OUTPUT

$h - hashref to the linkage row with safe title and alttitle

=item NOTES

=back

=cut
###############################################################################
sub linksafehtml {
    my ($h) = shift;
    my(@temp,$k,$v);
    
    @temp = sort keys %{$h};
    @temp = keys %{$h};
    while (@temp) {
        $k = shift @temp;
        next if ( $k !~ /(title|alttitle)/ );   # only want titles
        $h->{$k} = string_tohtml( $h->{$k} ) ;
    }
    return $h;
} #/linksafehtml

##############################################################################
=pod

=head2 loadObjidToNVP

loadObjidToNVP - &loadObjidToNVP($sku, $dbh, $site, \%NVP);

=over 4

=item DESCRIPTION

Loads objects data to NVP

=item INPUT

$sku - objid ID to load to NVP hash

$dbh - current active database handle

$site - site hashref 

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

=item OUTPUT

Updated NVP with the row's data loaded in

=item NOTES

Never returns 0, so return of 1 is not really necessary!

=back

=cut
###############################################################################
sub loadObjidToNVP {
    my $sku=shift;
    my $dbh = shift;
    my $site = shift;
    my $NVP = shift;
    
    my($h,$k,@temp,$k2);
    
    
    ## load parent objid's data from db
    #
    $h = {};
    $h=loadObjidToHashref($sku, $site, $dbh);
    
    @temp = keys %{$h};
    while (@temp) {
        $k = shift @temp;
        $$NVP{$k} = $h->{$k};
        $k2=$k."_pre";
        $$NVP{$k2} = $h->{$k};
        $k2=$k."_ini";
        $$NVP{$k2} = $h->{$k};
    }
    ## special cases
    #
    $$NVP{'lengthdescr'}=length($$NVP{'descript'});
    $$NVP{'lengthdescr'}.=" Characters";
    
    return 1;
} #/loadObjidToNVP


##############################################################################
=pod

=head2 getPageStyleText

getPageStyleText - &getPageStyleText($SELECTED_VAL, $site);

=over 4

=item DESCRIPTION

To get the text representation for the value of pagestyle that is retreived from the database

=item INPUT

$db_pagestyle - the db value stored for the pagestyle

$site - A hashref containing information required throughout about the current
site and session

=item OUTPUT

$val - the text representation for db_pagestyle, or $script_msg{NO_PAGESTYLE} if no pagestyle defined

=item NOTES

=back

=cut
###############################################################################
sub getPageStyleText {
    my ($db_pagestyle) = shift;
    my ($site) = shift;
    
    my (@raw, $row, $code, $val);
    
    &debug("IN getPageStyleText with $db_pagestyle, looking for sel_pagestyle.sel in $site->{'selskeletonpath'}",3,$site->{'debug_level'}, $site->{'debug_log'});
    $db_pagestyle = string_exdbi($db_pagestyle);
    open(PSFH, "<$site->{'selskeletonpath'}/sel_pagestyle.sel");
    @raw=<PSFH>;
    chomp @raw;
    close(PSFH);
    
    foreach $row (@raw) {
        ($code, $val) = split /\|/, $row;
        &debug("code is $code, val is $val, pagestyle is $db_pagestyle",3,$site->{'debug_level'}, $site->{'debug_log'});
        if ($code eq $db_pagestyle) {
            return $val
        }
    }
    
    return ($site->{'script_msg'}{NO_PAGESTYLE});
}

##############################################################################
=pod

=head2 modifyLinkages

modifyLinkages - &modifyLinkages (\%FORM, $site, $dbh, @links);

=over 4

=item DESCRIPTION

To validate user data, update or insert links if they are valid, or return
error messages if they are not valid.

=item INPUT

$FORM  - Hashref to the current page that is displayed.

$site - A hashref containing information required throughout about the current
site and session

$dbh - current active database handle

@links - array of hashrefs, each row consisting of a new link, an empty new row
(nothing entered by the user), or a modified link

=item OUTPUT

@links - Each modified link, new rows will have an 'objid' key and any row
with an error in it will have an error message in the _err keys.

=item NOTES

Be sure to leave the @links parameter as the last parameter to ensure retreival
of the entire links array.

=back

=cut
###############################################################################
sub modifyLinkages {
    my ($FORM) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my (@links) = @_;
    
    my ($i, $x, $q, $ip, $ix, $checkVarObjid, $vcodeq, $numVrows, @res);
    my ($k2, $h2, $numRows, $select_hash, $query_hashref);
    
    for ( $i=0; $i<=$#links; $i++ ) {
        # next if ( ! $links[$i]->{'KILL_inp'} );
        
        # there is a kill entry!
        
        if ( $links[$i]->{'objid'} =~ /^newrow/ && $links[$i]->{'KILL_inp'} ) {
            # just reinitialize
            $x=$links[$i]->{'objid'};
            
            $x =~ tr/A-Za-znewrow//d ;      # strip alphas
            $links[$i]=&createnewrow($x);
            
        }
        elsif ($links[$i]->{'objid'} !~ /^newrow/) {
            # existing sku, kill the record and the list entry
            #
            if ($links[$i]->{'KILL_inp'}) {
		&debug("Killing link: row is $links[$i]->{'objid'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
                $query_hashref = {};
                $query_hashref->{'TABLE'}->{'NAME'} = "linkage";
                $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15001-A";
#		$query_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("obj1'", $$FORM{'obj1'}, "and");
#		$query_hashref->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("obj2", $$FORM{'obj2_ini'}, "and"); 
		$query_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $links[$i]->{'objid'}, "and");
		$query_hashref->{'statement'} = "UPDATE";
		$q = &build_sql($query_hashref, $site, $dbh);
		&query_execute($q, $site, $dbh);
                
                &debug("$q",2,$site->{'debug_level'}, $site->{'debug_log'});
                $query_hashref = {};
                $query_hashref->{'TABLE'}->{'NAME'} = "extraprop";
                $query_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "15001-A";
		$query_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $links[$i]->{'objid'}, "and");
		$query_hashref->{'statement'} = "UPDATE";
		$q = &build_sql($query_hashref, $site, $dbh);
		&query_execute($q, $site, $dbh);
                &debug("$q",2,$site->{'debug_level'}, $site->{'debug_log'});
                
                ## kill it from the list
                #
                
                $ip=$i-1;
                $ix=$i+1;
                if ($ip > -1 && $#links >= $ix) {
                    @links=( @links[0..$ip],@links[$ix..$#links] );
                    } elsif ($ip < 0 and $#links >= 1) {
                    @links=@links[1..$#links];
                    } elsif ($ix > $#links && $ip >= 0) {
                    @links=@links[0..$ip];
                }
                #/ checking how to remake @links
                
                $i--;         # decrement one so auto-increment net zero
                
            }
        } # end of choosing how to kill the entry
    } # done killing links
    
    ## validate submitted data, write changes to tables
    #
    for ( $i=0; $i<=$#links; $i++ ) {
        
        if ($links[$i]->{'obj2_ini'}) { # existing links
            $checkVarObjid = $links[$i]->{'obj2_ini'};
        }
        else {   # new links
            $checkVarObjid = $links[$i]->{'obj2_inp'};
        }
        $select_hash = {};
        $select_hash->{'TABLE'}->{'NAME'} = "things";
        $select_hash->{'TABLE'}->{'FIELDS'}->{'vcode'} = "vcode";
	$select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $checkVarObjid, "and");
	$select_hash->{'statement'} = "SELECT";
	$vcodeq = &build_sql($select_hash, $site, $dbh);
        $numVrows = &query_getAll($vcodeq, $site, $dbh, \@res);
        if ($res[0]->{'vcode'} ne "") {
            $links[$i]->{'vcode'} = $res[0]->{'vcode'};
        }
        
        $links[$i] = validatelink( $links[$i], $FORM, $site, $dbh );
        &debug("$vcodeq\numVRows\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
        &debug("After validate link with \$links[$i]->{'changestowrite'} = $links[$i]->{'changestowrite'} and \$links[$i]->{'validationcheck'} = $links[$i]->{'validationcheck'}",3,$site->{'debug_level'}, $site->{'debug_log'});
        
        # Do not want to enter a row with a blank obj2 and valid other data!
        # We must check the _inp & _ini to handle both new and old links
        if ( ($links[$i]->{'obj2_inp'} eq '') && ($links[$i]->{'obj2_ini'} eq '') &&
            ($links[$i]->{'changestowrite'} eq "yes") &&
            ($links[$i]->{'validationcheck'} eq "ok")) {
            $links[$i]->{'changestowrite'} = "no";
            $links[$i]->{'validationcheck'} = "";
            $links[$i]->{'previous_error'} = "yes";
            $links[$i]->{'obj2_err'} = $site->{'script_msg'}{NO_OBJ};
        }
        
        # Check to see if it is already linked
        if ($links[$i]->{'objid'} =~ /newrow/) {
            $h2 = $links[$i];
            &debug_hashref($h2,3,$site->{'debug_level'}, $site->{'debug_log'});
            $select_hash = {};
            $select_hash->{'TABLE'}->{'NAME'} = "linkage";
            $select_hash->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
	    $select_hash->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("obj2", $links[$i]->{'obj2_inp'}, "and");
	    $select_hash->{'TABLE'}->{'WHERE'}[1] = &build_where_hash("obj1", $$FORM{'obj1_pre'}, "and");
	    $select_hash->{'TABLE'}->{'WHERE'}[2] = &build_where_hash("status", "15000-A", "and");
	    $select_hash->{'statement'} = "SELECT";
	    $q = &build_sql($select_hash, $site, $dbh);
	    $numRows = &query_getAll($q, $site, $dbh, \@res);
	    &debug("$q\n\t-->$numRows affected",2,$site->{'debug_level'}, $site->{'debug_log'});
	    if ($numRows > 0) {
		$links[$i]->{'validationcheck'} = "";
		$links[$i]->{'changestowrite'} = "no";
		# Error - Object $links[$i]->{'obj2_inp'} is already linked
		$links[$i]->{'obj2_err'} = "$site->{'script_msg'}{ERROR} $site->{'script_msg'}{OBJECT} $links[$i]->{'obj2_inp'} $site->{'script_msg'}{ALREADY_LINKED}";
	    }
	}
	
	# force update of alternate title
	if ( ( $links[$i]->{'validationcheck'} eq "ok" )
	     && ( $links[$i]->{'alttitle_changed'} eq "yes" ) ) {
	    $links[$i]->{'changestowrite'} =  "yes";
	}
	
	if ( ( $links[$i]->{'validationcheck'} eq "ok" )
	     && ( $links[$i]->{'changestowrite'} eq "yes" ) ) {
	    # write and log changes
	    if ( $links[$i]->{'objid'} =~ /^newrow/ ) {
		push @links, &writelink( $links[$i], $site , $FORM, $dbh);
		$x=$links[$i]->{'objid'};
		$x =~ tr/[A-Za-z_.]//d ;      # strip alphas
		$links[$i]=&createnewrow($x);
	    }
	    else {
		# it is an existing row, just update it
		$links[$i]=&writelink( $links[$i] , $site, $FORM, $dbh);
		
	    } # end of testing for new or not
	    #
	} # end of data write cycle
    }
    return @links;
}

1;
