#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# login.pm
#

use strict;

##############################################################################
=pod

=head1 login.pm

login.pm - subroutines for login.cgi and any routines required for checking login
           validity

=head1 DESCRIPTION

This library is used for all subroutines that are used by login.cgi, and other
scripts which may need to check certain conditions about an already logged in
user.

=head1 NOTES & BUGS

At a minimum, the calling script must require or include all of the following
files:
    login.pm
    language.pm  (for get_messages)
    filetools.pm (for debug)
    conversions.pm (for debug)
    db_layer.pm
    permission_layer.pm

=cut

##############################################################################

##############################################################################
=pod

=head2 check_login

check_login - &check_login($site, $dbh);

=over 4

=item DESCRIPTION

Check to be sure that the use that is logged in is still valid, that the time hasn't
expired or that the SID has been removed.  

Invoked after the form data read but before sendingout the http header (it may 
spew a redirect to the login script).

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$return_string - Error string or blank if all is ok

=item NOTES

=back

=cut
###############################################################################
sub check_login {
    my ($site) = shift;
    my ($dbh) = shift;
    
    # takes script, mode, SID as args
    # returns $USERNAME if sid for this script and mode, otherwise ""
    #
    my ($rv,$num,$life,$q,$return_string, $env_browswer);
    my ($select_hashref, @sql_results, $contacts_hashref, $session_hashref);
    my ($dt, $update_hashref, $x, @temp, $scriptpath);
    


    $rv=1;   # ok by default
    $return_string="";
    $life=3700;  # number of seconds to live... 
    
    ## check for a non-zero sid  
    if ( $site->{'sid'} < 0 ) { 
	$rv=0;
	# "ERROR - there is no session ID $site->{'sid'}";
	$return_string = &login_message($site, 1) . " $site->{'sid'}";
    }
    
    if ( $rv ) {
	
	$env_browswer=substr( $ENV{'HTTP_USER_AGENT'},0,40 );

	## gather the browser used to login
	#
	$select_hashref = {};
	$select_hashref->{'TABLE'}->{'NAME'} = "sessions";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'browser'} = "browser";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'custid'} = "custid";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'mdate'} = "mdate";
	$select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid", "$site->{'sid'}", "and");
	
	$select_hashref->{'statement'} = "SELECT";
	$q = &build_sql($select_hashref, $site, $dbh, 1);
	$num = &query_getAll($q, $site, $dbh, \@sql_results);
	&debug("$q\n\t-->$num rows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
	if ($num > 0) {
	    $session_hashref = $sql_results[0];
	    if  ( $session_hashref->{'browser'} eq $env_browswer ) {                       # now see if the 
	    # browsers match  
	    
		$dt=time() - $session_hashref->{'mdate'};
		
		if (  $dt > $life ) {                # login has expired
		    $rv=0;
		    # "ERROR - user logged out! Login inactive more than $life seconds!"
		    $return_string = &login_message($site, 2) . " $life " . &login_message($site, 3);
		    #/checking mdate 
		    
		} else {                             # update the time of 
		    # last click
		    $num=time();
		    $update_hashref = {};
		    $update_hashref->{'TABLE'}->{'NAME'} = "sessions";
		    $update_hashref->{'TABLE'}->{'FIELDS'}->{'mdate'} = $num;
		    $update_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("sid", "$site->{'sid'}", "and");
		    $update_hashref->{'statement'} = "UPDATE";
		    $q = &build_sql($update_hashref, $site, $dbh, 1);
		    $num = &query_getAll($q, $site, $dbh, \@sql_results);
		    
		} #/done checking freshness of login
		
	    } else {
		
		# too few or too many records
		if ( $num ) {
		    # "ERROR - there are too many records for $site->{'sid'}";
		    $return_string = &login_message($site, 4) . " $site->{'sid'} ";
		    # remove bad records for SID
		    
		} else {
		    # "ERROR - $site->{'sid'} bad sid! you must log into the system!";
		    $return_string = &login_message($site, 5) . " $site->{'sid'} " . &login_message($site, 6);
		}
		
		$rv=0;
	    }
	    
	} 
	else {   # browsers do not match, return error 
	    # "ERROR - Browsers do not match!"
	    $return_string = &login_message($site, 7);
	} #/browser checking
	
    } #/looking for sid
    
    ## now test the uid permissions 
    #
    &debug("$rv, $return_string... Now test uid permissions", 3, $site->{'debug_level'}, $site->{'debug_log'});
    if ($rv) {
	
	## get the user record
	#
	$select_hashref = {};
	$select_hashref->{'TABLE'}->{'NAME'} = "contacts";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'uname'} = "uname";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'objtype'} = "objtype";
	$select_hashref->{'TABLE'}->{'FIELDS'}->{'status'} = "status";
	$select_hashref->{'TABLE'}->{'WHERE'}[0] = &build_where_hash("objid", $session_hashref->{'custid'}, "and");
	$select_hashref->{'statement'} = "SELECT";
	$q = &build_sql($select_hashref, $site, $dbh, 1);
	$num = &query_getAll($q, $site, $dbh, \@sql_results);
	&debug("$q\n\t-->$num rows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});

	## verify exactly one row returned
	#
	if ( $num == 1 ) {
	    $contacts_hashref = $sql_results[0];
	    
	    ## now check permissions
	    #
	    if ( $contacts_hashref->{'objtype'} eq '39004-A' ) {
		# this is an employee
		
	      if ( $contacts_hashref->{'status'} eq '15000-A' ) { 
		  # active employee
		  $return_string=$contacts_hashref->{'uname'};
	      } 
	      else {
		  # "ERROR - account no longer active";
		  $return_string = &login_message($site, 8);
	      } #/ done checking status
	      
	  } else {
	      # this is not an employee
	      # "ERROR - this account is not authorized to access this area!"; 
	      $return_string = &login_message($site, 9);
	  } #/done with verify 
	    
	} 
	else {
	    # "ERROR - No accout found for this account!";
	    $return_string = &login_message($site, 10);
	} #/end testing on number of user records in system
    } #/testing user permissions
    
    ## test whether to redirect back to login screen or not 
    #
    if ( ! $rv ) {
	## redirect them
	$x = (split /\?/, $ENV{'REQUEST_URI'})[0];  # remove GET stuff
	@temp=split /\//, $x;                        
	pop @temp;                                  # throw away scriptname
	shift(@temp) if ( ! $temp[0] );
	$x= join "/", @temp;                         
	$scriptpath="$x/login.cgi";
	print "Location: /$scriptpath\n\n"; 
    }
    
    if ( length($return_string)<1 ) {
	# "ERROR - unknown reason for lack of error message at login";
	$return_string = &login_message($site, 11);
    }
    &debug("$return_string", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $return_string;
    
} #/check_login

##############################################################################
=pod

=head2 login_message

login_message - $ret = &login_message($site, 1);

=over 4

=item DESCRIPTION

To support multiple languages, all error messages need to be stripped out of
the script and translated to different languages.  This returns the error 
message based the number passed in.

=item INPUT

$site - A hashref containing information required throughout about the current
        site and session

$msg_number - The number of the message to be returned

=item OUTPUT

The text of the message in the appropriate language.

=item NOTES

=back

=cut
###############################################################################
sub login_message {
    my ($site) = shift;
    my ($msg_number) = shift;
    
    my (@temp);
    my (%script_msg);

    %script_msg = &get_messages($site->{'om_render_lang'}, "login.pm", $site);
    return $script_msg{$msg_number};
}

##############################################################################
=pod

=head2 get_system_pulldown

get_system_pulldown - &login_message();

=over 4

=item DESCRIPTION

To generate a pulldown to use for all languages that can be used for this partic
site.

=item INPUT

=item OUTPUT

$pulldown - A SELECT HTML object, with all languages that are supported by the site.

=item NOTES

=back

=cut
###############################################################################
sub get_system_pulldown {

	my (@sys,$sys);
	@sys = ('all|AC Admin','wms|WM Admin');
	$sys = &gen_select("all",1,@sys);
	return $sys;
}

1;
