#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# update_conf.pm
#    reads .conf files from the ./conf directory and 
#    creates a new site.pm file in that directory
#
#
###############################################################################

=pod

=head1 update_conf.pm

update_conf.pm - tools for updating site.pm using conf files manually 

=head1 SYNOPSIS

http://domain/os_allcommerce/lib/update_conf.pm

=head1 DESCRIPTION

This script is for updating the site.pm using conf files manually. User can 
manually go into the four conf files (site.conf, database.conf, machine.conf, 
and cluster.conf) and do editions, then run this script 
at the command line to update the site.pm.  

=head1 NOTES & BUGS

=cut

###############################################################################


##############################################################################

=pod

=head2 update_conf 

update_conf - &update_conf();

=over 4

=item DESCRIPTION

The main subroutine is resposible for determining the flow of the update_conf.pm
 script. 

=item INPUT

NONE

=item OUTPUT

Update site.pm according to the conf files (site.conf, database.conf, machine.conf, 
and cluster.conf)

=item NOTES

=back

=cut

###############################################################################
sub update_conf {

    ## determine path from store (site) root and store db name...
    #
    #  note - this script runs at server root or in crons,
    #    generally. Test these conditions first.
    #  $0 - may not contain a full path, bootstrap with "pwd"
    #

    my $s;

    my $ORACLE_SID=shift;
    my $ORACLE_HOME=shift; 

    if ( $0 !~ /^\// ) {  # $0 is not an absolute path
        $s=`pwd`;          # hope pwd on path...
        chomp $s;
        $s.="/$0";
    } else {              # $0 IS an absolute path
        $s=$0;
    } # done determining if $0 is relative or absolute

    my @temp=split /\//, $s;

    my @rpath=@temp;

    my $scriptname = pop @rpath;
    $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

    my $rundir= join "/", @rpath;
    chdir( $rundir );

    my $relpath="";
    my $siteshortname="";
    
    ## short circuit if $s does not contain crons
    if ( $s !~ /lib/ ) {
        $siteshortname = pop @rpath;
        $relpath=".";
        @temp=();
    } # done testing for degenerate case

    while ( @temp ) {
          my $u = pop @temp;

          ## skip references to the present directory
          next if ( $u eq '.' );
          next if ( length($u)==0 );        # skip null directory levels

          $relpath.="/.." if ( $relpath);
          $relpath.="." if ( ! $relpath );

          if ( $u eq "lib" ) {
              $siteshortname = $temp[$#temp];
              @temp=();
          } # note - crons is off the site root, so the next up is site root

    } # done constructing relative path to store root

    $relpath .= "/../$siteshortname";

    require "./lib/db_layer.pm";

    my $db=om_db();
    my %conf=();        # null hash for configuration parameters
    my %database=();    # temporary holder for parameters from database.conf
    my %site=();        # temporary holder for parameters from site.conf
    my %machine=();     # temporary holder for parameters from machine.conf

    $relpath="../os_allcommerce";
    $siteshortname = "os_allcommerce";
    opendir CONF, "$relpath/conf";
    my @allfiles = readdir CONF;
    closedir CONF;

    while (@allfiles) {
          my $f = pop @allfiles;              # get a file from the list
          next if ( length( $f ) <= 5 );   # .conf has five characters
          next if ( $f !~ /\.conf$/ );     # it has to end in .conf
          next if ( $` =~ /[^a-zA-Z]/ );   # compare name for non-letters

          open( FH, "<$relpath/conf/$f" );       # read the file
          my @lines=(<FH>);
          close(FH);

          my $h={};                           # blank hashref
          while ( @lines ) {               # go through the lines
               $_ = pop @lines;
               next if ( $_ =~ /^#/ );          # skip comment lines
               chomp;
               next if ( $_ =~ /^$/ );             # skip blank lines
               my ( $k,$v ) = split /\s/, $_, 2;
               $v =~ s/^\s+//g;
               $v =~ s/\s+$//g;
               $v =~ s/\@/\\\@/g if ( $v !~ /\\/ );
               $h->{$k} = $v;
          }                                # done reading conf data
         %site=%{$h} if ( $f eq "site.conf" );
         %machine=%{$h} if ( $f eq "machine.conf" );
         %database=%{$h} if ( $f eq "database.conf" );

         foreach my $k (keys %{$h} ) { $conf{$k}=$h->{$k}; }
    }                                   # done running through files

    sort %conf;
    ## print to file site.pm

    open SITEPM, ">./conf/site.pm"; 

    ## print out #!/usr/bin/perl
    print SITEPM "#!/usr/bin/perl \n\n";

    ## print out header info and warning not to edit this file anymore

    print SITEPM "# This file was generated automatically by update_conf.pm from the 
# contents of <name>.conf where <name> is an alpha charactered named file.  Don't edit 
# this file, edit the <name>.conf files instead. \n\n"; 

if ($ORACLE_SID) {
    print SITEPM "\$ENV{'OARCLE_SID'}=\"$ORACLE_SID\"\;\n\n";
}   
if ($ORACLE_HOME) {
    print SITEPM "\$ENV{'OARCLE_HOME'}=\"$ORACLE_HOME\"\;\n\n";
} 
    print SITEPM "sub setsite { 
	# returns a hashref
	my(\$h,\$m,\$c,\$sr,\$rr,\$dbc,\$dbkv);
	\$h={};

	## actually set environmental params now
	#	
	# For a detail information about the following fields,
	# please see the document DOCS/install.html
	\$h = { \n "; 

    ## Here is where the hash table gets printed.  Need to remove $sr.
    ## Also include fsrelativeroot
    my @quotemeta = ("sitename","currency_symbol");
    my @noquote = ("site_sku_numlen", "site_sku_sclen", "maxtopnav", "site_sku_totlen", "site_sku_prelen");
    my @singlequote =("site_database_serv", "sitecode", "ssl_prefix", "sitessldomain", "sitedomain", "siteserver", "sitesslserver");
    print SITEPM "\tfsrelativeroot\t=> '$siteshortname',\n"; 
    foreach my $k ( @noquote ) {  
          print SITEPM "\t$k\t=> $conf{$k}, \n" if $k; 
          delete $conf{$k}; 
    } 

    foreach my $k ( @singlequote ) { 
          print SITEPM "\t$k\t=> \'$conf{$k}\', \n" if $k;
          delete $conf{$k}; 
    } 

    foreach my $k ( keys %conf ) { 
            if ( $k !~ /\$sr/ ) { 
               foreach my $item (@quotemeta) {
                  $conf{$k} = quotemeta $conf{$k} if ($item eq $k);
               }
               print SITEPM "\t$k\t=> \"$conf{$k}\",\n" if $k; 
            }
    } 

    ## Done with hash table
    print SITEPM "     }; # done with  \$h \n";

       ## \$m is the machine specific (file system) bits hashref
       #      it's date is used to construct \$h, then it's keys 
       #      and values get read into \$h before returning \$h
       # \n";

    ## Here is where $sr gets placed.
    foreach my $k ( keys %conf ) {
          if ( $k =~ /\$sr/ ) { print SITEPM "       $k = \"$conf{$k}\";\n"; }
    }
    print SITEPM "       \$rr = \"\$h->{\'fsrelativeroot\'}\";

       \$m = {
\t   fsrootweb	=> \$sr,
\t   fsrootcgi	=> \"\$sr\/\$rr\/cgi-bin\",
\t   fsrootlib	=> \"\$sr\/\$rr\/lib\",
\t   fsroothtml	=> \"\$sr\",
\t   fsrootcron	=> \"\$sr\/\$rr\/crons\",
\t   fsrootlog	=> \"\$sr\/\$rr\/logs\",
\t   fsrootdata	=> \"\$sr\/\$rr\/data\"
\t}; # end of defining file system parameters\n";

    ## Database format from db_om.pm 
    my $site_driver = $conf{'site_datadriver'};
    if ($site_driver =~ /^$/ ) { $site_driver="mysql"; }
    $db->{'db_site_config'}{$site_driver} =~ s/sid\=/sid\=$ORACLE_SID/;

    print SITEPM "## Database format from db_om.pm 
\t \$h->{\'db_site_config\'}=\"$db->{'db_site_config'}{$site_driver}\";
\t \$h->{\'db_site_connect\'}=\"$db->{'db_site_config'}{$site_driver}\";  
\t## construct some deterministic composite elements
\t#
\t\$h->{\'siteurl\'}=\$h->{\'siteserver\'}\.\$h->{\'sitedomain\'};
\t\$h->{\'sitesslurl\'}=\$h->{\'sitesslserver\'}\.\$h->{\'sitessldomain\'};

\tforeach ( keys %{\$m} ) {
\t   \$h->{\$_}=\$m->{\$_} if ( ! defined(\$h->{\$_}) ); 
\t}

\t# Set no values to be undef
\tif (\$h->{'autosku'} eq \"no\") { 
\t\t\$h->{'autosku'} = undef \$h->{'autosku'};
\t};
\tif (\$h->{'override_permissions'} eq \"no\") { 
\t\t\$h->{'override_permissions'} = undef \$h->{'override_permissions'};
\t};
\treturn \$h;
} #\/setsite

1;\n"; 

    close SITEPM;

    # make sure it has the proper file permissions
    chmod 0755, "$relpath/conf/site.pm"; 
}
	
1;
