#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  variant.pm
#

##############################################################################
=pod

=head1 variant.pm

variant.pm - subroutines for dealing with variant products and master products

=head1 DESCRIPTION

This library is used for all subroutines for handling variant products and
master products.

=head1 NOTES & BUGS

DBI must be required in the calling script
eapi.pm must be included in calling script

=cut
##############################################################################

use strict;

####################### Start Routines from Version 1.0 ######################
##############################################################################
=pod

=head2 get_fields_and_title

get_fields_and_title - &get_fields_and_title($product_hashref, $objid, $dbh, $master_hashref, $site);

=over 4

=item DESCRIPTION

To retreive the title for objects, as well as any additional requested fields.
This is temporary until all calls to get_fields_and_title are removed! They are no
longer needed since variant titles are now stored in the record.

=item INPUT

$product_select - Extra fields to get from the product in format 
    $product_select->{'FIELDS'}->{'fieldname'} = "fieldname" for fields to select
    $product_select->{'WHERE'} = array of hashrefs defining where clause, see db_layer.pm NOTES

$objid - the primary key of things, to base the query on

$dbh - current active database handler

$master_select - Extra fields to get from the master in format $master_select->{'FIELDS'}->{'fieldname'} = "fieldname"

$site - A hashref containing information required throughout about the current site and session

=item OUTPUT

$results - a hash, to be referenced by the fields queried in things

=item NOTES

Only 1 row will ever be returned, as objid is the primary key of things.

=back

=cut
###############################################################################
sub get_fields_and_title {
    my ($product_select) = shift;
    my ($objid) = shift;
    my ($dbh) = shift;
    my ($master_select) = shift;
    my ($site) = shift;

    my ($q, $sth, @results_arr, @master_arr, @extraprop_arr, $results, $extraprop, $master_row, @rows, $row, $title);
    my ($multiple_var, $vcode_indx, @vcodes, @variants, $numRows, $key, @fields);
    my ($fld, $v_codes, $variant, $while_indx, $done, $code, @val, $val);
    my ($field, $select_hashref, @where_array);
    my ($where_indx, $hash, $while_indx, $where_hashref);

    &debug("In get_fields_and_title with $objid", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'title'} = "title";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'axisnames'} = "axisnames";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = "ref_from";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'vcode'} = "vcode";

    if (defined $product_select) {
	foreach $field (keys %{$product_select->{'FIELDS'}}) {
	    $select_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $product_select->{'FIELDS'}->{$field};
	}
    }
    $where_indx = $while_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");

    if (defined $product_select->{'WHERE'}) {
	while ($where_hashref = $product_select->{'WHERE'}[$while_indx++]) {
	    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = $where_hashref;
	    &debug("Added $where_hashref->{'field'} $where_hashref->{'value'} $where_hashref->{'and_or'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }

    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh);
    $numRows = &query_getAll($q, $site, $dbh, \@results_arr);
    &debug("$q\n\t-->$numRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
    
    # we know that there is only one row returned, so get it!
    $results = $results_arr[0];
    $results->{'numrows'} = $numRows;
    &debug_hashref($results, 3, $site->{'debug_level'}, $site->{'debug_log'});
    return ($results);
}

##############################################################################
=pod

=head2 isMasterProduct

isMasterProduct - &isMasterProduct($objid, $site, $dbh, \%NVP);

=over 4

=item DESCRIPTION

Determines if the product is a master product.

=item INPUT

$objid - object to determine if the object is a master product

$site - site hashref

$dbh - live database handler

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

=item OUTPUT

1 if it is master product, and set's the master_objid in the form
  
0 if it is not a master product

=item NOTES

=back

=cut
###############################################################################
sub isMasterProduct {
    my ($objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($NVP) = shift;
    
    my ($q, $numRows, @res, $h, $select_hashref, $where_indx);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = "ref_from";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@res);
    
    $h = $res[0];
    # if the ref_from equals the objid, it is a master product
    if ($h->{'ref_from'} eq $objid) {
        $$NVP{'master_objid'} = $objid;
        return 1;
    }
    
    return 0;
}

##############################################################################
=pod

=head2 isVariantProduct

isVariantProduct - &isVariantProduct($objid, $dbh, $site);

=over 4

=item DESCRIPTION

Determines if the product is a variant product

=item INPUT

$objid - object to determine if the object is a master product

$dbh - live database handler

$site - A hashref containing information required throughout about the current site and session

=item OUTPUT

objid of the master if it is a variant product

nothing if it is not a variant product

=item NOTES

=back

=cut
###############################################################################
sub isVariantProduct {
    my ($objid) = shift;
    my ($dbh) = shift;
    my ($site) = shift;
    
    my ($q, $numRows, @res, $h, $select_hashref, $where_indx);
    
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'ref_from'} = "ref_from";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", $objid, "and");
    $q = &build_sql($select_hashref, $site, $dbh, 1);
    $numRows = &query_getAll($q, $site, $dbh, \@res);
    &debug("$q\n\t-->$numRows affected",2, $site->{'debug_level'}, $site->{'debug_log'});

    $h = $res[0];
    &debug("isVariantProduct: Ref_from is $h->{'ref_from'}, objid is $objid!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # if the ref_from equals the objid, it is a master product
    if (($h->{'ref_from'} ne '') && ($h->{'ref_from'} ne $objid)) {
        return $h->{'ref_from'};  #return master objid
    }
}

####################### MASTER PRODUCT SUBROUTINES #########################
##############################################################################
=pod

=head2 setBackToMaster

setBackToMaster - &setBackToMaster($master_objid, $pr, "COLOR=#000000", \%FORM, \%NVP, $site);

=over 4

=item DESCRIPTION

Set a link to go back to master from a variant product

=item INPUT

$master_objid - objid of the master to go back to

$pr - partial string for the link, with:
    "&sid=$site->{'sid'}&mode=$mode&loadfrom=$loadfrom";
       Where $site->{'sid'} is the session id
                   $mode is the mode you want to go to
                   $loadfrom is where to load it from

$td_props - Properties for the TD that may want to be
              encorporated - COLOR, ALIGN, etc

$FORM - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current site and session

=item OUTPUT

Sets 'back_to_master' and 'master_objid' in NVP

=item NOTES

=back

=cut
###############################################################################
sub setBackToMaster {
    my ($master_objid) = shift;
    my ($pr) = shift;
    my ($td_props) = shift;
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    
    my ($k, $master, $master_link, $ln);
    my ($whtoken);
    
    
#    $k="objid=$master_objid";
#    $whtoken = "warehouse_inp=";
#    $master = "$site->{'scriptname'}\?$pr&$k&$whtoken";
    $master = $pr;
    if ($td_props) {
        $ln =  "<TD $td_props VALIGN=BOTTOM NOWRAP>";
    }
    else {
        $ln =  "<TD ALIGN=CENTER VALIGN=BOTTOM NOWRAP>";
    }
    $master_link = "$ln<A HREF=\"$master\"><FONT FACE=\"arial,helvetica\" SIZE=-1>$site->{'script_msg'}{10} $$FORM{'master_objid'}</FONT></TD>";
    $$NVP{'back_to_master'} = $master_link;
    $$NVP{'master_objid'} = $master_objid;
}

##############################################################################
=pod

=head2 setupMasterLinks

setupMasterLinks - &setupMasterLinks($objid, $pr, "price", \%FORM, \%NVP, $site, $dbh);

=over 4

=item DESCRIPTION

To set up links to all of the variant products for master
products

=item INPUT

$objid - master object id

$pr - partial string for the link, with:
     "&sid=$site->{'sid'}&mode=$mode&loadfrom=$loadfrom";
           Where $site->{'sid'} is the session id
                    $mode is the mode you want to go to
                    $loadfrom is where to load it from

$type_link - string price or inventory to concatenate
                      to the error messages

$FORM - Hashref to the current page that is displayed.

$NVP - A temporary hashref to hold information to update the next FORM to be displayed

$site - A hashref containing information required throughout about the current site and session

$dbh - Current active database handle

=item OUTPUT

$linkrows - the links that were generated

Updates the $NVP{'variants'}  with the linkrows.

=item NOTES

=back

=cut
###############################################################################
sub setUpMasterLinks {
    my ($objid) = shift;
    my ($pr) = shift;
    my ($type_link) = shift;
    my ($FORM) = shift;
    my ($NVP) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($q, $numVarRows, $linkrows, $i, $h, $where_indx);
    my (@res, $var_titles, $k, $links, $whtoken, $select_hashref);
    
    $select_hashref = {};
    $select_hashref->{'TABLE'}->{'NAME'} = "things";
    $select_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $where_indx = 0;
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("ref_from", $objid, "and");
    $select_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", "!$objid", "and");
    $select_hashref->{'statement'} = "SELECT";
    $q = &build_sql($select_hashref, $site, $dbh);

    $numVarRows = &query_getAll($q, $site, $dbh, \@res);

    &debug("$q\n\t-->$numVarRows affected", 2, $site->{'debug_level'}, $site->{'debug_log'});
    
    $linkrows = "";
    for ($i = 0; $i < $numVarRows; $i++) {
        &debug("In creating links with $i of $numVarRows!",3,$site->{'debug_level'}, $site->{'debug_log'});
        
        $h = $res[$i];
        $var_titles = &get_fields_and_title(undef, $h->{'objid'}, $dbh, undef, $site);
        $k="&objid=$h->{'objid'}";
#        $whtoken = "warehouse_inp=";
        
#        $links = "$site->{'scriptname'}\?$pr&$k&$whtoken";
	$links = $pr . $k;
        $linkrows .= "<TR><TD><A HREF=\"$links\"><FONT FACE=\"arial,helvetica\" SIZE=-1>$h->{'objid'}</FONT></TD>";
        $linkrows .= " <TD><FONT FACE=\"arial,helvetica\" SIZE=-1>$var_titles->{'title'}</FONT></TD></TR>";
    }
    if ($numVarRows == 0) {
        $linkrows = "<FONT FACE=\"arial,helvetica\" SIZE=+1 COLOR=\"#FF0000\">You must configure variant products prior to assigning $type_link\n";
    }
    
    $$NVP{'master_objid'} = $objid;
    
    &debug("Link rows is $linkrows!",3,$site->{'debug_level'}, $site->{'debug_log'});
    
    $$NVP{'variants'} = $linkrows;
    return $linkrows;
}

####################### End of Routines from Version 1.0 #####################
####################### Routines from Version 1.1 (admin_object)##############

##############################################################################
=pod

=head2 build_variant_title

build_variant_title - &build_variant_title("3-AA", "13-AA:4-AA", "eng", $site, $dbh);

=over 4

=item DESCRIPTION

Taking a master object id and a vcode, build the title for the variant product.

=item INPUT

$master_objid - objid of the master product

$vcodes	- vcode field from the variant product (group of objids appended by :)

$language - Default language to use, usually om_render_lang

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$master_hash - Optional, the hashref of the master product, if already retreived

=item OUTPUT

$title -    Variant title formed from the:
            <master title> - title of vcode[0], .. , title of vcode[3],

=item NOTES

Variant titles are now stored in the database, although they are still dynamically 
generated.

=back

=cut
###############################################################################
sub build_variant_title {
    my ($master_objid) = shift;
    my ($vcodes) = shift;
    my ($language) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($master_hash) = shift;  #optional, if already retreived

    my ($title, $q, $select_hashref, $numRows, @master_arr, $master_row, @tmp);
    my (@vcodes, $vcode, $object_hashref, $lang_order, $master_hashref);

    $lang_order = &getLanguageOrder($language, $site);

   # get the master title
    if (defined $master_hash) {
        $master_hashref = $master_hash;
    }
    else {
	$master_hashref = &load_object($master_objid, "things", "objid,language,title", \@tmp, $site, $dbh, 1);
    }

    $master_hashref = &set_fields_to_default_lang_val($master_hashref->{'c'}, "title", $lang_order, $site);
    $title = "$master_hashref->{'title'} - ";  # set the master title

    &debug("set_variant_title: Got master title, set to: $master_hashref->{'title'}, eng: $master_hashref->{'eng'}->{'title'}, fra: $master_hashref->{'fra'}->{'title'}", 3, $site->{'debug_level'}, $site->{'debug_log'});

    @vcodes = split /\:/, $vcodes;
    foreach $vcode (@vcodes) {
	$object_hashref = &load_object($vcode, "things", "objid,language,title", \@tmp, $site, $dbh, 1);
	$object_hashref = &set_fields_to_default_lang_val($object_hashref->{'c'}, "title", $lang_order, $site);
	$title .= "$object_hashref->{'title'}, ";
    }

    # get rid of last " ,"
    chop $title;
    chop $title;
    $title = &string_exdbi( $title);
    return ($title);
}

##############################################################################
=pod

=head2 check_variant_titles

check_variant_titles - &check_variant_titles($object_info, $site, $dbh);

=over 4

=item DESCRIPTION

Check if the changes made to the record could affect the automatic generation 
of variant titles.  If so, then we need to re-compute the variant titles and update them.

=item INPUT

$info - Hashref with flags of information.  See &get_object_info OUTPUT to
               see details.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - Key to $site->{'script_msg'} if there is an error that 
         occurred, otherwise, NULL if everything worked properly.

=item NOTES

=back

=cut
###############################################################################
sub check_variant_titles {
    my ($info) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($error, $variant, @variant_sets);
    &debug("check_variant_title:  In here!", 3, $site->{'debug_level'}, $site->{'debug_log'});
    &debug_hashref($info, 3, $site->{'debug_level'}, $site->{'debug_log'});

    # Nothing changed to affect variant titles
    if (!$info->{'objid'}) {
	return;
    }

    if ($info->{'objclass'} eq $site->{'OBJCLASSES'}->{'variant'}) {
	&find_variant_sets($info->{'objid'}, $site, $dbh, \@variant_sets);
	&debug("check_variant_titles, Just found $#variant_sets + 1", 3, $site->{'debug_level'}, $site->{'debug_log'});
	foreach $variant (@variant_sets) {
	    $error = &update_variant_products_by_axisname($variant, $site, $dbh);
	    if (defined $error) {
		&debug("check_variant_title:  Couldn't update $variant->{'objid'}, $site->{'script_msg'}{$error}", 1, $site->{'debug_level'}, $site->{'debug_log'});
	    }
	}
    }
    elsif ($info->{'objtype'} eq $site->{'OBJTYPES'}->{'variant_set'}) {
	$error = &update_variant_products_by_axisname($variant->{'objid'}, $site, $dbh);
	if (defined $error) {
	    &debug("check_variant_title:  Couldn't update $variant->{'objid'}, $site->{'script_msg'}{$error}", 1, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }
    elsif ($info->{'objtype'} eq $site->{'OBJTYPES'}->{'product'}) {
	$error = &update_masters_variants($info->{'objid'}, $site, $dbh);
    }
    else {
	&debug("check_variant_titles: Don't know what to do with $info->{'objtype'}.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return;
    }

    return $error;
}

##############################################################################
=pod

=head2 find_variant_sets

find_variant_sets - &find_variant_sets("3-AA", $site, $dbh, \@results);

=over 4

=item DESCRIPTION

Find all of the variant sets that a variant belongs to.

=item INPUT

$variant_objid - Objid of the variant to look for in variant sets

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

$p_variant_sets - Pointer to an array for output.

=item OUTPUT

$error - Error if error occurred, undef otherwise

=item NOTES

=back

=cut
###############################################################################
sub find_variant_sets {
    my ($variant_objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($p_variant_sets) = shift;

    my ($sql_hashref, $q, @all_possibilities, $indx, $numRows, $set);
    my (@objids, $objid, $where_indx);

    &debug("In find_variant_sets", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $sql_hashref = {};
    $sql_hashref->{'statement'} = "select";
    $sql_hashref->{'TABLE'}->{'NAME'} = "extraprop";
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{'body'} = "body";

    $where_indx = 0;
    $sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("body", "%$variant_objid%", "and");
    $sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("fieldname", "variant_set", "and");
    $q = &build_sql($sql_hashref, $site, $dbh, 1);
    if (!$q) {
	&debug("find_variant_sets: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{2};
    }	

    $numRows = &query_getAll($q, $site, $dbh, \@all_possibilities);
    if ($numRows == 0) {
	&debug("find_variant_sets: No variant sets found for $variant_objid.", 3, $site->{'debug_level'}, $site->{'debug_log'});	
	return;
    }

    foreach $set (@all_possibilities) {
	@objids = split /\:/, $set->{'body'};
	foreach $objid (@objids) {
	    # need to comare each time to make sure that it is exactly the same variant_objid, not just similar (1-AA, could match 21-AA)
	    if ($objid eq $variant_objid) {
		$$p_variant_sets[$indx++] = $set->{'objid'};
	    }
	}
    }

    &debug("find_variant_sets: Found $indx variant sets for $variant_objid.", 3, $site->{'debug_level'}, $site->{'debug_log'});	
    return;
    
}

##############################################################################
=pod

=head2 update_variant_sets_by_axisname

update_variant_sets_by_axisname - &update_variant_sets_by_axisname("3-AA", $site, $dbh);

=over 4

=item DESCRIPTION

Find all master objects which have part the variant set defined in the axisnames field.  
Re-generate the variant title for all variant products regardless of language.

=item INPUT

$variant_set_objid - Objid that describes the variant set in the axisnames field

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

=item NOTES

=back

=cut
###############################################################################
sub update_variant_products_by_axisname {
    my ($variant_set_objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    
    my ($sql_hashref, $q, $numRows, $error, @all_possibilities, @axisnames, $axisname, $master, @temp);

    my ($object_hashref, $where_indx);

    &debug("In update_variant_products_by_axisname with $variant_set_objid (variant set objid)",3, $site->{'debug_level'}, $site->{'debug_log'});
    $object_hashref = &load_object($variant_set_objid, "things", "objid,encoding,language", \@temp, $site, $dbh, 1);
    
    $sql_hashref->{'statement'} = "select";
    $sql_hashref->{'TABLE'}->{'NAME'} = "things";
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{'objid'} = "objid";
    $sql_hashref->{'TABLE'}->{'FIELDS'}->{'axisnames'} = "axisnames";
    $where_indx = 0;
    $sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("axisnames", "%$variant_set_objid%", "and");
    $q = &build_sql($sql_hashref, $site, $dbh, 1);
    if (!$q) {
	&debug("find_variant_sets: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{2};
    }	

    $numRows = &query_getAll($q, $site, $dbh, \@all_possibilities);
    if ($numRows == 0) {
	&debug("update_variant_products_by_axisname: No variant sets found for $variant_set_objid.", 3, $site->{'debug_level'}, $site->{'debug_log'});	
	return;
    }

    foreach $master (@all_possibilities) {
	@axisnames = split /\:/, $master->{'axisnames'};
	foreach $axisname (@axisnames) {
	    next if ($axisname ne $variant_set_objid);
	    &debug("update_variant_products_by_axisname: Going to go into update_masters_variants with $master->{'objid'}.", 3, $site->{'debug_level'}, $site->{'debug_log'});	
	    $error = &update_masters_variants($master->{'objid'}, $site, $dbh);
	    if ($error) {
		&debug("update_variant_products_by_axisname: $error: $site->{'script_msg'}{$error}.", 3, $site->{'debug_level'}, $site->{'debug_log'});			
		return $error;
	    }
	}
    }
    return;
}


##############################################################################
=pod

=head2 update_masters_variants

update_masters_variants - &update_masters_variants();

=over 4

=item DESCRIPTION

Update the variant titles for all variant products associated with the master, for 
each language that the master is defined in.  If the variant product does not exist 
in a language that the master is defined in, then it must be defined!  Note that title 
is the only item that must be regenerated for the variant product, therefore we will 
force an entry at this time. 

=item INPUT

$master_objid - Master's objid

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

The variant titles are all updated
$error - If an error occurred, under otherwise

=item NOTES

=back

=cut
###############################################################################
sub update_masters_variants {
    my ($master_objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($lang_order, $default_lang, $object_hashref, $master_hashref, $language, $variant, @variants);
    my ($h, $error, @lang_keys, @tmp);

    &debug("In update_masters_variants for $master_objid", 3, $site->{'debug_level'}, $site->{'debug_log'});
    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
    $default_lang = $site->{'om_render_lang'};
    $object_hashref = &load_object($master_objid, "things", "", \@tmp, $site, $dbh, 1);
    $master_hashref = $object_hashref->{'c'};

    @lang_keys = split /\,/, $master_hashref->{'lang_keys'};

    foreach $language (@lang_keys) {
	# if the master is defined in this language, then we need to create
	# all of the variant titles for this language.
	if (defined $master_hashref->{$language}->{'title'}) {
	    &get_master_product_variants($master_hashref->{'objid'}, "objid,vcode,ref_from,title", \@variants, $site, $dbh);
	    foreach $variant (@variants) {
		$h = {};
		$h->{'objtype_inp'} = $master_hashref->{'objtype'};
		$h->{'objtype_pre'} = $master_hashref->{'objtype'};
		$h->{'master_objid'} = $master_objid;
		$h->{'title_pre'} = $variant->{'title'};
		$h->{'title_inp'} = &build_variant_title($master_objid, $variant->{'vcode'}, $language, $site, $dbh);
		&debug("update_masters_variants: updating for $h->{'title_inp'}", 3, $site->{'debug_level'}, $site->{'debug_log'});
		# temporarily fool it for the update_object generation
		$site->{'om_render_lang'} = $language;
		$error = &update_object($variant->{'objid'}, $h, $site, $dbh);
		if ($error) {
		    &debug("update_masters_variants: update_object failed $error, $site->{'script_msg'}{$error}.", 3, $site->{'debug_level'}, $site->{'debug_log'});			
		    # reset it so that we don't redisplay in a different language
		    $site->{'om_render_lang'} = $default_lang;
		    return $error;
		}
	    }
	}
    }
    # reset it so that we don't redisplay in a different language
    $site->{'om_render_lang'} = $default_lang;
    &debug("update_masters_variants: Done successfully!.", 3, $site->{'debug_level'}, $site->{'debug_log'});			
    return;
}

##############################################################################
=pod

=head2 get_master_product_variants

get_master_product_variants - &get_master_product_variants();

=over 4

=item DESCRIPTION

To retrieve $fields from all products that have $master_objid as their master,
and return the information in an array, each consiting of the informaiton of the
variant product, with the $fields populated.

=item INPUT

$master_objid - Objid of the master product

$fields - Fields to retreive from the variant products, comma delimited list

$p_variants - pointer to an array, where the variant products information is returned.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$error - if an error occurred, undef otherwise

=item NOTES

TARA - DOES NOT CONSIDER LANGUAGE SENSITIVE INFORMAITON, AND ONLY GETS DATA FROM THINGS

=back

=cut
###############################################################################
sub get_master_product_variants {
    my ($master_objid) = shift;
    my ($fields) = shift;
    my ($p_variants) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($sql_hashref, $q, $numRows, @all_fields, $field, $where_indx);

    $sql_hashref->{'statement'} = "SELECT";
    $sql_hashref->{'TABLE'}->{'NAME'} = "things";

    if ($fields eq "") {
	$sql_hashref->{'TABLE'}->{'FIELDS'}->{'*'} = "*";
    }
    else {
	$fields =~ s/ //gms;
	@all_fields = split /\,/, $fields;
	foreach $field (@all_fields) {
	    $sql_hashref->{'TABLE'}->{'FIELDS'}->{$field} = $field;
	}
    }

    $where_indx = 0;
    $sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("objid", "!$master_objid", "and");
    $sql_hashref->{'TABLE'}->{'WHERE'}[$where_indx++] = &build_where_hash("ref_from", "$master_objid", "and");
    $q = &build_sql($sql_hashref, $site, $dbh, 1);
    if (!$q) {
	&debug("get_master_product_variants: Insufficient permissions to update!  Should NOT ever happend.", 1, $site->{'debug_level'}, $site->{'debug_log'});
	return $site->{'script_msg'}{2};
    }	

    $numRows = &query_getAll($q, $site, $dbh, $p_variants);
    if ($numRows == 0) {
	&debug("get_master_product_variants: No variants found for master $master_objid.", 3, $site->{'debug_level'}, $site->{'debug_log'});	
    }    

    return;
}

##############################################################################
=pod

=head2 inherit_master_fields

inherit_master_fields - &inherit_master_fields("3-AA", "msku", \%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Variant products are required to inherit certain fields from their masters.
For example, a variant product would inherit the msku from the master product.
It also uses it's title to build it's own special title.  Also in the site
generation, variant products use the master's brief and keywords in the 
display of the object.  The routine allow for you to specify what fields
you wish to inherit from the master product and returns a hashref with that
data from the master product.

=item INPUT

$master_objid - Object ID for the master product

$fields - List of fields which to inherit from the master, comma delimited

$input_hash - Pointer to the FORM hash normally.

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$inherited_hashref - Hashref with a key to each field which was to be inherited,
                     with the value from the master record.

=item NOTES

=back

=cut
###############################################################################
sub inherit_master_fields {
    my ($master_objid) = shift;
    my ($fields) = shift;
    my ($input_hash) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my ($field, $master_hashref, @all_fields, $inherited_hashref, $lang_order, $field_hashref, @tmp);

    &debug("inherit_master_fields: Need to inherit $fields", 3, $site->{'debug_level'}, $site->{'debug_log'});
    # always have language just in case
    if ($fields !~ /language/) {
	$fields .= ",language";
    }

    $fields =~ s/ //gms;
    @all_fields = split /\,/, $fields;
    $master_hashref = &load_object($master_objid, "things", $fields, \@tmp, $site, $dbh, 1);
    $lang_order = &getLanguageOrder($site->{'om_render_lang'}, $site);
    $field_hashref = &set_fields_to_default_lang_val($master_hashref->{'c'}, $fields, $lang_order, $site);

    &debug_hashref($field_hashref, 3, $site->{'debug_level'}, $site->{'debug_log'});
    foreach $field (@all_fields) {
	if ($field eq "title") {
	    $inherited_hashref->{'title'} = &build_variant_title($$input_hash{'master_objid'}, $$input_hash{'vcode_inp'},
						     $site->{'om_render_lang'}, $site, $dbh, $master_hashref);
	}
	else {
		$inherited_hashref->{$field} = $field_hashref->{$field};
		&debug("inherit_master_fields: Just inheritted $field = $field_hashref->{$field}", 3, $site->{'debug_level'}, $site->{'debug_log'});
	}
    }

    return ($inherited_hashref);
}


##############################################################################
=pod

=head2 build_variant_product_vcode

build_variant_product_vcode - &build_variant_product_vcode(\%FORM, $site, $dbh);

=over 4

=item DESCRIPTION

Take the input from the axis pulldowns and build the variant product vcode from
them.  

=item INPUT

$input_hashref - Pointer to FORM hash

$site - A hashref containing information required throughout about the current
        site and session

$dbh - current active database handle

=item OUTPUT

$vcode - build from the axis#_inp fields

=item NOTES

=back

=cut
###############################################################################
sub build_variant_product_vcode {
    my ($input_hashref) = shift;
    my ($site) = shift;
    my ($dbh) = shift;

    my (@results, $q, $numRows, $results, $axisnames, @axis, $numAxis);
    my ($error, $err, $select_hashref, $i, $axis_check, $vcode);

    &debug("build_variant_product_vcode: in here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $vcode = "";

    for ($i = 0; $i < 4; $i++) {
	$axis_check = "axis" . $i . "_inp";
	if (defined $$input_hashref{$axis_check}) {
	    $vcode .= "$$input_hashref{$axis_check}:";
	}
    }

    chop $vcode;  # get rid of last :
    &debug("build_variant_product_vcode: Just built $vcode", 3, $site->{'debug_level'}, $site->{'debug_log'});
    return $vcode;
}


##############################################################################
=pod

=head2 get_existing_variant_products

get_existing_variant_products - &get_existing_variant_products($mast

=over 4

=item DESCRIPTION

To retrieve all of the variant products defined for $master_objid.
Takes care of language.a

=item INPUT

$master_objid - Objid of the master product

$site - A hashref containing information required throughout about the current

$site - A hashref containing information required throughout about the current
        site and session

$p_variants - pointer to an array, where the variant products information is returned.

=item OUTPUT

$p_variants is populated with existing variant products

=item NOTES

=back

=cut
###############################################################################
sub get_existing_variant_products {
    my ($master_objid) = shift;
    my ($site) = shift;
    my ($dbh) = shift;
    my ($p_results) = shift;

    my ($sql_hashref, $numVariants, $where_indx, $object_hashref, @tmp, @WHERE);
    my ($num_variants, @variants, $next_start);

    &debug("get_existing_variant_products: In here", 3, $site->{'debug_level'}, $site->{'debug_log'});

    $where_indx = 0;
    $WHERE[$where_indx++] = &build_where_hash("ref_from", $master_objid, "and");
    $WHERE[$where_indx++] = &build_where_hash("objid", "!$master_objid", "and");
    ($num_variants, $next_start, @variants) = &load_objects("things", "*", \@WHERE, "", "", $site, $dbh, $site->{'override_permissions'});
    &debug("get_existing_variant_products: Found $num_variants for Master $master_objid!", 3, $site->{'debug_level'}, $site->{'debug_log'});

    @$p_results = @variants;
    return ($num_variants);

}

####################### End of Routines from Version 1.1 (admin_object) #####################

1;
