use strict;


sub create_po_main {
	my ($site, $form) = @_;
	
	# bail if we don't need to save anything
	return '' unless $form->{'save.main'};
	
	my $err;
	
	# Check the input
	$err = &check_po_input($form, $site);
	
	# Insert data if input is good
	$err = &insert_po($form, $site) unless ($err);
	
	# Scrub the form if there's no error
	my @fields = (
		"wh",
		"vendor",
		"ponum",
		"canceldate",
		"contact",
		"team"
	);

	my $ponum = $form->{ponum};
	#$form = &wipe_fields($form, @fields) unless ($form->{'inp_err'});
	#$err = "Created PO $ponum" unless ($err);

	# Return form
	return $err;
}

sub create_po_item {
	my ($site, $form) = @_;
	
	# bail if we don't need to save anything
	return '' unless $form->{'save.item'};
	
	my $err;
	
	# Check the input
	$err = &check_po_input($form, $site);

	# Insert data if input is good
	$err = &insert_po_item($form, $site) unless ($err);
	
	# Scrub the form if there's no error
	my @fields = (
		"sku",
		"qty",
		"overage",
		"rprice",
		"vprice"
	);

	my $sku = $form->{'sku'};
	$form = &wipe_fields($form, @fields) unless ($err);
	#$err = "Created PO Line for SKU $sku" unless ($err);

	# Return error
	return $err;
}
	
sub check_po_input {
	my ($form, $site) = @_;
	my $rv = 1;
	my $failed = {};
	my $err;
	my $type = ($form->{'save.main'}) ? 'main' : 'item';
	warn "Po type is $type\n";
	
	my $req;
	
	if ($type eq 'main') {
		$req = {
			'vendor'     => "Vendor",
			'wh'         => "Receiving Facility",
			'ponum'      => "PO Number",
			'canceldate' => "Cancellation Date",
		};
	} else {
		$req = {
			'sku'	  => 'SKU',
			'qty'	  => 'Quantity',
			'overage' => 'Overage Tolerance',
			'vprice'  => 'Vendor Price',
			'rprice'  => 'Retail Price'
		};
	}
	
	($rv, $failed) = &check_required_fields($form, $req);
	warn "Check for required fields returned $rv, $failed\n";
		
	unless ($rv) {
		$err .= "Please fill in all required fields!\n";
	}

	# Check that canceldate is valid
	if (($type eq 'main') && (!$err)) {
		$rv = &check_date($form->{'canceldate'}, 1);
		unless ($rv) {
			$err .= "Date is invalid.\n";
		}
	}
	
	# Check that the sku exists
	if ($type eq 'item') {
		my $sku_err;
		($rv, $sku_err) = &check_sku($site, $form->{'sku'}) if ($form->{'sku'});
		if ($rv) {
			$err .= "SKU $form->{'sku'} not found.\n";
		} else {
			$rv = &check_price($form->{'rprice'});
			$err .= "Retail Price is invalid\n" unless ($rv);
			$rv = &check_price($form->{'vprice'});
			$err .= "Vendor Price is invalid\n" unless ($rv);
		}
	}
		
	warn "Checked Input. Err is: <<$err>>\n";
	return $err;
}

sub insert_po {
	my ($form, $site) = @_;
	
	my $objid = &get_new_objid($site, $site->{dbh}, 'po');
	return "Could not create object id\n" unless ($objid);
	
	my $insert_q = {
		'statement' => 'insert',
		'TABLE' =>
			{
			 'NAME' => 'po',
			 'FIELDS' => 
			 	{
				 'objid' => $objid,
				 'wh' => $form->{'wh'},
				 'ponum' => $form->{'ponum'},
				 'canceldate' => $form->{'canceldate'},
				 'status' => 'pending'
				}
			}
	};

	my $q = &build_sql($insert_q, $site, $site->{dbh}, 0);
	my $rv = &query($q, $site->{dbh});
	
	warn "Inserted new PO. RV was <<$rv>>\n";
	unless ($rv) {
		return "DB error. Could not create new PO.";
	}
	
	# Add PO extraprops
	my @props = qw(vendor contact team);
	foreach my $eprop (@props) {
		$rv = &create_extraprop($objid, $eprop, $form->{$eprop}, $site);
	}
	
	return;
}

sub insert_po_item {
	my ($form, $site) = @_;
	
	my $objid = &get_new_objid($site, $site->{dbh}, 'poline');
	return "Could not create object id" unless ($objid);
	
	my $insert_q = {
		'statement' => 'insert',
		'TABLE' =>
			{
			 'NAME' => 'poline',
			 'FIELDS' => 
			 	{
				 'objid' => $objid,
				 'ponum' => $form->{'ponum'},
				 'tolerance' => $form->{'overage'},
				 'quant' => $form->{'qty'},
				 'sku' => $form->{'sku'},
				 'status' => 'open'
				}
			}
	};

	my $q = &build_sql($insert_q, $site, $site->{dbh}, 0);
	my $rv = &query($q, $site->{dbh});
	
	&create_extraprop($objid, 'vprice', $form->{'vprice'}, $site);
	&create_extraprop($objid, 'rprice', $form->{'rprice'}, $site);

	warn "Inserted new po line. RV was <<$rv>>\n";
	
	unless ($rv) {
		return "DB error. Could not create PO Item.";
	}
	return;
}


sub po_edit_cell {
	my ($site, $form) = @_;
	my $ponum = $form->{'ponum'};
	
	return '' if ($form->{'save.finalize'});
	
	my $current_cell = ($ponum) ? 'wm_cell_po_item' : 'wm_cell_po_main';
	my $cell;

	$cell = &localize_template($current_cell,$site->{'current_interface'},0,$site);
	$cell = &permutebit( $cell, $form );
	return $cell;	
}

sub current_po {
	my ($site, $form) = @_;
	warn "Getting current po ($form->{'ponum'})\n";
	
	return '' unless ($form->{'ponum'});
	
	
	my $curr = &po_head($form, $site);
	$curr .= &po_lines($form, $site);

	return $curr;
}


sub po_head {
	my ($form, $site) = @_;
	my $ponum = $form->{'ponum'};
	
	my $po = &get_po_head($ponum, $site);
	
	my $cell = &localize_template('wm_cell_po_disp_head',$site->{'current_interface'},0,$site);
	$cell = &permutebit( $cell, $po );

	return $cell;
}


sub get_po_head {
	my ($ponum, $site) = @_;
	
	my @where;
	$where[0] = &build_where_hash('ponum', $ponum, 'and');
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'po',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'ponum' => 'ponum',
				 'wh'    => 'wh',
				 'canceldate' => 'canceldate',
				 'objid' => 'objid',
				 'status' => 'status'
				}
			}
	};
	
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "Query is $q\n";
	
	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "Rows returned: $rows\n";
	my $po = $results->[0];
	$po->{'canceldate'} = sprintf("%08d", $po->{'canceldate'});
	
	my @props = &fetch_extraprops($po->{'objid'}, $site);
	
	return $po unless ($props[0] && ref($props[0]) eq 'HASH');
	foreach my $prop (@props) {
		$po->{$prop->{'fieldname'}} = $prop->{'body'};
	}
	
	return $po;
}	
	

sub po_lines {
	my ($form, $site) = @_;
	my $ponum = $form->{'ponum'};
	
	my $po = &get_po_head($ponum, $site);
	
	my $lines = [];
	$lines = &get_po_lines($ponum, $site);
	
	my $line_cells;
	my $cell = &localize_template('wm_cell_po_disp_item',$site->{'current_interface'},0,$site);
	
	foreach my $line (@{$lines}) {
		next unless $line->{'objid'};
		warn "Line SKU $line->{sku}\n";
		my $this_cell = &permutebit($cell, $line);
		$line_cells .= &permutebit($this_cell, $line->{'sku_data'});
		
	}
	
	my $fin_button;
	if ($po->{'status'} eq 'pending') {
		$fin_button = &localize_template('wm_cell_po_finalize_button',$site->{'current_interface'},0,$site);
		$fin_button = &permutebit($fin_button, $form);
	}
	
	my $ret;
	if ($line_cells) {
		my $h = {};
		$h->{items} = $line_cells;
		$h->{finalize} = $fin_button;
		
		$cell = &localize_template('wm_cell_po_disp_item_head',$site->{'current_interface'},0,$site);
		$ret = &permutebit($cell, $h);
	}
	
	return $ret;
}

sub get_po_lines {
	my ($ponum, $site) = @_;
	
	my @where;
	$where[0] = &build_where_hash('ponum', $ponum, 'and');	
	
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'poline',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 '*' => '*'
				}
			}
	};
	
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "Query is $q\n";
	
	my @results;
	my $rows = &query($q, $site->{dbh}, \@results);
	warn "Rows returned: $rows\n";
	
	my @return;
	foreach my $line (@results) {
		next unless ($line->{'objid'});
		my @poprops = &fetch_extraprops($line->{'objid'}, $site);
		foreach my $prop (@poprops) {
			$line->{$prop->{'fieldname'}} = $prop->{'body'};
		}
		$line->{'sku_data'} = &get_sku($line->{'sku'}, $site);
		push @return, $line;
	}
	
	return \@return;
}	
	
sub list_open_pos {
	my ($site, $form) = @_;
	my $cells;
	my $onecell=localize_template('wm_cell_open_po',$site->{'current_interface'},0,$site);
	my $row;
	
	my $results = [];
	$results = &get_open_pos($site, $form);
	warn "Results: $#{$results}\n";
	
	unless ($#{$results} >= 0) {
		$row->{'po_link'} = 'No open POs';
		$cells = &permutebit($onecell, $row);
		return $cells;
	}
	
	foreach my $line (@{$results}) {
		warn "Doing cell\nResult $line->{ponum}";
		$row->{'po_link'} = "$line->{'ponum'} (Objid $line->{'objid'})";
		$cells .= &permutebit($onecell, $row) if ($line->{'ponum'} && $line->{'objid'});
	}
	
	return $cells;
}

sub get_open_pos {
	my ($site, $form)  = @_;
	my @where;
	$where[0] = &build_where_hash('status', 'open', 'and');
	my $select_q = {
		'statement' => 'select',
		'TABLE' => 
			{
			 'NAME'   => 'po',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'ponum' => 'ponum',
				 'objid' => 'objid'
				}
			}
	};
	
	my $q = &build_sql($select_q, $site, $site->{dbh}, 0);
	warn "Query is $q\n";
	
	my $results = [];
	my $rows = &query($q, $site->{dbh}, $results);
	warn "Rows returned: $rows\n";
	return $results;
}


sub po_finalize {
	my ($site, $form) = @_;
	
	return '' unless $form->{'save.finalize'};
	
	# Change the PO's status to open
	my $rv = &change_po_status($form->{'ponum'}, 'open', $site);

	if ($rv) {
		return "Status of PO $form->{'ponum'} now open.";
	} else {
		return "Could not finalize PO $form->{'ponum'}.";
	}
}


sub change_po_status {	
	my ($ponum, $status, $site) = @_;
	
	my @where;
	$where[0] = &build_where_hash('ponum', $ponum, 'and');
	
	my $update_q = {
		'statement' => 'update',
		'TABLE' => 
			{
			 'NAME'   => 'po',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'status' => $status,
				}
			}
	};
	my $q = &build_sql($update_q, $site, $site->{dbh}, 0);
	
	warn "Query is $q\n";
	my $rv = &query($q, $site->{dbh});
	return $rv;
}
	
	
sub change_poline_status {
	my ($ponum, $sku, $status, $site) = @_;
	
	my @where;
	$where[0] = &build_where_hash('ponum', $ponum, 'and');
	$where[1] = &build_where_hash('sku', $sku, 'and');
	
	my $update_q = {
		'statement' => 'update',
		'TABLE' => 
			{
			 'NAME'   => 'poline',
			 'WHERE'  => \@where,
			 'FIELDS' => 
			 	{
				 'status' => $status,
				}
			}
	};
	my $q = &build_sql($update_q, $site, $site->{dbh}, 0);
	
	warn "Query is $q\n";
	my $rv = &query($q, $site->{dbh});
	return $rv;
}
	

1;
