use strict;

sub check_for_po {
	my ($ponum, $site) = @_;
	
	my $po = &get_po_head($ponum, $site);
	return "Cannot find PO $ponum." unless ($po->{'objid'});
	return '';
}

sub receive_po {
	my ($site, $form) = @_;
	
	my $polines = &get_po_lines($form->{'ponum'}, $site);
	
	my $line_cells;
	my $cell = &localize_template('wm_cell_receive_po_line',$site->{'current_interface'},0,$site);
	
	foreach my $line (@{$polines}) {
		next unless $line->{'objid'};
		next unless ($line->{'status'} ne 'received');
		warn "Line SKU $line->{sku}\n";
		$line->{'oquant'} = $line->{'quant'};
		$line_cells .= &permutebit($cell, $line);
	}
	
	my $ret;
	if ($line_cells) {
		my $h = {};
		$h->{'po_lines'} = $line_cells;
		$cell = &localize_template('wm_cell_receive_po_head',$site->{'current_interface'},0,$site);
		$ret = &permutebit($cell, $h);
	}
	
	return $ret;
}
	
sub check_receiving_input {
	my ($form, $site) = @_;
	
	return '';
}


sub receive_po_line {
	my ($form, $site) = @_;
	
	my $sku = $form->{'sku'};
		
	my $rv = &create_boxes($site, $form);
	return "Error receiving $form->{'sku'}." unless ($rv);
	
	# Expire the line from the po
	$rv = &change_poline_status($form->{'ponum'}, $form->{'sku'}, 'received', $site);
	return "Error receiving $form->{'sku'}." unless ($rv);
	
	# Set the po status to partially received
	$rv = &change_po_status($form->{'ponum'}, 'part_recv', $site);
	return "Error receiving $form->{'sku'}." unless ($rv);
	
	return '';
}

sub create_boxes {
	my ($site, $form) = @_;
	
	while ($form->{'numboxes'} > 0) {
		my ($rv, $err) = &make_one_box($form, $site);
		return $err if ($err);
		$form->{'numboxes'}--;
	}
	
	return 1;
}


sub make_one_box {
	my ($form, $site) = @_;
	
	my $ponum = $form->{'ponum'};
	warn "making new box for po $ponum\n";
	
	my $item = &get_sku($form->{'sku'}, $site);
	my $id = &make_id($site);
	
	warn "Creating box $id for item $item->{'objid'} (sku: $item->{'msku'})\n";

	my $box = {
		'objid'   => $id,
		'objclass' => 'wmsbox',
		'msku' => $item->{'msku'},
		'quant' => $form->{'pbquant'},
		'rec_by' => $form->{'uid'},
		'ponum' => $form->{'ponum'},
		'status' => 'new',
		'nosave' => 	
			{
			'objid' => 1,
			'msku' => 1,
			'objclass' => 1,
			}
	};
	
	my ($box, $err) = &store_box($site, $box);
	
	return ($box->{'objid'}, $err);
}

sub find_new_boxes {
	my ($form, $site) = @_;
	
	warn "Finding new boxes\n";
	
	my $boxes = &get_boxes_by_po($site, $form->{'ponum'});
		
	my $cell = &localize_template('wm_cell_new_box_line',$site->{'current_interface'},0,$site);
	my $box_lines;
	foreach my $box (@{$boxes}) {
		next unless ($box->{'status'} eq 'new');
		$box_lines .= &permutebit($cell, $box);
	}
	
	return '' unless ($box_lines);
	
	$cell = &localize_template('wm_cell_new_box_head',$site->{'current_interface'},0,$site);
	my $h = {'new_box_lines' => $box_lines};
	my $ret = &permutebit($cell, $h);
	
	return $ret;
}
	

sub new_box_report {
	my $site = shift;
	
	my $boxids = &get_objids_by_extraprop($site, 'status', 'new', 'cdate ASC');
	my $boxes = &get_boxes($site, $boxids);
	
	my $cell = &localize_template('wm_cell_box_new_line', $site->{'current_interface'},0,$site);
	my $h = {};
	foreach my $box (@{$boxes}) {
		next unless ($box->{'objclass'} eq 'wmsbox');
		$h->{'boxes'} .= permutebit($cell, $box);
	}
	
	$cell = &localize_template('wm_cell_box_new_head', $site->{'current_interface'},0,$site);
	my $return = &permutebit($cell, $h);
	
	return $return;
}

sub today_report {
	my $site = shift;
	my $day = shift;

	my $boxes = &get_boxes_by_date($site, $day);
	
	my $cell = &localize_template('wm_cell_box_today_line', $site->{'current_interface'},0,$site);
	my $h = {};
	foreach my $box (@{$boxes}) {
		next unless ($box->{'objclass'} eq 'wmsbox');
		$h->{'boxes'} .= permutebit($cell, $box);
	}
	
	$cell = &localize_template('wm_cell_box_today_head', $site->{'current_interface'},0,$site);
	my $return = &permutebit($cell, $h);
	
	return $return;
}

sub date_po_report {
	my $site = shift;
	my $ponum = shift;
	my $day = shift;
	
	my $boxes = &get_boxes_by_date($site, $day);
	my $poboxes = [];	
	
	foreach my $box (@{$boxes}) {
		next unless ($box->{'ponum'} eq $ponum);
		push @{$poboxes}, $box;
	}
	
	my $cell = &localize_template('wm_cell_box_today_line', $site->{'current_interface'},0,$site);
	my $h = {};
	foreach my $box (@{$poboxes}) {
		$h->{'boxes'} .= permutebit($cell, $box);
	}
	
	$cell = &localize_template('wm_cell_box_today_head', $site->{'current_interface'},0,$site);
	my $return = &permutebit($cell, $h);
	
	return $return;
}


sub get_rept_ponum {
	my $site = shift;
	my $form = shift;
	
	my $cell = &localize_template('wm_cell_box_enter_ponum', $site->{'current_interface'},0,$site);
	return &permutebit($cell, $form);
}

sub enter_boxid {
	my ($site, $form) = @_;
	my $cell = &localize_template('wm_cell_enter_boxid', $site->{'current_interface'},0,$site);
	return &permutebit($cell, $form);
}


sub box_edit_form {
	my ($site, $boxid) = @_;
	
	my $cell = &localize_template('wm_cell_edit_box_details', $site->{'current_interface'},0,$site);
	my $box = &get_one_box($site, $boxid);
	
	my $status_sel = &localize_template('wm_cell_box_status_select', $site->{'current_interface'},0,$site);
	my $h = {$box->{'status'} => ' SELECTED'};
	$box->{'status_select'} = &permutebit($status_sel, $h);

	return &permutebit($cell, $box);
}

sub edit_box_save {
	my ($site, $form) = @_;
	return unless ($form->{'box.save'});
	
	my $boxid = $form->{'boxid'};
	my $box = &get_one_box($site, $boxid);
	
	return "Can't find box $boxid" unless ($box->{'objid'});
	warn "Saving changes to box $boxid\n";
	
	my $updates = {
		'status' => $form->{'status'},
		'ponum' => $form->{'ponum'},
		'msku' => $form->{'msku'},
		'quant' => $form->{'quant'}
	};
	
	$box->{'edit'} = [] unless (ref($box->{'edit'}) eq 'ARRAY');
	my $changes = {};
	my $changed = 0;
	foreach my $field (keys %{$updates}) { 
		if (($updates->{$field} ne '') && ($updates->{$field} ne $box->{$field})) {
			warn "$field changed! Old: $box->{$field}, New: $updates->{$field}\n";
			my $editid = &make_id($site);
			$changes = 
				{'editid' => $editid,
				 'field'=> $field,
				 'from' => $box->{$field},
				 'to'   => $updates->{$field},
				 'by'   => $form->{'uid'}};
			unshift @{$box->{'edit'}}, $changes;
			warn "Edits now:\n";
			foreach my $e (@{$box->{'edit'}}) {
				warn "\t$e->{'editid'}\n";
			}
			
			$box->{$field} = $updates->{$field};
			$changed = 1;
		}
	}

	return unless ($changed);
	
	&store_box($site, $box);
		
	return;
}


sub box_status_form {
	my ($site, $form) = @_;
	
	my $select = &localize_template('wm_cell_box_status_select', $site->{'current_interface'},0,$site);
	$select = &permutebit($select, {});
	$form->{'status_select'} = $select;
	
	my $cell = &localize_template('wm_cell_box_status_change', $site->{'current_interface'},0,$site);
	$cell = &permutebit($cell, $form);
	
	return $cell;
}


sub change_box_status {
	my ($site, $form) = @_;
	my $status = $form->{'status'};
	my $boxid = $form->{'boxid'};
	my $uid = $form->{'uid'};
	
	warn "Changing box status for $boxid\n";
	my $box = &get_one_box($site, $boxid);
	
	return "Please fill out all fields\n" unless ($uid ne '' && $status ne '' && $boxid ne '');
	return "Cannot find box $boxid\n" unless ($box->{'objid'});
	return "Box $boxid is already marked \"$status\"\n" if ($box->{'status'} eq $status);

	
	my $editid = &make_id($site);
	my $change = {
		'editid' => $editid,
		'field' => 'status',
		'from' => $box->{'status'},
		'to' => $status,
		'by' => $uid
	};

	$box->{'status'} = $status;
	unshift @{$box->{'edit'}}, $change;
	
	my ($box, $err) = &store_box($site, $box);
	return $err if ($err);
	return "Box $boxid now marked \"$status\"\n";
}


sub find_boxes_by_sku {
	my ($site, $form) = @_;
	my $sku = $form->{'sku'};
	my $props = {
		1 => {
			   'field' => 'msku',
			   'value' => $sku,
			   'join'  => 'and'
			  },
		2 => {
			   'field' => 'objclass',
			   'value' => 'wmsbox',
			   'join'  => 'and'
			  }
		
	};
	my $boxids = &get_objids_by_property($site, $props, 0);
	
	my $boxes = &get_boxes($site, $boxids);
	
	my (@active, @reserve);
	
	foreach my $box (@{$boxes}) {
		push @active, $box if ($box->{'status'} eq 'active');
		push @reserve, $box if ($box->{'status'} eq 'reserve');
	}
	
	my $out = &localize_template('wm_cell_find_sku_head',$site->{'current_interface'},0,$site);
	my $line = &localize_template('wm_cell_find_sku_line',$site->{'current_interface'},0,$site);

	foreach my $box (@active) {
		$box->{'curr_location'} = '[unknown]';
		$box->{'curr_location'} = $box->{'location'}->[0]->{'loc'}
			if ($box->{'location'}->[0]->{'loc'});
		$form->{'active'} .= &permutebit($line, $box);
	}
	foreach my $box (@reserve) {
		$box->{'curr_location'} = '[unknown]';
		$box->{'curr_location'} = $box->{'location'}->[0]->{'loc'}
			if ($box->{'location'}->[0]->{'loc'});
		$form->{'reserve'} .= &permutebit($line, $box);
	}
	
	return &permutebit($out, $form);
}


sub list_bin_contents {
	my ($site, $form) = @_;
	my $binid = $form->{'bin'};
	# Get the bin, find out if it's reserve or active
	my $bin = &get_one_bin($site, $binid);
	
	return "Cannot find bin $binid" unless ($bin->{'objid'});
	
	my $contents;
	if ($bin->{'status'} eq 'active') {
		$contents = &list_act_bin_contents($site, $form, $bin);
	} else {
		$contents = &list_res_bin_contents($site, $form, $bin->{'objid'});
	} 
	
	return $contents;
}

sub list_act_bin_contents {
	my ($site, $form, $bin) = @_;
	
	my $binline = &localize_template('wm_cell_find_bin_line',$site->{'current_interface'},0,$site);
	my $binhead = &localize_template('wm_cell_find_bin_head',$site->{'current_interface'},0,$site);
	$form->{'bin_lines'} = &permutebit($binline, $bin);
	
	return &permutebit($binhead, $form);
}

sub list_res_bin_contents {
	my ($site, $form, $bin) = @_;

	# Tricky. First, we have to find the location ids
	my $locations = &get_objids_by_extraprop($site, 'loc', $bin);
	
	# Then, from those, get boxids
	my @boxids;
	foreach my $lid (@{$locations}) {
		next unless ($lid);
		my $this_box = &get_objids_by_extraprop($site, 'locationid', $lid);
		push @boxids, @{$this_box};
	}
	
	my $count = $#boxids + 1;
	warn "Found $count boxes in location $bin\n";

	# Now get boxes
	my $boxes = &get_boxes($site, \@boxids);
	
	my $binline = &localize_template('wm_cell_find_bin_line',$site->{'current_interface'},0,$site);
	my $binhead = &localize_template('wm_cell_find_bin_head',$site->{'current_interface'},0,$site);

	foreach my $box (@{$boxes}) {
		next unless ($box->{'objid'});
		$form->{'bin_lines'} .= &permutebit($binline, $box);
	}
	
	return &permutebit($binhead, $form);
}


1;
